package de.ubt.wm.sb.junit;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ubt.wm.sb.exceptions.StateResetException;
import de.ubt.wm.sb.model.BallProperty;
import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Hit;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.Move;
import de.ubt.wm.sb.model.PlayerAction;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.TeamAction;
import de.ubt.wm.sb.model.strategy.Strategy;

public class BlockerStateTest {

	private Game g;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		g = Game.getInstance();
		g.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", Strategy.BasicTeamA,
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", Strategy.BasicTeamB);
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void ballUebersNetz() {
		for (int i = 0; i< 50; i++){
			State s = new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.B22, null, 2, null, BallProperty.NORMAL, g.getTeamA().getPlayer1());
			g.setCurrentState(s);
			
			TeamAction tA_A = TeamAction.doNothing;
			TeamAction tA_B = new TeamAction(new PlayerAction(new Hit(HitTechnique.F_E, Field.A11), Move.stay), PlayerAction.doNothing);
			
			try {
				g.getTeamA().setTeamAction(tA_A);
				g.getTeamB().setTeamAction(tA_B);
				g.computeNextState();
			} catch (StateResetException e){
				continue;
			} catch (Exception e) {
				e.printStackTrace();
			}
			if (g.getCurrentState().ballOverNet()){
				assertTrue("Blocker wird auf null gesetzt bei neuem Angriff", g.getCurrentState().getBlocker() == null);
			}else{
				// wenn Zustand zurückgesetzt wurde, dann kann sich Blocker trotzdem noch ändern
				assertTrue("Blocker bleibt gleich wenn nichts durch Strategie geändert wird. State " + g.getCurrentState().toString() 
						+ "; Blocker sollte A1 sein.", 
						g.getCurrentState().getBlocker() == g.getTeamA().getPlayer1());
			}
		}
	}
	
	@Test
	public void gesetzterBlockerNichtAendern(){
		for (int i = 0; i< 50; i++){
			State s = new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.B22, null, 1, null, BallProperty.NORMAL, g.getTeamA().getPlayer1());
			g.setCurrentState(s);
			
			TeamAction tA_A = TeamAction.doNothing;
			TeamAction tA_B = new TeamAction(new PlayerAction(new Hit(HitTechnique.s, Field.B23), Move.stay), PlayerAction.doNothing);
			try {
				g.getTeamA().setTeamAction(tA_A);
				g.getTeamB().setTeamAction(tA_B);
				g.computeNextState();
			} catch (Exception e) {
				e.printStackTrace();
			}
			if (g.getCurrentState().ballOverNet()){
				assertTrue("Blocker wird auf null gesetzt bei neuem Angriff", g.getCurrentState().getBlocker() == null);
			}else{
				assertTrue("Blocker bleibt gleich wenn nichts durch Strategie geändert wird", g.getCurrentState().getBlocker() == g.getTeamA().getPlayer1());
			}
		}
		
	}

}
