package de.ubt.wm.sb.junit;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ubt.wm.sb.model.BallProperty;
import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Hit;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.strategy.Strategy;

public class CheckIfHitIsAllowedTest {
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", Strategy.BasicTeamA,
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", Strategy.BasicTeamB);
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void ServeTest() {
		Game game = Game.getInstance();
		State start_state = new State(Field.A02, Field.A13, Field.B12, Field.B13, Field.A02, Game.getInstance().getTeamA().getPlayer1(), -1, null, BallProperty.NORMAL, null);
		State.START_STATES_A1 = new State[]{start_state};
	
		game.setCurrentState(start_state);
		
		assertTrue("Aufschlag A1 S_J target B12", game.getTeamA().getPlayer1().isHitAllowed(new Hit(HitTechnique.S_J, Field.B12)));
		assertFalse("Aufschlag A1 S_J target B32", game.getTeamA().getPlayer1().isHitAllowed(new Hit(HitTechnique.S_J, Field.B32)));
	}
	
	@Test
	public void ReceptionTest(){
		Game game = Game.getInstance();
		game.setCurrentState(new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.B33, game.getTeamA().getPlayer1(), -1, null, BallProperty.NORMAL, null));
		
		assertTrue("Annahme von B1", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.r_m, Field.B23)));
		assertFalse("Annahme von B1 mit r obwohl Ball im Nachbarfeld", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.r, Field.B23)));
		assertFalse("Annahme von B1 ZF andere Seite", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.r_m, Field.A23)));
		assertTrue("Annahme von B1 ZF weit weg Seite", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.r_m, Field.B11)));
		assertFalse("Annahme von A1 nicht erlaubt", game.getTeamA().getPlayer1().isHitAllowed(new Hit(HitTechnique.r_m, Field.A23)));
		
		game.getCurrentState().setCounter(0);
		assertFalse("Annahme von B1 nicht erlaubt wegen Counter = 0", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.r_m, Field.B23)));
	}
	
	@Test
	public void DefenceTest(){
		Game game = Game.getInstance();
		game.setCurrentState(new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.B33, game.getTeamA().getPlayer1(), 0, null, BallProperty.NORMAL, null));
		
		assertTrue("Defence von B1", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.d_m, Field.B23)));
		assertFalse("Defence von B1 mit r obwohl Ball im Nachbarfeld", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.d, Field.B23)));
		assertFalse("Defence von B1 ZF andere Seite", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.d_m, Field.A23)));
		assertTrue("Defence von B1 ZF weit weg Seite", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.d_m, Field.B11)));
		assertFalse("Defence von A1 nicht erlaubt", game.getTeamA().getPlayer1().isHitAllowed(new Hit(HitTechnique.d_m, Field.A23)));
		
		game.getCurrentState().setCounter(-1);
		assertFalse("Defence von B1 nicht erlaubt wegen Counter = 0", game.getTeamB().getPlayer1().isHitAllowed(new Hit(HitTechnique.d_m, Field.B23)));
	}
	
	@Test
	public void SettingTest(){
		Game game = Game.getInstance();
		game.setCurrentState(new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.B23, game.getTeamA().getPlayer1(), 1, null, BallProperty.NORMAL, null));
		
		assertTrue("Zuspiel von B2", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.s, Field.B13)));
		assertFalse("Zuspiel von B2 - ZF zu weit weg", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.s, Field.B31)));
		assertFalse("Zuspiel von B2 - ZF andere Seite", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.s, Field.A31)));
		assertFalse("Zuspiel von A2 - Ball andere Seite", game.getTeamA().getPlayer2().isHitAllowed(new Hit(HitTechnique.s, Field.A31)));
		
		game.setCurrentState(new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.B23, game.getTeamA().getPlayer1(), 0, null, BallProperty.NORMAL, null));
		assertFalse("Zuspiel von B2 - counter aber 0", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.s, Field.B13)));
	}
	
	@Test
	public void FieldAttackTest(){
		Game game = Game.getInstance();
		game.setCurrentState(new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.B23, game.getTeamA().getPlayer1(), 1, null, BallProperty.NORMAL, null));
		
		assertTrue("Poke von B2", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_P, Field.A23)));
		assertFalse("Shot von B2 aber counter 1", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_E, Field.A23)));
		assertFalse("Smash von B2 aber counter 1", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_SM, Field.A23)));
		
		game.getCurrentState().setCounter(2);
		
		assertTrue("Shot von B2", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_E, Field.A23)));
		assertTrue("Smash von B2", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_SM, Field.A23)));
		assertFalse("Smash von B2 - falsches ZF", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_SM, Field.A32)));
		
		game.getCurrentState().setFieldPosBall(Field.B33);
		
		assertTrue("Shot von B2 Ball vor ihm", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_E, Field.A23)));
		assertTrue("Smash von B2 Ball vor ihm", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_SM, Field.A23)));
		
		game.getCurrentState().setFieldPosBall(Field.B24);
		
		assertTrue("Shot von B2 Ball neben ihm", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_E, Field.A23)));
		assertFalse("Smash von B2 Ball neben ihm", game.getTeamB().getPlayer2().isHitAllowed(new Hit(HitTechnique.F_SM, Field.A23)));
		
		game.getCurrentState().setFieldPosBall(Field.A32);
		
		assertTrue("Smash von A1 Ball vor ihm", game.getTeamA().getPlayer1().isHitAllowed(new Hit(HitTechnique.F_SM, Field.B23)));
		
		game.getCurrentState().setFieldPosBall(Field.A21);
		
		assertFalse("Smash von A1 Ball neben ihm", game.getTeamA().getPlayer1().isHitAllowed(new Hit(HitTechnique.F_SM, Field.B23)));
		
	}
	
	
	
}
