package de.ubt.wm.sb.junit;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ubt.wm.sb.model.BallProperty;
import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Hit;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.Move;
import de.ubt.wm.sb.model.PlayerAction;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.TeamAction;
import de.ubt.wm.sb.model.strategy.Strategy;

public class CheckIfTeamActionIsAllowedTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", Strategy.BasicTeamA,
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", Strategy.BasicTeamB);
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void TwoPlayersAreHitting() {
		Game g = Game.getInstance();
		State s = new State(Field.A22, Field.A23, Field.B22, Field.B23, Field.A22, Game.getInstance().getTeamA().getPlayer1(), 0, null, BallProperty.NORMAL, null);
		g.setCurrentState(s);
		// some Random tested things
		TeamAction tA = new TeamAction(new PlayerAction(new Hit(HitTechnique.d, Field.A23), Move.stay), new PlayerAction(new Hit(HitTechnique.d_m, Field.A23), Move.stay));
		assertFalse("TeamAction - es duerfen nicht beide Spielder den Ball schlagen", g.getTeamA().isTeamActionAllowed(tA));
		tA = new TeamAction(new PlayerAction(new Hit(HitTechnique.d, Field.A23), Move.stay), new PlayerAction(Hit.nohit, Move.stay));
		assertTrue("TeamAction - nur einer schlaegt ist erlaubt", g.getTeamA().isTeamActionAllowed(tA));
	}
	
	
	@Test
	public void blockingActionOfOnePlayer(){
		Game g = Game.getInstance();
		State s = new State(Field.A22, Field.A23, Field.B32, Field.B23, Field.A23, Game.getInstance().getTeamB().getPlayer1(),
				2, Game.getInstance().getTeamA().getPlayer1(), BallProperty.NORMAL, g.getTeamB().getPlayer1());
		g.setCurrentState(s);
		TeamAction tA = new TeamAction(new PlayerAction(Hit.nohit, Move.b), new PlayerAction(Hit.nohit, Move.stay));
		assertTrue("TeamAction - einer blockt erlaubt", g.getTeamB().isTeamActionAllowed(tA));
	}
	
	@Test
	public void aufschlagWithMove(){
		Game g = Game.getInstance();
		g.setCurrentState(State.chooseRandomStateOfArray(State.START_STATES_A1));
		TeamAction tA = new TeamAction(new PlayerAction(new Hit(HitTechnique.S_F, Field.B22), Move.m_f), new PlayerAction(Hit.nohit, Move.stay));
		assertTrue("TeamAction - Aufschlag mit move south", g.getTeamA().isTeamActionAllowed(tA));
	}

}
