package de.ubt.wm.sb.junit;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;

import org.junit.Before;
import org.junit.Test;

import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.strategy.Strategy;

public class ServingOrder {
	
	 @Before
	    public void setUp() {
			Game g = Game.getInstance();
			g.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", Strategy.BasicTeamA,
			"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", Strategy.BasicTeamB);
			// Testreihenfolge A2, B2, A1, B1
			g.setServingOrder(new ArrayList<Player>(Arrays.asList(g.getTeamA().getPlayer2(), g.getTeamB().getPlayer2(), g.getTeamA().getPlayer1(), g.getTeamB().getPlayer1())));
//			System.out.println("1: " + g.getTeamA().getPlayer2().getId());
	    }
	 

	@Test 
	public void chooseNextServingPlayerTest(){
		Game g = Game.getInstance();
		
		Player nextServer = g.chooseNextServingPlayer(g.getTeamA().getPlayer1());
		assertSame("chooseNextServingPlayer gibt richtigen Player zurück", g.getTeamB().getPlayer1(), nextServer);
		
		nextServer = g.chooseNextServingPlayer(g.getTeamB().getPlayer1());
		assertSame("chooseNextServingPlayer gibt richtigen Player zurück", g.getTeamA().getPlayer2(), nextServer);
	}
	
	@Test
	public void getNextServingStateTest(){
		Game g = Game.getInstance();
//		System.out.println("2: " + g.getTeamA().getPlayer2().getId());
		g.setCurrentState(State.POINT_B_A2);
//		System.out.println(State.POINT_B_A2.hashCode());
//		System.out.println(State.POINT_A_A2.hashCode());
//		System.out.println("3: " + State.POINT_B_A2.getLastServer().getId());
		State nextServingState = g.getNextServingState(g.getCurrentState());
		assertTrue("getNextServingState nach PunktB-A2 gibt richtigen State zurück", Arrays.asList(State.START_STATES_B2).contains(nextServingState));
		assertSame("richtige Aufschlagspieler nach A2", nextServingState.getLastServer(), g.getTeamB().getPlayer2());
		
		g.setCurrentState(State.POINT_A_B2);
		nextServingState = g.getNextServingState(g.getCurrentState());
		assertTrue("getNextServingState nach PunktA-B2 gibt richtigen State zurück", Arrays.asList(State.START_STATES_A1).contains(nextServingState));
		assertSame("richtige Aufschlagspieler nach B2", nextServingState.getLastServer(), g.getTeamA().getPlayer1());
		
		g.setCurrentState(State.POINT_B_A1);
		nextServingState = g.getNextServingState(g.getCurrentState());
		assertTrue("getNextServingState nach PunktB-A1 gibt richtigen State zurück", Arrays.asList(State.START_STATES_B1).contains(nextServingState));
		assertSame("richtige Aufschlagspieler nach A1", nextServingState.getLastServer(), g.getTeamB().getPlayer1());
		
		g.setCurrentState(State.POINT_A_B1);
		nextServingState = g.getNextServingState(g.getCurrentState());
		assertTrue("getNextServingState nach PunktA-B1 gibt richtigen State zurück", Arrays.asList(State.START_STATES_A2).contains(nextServingState));	
		assertSame("richtige Aufschlagspieler nach B1", nextServingState.getLastServer(), g.getTeamA().getPlayer2());
	}
	

}
