package de.ubt.wm.sb.junit.sim;

import static org.junit.Assert.*;

import java.util.Arrays;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ubt.wm.sb.model.BallProperty;
import de.ubt.wm.sb.model.BlockTechnique;
import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.strategy.Strategy;
import de.ubt.wm.sb.probs.Prob_Attack;
import de.ubt.wm.sb.probs.Prob_Block;
import de.ubt.wm.sb.probs.Prob_DefRec;
import de.ubt.wm.sb.probs.Prob_Hit;
import de.ubt.wm.sb.probs.Prob_Serve;
import de.ubt.wm.sb.sim.MainSim;
import de.ubt.wm.sb.sim.SimulationManager;

public class ExtremeProbTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void extremeServingProbs_TeamA() {
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", Strategy.cali_finalMatch_TeamA_read,
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", Strategy.cali_finalMatch_TeamB_read);
		
		for (Field f : Field.INSIDE_B){
			if (!Arrays.asList(Field.AT_THE_NET_B).contains(f)){
				try {
					game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F, 100, null,  0, 1, f));
					game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J,100,null, 0, 1, f));
					game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F, 100,null, 0, 1, f));
					game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J,100,null, 0, 1, f));
				} catch (Exception e) {
					e.printStackTrace();
				};
			}
		}
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
				
		SimulationManager simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
		assertTrue("direct point prob serve is zero", simManager.getServeP() < 0.000000001);
		assertTrue("direct fault prob serve is one", simManager.getServePOverline() > 0.99999);
		
		simManager.reset();
		
		for (Field f : Field.INSIDE_B){
			if (!Arrays.asList(Field.AT_THE_NET_B).contains(f)){
				try {
					game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F,100,null, 1, 0, f));
					game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J, 100,null,1, 0, f));
					game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F,100, null,1, 0, f));
					game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J, 100,null,1, 0, f));
				} catch (Exception e) {
					e.printStackTrace();
				};
			}
		}
				
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
		
		simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
		assertTrue("direct point or go on prob serve is one", simManager.getServeP()+(1-simManager.getServeP()-simManager.getServePOverline()) > 0.999999);
		assertTrue("direct fault prob serve is zero", simManager.getServePOverline() < 0.00001);
	}

	@Test
	public void extremeServingProbs_TeamB() {
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamA(),
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamB());
		
		for (Field f : Field.INSIDE_A){
			if (!Arrays.asList(Field.AT_THE_NET_A).contains(f)){
				try {
					game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F,100,null, 0, 1, f));
					game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J,100,null, 0, 1, f));
					game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F,100,null, 0, 1, f));
					game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J,100,null, 0, 1, f));
				} catch (Exception e) {
					e.printStackTrace();
				};
			}
		}
				
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
		
		SimulationManager simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
		assertTrue("direct point prob serve is zero", simManager.getServeQ() < 0.000000001);
		assertTrue("direct fault prob serve is one", simManager.getServeQOverline() > 0.99999);
		
		simManager.reset();
		
		for (Field f : Field.INSIDE_A){
			if (!Arrays.asList(Field.AT_THE_NET_A).contains(f)){
				try {
					game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F,100,null, 1, 0, f));
					game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J,100,null, 1, 0, f));
					game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_F,100,null, 1, 0, f));
					game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Serve(HitTechnique.S_J,100,null, 1, 0, f));
				} catch (Exception e) {
					e.printStackTrace();
				};
			}
		}
		
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
				
		simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
		assertTrue("direct point or go on prob serve is one", simManager.getServeQ()+(1-simManager.getServeQ()-simManager.getServeQOverline()) > 0.999999);
		assertTrue("direct fault prob serve is zero", simManager.getServeQOverline() < 0.00001);
	}
	
	@Test
	public void extremeFieldAttackProbs_TeamA() {
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamA(),
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamB());
		
		for (Field f : Field.INSIDE_B){
			for (Field pf : Field.INSIDE_A){
					try {
						game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P, 100,null,0, 1, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_B).contains(f)){
							game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E,100,null, 0, 1, f, pf));
							game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100,null, 0, 1, f, pf));
						}
						game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P,100,null, 0, 1, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_B).contains(f)){
							game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E,100,null, 0, 1, f, pf));
							game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100,null, 0, 1, f, pf));
						}
					} catch (Exception e) {
						e.printStackTrace();
					};
				}
		}
		
//		System.out.println(game.getTeamA().getPlayer1().getIndProbs().printAnalogToExcelInputFormat(game.getTeamA().getPlayer1()));
		
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
		
		SimulationManager simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
		assertTrue("direct point prob fA is zero", simManager.getfAP() < 0.000000001);
		assertTrue("direct fault prob fA is one", simManager.getfAPOverline() > 0.99999);
		
		simManager.reset();
		
		for (Field f : Field.INSIDE_B){
			for (Field pf : Field.INSIDE_A){
					try {
						game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P,100, null,1, 0, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_B).contains(f)){
							game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E, 100,null,1, 0, f, pf));
							game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100,null, 1, 0, f, pf));
						}
						game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P, 100,null,1, 0, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_B).contains(f)){
							game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E,100, null,1, 0, f, pf));
							game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100,null, 1, 0, f, pf));
						}
					} catch (Exception e) {
						e.printStackTrace();
					};
				}
		}
		
		try {
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.HARD));
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.HARD));
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100,null, 1, 0, BallProperty.HARD));
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.HARD));
			
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.HARD));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.HARD));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100, null,1, 0, BallProperty.HARD));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.HARD));
			
			game.getTeamA().getPlayer1().getIndProbs().overrideProbHit(new Prob_Hit(HitTechnique.s,100,null, 1, 0));
			game.getTeamA().getPlayer2().getIndProbs().overrideProbHit(new Prob_Hit(HitTechnique.s,100,null, 1, 0));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		game.getTeamB().getPlayer1().getIndProbs().setProbBlock(new Prob_Block(BlockTechnique.b,100, 0, 0, 1));
		game.getTeamB().getPlayer2().getIndProbs().setProbBlock(new Prob_Block(BlockTechnique.b,100, 0, 0, 1));
		
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
		
		simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
		System.out.println(simManager.printResultsSummary());
		
		assertTrue("direct point or go on prob fA is one", simManager.getfAP()+(1-simManager.getfAP()-simManager.getfAPOverline()) > 0.999999);
		assertTrue("direct fault prob fA is zero", simManager.getfAPOverline() < 0.00001);
	}
	
	@Test
	public void extremeFieldAttackProbs_TeamB() {
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamA(),
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamB());
		
		for (Field f : Field.INSIDE_A){
			for (Field pf : Field.INSIDE_B){
					try {
						game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P,100,null, 0, 1, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_A).contains(f)){
							game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E,100,null, 0, 1, f, pf));
							game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100, null,0, 1, f, pf));
						}
						game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P,100,null, 0, 1, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_A).contains(f)){
							game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E,100,null, 0, 1, f, pf));
							game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100,null, 0, 1, f, pf));
						}
					} catch (Exception e) {
						e.printStackTrace();
					};
				}
		}
		
//		System.out.println(game.getTeamB().getPlayer1().getIndProbs().printAnalogToExcelInputFormat(game.getTeamB().getPlayer1()));
		
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
		
		SimulationManager simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
		assertTrue("direct point prob fA is zero", simManager.getfAQ() < 0.000000001);
		assertTrue("direct fault prob fA is one", simManager.getfAQOverline() > 0.99999);
		
		simManager.reset();
		
		for (Field f : Field.INSIDE_A){
			for (Field pf : Field.INSIDE_B){
					try {
						game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P,100,null, 1, 0, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_A).contains(f)){
							game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E,100,null, 1, 0, f, pf));
							game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100,null, 1, 0, f, pf));
						}
						game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_P,100,null, 1, 0, f, pf));
						if (!Arrays.asList(Field.AT_THE_NET_A).contains(f)){
							game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_E,100,null, 1, 0, f, pf));
							game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Attack(HitTechnique.F_SM,100,null,1, 0, f, pf));
						}
					} catch (Exception e) {
						e.printStackTrace();
					};
				}
		}
		
		try {
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.HARD));
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.HARD));
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100, null,1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100, null,1, 0, BallProperty.HARD));
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.HARD));
			
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d,100,null, 1, 0, BallProperty.HARD));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.d_m,100,null, 1, 0, BallProperty.HARD));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r,100,null, 1, 0, BallProperty.HARD));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.NORMAL));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_DefRec(HitTechnique.r_m,100,null, 1, 0, BallProperty.HARD));
			
			game.getTeamB().getPlayer1().getIndProbs().overrideProbHit(new Prob_Hit(HitTechnique.s,100,null, 1, 0));
			game.getTeamB().getPlayer2().getIndProbs().overrideProbHit(new Prob_Hit(HitTechnique.s,100,null, 1, 0));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		// gegnerische Blockfaehigkeiten auf null setzen
		game.getTeamA().getPlayer1().getIndProbs().setProbBlock(new Prob_Block(BlockTechnique.b,100, 0, 0, 1));
		game.getTeamA().getPlayer2().getIndProbs().setProbBlock(new Prob_Block(BlockTechnique.b,100, 0, 0, 1));
		
		game.getTeamA().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamA().getPlayer2().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer1().getIndProbs().computeModifiedProbs();
		game.getTeamB().getPlayer2().getIndProbs().computeModifiedProbs();
		
		simManager = MainSim.startSim(100, true, 0, true, null, "extreme");
		simManager.computeSummary();
		
//		System.out.println(simManager.getfAQ());
		
		assertTrue("direct point or go on prob fA is one", simManager.getfAQ()+(1-simManager.getfAQ()-simManager.getfAQOverline()) > 0.999999);
		assertTrue("direct fault prob fA is zero", simManager.getfAQOverline() < 0.00001);
	}
	
}
