package de.ubt.wm.sb.junit.sim;

import java.util.Arrays;

import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Hit;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.Move;
import de.ubt.wm.sb.model.PlayerAction;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.TeamAction;
import de.ubt.wm.sb.model.strategy.Strategy;

/**
 * Speical Basic unterscheidet sich von der normalen Basic Strategie darin, dass feste Zielfelder bei den Angriffen ausgewählt werden
 * @author susanne
 *
 */
public class SpecialBASIC extends Strategy {

	protected SpecialBASIC(String name, boolean isTeamA, double pi_b, double[][] pi_h) {
		super(name, isTeamA, pi_b, pi_h, 0.5);
	}
	
	private static SpecialBASIC sstratBasicTeamA = new SpecialBASIC("Special Basic Team A", true, 0.5, new double[][]{{0.5, 0.5, 0.5, 0.5},{0.5,0.5,0.5,0.5}});
	private static SpecialBASIC sstratBasicTeamB = new SpecialBASIC("Special Basic Team B", false, 0.5, new double[][]{{0.5, 0.5, 0.5, 0.5},{0.5,0.5,0.5,0.5}});
	
	public static SpecialBASIC getInstanceTeamA(){
		centralFieldP1 = Field.A22;
		centralFieldP2 = Field.A23;
		return sstratBasicTeamA;
	}
	
	public static SpecialBASIC getInstanceTeamB(){
		centralFieldP1 = Field.B22;
		centralFieldP2 = Field.B23;
		return sstratBasicTeamB;
	}
	
	@Override
	public TeamAction servingRule(State s, double r) {
		Hit h;
		if (isTeamA){
			h = new Hit(HitTechnique.S_F, Field.B22);
		}else{
			h = new Hit(HitTechnique.S_F, Field.A22);
		}

		Move moveServer;
		if (h.isHitWithJump()){
			moveServer = Move.m_f;
		}else{
			if (Arrays.asList(Field.OUT_A).contains(super.fP1)){
				moveServer = fP1.moveTowardsField(getCentralFieldOfOtherHalf(fP2));
			}else{
				moveServer = fP2.moveTowardsField(getCentralFieldOfOtherHalf(fP1));
			}
		}
		
		if (Arrays.asList(Field.OUT_A).contains(fP1) || Arrays.asList(Field.OUT_B).contains(fP1))
			return new TeamAction(new PlayerAction(h,moveServer), new PlayerAction(Hit.nohit, Move.stay));
		else
			return new TeamAction(new PlayerAction(Hit.nohit, Move.stay), new PlayerAction(h,moveServer));
	}

	@Override
	public TeamAction attackingRule(State s, double r) {
		Hit h;
		if (isTeamA){
			if(fP1==fBall && s.getLastContact() != Game.getInstance().getTeamA().getPlayer1()){
				if (Arrays.asList(Field.AT_THE_NET_A).contains(fP1))
					h = new Hit(HitTechnique.F_P, Field.B22);
				else{
					h = new Hit(HitTechnique.F_SM, Field.B22);
				}
				return  new TeamAction(new PlayerAction(h, Move.stay), PlayerAction.doNothing);
			}
			if(fP2==fBall && s.getLastContact() != Game.getInstance().getTeamA().getPlayer2()){
				if (Arrays.asList(Field.AT_THE_NET_A).contains(fP2))
					h = new Hit(HitTechnique.F_P, Field.B22);
				else{
					h = new Hit(HitTechnique.F_SM, Field.B22);
				}
				return  new TeamAction(PlayerAction.doNothing, new PlayerAction(h, Move.stay));
			}
		}else{
			if(fP1==fBall && s.getLastContact() != Game.getInstance().getTeamB().getPlayer1()){
				if (Arrays.asList(Field.AT_THE_NET_B).contains(fP1))
					h = new Hit(HitTechnique.F_P, Field.A22);
				else{
					h = new Hit(HitTechnique.F_SM, Field.A22);
				}
				return  new TeamAction(new PlayerAction(h, Move.stay), PlayerAction.doNothing);
			}
			if(fP2==fBall && s.getLastContact() != Game.getInstance().getTeamB().getPlayer2()){
				if (Arrays.asList(Field.AT_THE_NET_B).contains(fP2))
					h = new Hit(HitTechnique.F_P, Field.A22);
				else{
					h = new Hit(HitTechnique.F_SM, Field.A22);
				}
				return  new TeamAction(PlayerAction.doNothing, new PlayerAction(h, Move.stay));
			}
		}
		return TeamAction.doNothing;
	}
	
}
