package de.ubt.wm.sb.junit.sim;

import static org.junit.Assert.assertTrue;

import java.util.ArrayList;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.sim.MainSim;
import de.ubt.wm.sb.sim.SimulationManager;

public class WinningProbTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void doubleRunWithSameSeed() {
		int n = 100;
		System.out.println("n: " + n);
//		long seed = new Long("1502739489711");
		long seed = System.currentTimeMillis();
		System.out.println("Seed is: " + seed);
		MainSim.r.setSeed(seed);
		
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamA(),
		"Brasilien", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamB());
		
		SimulationManager simManager = MainSim.startSim(n, true, 0, true, null, "allMatchesSkills");
		simManager.computeSetWinningProb();
		double winningProbA_1 = simManager.teamA_SetWinningProb;
		System.out.println("normal probs, winning Prob Team A:" + winningProbA_1);
	
		// zur Kontrolle zwei mal hintereinander
		System.out.println("Seed is: " + seed);
		MainSim.r.setSeed(seed);
		
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamA(),
		"France", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamB());
		
		simManager.reset();
		simManager = MainSim.startSim(n, true, 0, true, null, "allMatchesSkills");
		simManager.computeSetWinningProb();
		double winningProbA_copy = simManager.teamA_SetWinningProb;
		System.out.println("normal probs, winning Prob Team A:" + winningProbA_copy);
		assertTrue("zweimal gleiche Gewinnwahrscheinlcihkeit", Math.abs(winningProbA_1 - winningProbA_copy) <= 0.001);
		
	}
	
	@Test
	public void switchTeamSkills() {
		int n = 100;
		System.out.println("n: " + n);
//		long seed = new Long("1502739489711");
		long seed = System.currentTimeMillis();
		System.out.println("Seed is: " + seed);
		MainSim.r.setSeed(seed);
		
		Game game = Game.getInstance();
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamA(),
		"Brasilien", "Alison", "Emanuel", "data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamB());
		
		SimulationManager simManager = MainSim.startSim(n, true, 0, true, null, "allMatchesSkills");
		simManager.computeSetWinningProb();
		double winningProbA_1 = simManager.teamA_SetWinningProb;
		System.out.println("normal probs, winning Prob Team A:" + winningProbA_1);
		
		// Seed zurücksetzen
		System.out.println("Seed is: " + seed);
		MainSim.r.setSeed(seed);
		MainSim.calls = 0;
		
		//Skills tauschen, beide Mannschaften spielen gleiche Strategie
		game.initGame("Germany", "Brink", "Reckerman","data/allMatches/BRA_Alison_InputData_allMatches.xls","data/allMatches/BRA_Emanuel_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamA(),
		"France", "Alison", "Emanuel", "data/allMatches/GER_Brink_InputData_allMatches.xls","data/allMatches/GER_Reckermann_InputData_allMatches.xls", SpecialBASIC.getInstanceTeamB());
		
		// Aufschlagrecht vertauschen
		game.lowerbound = 0.5;
		game.upperbound = 1;
		// Serving Order muss auch geändert werden
		ArrayList<Player> servingOrder = new ArrayList<Player>();
		servingOrder.add(game.getTeamB().getPlayer1());
		servingOrder.add(game.getTeamA().getPlayer1());
		servingOrder.add(game.getTeamB().getPlayer2());
		servingOrder.add(game.getTeamA().getPlayer2());
		game.setServingOrder(servingOrder);
		
		simManager.reset();
		simManager = MainSim.startSim(n, true, 0, true, null, "allMatchesSkills");
		simManager.computeSetWinningProb();
		double winningProbA_2 = simManager.teamA_SetWinningProb;
		System.out.println("switched probs, winning Prob Team A:" + winningProbA_2);
		
		System.out.println("Diff to 1: " + (1 - winningProbA_1 - winningProbA_2));
		assertTrue("Gewinnwahrscheinlichkeiten sind ungefaehr Gegenwahrscheinlichkeiten", Math.abs(1 - winningProbA_1 - winningProbA_2) <= 0.001);
	}
	
}
