package de.ubt.wm.sb.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.WindowConstants;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.strategy.Strategy;

public class MainFrame extends JFrame{

	static final Logger log4j = LogManager.getLogger(JFrame.class.getName());
	private static final long serialVersionUID = 2304578662659984426L;
	
	private EditPane editPane;
	private JPanel buttonPane;
	WaitDialog dialog;
	
	SimSwingWorker mySimSwingWorker;
	
	public MainFrame(){
		super();
		constructMainFrame();
	}
	
	private void constructMainFrame()
	  {
	    this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
	    
	    editPane = new EditPane();
	    editPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
	    this.getContentPane().add(editPane);
	    
	    buttonPane = new JPanel();
	    FlowLayout flowLayout = (FlowLayout) buttonPane.getLayout();
	    flowLayout.setAlignment(FlowLayout.RIGHT);
	    this.getContentPane().add(buttonPane, BorderLayout.SOUTH);
	    
	    JButton btnCancel = new JButton("Cancel");
	    btnCancel.addActionListener(new ActionListener() {		
			@Override
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});
	    buttonPane.add(btnCancel);
	       
	    
	    JButton btnStartSim = new JButton("Start Sim");
	    btnStartSim.setSelected(true);
	    
		btnStartSim.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				// TODO: auslesen der Werte von Parametern und neue Strategie erzeugen
				double pi_h_serve_tech_A1 = editPane.slider_pi_h_serve_tech_A1.getValue()*1.0/100;
				double pi_h_serve_field_A1 = editPane.slider_pi_h_serve_field_A1.getValue()*1.0/100;
				double pi_h_serve_tech_A2 = editPane.slider_pi_h_serve_tech_A2.getValue()*1.0/100;
				double pi_h_serve_field_A2 = editPane.slider_pi_h_serve_field_A2.getValue()*1.0/100;
				double pi_h_field_tech_A1 = editPane.slider_pi_h_field_tech_A1.getValue()*1.0/100;
				double pi_h_field_field_A1 = editPane.slider_pi_h_field_field_A1.getValue()*1.0/100;
				double pi_h_field_tech_A2 = editPane.slider_pi_h_field_tech_A2.getValue()*1.0/100;
				double pi_h_field_field_A2 = editPane.slider_pi_h_field_field_A2.getValue()*1.0/100;
				double pi_s = editPane.slider_pi_s.getValue()*1.0/100;
				double pi_b = editPane.slider_pi_b.getValue()*1.0/100;
				// pi_b, pi_h = serve/tech, serve/field, field/tech, field/field, pi_s
				Strategy strat_A = new Strategy("stratAfromSlider", true, pi_b,
						new double[][]{{pi_h_serve_tech_A1, pi_h_serve_field_A1, pi_h_field_tech_A1, pi_h_field_field_A1},
							{pi_h_serve_tech_A2, pi_h_serve_field_A2, pi_h_field_tech_A2, pi_h_field_field_A2}}, pi_s);
				
				double pi_h_serve_tech_B1 = editPane.slider_pi_h_serve_tech_B1.getValue()*1.0/100;
				double pi_h_serve_field_B1 = editPane.slider_pi_h_serve_field_B1.getValue()*1.0/100;
				double pi_h_serve_tech_B2 = editPane.slider_pi_h_serve_tech_B2.getValue()*1.0/100;
				double pi_h_serve_field_B2 = editPane.slider_pi_h_serve_field_B2.getValue()*1.0/100;
				double pi_h_field_tech_B1 = editPane.slider_pi_h_field_tech_B1.getValue()*1.0/100;
				double pi_h_field_field_B1 = editPane.slider_pi_h_field_field_B1.getValue()*1.0/100;
				double pi_h_field_tech_B2 = editPane.slider_pi_h_field_tech_B2.getValue()*1.0/100;
				double pi_h_field_field_B2 = editPane.slider_pi_h_field_field_B2.getValue()*1.0/100;
				double pi_s_B = editPane.slider_pi_s_B.getValue()*1.0/100;
				double pi_b_B = editPane.slider_pi_b_B.getValue()*1.0/100;
				
				Strategy strat_B = new Strategy("stratBfromSlider", false, pi_b_B,
						new double[][]{{pi_h_serve_tech_B1, pi_h_serve_field_B1, pi_h_field_tech_B1, pi_h_field_field_B1},
							{pi_h_serve_tech_B2, pi_h_serve_field_B2, pi_h_field_tech_B2, pi_h_field_field_B2}}, pi_s_B);
								
				Game game = Game.getInstance();
				game.initGameFromEvalCSV(editPane.getTxt_teamA().getText(),
						editPane.getTxt_nameA1().getText(),
						editPane.getTxt_nameA2().getText(),
						editPane.getTxt_skillsTeamA().getText(),
						strat_A,
						editPane.getTxt_teamB().getText(),
						editPane.getTxt_nameB1().getText(),
						editPane.getTxt_nameB2().getText(),
						editPane.getTxt_skillsTeamB().getText(),
						strat_B);
	
				mySimSwingWorker = new SimSwingWorker(editPane.getChckbx_Gui().isSelected(), 2001-editPane.getSLider_speed().getValue(), false);
				mySimSwingWorker.execute();
				
				if (!mySimSwingWorker.gui){
					dialog = new WaitDialog();
					dialog.setLocationRelativeTo(GUIManager.getInstance().getMainFrame());
					dialog.setVisible(true);
				}else{
					ViewSimFrame viewSimFrame = new ViewSimFrame();
					GUIManager.getInstance().setViewSimFrame(viewSimFrame);
					viewSimFrame.setLocationRelativeTo(GUIManager.getInstance().getMainFrame());
					viewSimFrame.setVisible(true);
				}		

			}
		});
	    
	    
	    buttonPane.add(btnStartSim);
	    
	    this.pack();
	    this.setLocationRelativeTo(null);
	    this.setVisible(true);
	  }
	
	
	public EditPane getEditPane() {
		return editPane;
	}

	public JPanel getButtonPane() {
		return buttonPane;
	}
	
	public WaitDialog getWaitDialog(){
		return dialog;
	}
	
	public void setWaitDialog(WaitDialog dialog){
		this.dialog = dialog;
	}
	
	public SimSwingWorker getMySimSwingWorker() {
		return mySimSwingWorker;
	}

	public void setMySimSwingWorker(SimSwingWorker mySimSwingWorker) {
		this.mySimSwingWorker = mySimSwingWorker;
	}
}
