package de.ubt.wm.sb.gui;

import java.util.List;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.gui.SimSwingWorker.PublishObject;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.sim.MainSim;
import de.ubt.wm.sb.sim.SimulationManager;

//erster Raeckgabetyp faer Background, zweiter faer Intermins
public class SimSwingWorker extends SwingWorker<SimulationManager, PublishObject> {

	static final Logger log4j = LogManager.getLogger(SimSwingWorker.class.getName());
	
	int waitTime;
	public boolean interruped;
	boolean gui;
	public String hitInfoText;
	public String attackInfoText;
	public String blockInfoText;
	
	public SimulationManager simManager;
		
	public SimSwingWorker(boolean gui, int waitTime, boolean interruped) {
		super();
		this.gui = gui;
		this.waitTime = waitTime;
		this.interruped = interruped;
	}

	public void myPublish(int numberOfSets, boolean bool, State s){
		publish(new PublishObject(numberOfSets, bool, s));
		waitTime();
	}
	
	@Override
	protected SimulationManager doInBackground() throws Exception {

		// Objekte besorgen die man braucht fuer das Spiel
		MainFrame mainFrame = GUIManager.getInstance().getMainFrame();
		EditPane tabbedPane = mainFrame.getEditPane();
		SimulationManager simMan = MainSim.startSim(((Number) tabbedPane.getSpinner_MaxPoints().getValue()).intValue(), true, 0, true, this, "skillsGUI");
		this.simManager = simMan;
		return simMan;
	}

	@Override
	protected void process(List<PublishObject> publishObjects) {
		log4j.debug("process");
		
		State state = publishObjects.get(publishObjects.size()-1).getCurrentState();
		boolean newState = publishObjects.get(publishObjects.size()-1).isNewState();
		
		if(!this.gui){		
			WaitDialog dialog = GUIManager.getInstance().getMainFrame().getWaitDialog();
			dialog.updateWaitDialog(publishObjects.get(publishObjects.size()-1).getNumberOfplayedSets());
		}else{
			ViewSimFrame viewSimFrame = GUIManager.getInstance().getViewSimFrame();
			viewSimFrame.repaintState(newState, state, hitInfoText, attackInfoText, blockInfoText);
			SwingUtilities.updateComponentTreeUI(viewSimFrame);			
		}

	}
	
	
	@Override
	protected void done() {
		log4j.debug("fertig");
		
		SimulationManager simManager = this.simManager;
//		simManager.computeSummary();
		
		// Attack Statistic in ExcelFile schreiben
//		try {
//			simManager.writeResultsToFile("attackStatistics");
//		} catch (Exception e) {
//			log4j.error("Could not write attack Statistics to Excel File");
//			e.printStackTrace();
//		}
		
		System.out.println("Results Summary:\n" + simManager.printResultsSummary());
			
		// Fenster schlieaeen (je nachdem WaitDialog oder Spielansicht)
		if(!this.gui){
			WaitDialog dialog = GUIManager.getInstance().getMainFrame().getWaitDialog();
			dialog.dispose();
		}else{
			ViewSimFrame viewSim = GUIManager.getInstance().getViewSimFrame();
			viewSim.dispose();
		}
		
		// ResultFrame anzeigen
		ResultFrame resultFrame = new ResultFrame(simManager);
		resultFrame.pack();
		resultFrame.setLocationRelativeTo(null);
		resultFrame.setVisible(true);
				
	}
	
	private void waitTime(){
		if(gui){
			try {
                synchronized (this) {
                    this.wait(waitTime);//Waittime
                }
            } catch (Exception ex) {
            }
		}
	}
		

	public class PublishObject{
		
	private int numberOfplayedSets;
	private boolean newState;
	private State currentState;
	
	public PublishObject(int numberOfSets, boolean newState, State currentState){
		this.numberOfplayedSets = numberOfSets;
		this.newState = newState;
		this.currentState = currentState;
	}

	public boolean isNewState() {
		return newState;
	}
	
	public State getCurrentState() {
		return currentState;
	}

	public int getNumberOfplayedSets() {
		return numberOfplayedSets;
	}

	public void setNumberOfplayedSets(int numberOfplayedSets) {
		this.numberOfplayedSets = numberOfplayedSets;
	}
	
	}
}
