package de.ubt.wm.sb.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.exceptions.StateResetException;
import de.ubt.wm.sb.gui.GUIManager;
import de.ubt.wm.sb.gui.MainGui;
import de.ubt.wm.sb.model.strategy.RealStrat;
import de.ubt.wm.sb.probs.Prob_Block;
import de.ubt.wm.sb.probs.Prob_Hit;
import de.ubt.wm.sb.probs.Prob_IndPlayer;
import de.ubt.wm.sb.sim.MainSim;



/**
 * Spiel
 */	
public class Game {

	static final Logger log4j = LogManager.getLogger(Game.class.getName());
	public static boolean useModProbs = true;
	
	private static Game game = new Game();
	public double lowerbound = 0;
	public double upperbound = 0.5;
	
	private Team teamA;	
	private Team teamB;
	
	private int pointsTeamA;	
	private int pointsTeamB;

	private State currentState;

	private List<State> stateHist;
	private ArrayList<Player> servingOrder;
	
	/**
	 * leeres Spiel
	 */
	private Game(){
		super();
	}
	
	
	public static Game getInstance(){
		return game;
	}
	
	/**
	 * Defines Players of Team A with probabilites from automatic generated CSV Files
	 * Fuegt Teams dem Spiel hinzu
	 * Ruft resetGame() am Schluss auf, sodass Startzustand zufuellig gewuehlt wird
	 * @param nameTeamA - Team Name Team A
	 * @param namePlayerA1 - Spielername A1, erster Aufschläger in Team A
	 * @param namePlayerA2 - Spielername A2
	 * @param dataFileTeamA - Pfad zu CSV-heet mit Spielerwkeiten "Name.eval.csv"
	 * @param strat - Strategie von Team A
	 * @param nameTeamB - Team Name Team B
	 * @param namePlayerB1 - Spielername B1, erster Aufschläger in Team B
	 * @param namePlayerB2 - Spielername B2
	 * @param dataFileTeamB - Pfad zu CSV-heet mit Spielerwkeiten "Name.eval.csv"
	 * @param strat_B 
	 */
	public void initGameFromEvalCSV(String nameTeamA, String namePlayerA1, String namePlayerA2, String dataFileTeamA, RealStrat strat,
			String nameTeamB, String namePlayerB1, String namePlayerB2, String dataFileTeamB, RealStrat strat_B){
		
		Player A1 = new Player(namePlayerA1, dataFileTeamA);
		Player A2 = new Player(namePlayerA2, dataFileTeamA);
		Team A = new Team(nameTeamA);
		A.addPlayersToTeam(A1,A2);
		A.setStrat(strat);
		
		Player B1 = new Player(namePlayerB1, dataFileTeamB);
		Player B2 = new Player(namePlayerB2, dataFileTeamB);
		Team B = new Team(nameTeamB);
		B.addPlayersToTeam(B1, B2);
		B.setStrat(strat_B);
		
		game.teamA = A;
		game.teamB = B;
		
		try{
			
			Prob_IndPlayer indProbsA1 = new Prob_IndPlayer();
			indProbsA1.readFrom_BVEVAL_CSV(A1, true);
			A1.setIndProbs(indProbsA1);
			
			Prob_IndPlayer indProbsA2 = new Prob_IndPlayer();
			indProbsA2.readFrom_BVEVAL_CSV(A2, false);
			A2.setIndProbs(indProbsA2);
			
			Prob_IndPlayer indProbsB1 = new Prob_IndPlayer();
			indProbsB1.readFrom_BVEVAL_CSV(B1, true);
			B1.setIndProbs(indProbsB1);
			
			Prob_IndPlayer indProbsB2 = new Prob_IndPlayer();
			indProbsB2.readFrom_BVEVAL_CSV(B2, false);
			B2.setIndProbs(indProbsB2);
		}
		catch (Exception e) {
			log4j.error("... could not load probabilities");
			e.printStackTrace();
			System.exit(1);
		}
		
		// falls Spiel neu initialisiert wird -> neue statische Zustände, da dort Referenzen auf Spieler enthalten sind
		State.reinitializeStates();
		
		// Aufschlagsreihenfolge festlegen
		// TeamA wechselt mit TeamB ab!
		servingOrder = new ArrayList<Player>();
		servingOrder.add(A1);
		servingOrder.add(B1);
		servingOrder.add(A2);
		servingOrder.add(B2); 
		
		// Reset Game braucht Spieler und Teams und zufuelligen Startzustand
		game.resetGame();
	}
	
	/**
	 * Defines Players of Team A with probabilites from ExcelFiles
	 * Fuegt Teams dem Spiel hinzu
	 * Ruft resetGame() am Schluss auf, sodass Startzustand zufuellig gewuehlt wird
	 * @param nameTeamA - Team Name Team A
	 * @param namePlayerA1 - Spielername A1, erster Aufschläger in Team A
	 * @param namePlayerA2 - Spielername A2
	 * @param dataFileA1 - Pfad zu Excelsheet mit Spielerwkeiten von Spieler A1
	 * @param dataFileA2 - Pfad zu Excelsheet mit Spielerwkeiten von Spieler A2
	 * @param strat - Strategie von Team A
	 * @param nameTeamA - Team Name Team B
	 * @param namePlayerA1 - Spielername B1, erster Aufschläger in Team B
	 * @param namePlayerA2 - Spielername B2
	 * @param dataFileA1 - Pfad zu Excelsheet mit Spielerwkeiten von Spieler B1
	 * @param dataFileA2 - Pfad zu Excelsheet mit Spielerwkeiten von Spieler B2
	 * @param strat_B 
	 */
	public void initGame(String nameTeamA, String namePlayerA1, String namePlayerA2, String dataFileA1, String dataFileA2, RealStrat strat,
			String nameTeamB, String namePlayerB1, String namePlayerB2, String dataFileB1, String dataFileB2, RealStrat strat_B){
		
		Player A1 = new Player(namePlayerA1, dataFileA1);
		Player A2 = new Player(namePlayerA2, dataFileA2);
		Team A = new Team(nameTeamA);
		A.addPlayersToTeam(A1,A2);
		A.setStrat(strat);
		
		Player B1 = new Player(namePlayerB1, dataFileB1);
		Player B2 = new Player(namePlayerB2, dataFileB2);
		Team B = new Team(nameTeamB);
		B.addPlayersToTeam(B1, B2);
		B.setStrat(strat_B);
		
		game.teamA = A;
		game.teamB = B;
		
		try{
			
			Prob_IndPlayer indProbsA1 = new Prob_IndPlayer();
			indProbsA1.readFromExcelTemplate(A1);
			A1.setIndProbs(indProbsA1);
			
			Prob_IndPlayer indProbsA2 = new Prob_IndPlayer();
			indProbsA2.readFromExcelTemplate(A2);
			A2.setIndProbs(indProbsA2);
			
			Prob_IndPlayer indProbsB1 = new Prob_IndPlayer();
			indProbsB1.readFromExcelTemplate(B1);
			B1.setIndProbs(indProbsB1);
			
			Prob_IndPlayer indProbsB2 = new Prob_IndPlayer();
			indProbsB2.readFromExcelTemplate(B2);
			B2.setIndProbs(indProbsB2);
		}
		catch (Exception e) {
			log4j.error("... could not load probabilities");
			e.printStackTrace();
			System.exit(1);
		}
		
		// falls Spiel neu initialisiert wird -> neue statische Zustände, da dort Referenzen auf Spieler enthalten sind
		State.reinitializeStates();
		
		// Anteilig wer ist wieviel Blockspieler
//		Strat_BASIC_BASIC.getInstanceTeamA().setIsBlockingP1(A1.getIndProbs().getIsBlockingPlayer());
//		Strat_BASIC_BASIC.getInstanceTeamA().setIsBlockingP2(A2.getIndProbs().getIsBlockingPlayer());
//		Strat_BASIC_BASIC.getInstanceTeamB().setIsBlockingP1(B1.getIndProbs().getIsBlockingPlayer());
//		Strat_BASIC_BASIC.getInstanceTeamB().setIsBlockingP2(B2.getIndProbs().getIsBlockingPlayer());
		
		// Aufschlagsreihenfolge festlegen
		// TeamA wechselt mit TeamB ab!
		servingOrder = new ArrayList<Player>();
		servingOrder.add(A1);
		servingOrder.add(B1);
		servingOrder.add(A2);
		servingOrder.add(B2); 
		
		// Reset Game braucht Spieler und Teams und zufuelligen Startzustand
		game.resetGame();
	}
	
	
	/**
	 * Setzt alle Punkte auf 0 zurueck,
	 * Der aktuelle Zustand wird zufuellig auf Startzustand von TeamA oder TeamB gesetzt
	 * Die Zustandshistorie wird ueberschrieben mit einer neuen leeren Liste 
	 */
	public void resetGame(){
		this.pointsTeamA = 0;
		this.pointsTeamB = 0;
		this.stateHist = new ArrayList<State>();
		
		double r = MainSim.countRandomCalls();
		if(lowerbound <= r && r < upperbound){
			setCurrentState(State.chooseRandomStateOfArray(State.START_STATES_A1));
			currentState.setLastServer(teamA.getPlayer1());
		}else{
			setCurrentState(State.chooseRandomStateOfArray(State.START_STATES_B1));
			currentState.setLastServer(teamB.getPlayer1());
		}
	}

	public State getCurrentState(){
		return currentState;
	}	
	
	/**
	 * aktualisiert den aktuellen Zustand des Spiels 
	 * erhueht den Spielstand, falls neuer Zustand Punkt
	 * fuegt den neuen Zustand der StateHistorie hinzu
	 * setzt die Felder der Spieler
	 **/
	public void setCurrentState(State s){
		if (State.POINT_A.contains(s)){
			pointsTeamA++;
		}
		if (State.POINT_B.contains(s)){
			pointsTeamB++;
		}
		
		this.currentState = s;
		this.stateHist.add(s);
		
		// Falls kein Punkt: die Felder der Spieler setzen 
		if (!State.POINT_A.contains(s) && !State.POINT_B.contains(s)){
			this.teamA.getPlayer1().setField(s.getFieldA1());
			this.teamA.getPlayer2().setField(s.getFieldA2());
			this.teamB.getPlayer1().setField(s.getFieldB1());
			this.teamB.getPlayer2().setField(s.getFieldB2());
		}
	}
	
	/**
	 * Berechnet aus den TeamAktionen und dem letzten Zustand den nuechsten neuen Zustand
	 * @throws Exception 
	 */
	public void computeNextState() throws Exception{		
		TeamAction actionA = teamA.getTeamAction();
		TeamAction actionB = teamB.getTeamAction();
		
		if (actionA.getActionPlayer2().getMove() == Move.b) MainSim.numBlocksA2++;
		
		//alten Zustand zum Vergleichen abspeichern
		State lastState = this.stateHist.get(this.stateHist.size()-1);
		
		// ACHTUNG: dieser Fall sollte nicht passieren, da im MySimSwingWorker behandelt
		// Falls wir in einem terminalen Zustand sind mit dem naechsten Startzustand beginnen.
		if(State.POINT_A.contains(currentState) || State.POINT_B.contains(currentState)){
			log4j.warn("terminale Zustand in computeNextState, sollte eigentlich nicht vorkommen, da in simulation behandelt");
			setCurrentState(getNextServingState(currentState));
		}
		
		
		// Alles was zu einem Fehler fuehren kann:
		
		//--------------------------------------------------------------------------------------------------		
		// Ball nicht gespielt
		//--------------------------------------------------------------------------------------------------
		
		//Ball auf der Seite von A und Team A hat keine Ballberuehrung
		if (lastState.getFieldBall().belongsToSideA() && actionA.getActionPlayer1().getHit() == Hit.nohit && actionA.getActionPlayer2().getHit() == Hit.nohit){
			log4j.debug("Ball auf der Seite von A und A spielt den Ball nicht!");
			
			//einziger Fall, dass es einen Punkt fuer Team A geben kann, wenn
			//Ball im Aus von A und Ball wurde uebers Netz gespielt
			if(Arrays.asList(Field.OUT_A).contains(lastState.getFieldBall()) && lastState.ballOverNet()){
				log4j.debug("Punkt Team A, Fehler Team B!");
				setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
				return;
			}
			else {
				log4j.debug("Punkt Team B, Fehler Team A!");
				setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
				return;
				
			}
			
		}
		
		//Ball auf der Seite von B und Team B hat keine Ballberuehrung
		if (lastState.getFieldBall().belongsToSideB() && actionB.getActionPlayer1().getHit() == Hit.nohit && actionB.getActionPlayer2().getHit() == Hit.nohit){
			log4j.debug("Ball auf der Seite von B und B spielt den Ball nicht!");
			
			//einziger Fall, dass es einen Punkt fuer Team B geben kann, wenn
			//Ball im Aus von B und Ball wurde uebers Netz gespielt
			if(Arrays.asList(Field.OUT_B).contains(lastState.getFieldBall()) && lastState.ballOverNet()){
				log4j.debug("Punkt Team B, Fehler Team A!");
				setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
				return;
			}
			else {
				log4j.debug("Punkt Team A, Fehler Team B!");
				setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
				return;
			}
			
		}
		
		//--------------------------------------------------------------------------------------------------		
		// 4te Ballberuehrung
		//--------------------------------------------------------------------------------------------------
		if ((actionA.getActionPlayer1().getHit()!=Hit.nohit || actionA.getActionPlayer2().getHit()!=Hit.nohit)
				&& lastState.getCounter() == 3){
			setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
			return;
		}
		
		if ((actionB.getActionPlayer1().getHit()!=Hit.nohit || actionB.getActionPlayer2().getHit()!=Hit.nohit)
				&& lastState.getCounter() == 3){
			setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
			return;
		}
		
		//--------------------------------------------------------------------------------------------------		
		// Doppelkontakt
		//--------------------------------------------------------------------------------------------------
		if ((actionA.getActionPlayer1().getHit() != Hit.nohit && lastState.getLastContact() == this.teamA.getPlayer1())
				|| (actionA.getActionPlayer2().getHit() != Hit.nohit && lastState.getLastContact() == this.teamA.getPlayer2())){
			setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
			return;
		}
		
		if ((actionB.getActionPlayer1().getHit() != Hit.nohit && lastState.getLastContact() == this.teamB.getPlayer1())
				|| (actionB.getActionPlayer2().getHit() != Hit.nohit && lastState.getLastContact() == this.teamB.getPlayer2())){
			setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
			return;
		}
		
		
		//==================================================================================================
		// nur noch Ausfuehrungsfehler moeglich
		//==================================================================================================
		
		//konstruieren neuen Zustand aus letztem Zustand
		State newState = new State(lastState);

		
		/*
		 * Durchgehen welcher Spieler schlaegt und playerPlaysBall aufrufen
		 */
		Field newFieldBall;
		Player hittingPlayer = null;
		Player lastContactPlayer = null;
		int counter = 0;
		if (actionA.getActionPlayer1().getHit() != Hit.nohit){
			hittingPlayer = this.getTeamA().getPlayer1();
			counter++;
		}
		if (actionA.getActionPlayer2().getHit() != Hit.nohit){
			hittingPlayer = this.getTeamA().getPlayer2();
			counter++;
		}
		if (actionB.getActionPlayer1().getHit() != Hit.nohit){
			hittingPlayer = this.getTeamB().getPlayer1();
			counter++;
		}
		if (actionB.getActionPlayer2().getHit() != Hit.nohit){
			hittingPlayer = this.getTeamB().getPlayer2();
			counter++;
		}
		// Redundanz einbauen, dass keine zwei Spieler gleichzeitig den Ball schlagen (ist eigentlich bereits durch Teamaktion ausgeschlossen)
		if (counter != 1){
			log4j.error("Anzahl der ballspielenden Spieler: " + counter + "\n"
					+ " in Zustand " + lastState.toStringLong() + "\n"
					+ " Aktionen: " + game.getTeamA().getTeamAction() + " " + game.getTeamB().getTeamAction());
		}

		// schaue ob Wahrscheinlichkeiten für Schlag über den Spieler bekannt sind
		Prob_Hit probHit = hittingPlayer.getIndProbs().getProbHit(hittingPlayer.getPlayerAction().getHit().getTechnique(), 
				hittingPlayer.getPlayerAction().getHit().getTargetField(), hittingPlayer.getField(), lastState.getBallProp());
		if (probHit == null){
			log4j.debug("Player: " + hittingPlayer.toString() + " PlayerAction: " + hittingPlayer.getPlayerAction());
			lastState = helper_findLastServingState();
			game.setCurrentState(lastState);
			throw new StateResetException("Wahrscheinlichkeit nicht verfügbar. Zustand zurückgesetzt.", hittingPlayer, hittingPlayer.getPlayerAction());
		}
		
		// evaluiere Schlag		
		newFieldBall = playerHitsBall(hittingPlayer, lastState.getBallProp());
		if (newFieldBall == null) {
			if (hittingPlayer.isInTeamA())
				setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
			else
				setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
			return;
		}else{
			lastContactPlayer = hittingPlayer;
		}
		
		// Allgemein steht der hitting Player dort wo er den Ball geschlagen hat.
		newState.setField(hittingPlayer, lastState.getFieldBall());
		
		// es gibt immer nur maximal einen Block Spieler pro Angriff
		Player blockingPlayer = null;
		
		// ALT: hier durfte nur der designated Blocker blocken
//		if (lastState.getBlocker() != null && lastState.getBlocker().getPlayerAction().getMove().isBlockingAction()){
//			blockingPlayer = lastState.getBlocker();
//		}
		// jeder Spieler kann blocken, Effekt aber nur wenn Spieler aus verteidigendem Team blockt
		counter = 0;
		if (actionA.getActionPlayer1().getMove() == Move.b && game.getTeamA().isDefendingTeam()){
			blockingPlayer = this.getTeamA().getPlayer1();
			counter++;
		}
		if (actionA.getActionPlayer2().getMove() == Move.b && game.getTeamA().isDefendingTeam()){
			blockingPlayer = this.getTeamA().getPlayer2();
			counter++;
		}
		if (actionB.getActionPlayer1().getMove() == Move.b  && game.getTeamB().isDefendingTeam()){
			blockingPlayer = this.getTeamB().getPlayer1();
			counter++;
		}
		if (actionB.getActionPlayer2().getMove() == Move.b && game.getTeamB().isDefendingTeam()){
			blockingPlayer = this.getTeamB().getPlayer2();
			counter++;
		}
		// mehr als ein Spieler blockt
		if (counter > 1){
			Player maybeBlocker1;
			Player maybeBlocker2;
			if (game.getTeamA().isDefendingTeam()){
				maybeBlocker1 = this.getTeamA().getPlayer1();
				maybeBlocker2 = this.getTeamA().getPlayer2();
			}else{
				maybeBlocker1 = this.getTeamB().getPlayer1();
				maybeBlocker2 = this.getTeamB().getPlayer2();
			}
			System.out.println(maybeBlocker1.getField().getColumn());
			System.out.println(hittingPlayer.getField().getColumn());
			// beide moeglichen Blocker stehen richtig
			boolean maybeBlocker1_MayHaveImpact = false;
			boolean maybeBlocker2_MayHaveImpact = false;
			if (maybeBlocker1 != null && maybeBlocker1.getField().getColumn() == hittingPlayer.getField().getColumn() 
					&& maybeBlocker1.getField().isAtTheNet() )
				maybeBlocker1_MayHaveImpact = true;
			
			if (maybeBlocker2 != null && maybeBlocker2.getField().getColumn() == hittingPlayer.getField().getColumn() 
					&& maybeBlocker2.getField().isAtTheNet() )
				maybeBlocker2_MayHaveImpact = true;
			
			if (maybeBlocker1_MayHaveImpact && maybeBlocker2_MayHaveImpact){
				// both may have impact choose equally
				double r = MainSim.countRandomCalls();
				if ( r < 0.5){
					blockingPlayer = maybeBlocker1;
				}else{
					blockingPlayer = maybeBlocker2;
				}
			}else{
				if(maybeBlocker1_MayHaveImpact){
					blockingPlayer = maybeBlocker1;
				}else{
					if(maybeBlocker2_MayHaveImpact){
						blockingPlayer = maybeBlocker2;
					}else{
						// kein Spieler kann Einfluss nehmen, daher kann blockingPlayer auch gleich auf null gesetzt werden
						blockingPlayer = null;
					}
				}
			}
		}
		
		
		// um gegebenenfalls spaeter counter zurückzusetzen
		boolean blockOverNet = false;
		boolean okBlock = false;
		
		// falls Blocker ausgezeichnet und blockt in der Richtigen Spalte, sowie dass er am Netz steht
		if (blockingPlayer != null && blockingPlayer.getField().getColumn() == hittingPlayer.getField().getColumn() 
				&& blockingPlayer.getField().isAtTheNet() ){
			// check if Blockwahrscheinlichkeit vorhanden, ansonsten Zustand zurücksetzen
			if (blockingPlayer.getIndProbs().getProbBlock() != null){
				log4j.debug("Block by player " + blockingPlayer.getName() + " is possible");
				BlockScenario blockscenario = playerBlocks(blockingPlayer);
				
				// Block wurde durchgefüht und berührte Ball obwohl es ein Aufschlag war, das ist verboten.
				if (blockscenario != BlockScenario.MISSES_BALL && lastState.getCounter() == -1){
					log4j.warn("Player has blocked in state :" + game.getCurrentState().toString() + "It is a serving state and not allowed!");
					if (blockingPlayer.isInTeamA()){
						setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
						return;
					}else{
						setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
						return;
					}	
				}
				
				if (blockscenario == BlockScenario.DIRECT_POINT){
					if (blockingPlayer.isInTeamA()){
						setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
						return;
					}else{
						setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
						return;
					}					
				}
				
				if (blockscenario == BlockScenario.FAULT){
					if (blockingPlayer.isInTeamA()){
						setCurrentState(State.getPointStateTeamB(lastState.getLastServer()));
						return;
					}else{
						setCurrentState(State.getPointStateTeamA(lastState.getLastServer()));
						return;
					}					
				}
				
				if (blockscenario == BlockScenario.OK_BLOCK){
					okBlock = true;
					
					// alle Felder als nachfolge Feld möglich
					double r = MainSim.countRandomCalls();
					if (lowerbound <= r && r < upperbound){
						newFieldBall = Field.chooseRandomField(Field.INSIDE_A);
					}else{
						newFieldBall = Field.chooseRandomField(Field.INSIDE_B);
					}

					if (newFieldBall.belongsToSideA() && blockingPlayer.isInTeamB()
							|| newFieldBall.belongsToSideB() && blockingPlayer.isInTeamA())
						blockOverNet = true;
				}
			}else{
				// Zustand zurücksetzen, falls kein B
				lastState = helper_findLastServingState();
				game.setCurrentState(lastState);
				throw new StateResetException("Wahrscheinlichkeit nicht verfügbar. Zustand zurückgesetzt.", blockingPlayer, blockingPlayer.getPlayerAction());
			}
		}
		
		/**
		 *  setze neuen Zustand zusammen
		 */
		
		// neuen Blockspieler setzen, falls durch TeamAktion bestimmt
		if (actionA.getDesignatedBlockingPlayer() != null && this.teamA.isDefendingTeam()){
			newState.setBlocker(actionA.getDesignatedBlockingPlayer());
		}
		if (actionB.getDesignatedBlockingPlayer() != null && this.teamB.isDefendingTeam()){
			newState.setBlocker(actionB.getDesignatedBlockingPlayer());
		}
		
		// Bewegungen nach dem Schlag
		// nun sind noch die Bewegungen mögliche
		if (hittingPlayer != this.teamA.getPlayer1())
			newState.setFieldA1(this.teamA.getPlayer1().makeStep(actionA.getActionPlayer1().getMove()));
		if (hittingPlayer != this.teamA.getPlayer2())
			newState.setFieldA2(this.teamA.getPlayer2().makeStep(actionA.getActionPlayer2().getMove()));
		if (hittingPlayer != this.teamB.getPlayer1())
			newState.setFieldB1(this.teamB.getPlayer1().makeStep(actionB.getActionPlayer1().getMove()));
		if (hittingPlayer != this.teamB.getPlayer2())
			newState.setFieldB2(this.teamB.getPlayer2().makeStep(actionB.getActionPlayer2().getMove()));
		// hitting Player darf sich nach Schlagaktion generell doch noch bewegen... ausgehend von seiner neuen Position die dort ist wo er geschlagen hat
		if (hittingPlayer == this.teamA.getPlayer1()){
			this.teamA.getPlayer1().setField(lastState.getFieldBall());
			newState.setFieldA1(this.teamA.getPlayer1().makeStep(actionA.getActionPlayer1().getMove()));
		}
		if (hittingPlayer == this.teamA.getPlayer2()){
			this.teamA.getPlayer2().setField(lastState.getFieldBall());
			newState.setFieldA2(this.teamA.getPlayer2().makeStep(actionA.getActionPlayer2().getMove()));
		}	
		if (hittingPlayer == this.teamB.getPlayer1()){
			this.teamB.getPlayer1().setField(lastState.getFieldBall());
			newState.setFieldB1(this.teamB.getPlayer1().makeStep(actionB.getActionPlayer1().getMove()));
		}	
		if (hittingPlayer == this.teamB.getPlayer2()){
			this.teamB.getPlayer2().setField(lastState.getFieldBall());
			newState.setFieldB2(this.teamB.getPlayer2().makeStep(actionB.getActionPlayer2().getMove()));
		}
			
		// unerlaubte Bewegung eventuell wieder rückgängig machen
		if ((hittingPlayer.getPlayerAction().getHit().getTechnique() == HitTechnique.d_m || hittingPlayer.getPlayerAction().getHit().getTechnique() == HitTechnique.r_m )
				&& hittingPlayer.getPlayerAction().getMove() != Move.stay){
			log4j.warn("Bewegung nach r_m oder d_m nicht erlaubt und nicht durchgefuehrt.");
		}
			
		newState.setLastContact(lastContactPlayer);
		newState.setFieldPosBall(newFieldBall);
		// wenn Ball hart geschlagen wurde, wäre er auch nach einem Abpraller vom Block weiterhin hart
		BallProperty ballProp = BallProperty.NORMAL;
		if (hittingPlayer.getPlayerAction().getHit().isHardHit())
			ballProp = BallProperty.HARD;
		newState.setBallProp(ballProp);
		
		/** 
		* falls Ball die Seite wechselt nach einem Aufschlag -> Counter auf -1
		* falls Ball die Seite wechselt nach einem Angriff aus dem Spiel -> Counter auf 0 oder 1 je nachdem ob schon Berührung durch ok Block
		* sonst Counter um eins erhoehen
		*/
		if((lastState.getFieldBall().belongsToSideA() && newState.getFieldBall().belongsToSideB()) 
				|| (lastState.getFieldBall().belongsToSideB() && newState.getFieldBall().belongsToSideA())){
			log4j.debug("Ball hat Seite gewechselt");
			newState.setBlocker(null);
			
			if (Arrays.asList(State.START_STATES_A).contains(lastState) || Arrays.asList(State.START_STATES_B).contains(lastState)){
				newState.setCounter(-1);
				log4j.debug("... nach Aufschlag");
			}else
			{
				newState.setCounter(0);
				// Ball ist durchgegangen trotz Block, hat also die Seite gewechselt, wurde aber schon geblockt, daher Counter auf 1 erhoehen
				if(okBlock){
					newState.setCounter(1);
					// beide Spieler dürfen nun weiterspielen
					newState.setLastContact(null);
					log4j.debug("... trotz OK_BLOCK");
				}
				log4j.debug("... nach Angriff aus dem Spiel");
			}
			
		}
		else {
			// nach erfolgreicher Annahme eines Aufschlage von -1 auf 1 Counter inkrementieren
			if (lastState.getCounter() == -1){
				newState.setCounter(1);
			}else if(blockOverNet){
				// Ball ist auf der gleiche Seite wurde dort aber hingeblockt -> Counter auf 0
				newState.setCounter(0);
				// beide Spieler dürfen nun auch wieder den Ball annehmen, also LastContact auf null setzen.
				newState.setLastContact(null);
				log4j.debug("... nach Block");
			}
			else{
				newState.setCounter(lastState.getCounter()+1);
				log4j.debug("Counter erhoeht!");

			}
		}	
		
		if(newState.getFieldA1().belongsToSideB() || newState.getFieldA2().belongsToSideB()){
			log4j.warn("Spieler A1 oder A2 auf der Seite von B!");
		}
		if(newState.getFieldB1().belongsToSideA() || newState.getFieldB2().belongsToSideA()){
			log4j.warn("Spieler B1 oder B2 auf der Seite von A!");
		}
		
		setCurrentState(newState);
	}
	
	private BlockScenario playerBlocks(Player blockingPlayer){		
		log4j.debug("Blocking Action:" + blockingPlayer.getPlayerAction().toStringLong());
					
		Prob_Block bprob = blockingPlayer.getIndProbs().getProbBlock();
		double directPointProb = bprob.getDirectPointProb();
		double overNetNoFaultProb = bprob.getOverNetNoFaultProb();
		double faultProb = bprob.getFaultProb();
		
		double r = MainSim.countRandomCalls();
//		double r = Math.random();
		if (r < directPointProb){
			if (blockingPlayer == game.getTeamA().getPlayer2())  MainSim.numSuccBlocksA2++;
			log4j.debug("Block führt zu direktem Punkt");
			if (MainGui.isRunning == true){
				if (GUIManager.getInstance().getViewSimFrame() != null)
					GUIManager.getInstance().getMainFrame().getMySimSwingWorker().blockInfoText = "geblockt & Punkt";
			}
			return BlockScenario.DIRECT_POINT;
		}else if (directPointProb <= r && r < directPointProb + overNetNoFaultProb){
			log4j.debug("Block aber kein Punkt oder Fehler");
			if (MainGui.isRunning == true){
				if (GUIManager.getInstance().getViewSimFrame() != null)
					GUIManager.getInstance().getMainFrame().getMySimSwingWorker().blockInfoText = "geblockt";
			}
			return BlockScenario.OK_BLOCK;
		}else if (directPointProb + overNetNoFaultProb <= r && r < directPointProb + overNetNoFaultProb + faultProb){
			log4j.debug("Block führt zu Fehler");
			if (MainGui.isRunning == true){
				if (GUIManager.getInstance().getViewSimFrame() != null)
					GUIManager.getInstance().getMainFrame().getMySimSwingWorker().blockInfoText = "geblockt & Fehler";
			}
			return BlockScenario.FAULT;
		}else{
			log4j.debug("Block nicht erfolgreich, Ball kommt durch");
			if (MainGui.isRunning == true){
				if (GUIManager.getInstance().getViewSimFrame() != null)
					GUIManager.getInstance().getMainFrame().getMySimSwingWorker().blockInfoText = "nicht geblockt";
			}
			return BlockScenario.MISSES_BALL;
		}
	}
	
	/**
	 * Mit Hilfe der individuellen Spieler-W'keiten das naechste Feld fuer den Ball bestimmen
	 * gibt null zurueck falls "Ausfuehrungsfehler!"
	 **/
	private Field playerHitsBall(Player player, BallProperty ballProp){				
		log4j.debug("Player hits Ball");
		log4j.debug("Attacking Player:" + player.getPlayerAction().toStringLong());
		
		PlayerAction playeraction = player.getPlayerAction();
		double prob_succ;
		double prob_false;
		try {
			
			Prob_Hit probHit = player.getIndProbs().getProbHit(playeraction.getHit().getTechnique(), playeraction.getHit().getTargetField(), player.getField(), ballProp);
			if (probHit == null){
				throw new Exception("Prob_Hit for tech " + playeraction.getHit().getTechnique() + " and target field " + playeraction.getHit().getTargetField() + " does not exist");
			}
			if (useModProbs){
				prob_succ = probHit.getSuccProb_mod();
				prob_false = probHit.getFaultProb_mod();
			}else{
				prob_succ = probHit.getSuccProb();
				prob_false = probHit.getFaultProb();
			}
			
			
			double r = MainSim.countRandomCalls();
			if (r < prob_succ){
				//erfolgreicher Schlag		
				log4j.debug("erfolgreicher Schlag!");
				if (MainGui.isRunning == true){
					if (GUIManager.getInstance().getViewSimFrame() != null)
						GUIManager.getInstance().getMainFrame().getMySimSwingWorker().hitInfoText = "erfolgreicher Schlag";
				}
				return playeraction.getHit().getTargetField();
			}
			if (r >= prob_succ && r < prob_succ + prob_false){
				// Punkt fuer das andere Team
				log4j.debug("Fehler! von " + player.getTeam().name);
				if (MainGui.isRunning == true){
					if (GUIManager.getInstance().getViewSimFrame() != null)
						GUIManager.getInstance().getMainFrame().getMySimSwingWorker().hitInfoText = "Fehler von " + player.getTeam().name;
				}
				return null;
			}
			if (r >= prob_succ + prob_false){
				//Nachbarfeld bestimmen
				Field neighbour;
				// nur bei Aufschlägen und Angriffsschlägen soll Abweichung nur zum Ausfeld passieren!
				if (MainSim.M1_devToOutsideFieldOnlyForServesAndAttacks && Arrays.asList(new HitTechnique[]{HitTechnique.S_F, HitTechnique.S_J, HitTechnique.F_SM, HitTechnique.F_E, HitTechnique.F_P}).contains(playeraction.getHit().getTechnique())){
					Field[] outsideNeighbours = Field.getNeighbourOutsideFieldsOf(playeraction.getHit().getTargetField());
					if (outsideNeighbours.length != 0){
						neighbour = Field.chooseRandomField(outsideNeighbours);
					}else{
						neighbour = Field.chooseRandomField(playeraction.getHit().getTargetField().getNeighbours());
					}
				}else{
					neighbour = Field.chooseRandomField(playeraction.getHit().getTargetField().getNeighbours());
				}
				log4j.debug("Hit: " + playeraction.getHit().toString() + " Nachbarfeld: " + neighbour.toString());
				if (MainGui.isRunning == true){
					if (GUIManager.getInstance().getViewSimFrame() != null)
						GUIManager.getInstance().getMainFrame().getMySimSwingWorker().hitInfoText = "Nachbarfeld: " + neighbour.toString();
				}
				return neighbour;
			}
			log4j.error("playerPlaysBall bis zum Ende durchgelaufen!!!! -> das sollte nicht passieren!");
			
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
				
		log4j.error("playerPlaysBall bis zum Ende durchgelaufen!!!! -> das sollte nicht passieren!");
		return null;
	}
	
	public State getNextServingState(State s){
		if (State.POINT_A.contains(s)){
			if (s.getLastServer().getTeam() == teamA){
				return getServingStateOf(s.getLastServer());
			}else{
				return getServingStateOf(chooseNextServingPlayer(s.getLastServer()));
			}
		}
		if (State.POINT_B.contains(s)){
			if (s.getLastServer().getTeam() == teamB){
				return getServingStateOf(s.getLastServer());
			}else{
				return getServingStateOf(chooseNextServingPlayer(s.getLastServer()));
			}
		}
		log4j.error("Last Server " + s.getLastServer().toString() + " is not in the match. Can't compute next serving state.");
		return null;
	}
	
	public State getRandomServingState(){
		return State.chooseRandomStateOfArray(State.START_STATES);
	}
	
	public State getServingStateOf(Player nextServer){
		if (nextServer == teamA.getPlayer1()){
			return State.chooseRandomStateOfArray(State.START_STATES_A1);
		}
		if (nextServer == teamA.getPlayer2()){
			return State.chooseRandomStateOfArray(State.START_STATES_A2);
		}
		if (nextServer == teamB.getPlayer1()){
			return State.chooseRandomStateOfArray(State.START_STATES_B1);
		}
		if (nextServer == teamB.getPlayer2()){
			return State.chooseRandomStateOfArray(State.START_STATES_B2);
		}
		log4j.error("Server " + nextServer.toString() + " is no Player of the game!");
		return null;
	}
	
	public Player chooseNextServingPlayer(Player lastServer){
		int index = servingOrder.indexOf(lastServer);
		index = index +1;
		index = index % servingOrder.size();
		Player nextServer = servingOrder.get(index);
		return nextServer;
	}	
	
	public void chooseTeamActions() {
		// es wird eine Zufallszahl gezogen, die fuer beide Teamaktionen gilt, damit die Reihenfolge ob zuerst die Teamaktion
		// von Team A oder von Team B bestimmt wird, keine Rolle spielt
		double r = MainSim.countRandomCalls();
		
		try {
			game.getTeamA().chooseTeamAction(r);
//			game.getTeamA().chooseTeamActionAtRandom(r);
		} catch (Exception e) {
			e.printStackTrace();
			try {
				game.getTeamA().setTeamAction(TeamAction.doNothing);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
		try {
			log4j.debug("Team A spielt Aktion:" + game.getTeamA().getTeamAction().toString());
		} catch (Exception e2) {
			e2.printStackTrace();
		}

		try {
			game.getTeamB().chooseTeamAction(r);
//			game.getTeamB().chooseTeamActionAtRandom(r);
		} catch (Exception e) {
			e.printStackTrace();
			try {
				game.getTeamB().setTeamAction(TeamAction.doNothing);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
		try {
			log4j.debug("Team B spielt Aktion:" + ((Team) game.getTeamB()).getTeamAction().toStringLong());
		} catch (Exception e) {
			e.printStackTrace();
		}	
	}
		
	public int getPointsTeamA(){
		return this.pointsTeamA;
	}
	
	public void setPointsTeamA(int p){
		this.pointsTeamA = p;
	}

	public int getPointsTeamB(){
		return this.pointsTeamB;
	}
	
	public void setPointsTeamB(int p){
		this.pointsTeamB = p;
	}
	
	public Team getTeamB() {
		return teamB;
	}

	public void setTeamB(Team teamB) {
		this.teamB = teamB;
	}

	public Team getTeamA() {
		return teamA;
	}

	public void setTeamA(Team teamA) {
		this.teamA = teamA;
	}
		
	public List<State> getStateHist() {
		return stateHist;
	}

	public ArrayList<Player> getServingOrder() {
		return servingOrder;
	}


	public void setServingOrder(ArrayList<Player> servingOrder) {
		this.servingOrder = servingOrder;
	}


	@Override
	public String toString() {			
		return super.toString();
	}
	
	public String printScore(){
//		return this.getTeamA().getTeamName() + " " + this.getPointsTeamA() + " : " + this.getPointsTeamB() + " " + this.getTeamB().getTeamName();
		return this.getPointsTeamA() + " : " + this.getPointsTeamB();
	}
	
	public StringBuffer printStateHist(){
		
		StringBuffer string = new StringBuffer();
		for (State s : stateHist){
			string = string.append(s.toString() + ", ");
		}
		
		return string;
	}

	// auskommentiert, da HashSet verwendet wird->zufällige Reihenfolge
//	public Field chooseRandomField(HashSet<Field> fields){
//    	Field[] possibleFieldsArray = fields.toArray(new Field[fields.size()]);
//		int l = possibleFieldsArray.length;
//		int n = MainSim.r.nextInt(l);
//		return possibleFieldsArray[n];
//    	
//    }

	public void setEmptyTeamActions() {
		try {
			game.getTeamA().setTeamAction(new TeamAction());
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (game.getTeamB().getClass() == Team.class){
			try {
				((Team) game.getTeamB()).setTeamAction(new TeamAction());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}		
	}
	
	private State helper_findLastServingState(){
		while(!Arrays.asList(State.START_STATES_A).contains(stateHist.get(stateHist.size()-1)) &&
				!Arrays.asList(State.START_STATES_B).contains(stateHist.get(stateHist.size()-1))){
			stateHist.remove(stateHist.size()-1);
		}
		return stateHist.get(stateHist.size()-1);
	}

}