package de.ubt.wm.sb.model;
import java.util.HashSet;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.sim.MainSim;


public class Hit{
	
	static final Logger log4j = LogManager.getLogger(Hit.class.getName());

	public static final HashSet<Hit> JUMP_SERVE_A;
	public static final HashSet<Hit> FLOAT_SERVE_A;
	
	static{
		JUMP_SERVE_A = new HashSet<Hit>();
		FLOAT_SERVE_A = new HashSet<Hit>();
		for (int i = 0; i < Field.SECOND_THIRD_ROW_B.length; i++){
			JUMP_SERVE_A.add(new Hit(HitTechnique.S_J, Field.SECOND_THIRD_ROW_B[i]));
			FLOAT_SERVE_A.add(new Hit(HitTechnique.S_F, Field.SECOND_THIRD_ROW_B[i]));
		}
	}
	
	public static final HashSet<Hit> SERVES_TEAMA; 
	
	static {
		SERVES_TEAMA = new HashSet<Hit>();
		SERVES_TEAMA.addAll(JUMP_SERVE_A);
		SERVES_TEAMA.addAll(FLOAT_SERVE_A);
	}
	
	public static final HashSet<Hit> SMASH_TEAMA;
	public static final HashSet<Hit> SHOT_TEAMA; 
	
	static {
		SMASH_TEAMA = new HashSet<Hit>();
		SHOT_TEAMA = new HashSet<Hit>();
		for (Field field : Field.SECOND_THIRD_ROW_B){
			SMASH_TEAMA.add(new Hit(HitTechnique.F_SM, field));
			SHOT_TEAMA.add(new Hit(HitTechnique.F_E, field));
		}
	}
	
	public static final HashSet<Hit> POKE_TEAMA; 
	
	static {
		POKE_TEAMA = new HashSet<Hit>();
		for(Field field : Field.INSIDE_B){
			POKE_TEAMA.add(new Hit(HitTechnique.F_P, field));
		}
	}
	
	public static final HashSet<Hit> SMASHSHOTS_TEAMA;
	
	static{
		SMASHSHOTS_TEAMA = new HashSet<Hit>();
		SMASHSHOTS_TEAMA.addAll(SMASH_TEAMA);
		SMASHSHOTS_TEAMA.addAll(SHOT_TEAMA);
	}
	
	public static final HashSet<Hit> ATTACKS_TEAMA;
	
	static{
		ATTACKS_TEAMA = new HashSet<Hit>();
		ATTACKS_TEAMA.addAll(SMASH_TEAMA);
		ATTACKS_TEAMA.addAll(SHOT_TEAMA);
		ATTACKS_TEAMA.addAll(POKE_TEAMA);
	}
	
	public static final HashSet<Hit> JUMP_SERVE_B;
	public static final HashSet<Hit> FLOAT_SERVE_B;
	
	static{
		JUMP_SERVE_B = new HashSet<Hit>();
		FLOAT_SERVE_B = new HashSet<Hit>();
		for (int i = 0; i < Field.SECOND_THIRD_ROW_A.length; i++){
			JUMP_SERVE_B.add(new Hit(HitTechnique.S_J, Field.SECOND_THIRD_ROW_A[i]));
			FLOAT_SERVE_B.add(new Hit(HitTechnique.S_F, Field.SECOND_THIRD_ROW_A[i]));
		}
	}
	
	public static final HashSet<Hit> SERVES_TEAMB; 
	
	static {
		SERVES_TEAMB = new HashSet<Hit>();
		SERVES_TEAMB.addAll(JUMP_SERVE_B);
		SERVES_TEAMB.addAll(FLOAT_SERVE_B);
	}
	
	
	public static final HashSet<Hit> SMASH_TEAMB; 
	public static final HashSet<Hit> SHOT_TEAMB;
	
	static {
		SMASH_TEAMB = new HashSet<Hit>();
		SHOT_TEAMB = new HashSet<Hit>();
		for(Field field : Field.SECOND_THIRD_ROW_A){
			SMASH_TEAMB.add(new Hit(HitTechnique.F_SM, field));
			SHOT_TEAMB.add(new Hit(HitTechnique.F_E, field));
			}
	}
	
	
	public static final HashSet<Hit> POKE_TEAMB; 
	
	static {
		POKE_TEAMB = new HashSet<Hit>();
		for(Field field : Field.INSIDE_A){
			POKE_TEAMB.add(new Hit(HitTechnique.F_P, field));
		}
	}
	
	public static final HashSet<Hit> SMASHSHOTS_TEAMB;
	
	static{
		SMASHSHOTS_TEAMB = new HashSet<Hit>();
		SMASHSHOTS_TEAMB.addAll(SMASH_TEAMB);
		SMASHSHOTS_TEAMB.addAll(SHOT_TEAMB);
	}
	
	public static final HashSet<Hit> ATTACKS_TEAMB;
	
	static{
		ATTACKS_TEAMB = new HashSet<Hit>();
		ATTACKS_TEAMB.addAll(SMASH_TEAMB);
		ATTACKS_TEAMB.addAll(SHOT_TEAMB);
		ATTACKS_TEAMB.addAll(POKE_TEAMB);
	}
	
	
	//--------------- team independent hits ---------------------------------------------------------
		
	public static final Hit nohit = new Hit(null, null);	
	
	
	public static final HashSet<Hit> RECEPTIONS;
	
	static{
		RECEPTIONS = new HashSet<Hit>();
		for (Field field : Field.ALL){
			RECEPTIONS.add(new Hit(HitTechnique.r, field));
			RECEPTIONS.add(new Hit(HitTechnique.r_m, field));
		}
	}
	
	public static final HashSet<Hit> SETTINGS;
	
	static{
		SETTINGS = new HashSet<Hit>();
		for (Field field : Field.ALL){
			SETTINGS.add(new Hit(HitTechnique.s, field));
		}
	}
	
	public static final HashSet<Hit> DEFENCES;
	
	static{
		DEFENCES = new HashSet<Hit>();
		for (Field field : Field.ALL){
			DEFENCES.add(new Hit(HitTechnique.d, field));
			DEFENCES.add(new Hit(HitTechnique.d_m, field));
		}
	}
	
	
	
	//------------------------------ Ende der statischen Variablen ---------------------------------------------------
	
	private Field targetField;
	
	private HitTechnique technique;
	
	public Hit(HitTechnique technique, Field targetfield){   		
    		this.targetField = targetfield;
    		this.technique = technique;
    }
    
    public boolean isHitWithJump(){
    	if (this.technique == HitTechnique.F_SM || this.technique == HitTechnique.S_J){
    		return true;
    	}
    	return false;
    }
    
    public boolean isHardHit(){
    	if (this.technique == HitTechnique.F_SM || this.technique == HitTechnique.S_J){
    		return true;
    	}
    	return false;
    }
    
    public Field getTargetField(){
    	return this.targetField;
    }
    
    
    public HitTechnique getTechnique(){
    	return this.technique;
    } 
       
	/**
	 * Sucht aus einem HashSet von Hits zufällig und gleichverteilt einen Hit aus
	 * falls übergebenes HashSet leer => warning
	 * @return Hit
	 */
	public static Hit chooseRandomHit(HashSet<Hit> hits){
		if (hits.size()>0){
			Hit[] hitsArray = hits.toArray(new Hit[hits.size()]);
			int l = hitsArray.length;
			int n = MainSim.r.nextInt(l);
			return hitsArray[n];
		}else{
			log4j.warn("chooseRandomHit mit leerer Menge aufgerufen!");
			return null;
		}
		
	}
    
	public static Hit chooseServeAfterStrat(boolean isTeamA, double pi_serve_tech, double pi_serve_field, double pi_o1){
		HitTechnique tech;
		Field f;
		
		if (MainSim.countRandomCalls() < pi_serve_tech){
			tech = HitTechnique.S_J;
		}else{
			tech = HitTechnique.S_F;
		}
		
		if (MainSim.countRandomCalls() < pi_serve_field){
			if (isTeamA){
				if(MainSim.countRandomCalls() < pi_o1){
					// es wird auf die Seite von Opponent 1 gespielt
					f = Field.chooseRandomField(new Field[]{Field.B11, Field.B21});
				}else{
					f = Field.chooseRandomField(new Field[]{Field.B14, Field.B24});
				}
			}else{
				if(MainSim.countRandomCalls() < pi_o1){
					// es wird auf die Seite von Opponent 1 gespielt
					f = Field.chooseRandomField(new Field[]{Field.A11, Field.A21});
				}else{
					f = Field.chooseRandomField(new Field[]{Field.A14, Field.A24});
				}}
		}else{
			if (isTeamA){
				if(MainSim.countRandomCalls() < pi_o1){
					// es wird auf die Seite von Opponent 1 gespielt
					f = Field.chooseRandomField(new Field[]{Field.B12, Field.B22});
				}else{
					f = Field.chooseRandomField(new Field[]{Field.B13, Field.B23});
				}
			}else{
				if(MainSim.countRandomCalls() < pi_o1){
					// es wird auf die Seite von Opponent 1 gespielt
					f = Field.chooseRandomField(new Field[]{Field.A12, Field.A22});
				}else{
					f = Field.chooseRandomField(new Field[]{Field.A13, Field.A23});
				}}
		}
		return new Hit(tech, f);		
	}
		
	public static Hit chooseFieldAttackAfterStrat(boolean isTeamA, double pi_field_tech, double pi_field_field){
		HitTechnique tech;
		Field f;
		
		if (MainSim.countRandomCalls() < pi_field_tech){
			tech = HitTechnique.F_SM;
		}else{
			tech = HitTechnique.F_P;
		}
		
		if (MainSim.countRandomCalls() < pi_field_field){
			if (isTeamA){
				if (tech!=HitTechnique.F_SM){
					f = Field.chooseRandomField(Field.BORDER_LEFT_RIGHT_B);
				}else{
					f = Field.chooseRandomField(new Field[]{Field.B11, Field.B21, Field.B14, Field.B24});
				}
			}else{
				if (tech!=HitTechnique.F_SM){
					f = Field.chooseRandomField(Field.BORDER_LEFT_RIGHT_A);
				}else{
					f = Field.chooseRandomField(new Field[]{Field.A11, Field.A21, Field.A14, Field.A24});
				}
			}
		}else{
			if (isTeamA){
				if (tech!=HitTechnique.F_SM){
					f = Field.chooseRandomField(Field.NON_BORDER_FIELD_B);
				}else{
					f = Field.chooseRandomField(new Field[]{Field.B12, Field.B22, Field.B13, Field.B23});
				}
			}else{
				if (tech!=HitTechnique.F_SM){
					f = Field.chooseRandomField(Field.NON_BORDER_FIELD_A);
				}else{
					f = Field.chooseRandomField(new Field[]{Field.A12, Field.A22, Field.A13, Field.A23});
				}
			}
		}
		return new Hit(tech, f);		
	}
	
    
    @Override
    public boolean equals(Object obj) {
    	if (obj instanceof Hit){
    		Hit compareTo = (Hit) obj;
    		if (this.technique == compareTo.technique && this.targetField == compareTo.targetField){
    			return true;
    		}else{
    			return false;
    		}
    	}else{
    		return false;
    	}
    };
    
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 17 * hash + (this.technique != null ? this.technique.hashCode() : 0) 
        		+ (this.targetField != null ? this.targetField.getName().hashCode() : 0);
        return hash;
    }
    
    
	@Override
	public String toString() {			
		if (this == Hit.nohit){
			return "noHit";
		}else{
			return "(" + this.technique + ", " + this.targetField + ")" ;
		}		
	}

}