package de.ubt.wm.sb.model;

import java.util.HashSet;
import java.util.Objects;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.sim.MainSim;

/**
 * TeamAction
 */	
@SuppressWarnings("serial")
public class TeamAction implements java.io.Serializable {

	static final Logger log4j = LogManager.getLogger(TeamAction.class.getName());
	
	public static TeamAction doNothing =  new TeamAction(new PlayerAction(Hit.nohit, Move.stay), new PlayerAction(Hit.nohit, Move.stay),null);

	private PlayerAction actionPlayer1;
	private PlayerAction actionPlayer2;
	private Player designatedBlockingPlayer;

	/**
	 * Legt eine neue Team Aktion an, in der beide Spieler nichts tun. 
	 */	
	public TeamAction(){
    	this.actionPlayer1 = new PlayerAction(Hit.nohit, Move.stay);
    	this.actionPlayer2 = new PlayerAction(Hit.nohit, Move.stay);
    	this.designatedBlockingPlayer = null;
	};
	
    public TeamAction(PlayerAction playerAction1, PlayerAction playerAction2){
    	this.actionPlayer1 = playerAction1;
    	this.actionPlayer2 = playerAction2;
    }
    
    public TeamAction(PlayerAction playerAction1, PlayerAction playerAction2, Player blockingPlayer){
    	this.actionPlayer1 = playerAction1;
    	this.actionPlayer2 = playerAction2;
    	this.designatedBlockingPlayer = blockingPlayer;
    }
         
    public void setPlayerActionOfPlayer(Player p, PlayerAction a){
    	if (p == Game.getInstance().getTeamA().getPlayer1() || p == Game.getInstance().getTeamB().getPlayer1()){
    		this.actionPlayer1 = a;
    		return;
    	}
    	if (p == Game.getInstance().getTeamA().getPlayer2() || p == Game.getInstance().getTeamB().getPlayer2()){
    		this.actionPlayer2 = a;
    		return;
    	}
    	log4j.warn("Player unknown, not in TeamA nor in TeamB, can not set Playeraciton " + a + " of player " + p + " in state " + Game.getInstance().getCurrentState().toString());
    }
    
    public PlayerAction getActionPlayer1(){
    	return this.actionPlayer1;
    }
    
    public void setActionPlayer1(PlayerAction playerAction){
    	this.actionPlayer1 = playerAction;
    }

    public PlayerAction getActionPlayer2(){
    	return this.actionPlayer2;
    }
    
    public void setActionPlayer2(PlayerAction playerAction){
    	this.actionPlayer2 = playerAction;
    }
    
	public Player getDesignatedBlockingPlayer() {
		return designatedBlockingPlayer;
	}

	public void setDesignatedBlockingPlayer(Player designatedBlockingPlayer) {
		this.designatedBlockingPlayer = designatedBlockingPlayer;
	}

	/**
	 * Sucht aus einem HashSet von TeamAktionen zufaellig und gleichverteilt eine TeamAktion aus
	 * falls uebergebenes HashSet leer => warning
	 * @return TeamAction
	 */
	public static TeamAction chooseRandomTeamAction(HashSet<TeamAction> teamActions){
		
		if (teamActions.size()>0){
			TeamAction[] teamActionsArray = teamActions.toArray(new TeamAction[teamActions.size()]);
			int l = teamActionsArray.length;
			int n = MainSim.r.nextInt(l);
			return teamActionsArray[n];
		}else{
			log4j.warn("chooseRandomTeamAction mit leerer Menge aufgerufen!");
			return null;
		}
		
	}
	
//    @Override
//    public boolean equals(Object obj) { 	
//    	if(obj instanceof TeamAction){
//    	    TeamAction toCompare = (TeamAction) obj;
//    	    if(this.getActionPlayer1().getHit().equals(toCompare.getActionPlayer1().getHit())
//    				&& this.getActionPlayer2().getHit().equals(toCompare.getActionPlayer2().getHit())
//    				&& this.getActionPlayer1().getMove() == toCompare.getActionPlayer1().getMove()
//    				&& this.getActionPlayer2().getMove() == toCompare.getActionPlayer2().getMove()){
//    			return true;
//    		}else{
//    			return false;
//    		}
//    	  }
//    	  return false;
//    };
    
	@Override
	public String toString() {	
		if (designatedBlockingPlayer == null){
			return "[" + actionPlayer1.toString() + actionPlayer2.toString() + ", - ]";
		}else{
			return "[" + actionPlayer1.toString() + actionPlayer2.toString() + ", " + designatedBlockingPlayer.getName() +"]";
		}
	}
	
	public String toStringLong() {			
		if (designatedBlockingPlayer == null){
			return "[" + actionPlayer1.toStringLong() + actionPlayer2.toStringLong() + ", - ]";
		}else{
			return "[" + actionPlayer1.toStringLong() + actionPlayer2.toStringLong() + ", " + designatedBlockingPlayer.toString() + "]";			
		}
	}

	
	@Override
	public int hashCode(){
		return Objects.hash(
				actionPlayer1,actionPlayer2,designatedBlockingPlayer);
	}
	
	@Override
	public boolean equals(Object o){
		if (!(o instanceof TeamAction)){
			return false;
		}
		if (o == this) return true;
		TeamAction ta = (TeamAction) o;
		
		return actionPlayer1.equals(ta.actionPlayer1) && actionPlayer2.equals(ta.actionPlayer2) && designatedBlockingPlayer==ta.designatedBlockingPlayer;
	}
	
}