package de.ubt.wm.sb.model.strategy;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Hit;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.Move;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.PlayerAction;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.TeamAction;
import de.ubt.wm.sb.sim.MPC_TableRuns_Part1;

public class Strategy implements RealStrat{

	static final Logger log4j = LogManager.getLogger(RealStrat.class.getName());

	
	public static Strategy BasicTeamA = new Strategy("BASIC TeamA", true, 0.5, new double[][] {{0.5, 0.5, 0.4, 0.5},{0.5, 0.5, 0.4, 0.5}}, 0.5);
	public static Strategy BasicTeamB = new Strategy("BASIC TeamB", false, 0.5, new double[][] {{0.5, 0.5, 0.4, 0.5},{0.5, 0.5, 0.4, 0.5}}, 0.5);
	
	public static Strategy RiskyTeamA = new Strategy("Risky TeamA", true, 0.5, new double[][] {{1, 1, 1, 1},{1, 1, 1, 1}}, 0.5);
	public static Strategy RiskyTeamB = new Strategy("Risky TeamB", false, 0.5, new double[][] {{1, 1, 1, 1},{1, 1, 1, 1}}, 0.5);
	
	public static Strategy SafeTeamA = new Strategy("Safe TeamA", true, 0.5, new double[][] {{0, 0, 0, 0},{0, 0, 0, 0}}, 0.5);
	public static Strategy SafeTeamB = new Strategy("Safe TeamB", false, 0.5, new double[][] {{0, 0, 0, 0},{0, 0, 0, 0}}, 0.5);
	
	public static Strategy RiskySafeTeamA = new Strategy("Risky-Safe TeamA", true, 0.5, new double[][] {{1, 1, 0, 0},{1, 1, 0, 0}}, 0.5);
	public static Strategy RiskySafeTeamB = new Strategy("Risky-Safe TeamB", false, 0.5, new double[][] {{1, 1, 0, 0},{1, 1, 0, 0}}, 0.5);
	
	public static Strategy SafeRiskyTeamA = new Strategy("Safe-Risky TeamA", true, 0.5, new double[][] {{0, 0, 1, 1},{0, 0, 1, 1}}, 0.5);
	public static Strategy SafeRiskyTeamB = new Strategy("Safe-Risky TeamB", false, 0.5, new double[][] {{0, 0, 1, 1},{0, 0, 1, 1}}, 0.5);

//	// update: 23.03.2018
//	public static Strategy cali_finalMatch_TeamA = new Strategy("caliFinalA", true, 0.0156, new double[][] {{0.1923, 0.0385, 0.7391, 0.2609},{0.2414, 0.1724, 0.6923, 0.3333}}, 0.2727);
//	public static Strategy cali_finalMatch_TeamB = new Strategy("caliFinalB", false, 0.9608, new double[][] {{0.68, 0.16, 0.9286, 0.4286},{0.3667, 0.2667, 0.8148, 0.537}}, 0.3455);
//
//	public static Strategy cali_allMatchesExceptFinal_TeamA = new Strategy("caliAllExceptA", true, 0.0244, new double[][] {{0.3937, 0.189, 0.6532, 0.3871},{0.5041, 0.3636, 0.7167, 0.45}}, 0.5);
//	public static Strategy cali_allMatchesExceptFinal_TeamB = new Strategy("caliAllExceptB", false, 0.9585, new double[][] {{0.3724,0.1793, 0.82,0.36},{0.3534,0.1729, 0.8619,0.3094}}, 0.5);

	/**
	 * read actual strategies of final and prefinal Matches from file
	 */
	public static Strategy cali_allMatchesExceptFinal_TeamA_read;
	public static Strategy cali_allMatchesExceptFinal_TeamB_read;
	public static Strategy cali_allMatches_TeamA_read;
	public static Strategy cali_allMatches_TeamB_read;
	public static Strategy cali_finalMatch_TeamA_read;
	public static Strategy cali_finalMatch_TeamB_read;

	static {
		// fuer allMatchesExceptFinale & finalMacht
		for (int dataFileType=0; dataFileType<MPC_TableRuns_Part1.skillDataFiles.length; dataFileType++){
			for (int team=0; team<MPC_TableRuns_Part1.skillDataFiles[dataFileType].length; team++){
				String csvFile = MPC_TableRuns_Part1.skillDataFiles[dataFileType][team];
				
				double[][] pi_h = new double[2][4];
				double pi_b = -1.0;
				double pi_s = -1.0;
				
		        BufferedReader br = null;
		        String line = "";
		        String cvsSplitBy = ":";
				try {
				
				    br = new BufferedReader(new FileReader(csvFile));
				    while ((line = br.readLine()) != null) {
				
				        // use cvsSplitBy as separator
				        String[] myline = line.split(cvsSplitBy);
				        if (myline!= null && myline[0].length() > 3){
				        	if (myline[0].substring(0, 4).equals("# P1")){
				        		String pihP1 = myline[1].substring(myline[1].indexOf('[')+1, myline[1].lastIndexOf(']'));
				        		String[] pihP1split = pihP1.split(",");
				        		for (int i=0; i<pihP1split.length; i++){
				        			pihP1split[i]=pihP1split[i].replace("[", "");
				        			pihP1split[i]=pihP1split[i].replace("]", "");
				        			pihP1split[i]=pihP1split[i].replace(" ", "");
				        			pi_h[0][i]=Double.parseDouble(pihP1split[i]);
				        		}
				        	}
				        	if (myline[0].substring(0, 4).equals("# P2")){
				        		String pihP2 = myline[1].substring(myline[1].indexOf('[')+1, myline[1].lastIndexOf(']'));
				        		String[] pihP2split = pihP2.split(",");
				        		for (int i=0; i<pihP2split.length; i++){
				        			pihP2split[i]=pihP2split[i].replace("[", "");
				        			pihP2split[i]=pihP2split[i].replace("]", "");
				        			pihP2split[i]=pihP2split[i].replace(" ", "");
				        			pi_h[1][i]=Double.parseDouble(pihP2split[i]);
				        		}
				        	}
				        }
				        if (myline!= null && myline[0].length() > 5){
				        	if (myline[0].substring(0, 6).equals("# pi_b")){
				        		String pi_b_str = myline[1].trim();
				        		pi_b = Double.parseDouble(pi_b_str);
				        	}
				        }
				        if (myline!= null && myline[0].length() > 5){
				        	if (myline[0].substring(0, 6).equals("# pi_s")){
				        		String pi_s_str = myline[1].trim();
				        		pi_s = Double.parseDouble(pi_s_str);
				        	}
				        }
				    }
				
				} catch (FileNotFoundException e) {
				    e.printStackTrace();
				} catch (IOException e) {
				    e.printStackTrace();
				} finally {
				    if (br != null) {
				        try {
				            br.close();
				        } catch (IOException e) {
				            e.printStackTrace();
				        }
				    }
				}
				if (csvFile.contains("final")){
					if (csvFile.contains("GER")){
							// 1-pi_s da Brink Spieler 1 ist und im Finale in Spalte *3,*4 (=links von hinten gesehen)
							cali_finalMatch_TeamA_read = new Strategy("caliFinalA", true, pi_b, pi_h,1-pi_s);
					}else{
						if (csvFile.contains("BRA")){
							// 1-pi_s da Alison Spieler 1 ist und im Finale in Spalte *3,*4 (=links von hinten gesehen)
							cali_finalMatch_TeamB_read = new Strategy("caliFinalB", false, pi_b, pi_h,1-pi_s);
						}
					}
				}else{
					if (csvFile.contains("allMatchesExceptFinal")){
						if (csvFile.contains("GER")){
							// pi_s bei all Except Final wird serve auf 0.5 gesetzt
							cali_allMatchesExceptFinal_TeamA_read = new Strategy("caliAllExceptA", true, pi_b, pi_h, 0.5);
						}else{
							if (csvFile.contains("BRA")){
								// bei all Except Final wird serve auf 0.5 gesetzt
								cali_allMatchesExceptFinal_TeamB_read = new Strategy("caliAllExceptB", false, pi_b, pi_h, 0.5);
							}
						}
					}else
						if (csvFile.contains("allMatches.")){
							if (csvFile.contains("GER")){
								// pi_s bei all Except Final wird serve auf 0.5 gesetzt
								cali_allMatches_TeamA_read = new Strategy("caliAllA", true, pi_b, pi_h, 0.5);
							}else{
								if (csvFile.contains("BRA")){
									// bei all Except Final wird serve auf 0.5 gesetzt
									cali_allMatches_TeamB_read = new Strategy("caliAllB", false, pi_b, pi_h, 0.5);
								}
							}
						}else
						{
							System.err.println("Fehler beim Einlesen der Strategien."
									+"Skill Data Files Namen nicht erkannt expected: <final>, <allMatchesExceptFinal>, <allMatches>!");
						}
				}
				
			} // Ende for über Teams
		} // Ende for über all Matches Except, final Dateien	
	}
	
	//true und false müssen gedreht sein
	public static Strategy cali_finalMatch_TeamA_test = new Strategy("caliFinalA_test", false, 0.0156, new double[][] {{0.5, 0.0769, 0.7391, 0.24},{0.3103, 0.1724, 0.6923, 0.325}}, 0.2453);
	public static Strategy cali_finalMatch_TeamB_test = new Strategy("caliFinalB_test", true, 0.9608, new double[][] {{0.76, 0.24, 0.9286, 0.5},{0.4194, 0.3226, 0.92, 0.5439}}, 0.3265);
	public static Strategy cali_allMatches_TeamAOLD = new Strategy("caliAllA", true, 0.0224, new double[][] {{0.4145, 0.1645, 0.6667, 0.3434},{0.4667, 0.3267, 0.7044, 0.4080}}, 0.3206);
	public static Strategy cali_allMatches_TeamBOLD = new Strategy("caliAllB", false, 0.9589, new double[][] {{0.4226, 0.1964, 0.8348, 0.378},{0.3659, 0.2073, 0.8745, 0.3658}}, 0.5);
	public static Strategy cali_allMatchesExceptFinal_TeamAOLD = new Strategy("caliAllExcfA", true, 0.0245, new double[][] {{0.3968, 0.1825, 0.6532, 0.3617},{0.5041, 0.3636, 0.7083, 0.4328}}, 0.3397);
	public static Strategy cali_allMatchesExceptFinal_TeamBOLD = new Strategy("caliAllExfB", false, 0.9585, new double[][] {{0.3636, 0.0839, 0.8218, 0.3578},{0.3534, 0.1805, 0.8619, 0.3150}}, 0.4974);
	public static Strategy cali_5GERBRACR_GER = new Strategy("Calibrated Match 5 GER-BRA(CR) Team GER", true, 0.0333, new double[][] {{0.1905, 0.2381, 0.7917, 0.28},{0.45, 0.30, 0.8095, 0.3846}}, 0.5);
	public static Strategy test_5GERBRACR_GER = new Strategy("Test Match 5 GER-BRA(CR) Team GER", true, 0.0333, new double[][] {{1, 1, 0.7917, 0.28},{1, 1, 0.8095, 0.3846}}, 0.5);
	
	
	/** prefinal strategies **/
	public static double prefinal_pi_b_TeamA = cali_allMatchesExceptFinal_TeamA_read.getPi_b();
	public static double prefinal_pi_s_TeamA = 0.5;
	public static double prefinal_pi_b_TeamB = cali_allMatchesExceptFinal_TeamB_read.getPi_b();
	public static double prefinal_pi_s_TeamB = 0.5;
	
	public static Strategy prefinalRiskyA = new Strategy("prefinal Risky A", true, prefinal_pi_b_TeamA, new double[][] {{1, 1, 1, 1},{1, 1, 1, 1}}, prefinal_pi_s_TeamA);
	public static Strategy prefinalRiskySafeA = new Strategy("prefinal Risky-Safe A", true, prefinal_pi_b_TeamA, new double[][] {{1, 1, 0, 0},{1, 1, 0, 0}}, prefinal_pi_s_TeamA);
	public static Strategy prefinalSafeRiskyA = new Strategy("prefinal Safe-Risky A", true, prefinal_pi_b_TeamA, new double[][] {{0, 0, 1, 1},{0, 0, 1, 1}}, prefinal_pi_s_TeamA);
	public static Strategy prefinalSafeA = new Strategy("prefinal Safe A", true, prefinal_pi_b_TeamA, new double[][] {{0, 0, 0, 0},{0, 0, 0, 0}}, prefinal_pi_s_TeamA);
	
	public static Strategy prefinalRiskyB = new Strategy("prefinal Risky B", false, prefinal_pi_b_TeamB, new double[][] {{1, 1, 1, 1},{1, 1, 1, 1}}, prefinal_pi_s_TeamB);
	public static Strategy prefinalRiskySafeB = new Strategy("prefinal Risky-Safe B", false, prefinal_pi_b_TeamB, new double[][] {{1, 1, 0, 0},{1, 1, 0, 0}}, prefinal_pi_s_TeamB);
	public static Strategy prefinalSafeRiskyB = new Strategy("prefinal Safe-Risky B", false, prefinal_pi_b_TeamB, new double[][] {{0, 0, 1, 1},{0, 0, 1, 1}}, prefinal_pi_s_TeamB);
	public static Strategy prefinalSafeB = new Strategy("prefinal Safe B", false, prefinal_pi_b_TeamB, new double[][] {{0, 0, 0, 0},{0, 0, 0, 0}}, prefinal_pi_s_TeamB);
	
	/** final strategies **/
	public static double final_pi_b_TeamA = cali_finalMatch_TeamA_read.getPi_b();
	public static double final_pi_s_TeamA = cali_finalMatch_TeamA_read.getPi_o1();
	public static double final_pi_b_TeamB = cali_finalMatch_TeamB_read.getPi_b();
	public static double final_pi_s_TeamB = cali_finalMatch_TeamB_read.getPi_o1();
	
	public static Strategy finalRiskyA = new Strategy("final Risky A", true, final_pi_b_TeamA, new double[][] {{1, 1, 1, 1},{1, 1, 1, 1}}, final_pi_s_TeamA);
	public static Strategy finalRiskySafeA = new Strategy("final Risky-Safe A", true, final_pi_b_TeamA, new double[][] {{1, 1, 0, 0},{1, 1, 0, 0}}, final_pi_s_TeamA);
	public static Strategy finalSafeRiskyA = new Strategy("final Safe-Risky A", true, final_pi_b_TeamA, new double[][] {{0, 0, 1, 1},{0, 0, 1, 1}}, final_pi_s_TeamA);
	public static Strategy finalSafeA = new Strategy("final Safe A", true, final_pi_b_TeamA, new double[][] {{0, 0, 0, 0},{0, 0, 0, 0}}, final_pi_s_TeamA);
	
	public static Strategy finalRiskyB = new Strategy("final Risky B", false, final_pi_b_TeamB, new double[][] {{1, 1, 1, 1},{1, 1, 1, 1}}, final_pi_s_TeamB);
	public static Strategy finalRiskySafeB = new Strategy("final Risky-Safe B", false, final_pi_b_TeamB, new double[][] {{1, 1, 0, 0},{1, 1, 0, 0}}, final_pi_s_TeamB);
	public static Strategy finalSafeRiskyB = new Strategy("final Safe-Risky B", false, final_pi_b_TeamB, new double[][] {{0, 0, 1, 1},{0, 0, 1, 1}}, final_pi_s_TeamB);
	public static Strategy finalSafeB = new Strategy("final Safe B", false, final_pi_b_TeamB, new double[][] {{0, 0, 0, 0},{0, 0, 0, 0}}, final_pi_s_TeamB);	
	
	
	
	
	
	
	
	public static boolean M2_setAllowedFromNeighbourfield = false;
	public static boolean M3_randomizedPlayerInRefDecSet = true;
	public static boolean M4_targetFieldOfRecDefCanOnlyBeNeighbourfield = false;
	public static boolean M5_teamCoordinationError = false;
	
	private String name;
	protected boolean isTeamA;
	
	protected Field fP1;
	protected Field fP2;
	protected Field fBall;
	
	protected static Field centralFieldP1;
	protected static Field centralFieldP2;
	
	// blocking strategy: blocking probability of player 1
	private double pi_b;
	// hitting strategy: player dependent, order of values serve/tech, serve/field, field/tech, field/field
	private double[][] pi_h;
	// percentage os serves in direction of opponent player 1
	private double pi_o1;
	
	public Strategy(String name, boolean isTeamA, double pi_b, double[][] pi_h, double pi_o1){
		this.name = name;
		this.isTeamA = isTeamA;
		this.pi_b = pi_b;
		this.pi_h = pi_h;
		this.pi_o1 = pi_o1;
		
		if (isTeamA){
			centralFieldP1 = Field.A22;
			centralFieldP2 = Field.A23;			
		}else{
			centralFieldP1 = Field.B22;
			centralFieldP2 = Field.B23;
		}
	}
	
	
	
	
//	public static Strat_BASIC_BASIC getInstanceTeamA(){
//		centralFieldP1 = Field.A22;
//		centralFieldP2 = Field.A23;
//		return stratBasicTeamA;
//	}
//	
//	public static Strat_BASIC_BASIC getInstanceTeamB(){
//		centralFieldP1 = Field.B22;
//		centralFieldP2 = Field.B23;
//		return stratBasicTeamB;
//	}
	
	public double getPi_b() {
		return pi_b;
	}




	public double[][] getPi_h() {
		return pi_h;
	}




	public double getPi_o1() {
		return pi_o1;
	}




	public String getName() {
		return name;
	}



	@Override
	public final TeamAction getTeamAction(State s, double r) {
		
		fBall = s.getFieldBall();
		
		if(isTeamA){
			fP1 = s.getFieldA1();
			centralFieldP1 = Field.A22;
			fP2 = s.getFieldA2();
			centralFieldP2 = Field.A23;
		}else{
			fP1 = s.getFieldB1();
			centralFieldP1 = Field.B22;
			fP2 = s.getFieldB2();
			centralFieldP2 = Field.B23;
		}

		if (isTeamA){
			if (s.isServingStateTeamA()) return servingRule(s,r);
			if (s.isReceptionStateTeamA()) return receptionRule(s,r);
			if (s.isDefenceStateTeamA()) return defendingRule(s,r);
			if (s.isSettingStateTeamA()) return settingRule(s,r);
			if (s.isAttackingStateTeamA()) return attackingRule(s,r);
			
			if (s.isServingStateTeamB()) return otherTeamServingRule(s,r);
			if (s.isReceptionStateTeamB()) return otherTeamReceptionRule(s,r);
			if (s.isDefenceStateTeamB()) return otherTeamDefendingRule(s,r);
			if (s.isSettingStateTeamB()) return otherTeamSettingRule(s,r);
			if (s.isAttackingStateTeamB()) return otherTeamAttackingRule(s,r);			
 		}else{
 			if (s.isServingStateTeamB()) return servingRule(s,r);
			if (s.isReceptionStateTeamB()) return receptionRule(s,r);
			if (s.isDefenceStateTeamB()) return defendingRule(s,r);
			if (s.isSettingStateTeamB()) return settingRule(s,r);
			if (s.isAttackingStateTeamB()) return attackingRule(s,r);
			
			if (s.isServingStateTeamA()) return otherTeamServingRule(s,r);
			if (s.isReceptionStateTeamA()) return otherTeamReceptionRule(s,r);
			if (s.isDefenceStateTeamA()) return otherTeamDefendingRule(s,r);
			if (s.isSettingStateTeamA()) return otherTeamSettingRule(s,r);
			if (s.isAttackingStateTeamA()) return otherTeamAttackingRule(s,r);			
 		}
		
		// normal, dass bei 3 Schlägen keine Regel angewandt wird, da der Ballwechsel eh verloren ist
		if (s.getCounter() < 3){
			log4j.error("keine Zustandsregel angewandt in Zustand " + s.toString() );
		}		
		return TeamAction.doNothing;
	}

	@Override
	public TeamAction servingRule(State s, double r) {
		// zuerst bestimmen welcher Spieler schlägt
		int playerNumber;
		if (Arrays.asList(State.START_STATES_A1).contains(s) || Arrays.asList(State.START_STATES_B1).contains(s)){
			playerNumber = 1;
		}else{
			playerNumber = 2;
		}
		
		// Aufschlag bestimmen
		Hit h = Hit.chooseServeAfterStrat(isTeamA, pi_h[playerNumber-1][0], pi_h[playerNumber-1][1], pi_o1);

		Move moveServer;
		if (h.isHitWithJump()){
			moveServer = Move.m_f;
		}else{
			if (Arrays.asList(Field.OUT_A).contains(fP1)){
				moveServer = fP1.moveTowardsField(getCentralFieldOfOtherHalf(fP2));
			}else{
				moveServer = fP2.moveTowardsField(getCentralFieldOfOtherHalf(fP1));
			}
		}
		
		if (playerNumber == 1)
			return new TeamAction(new PlayerAction(h,moveServer), new PlayerAction(Hit.nohit, Move.stay));
		else
			return new TeamAction(new PlayerAction(Hit.nohit, Move.stay), new PlayerAction(h,moveServer));
	}

	@Override
	public TeamAction receptionRule(State s, double r) {
		// && ballIsInsideMyCourtSide() verhindert, dass Ball, obwohl er ins aus geht, angenommen wird
				
		/* M3 randomized Player */
//		double r = MainSim.countRandomCalls();
		int order = 1;
		if (M3_randomizedPlayerInRefDecSet){
			if (r < 0.5){
				order = 1;
			}else{
				order = 2;
			}
		}
		
		for (int i = 0; i < 2; i++){
			switch (order){
			case 1:
				if (fP1 == fBall && ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP1);
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP1);
					}
					Hit h = new Hit(HitTechnique.r, targetField);
					return  new TeamAction(new PlayerAction(h, Move.stay), new PlayerAction(Hit.nohit, fP2.moveTowardsField(targetField)));
				}
				order = 2;
				break;
			case 2:
				if (fP2 == fBall && ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP2);
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP2);
					}
					Hit h = new Hit(HitTechnique.r, targetField);
					return  new TeamAction(new PlayerAction(Hit.nohit, fP1.moveTowardsField(targetField)), new PlayerAction(h, Move.stay));
				}
				order = 1;
				break;
			}
		}
		
		for (int i = 0; i < 2; i++){
			switch (order){
			case 1:
				if (fP1.isNeighbour(fBall) && ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP1);
					
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP1);
					}
					
					Hit h = new Hit(HitTechnique.r_m, targetField);
					return  new TeamAction(new PlayerAction(h, Move.stay), new PlayerAction(Hit.nohit, fP2.moveTowardsField(targetField)));
				}
				order = 2;
				break;
			case 2:
				if (fP2.isNeighbour(fBall) && ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP2);
					
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP2);
					}
					
					Hit h = new Hit(HitTechnique.r_m, targetField);
					return  new TeamAction(new PlayerAction(Hit.nohit, fP1.moveTowardsField(targetField)), new PlayerAction(h, Move.stay));
				}
				order = 1;
				break;
			}
		}
		
		return TeamAction.doNothing;
	}

	@Override
	public TeamAction defendingRule(State s, double r) {
		/* M3 randomized Player */
//		double r = MainSim.countRandomCalls();
		int order = 1;
		if (M3_randomizedPlayerInRefDecSet){
			if (r < 0.5){
				order = 1;
			}else{
				order = 2;
			}
		}
		
		
		for (int i = 0; i < 2; i++){
			switch (order){
			case 1:
				if (fP1 == fBall&& (!Arrays.asList(Field.AT_THE_NET_A).contains(fP1) && !Arrays.asList(Field.AT_THE_NET_B).contains(fP1))&& ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP1);
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP1);
					}
					Hit h = new Hit(HitTechnique.d, targetField);
					return  new TeamAction(new PlayerAction(h, Move.stay), new PlayerAction(Hit.nohit, fP2.moveTowardsField(targetField)));
				}
				order=2;
				break;
			case 2:
				if (fP2 == fBall&& (!Arrays.asList(Field.AT_THE_NET_A).contains(fP2) && !Arrays.asList(Field.AT_THE_NET_B).contains(fP2))&& ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP2);
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP2);
					}
					Hit h = new Hit(HitTechnique.d, targetField);
					return  new TeamAction(new PlayerAction(Hit.nohit, fP1.moveTowardsField(targetField)), new PlayerAction(h, Move.stay));
				}
				order=1;
				break;
			}
		}
		
		for (int i = 0; i < 2; i++){
			switch (order){
			case 1:
				if (fP1.isNeighbour(fBall) && (!Arrays.asList(Field.AT_THE_NET_A).contains(fP1) && !Arrays.asList(Field.AT_THE_NET_B).contains(fP1))&& ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP1);
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP1);
					}
					Hit h = new Hit(HitTechnique.d_m, targetField);
					return  new TeamAction(new PlayerAction(h, Move.stay), new PlayerAction(Hit.nohit, fP2.moveTowardsField(targetField)));
				}
				order=2;
				break;
			case 2:
				if (fP2.isNeighbour(fBall) && (!Arrays.asList(Field.AT_THE_NET_A).contains(fP2) && !Arrays.asList(Field.AT_THE_NET_B).contains(fP2))&& ballIsInsideMyCourtSide()){
					Field targetField = getCentralFieldOfOtherHalf(fP2);
					/* hier Modifikation M4, dass nur Nachbarfeld als Zielfeld erlaubt */
					if (M4_targetFieldOfRecDefCanOnlyBeNeighbourfield){
						targetField = playInMyNeighbourFieldInDirectionOfTargetField(targetField, fP2);
					}
					Hit h = new Hit(HitTechnique.d_m, targetField);
					return  new TeamAction(new PlayerAction(Hit.nohit, fP1.moveTowardsField(targetField)), new PlayerAction(h, Move.stay));
				}
				order=1;
				break;
			}
		}
		return TeamAction.doNothing;
	}

	@Override
	public TeamAction settingRule(State s, double r) {
		/* M3 randomized Player */
//		double r = MainSim.countRandomCalls();
		int order = 1;
		if (M3_randomizedPlayerInRefDecSet){
			if (r < 0.5){
				order = 1;
			}else{
				order = 2;
			}
		}
		
		
		for (int i = 0; i < 2; i++){
			switch (order){
			case 1:
				if((fP1 == fBall || fP1.isNeighbourOrOwnField(fBall) && M2_setAllowedFromNeighbourfield)
				&& s.getLastContact() != Game.getInstance().getTeamA().getPlayer1() 
				&& s.getLastContact() != Game.getInstance().getTeamB().getPlayer1()){
					Field targetField = getCentralFieldOfOtherHalf(fP1);
					if (targetField.isNeighbourOrOwnField(fP1)){
						Hit hitP1 = new Hit(HitTechnique.s,targetField);
//						// Abstimmungsfehler einbauen
//						double r2 = MainSim.countRandomCalls();
						Field moveTargetField;
//						if (r2 < 1){
							moveTargetField = targetField;
//						}else{
//							moveTargetField = Field.chooseRandomField(targetField.getNeighbours());
//						}
						return  new TeamAction(new PlayerAction(hitP1, Move.stay), new PlayerAction(Hit.nohit, fP2.moveTowardsField(moveTargetField)));
					}
				}
				order = 2;
				break;
			case 2:
				if((fP2 == fBall || fP2.isNeighbourOrOwnField(fBall) && M2_setAllowedFromNeighbourfield)	
				&& s.getLastContact() != Game.getInstance().getTeamA().getPlayer2() 
				&& s.getLastContact() != Game.getInstance().getTeamB().getPlayer2()){
					Field targetField = getCentralFieldOfOtherHalf(fP2);
					if (targetField.isNeighbourOrOwnField(fP2)){
						Hit hitP2 = new Hit(HitTechnique.s,targetField);
						// Abstimmungsfehler einbauen
//						double r2 = MainSim.countRandomCalls();
						Field moveTargetField;
//						if (r2 < 1){
							moveTargetField = targetField;
//						}else{
//							moveTargetField = Field.chooseRandomField(targetField.getNeighbours());
//						}
						return  new TeamAction(new PlayerAction(Hit.nohit, fP1.moveTowardsField(moveTargetField)), new PlayerAction(hitP2, Move.stay));
					}
				}
				order = 1;
				break;
			}
		}
		
  		 return receptionRule(s,r);
		// return defendingRule(s,r);
	}

	@Override
	public TeamAction attackingRule(State s, double r) {
		
		int playerNumber = 0;
		
		// attacking Player bestimmen
		if (isTeamA){
			if(fP1==fBall && s.getLastContact() != Game.getInstance().getTeamA().getPlayer1()){
				playerNumber=1;
			}
			if(fP2==fBall && s.getLastContact() != Game.getInstance().getTeamA().getPlayer2()){
				playerNumber=2;
			}
		}else{
			if(fP1==fBall && s.getLastContact() != Game.getInstance().getTeamB().getPlayer1()){
				playerNumber=1;
			}
			if(fP2==fBall && s.getLastContact() != Game.getInstance().getTeamB().getPlayer2()){
				playerNumber=2;
			}
		}
		
		Hit h;
		if (playerNumber != 0){
			// Schlag nach Strategie auswählen
			h = Hit.chooseFieldAttackAfterStrat(isTeamA, pi_h[playerNumber-1][2], pi_h[playerNumber-1][3]);
			if (playerNumber == 1){
				return new TeamAction(new PlayerAction(h, Move.stay), PlayerAction.doNothing);
			} else if (playerNumber == 2){
				return new TeamAction(PlayerAction.doNothing, new PlayerAction(h, Move.stay));
			}
		}else{
			// kein Spieler bei Ball -> Notschlag SHOT
			if (isTeamA){
				// Ball in Nachbarfeld
				if(fBall.isNeighbour(fP1) && s.getLastContact() != Game.getInstance().getTeamA().getPlayer1()){
					h = Hit.chooseRandomHit(Hit.SHOT_TEAMA);
					return  new TeamAction(new PlayerAction(h, Move.stay), PlayerAction.doNothing);
				}
				if(fBall.isNeighbour(fP2) && s.getLastContact() != Game.getInstance().getTeamA().getPlayer2()){
					h = Hit.chooseRandomHit(Hit.SHOT_TEAMA);
					return  new TeamAction(PlayerAction.doNothing, new PlayerAction(h, Move.stay));
				}
			}else{
				// Ball in Nachbarfeld
				if(fBall.isNeighbour(fP1) && s.getLastContact() != Game.getInstance().getTeamB().getPlayer1()){
					h = Hit.chooseRandomHit(Hit.SHOT_TEAMB);
					return  new TeamAction(new PlayerAction(h, Move.stay), PlayerAction.doNothing);
				}
				if(fBall.isNeighbour(fP2) && s.getLastContact() != Game.getInstance().getTeamB().getPlayer2()){
					h = Hit.chooseRandomHit(Hit.SHOT_TEAMB);
					return  new TeamAction(PlayerAction.doNothing, new PlayerAction(h, Move.stay));
				}
			}
		}
		
		return TeamAction.doNothing;
	}

	@Override
	public TeamAction otherTeamServingRule(State s, double r) {
		return TeamAction.doNothing;
	}

	@Override
	public TeamAction otherTeamReceptionRule(State s, double r) {
		// der Nichtblocker soll sich in Richtung des zentralen Felds auf der anderen Seite als der Blockspieler bewegen
		Move moveP1 = fP1.moveTowardsField(getCentralFieldOfOtherHalf(fP2));
		Move moveP2 = fP2.moveTowardsField(getCentralFieldOfOtherHalf(fP1));
		TeamAction result = new TeamAction(new PlayerAction(Hit.nohit, moveP1), new PlayerAction(Hit.nohit, moveP2));
		
		Player blockingPlayer = determineBlockingPlayer(r);
		result.setDesignatedBlockingPlayer(blockingPlayer);
		log4j.debug("Blocking Player " + blockingPlayer.toString());
		
		// der Blocker soll sich nach Süden bewegen oder am Netz stehen bleiben, wenn er da schon steht
		Move moveBlocker;
		if (blockingPlayer.isAtTheNet()){
			moveBlocker = Move.stay;
		}else{
			moveBlocker = Move.m_f;
		}
		// PlayerAction für Blocker überschreiben
		result.setPlayerActionOfPlayer(blockingPlayer, new PlayerAction(Hit.nohit, moveBlocker));
		
		return result;
	}

	@Override
	public TeamAction otherTeamDefendingRule(State s, double r) {
		// der Nichtblocker soll sich in Richtung des zentralen Felds auf der anderen Seite als der Blockspieler bewegen
		Move moveP1 = fP1.moveTowardsField(centralFieldP1);
		Move moveP2 = fP2.moveTowardsField(centralFieldP2);
		TeamAction result = new TeamAction(new PlayerAction(Hit.nohit, moveP1), new PlayerAction(Hit.nohit, moveP2));
		
		Player blockingPlayer = determineBlockingPlayer(r);
		result.setDesignatedBlockingPlayer(blockingPlayer);
		log4j.debug("Blocking Player " + blockingPlayer.toString());
		
		// der Blocker soll sich nach Süden bewegen oder am Netz stehen bleiben, wenn er da schon steht
		Move moveBlocker;
		if (blockingPlayer.isAtTheNet()){
			moveBlocker = Move.stay;
		}else{
			moveBlocker = Move.m_f;
		}
		// PlayerAction für Blocker überschreiben
		result.setPlayerActionOfPlayer(blockingPlayer, new PlayerAction(Hit.nohit, moveBlocker));
		
		return result;
	}

	@Override
	public TeamAction otherTeamSettingRule(State s, double r) {

		// finde in die Spalte des Spielers, der nicht den Ball hat und wahrscheinlich angreifen wird
		int col_possiblyAttackingCol = findColOfPossiblyAttackingPlayer(s);
		
		PlayerAction blockPlayerAction;
		Field blockerField;
		Player designatedBlocker;
		if (s.getBlocker() == null){
			// dieser Fall kann eintreten falls geblockt wurde und der Ball die Seite des Blockers fällt
			designatedBlocker = determineBlockingPlayer(r);
		}else{		
			designatedBlocker = s.getBlocker();
		}

		if (designatedBlocker == Game.getInstance().getTeamA().getPlayer1() || designatedBlocker == Game.getInstance().getTeamB().getPlayer1())
			blockerField = fP1;
		else 
			blockerField = fP2;
				
		if (isTeamA){
			if(col_possiblyAttackingCol < blockerField.getColumn()){
				if (Arrays.asList(Field.AT_THE_NET_A).contains(blockerField)){
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_r);
				}else{
					// blocker noch einen Schritt nach vorne
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_fr);
				}
			}else if (col_possiblyAttackingCol > blockerField.getColumn()){
				if (Arrays.asList(Field.AT_THE_NET_A).contains(blockerField)){
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_l);
				}else{
					// blocker noch einen Schritt nach vorne
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_lf);
				}
			}else{
				if (Arrays.asList(Field.AT_THE_NET_A).contains(blockerField)){
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.stay);
				}else{
					// blocker noch einen Schritt nach vorne
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_f);
				}
				
			}
			
		}else{
			// Blocker aus Team B
			if(col_possiblyAttackingCol < blockerField.getColumn()){
				if (Arrays.asList(Field.AT_THE_NET_B).contains(blockerField)){
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_l);
				}else{
					// blocker noch einen Schritt nach vorne
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_lf);
				}
			}else if(col_possiblyAttackingCol > blockerField.getColumn()){
				if (Arrays.asList(Field.AT_THE_NET_B).contains(blockerField)){
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_r);
				}else{
					// blocker noch einen Schritt nach vorne
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_fr);
				}
			}else{
				if (Arrays.asList(Field.AT_THE_NET_B).contains(blockerField)){
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.stay);
				}else{
					// blocker noch einen Schritt nach vorne
					blockPlayerAction = new PlayerAction(Hit.nohit, Move.m_f);
				}
			}
		}
		// der nicht blockende Spieler läuft auf das zentrale Feld in der anderen Spielfeldhaelfte
		PlayerAction nonblockingPlayerAction;
		Field nonBlockingPlayerField;
		if (designatedBlocker == Game.getInstance().getTeamA().getPlayer1() 
				|| designatedBlocker == Game.getInstance().getTeamB().getPlayer1()){
			nonBlockingPlayerField = fP2;
		}else{
			nonBlockingPlayerField = fP1;
		}
		if (isTeamA){
			if (col_possiblyAttackingCol <= nonBlockingPlayerField.getColumn()){
				nonblockingPlayerAction = new PlayerAction(Hit.nohit, nonBlockingPlayerField.moveTowardsField(centralFieldP2));
			}else{
				nonblockingPlayerAction = new PlayerAction(Hit.nohit, nonBlockingPlayerField.moveTowardsField(centralFieldP1));
			}
		}else{
			if (col_possiblyAttackingCol < nonBlockingPlayerField.getColumn()){
				nonblockingPlayerAction = new PlayerAction(Hit.nohit, nonBlockingPlayerField.moveTowardsField(centralFieldP1));
			}else{
				nonblockingPlayerAction = new PlayerAction(Hit.nohit, nonBlockingPlayerField.moveTowardsField(centralFieldP2));
			}
		}
		
		if (designatedBlocker == Game.getInstance().getTeamA().getPlayer1() 
				|| designatedBlocker == Game.getInstance().getTeamB().getPlayer1()){
			return new TeamAction(blockPlayerAction, nonblockingPlayerAction, designatedBlocker);
		}else{
			return new TeamAction(nonblockingPlayerAction, blockPlayerAction, designatedBlocker);
		}

	}

	@Override
	public TeamAction otherTeamAttackingRule(State s, double r) {
		// Ball wurde vom Gegner erfolgreich gestellt -> nun evtl blocken
		// wenn Ball in der selben Spalte UND Verteidigungsspieler am Netz!
		Player blocker = s.getBlocker();
		PlayerAction blockingPlayerAction;
		Field blockingPlayerField;
		PlayerAction nonBlockingPlayerAction;
		Field nonBlockerField;
		if (s.getBlocker() == Game.getInstance().getTeamA().getPlayer1() 
				|| s.getBlocker() == Game.getInstance().getTeamB().getPlayer1()){
			blockingPlayerField = fP1;
			nonBlockerField = fP2;
		}
		else {
			blockingPlayerField = fP2;
			nonBlockerField = fP1;
		}
		
		if (s.getFieldBall().isInSameColumn(blockingPlayerField) 
				&& blocker.isAtTheNet()){
			blockingPlayerAction = new PlayerAction(Hit.nohit, Move.b);
		}else{
			blockingPlayerAction = new PlayerAction(Hit.nohit, Move.m_b);
		}
		
		if (isTeamA){
			if (blockingPlayerField.getColumn() <= 2){
				nonBlockingPlayerAction = new PlayerAction(Hit.nohit, nonBlockerField.moveTowardsField(centralFieldP2));
			}else{
				nonBlockingPlayerAction = new PlayerAction(Hit.nohit, nonBlockerField.moveTowardsField(centralFieldP1));
			}
		}else{
			if (blockingPlayerField.getColumn() >= 3){
				nonBlockingPlayerAction = new PlayerAction(Hit.nohit, nonBlockerField.moveTowardsField(centralFieldP2));
			}else{
				nonBlockingPlayerAction = new PlayerAction(Hit.nohit,nonBlockerField.moveTowardsField(centralFieldP1));
			}
		}
		
		if (blocker == Game.getInstance().getTeamA().getPlayer1() 
				|| blocker == Game.getInstance().getTeamB().getPlayer1()){
			return new TeamAction(blockingPlayerAction, nonBlockingPlayerAction);
		}else{
			return new TeamAction(nonBlockingPlayerAction, blockingPlayerAction); 
		}
		
	}
	
	/**
	 * Funktion bestimmt Blockspieler nach Wahrscheinlichkeit pi_b
	 * Darf den Blockspieler aber nicht im Zustand setzen -> das soll nur in ComputeNextState passieren
	 * @return Blockspieler
	 */
	private Player determineBlockingPlayer(double r){
//		double r = MainSim.countRandomCalls();
		if (r < pi_b){
			if (isTeamA){
				return Game.getInstance().getTeamA().getPlayer1();
			}else{
				return Game.getInstance().getTeamB().getPlayer1();
			}
		}else{
			if (isTeamA){
				return Game.getInstance().getTeamA().getPlayer2();
			}else{
				return Game.getInstance().getTeamB().getPlayer2();
			}
		}
	}
	
	/**
	 * @param s current state
	 * @return column of Attacking player, or column of the Ball if it is not clear which player attacks
	 */
	private int findColOfPossiblyAttackingPlayer(State s){
		// finde in die Spalte des Spielers, der nicht den Ball hat und wahrscheinlich angreifen wird
		int col_possiblyAttackingCol;
		if (isTeamA){
			if (s.getFieldBall() == s.getFieldB1())
				col_possiblyAttackingCol = s.getFieldB2().getColumn();
			else if (s.getFieldBall() == s.getFieldB2())
				col_possiblyAttackingCol = s.getFieldB1().getColumn();
			else 
				col_possiblyAttackingCol = s.getFieldBall().getColumn();
		}else{
			if (s.getFieldBall() == s.getFieldA1())
				col_possiblyAttackingCol = s.getFieldA2().getColumn();
			else if (s.getFieldBall() == s.getFieldA2())
				col_possiblyAttackingCol = s.getFieldA1().getColumn();
			else 
				col_possiblyAttackingCol = s.getFieldBall().getColumn();
			}
		return col_possiblyAttackingCol;
	}
	
	protected Field getCentralFieldOfOtherHalf(Field myField){
		if (isTeamA){
			if (myField.getColumn() <= 2){
				return centralFieldP2;
			}else{
				return centralFieldP1;
			}
		}else{
			if (myField.getColumn() <= 2){
				return centralFieldP1;
			}else{
				return centralFieldP2;
			}
		}
				
	}
	
	protected Field playInMyNeighbourFieldInDirectionOfTargetField(Field targetField, Field myField){
		if (targetField.isNeighbourOrOwnField(myField)){
			return targetField;
		}
		Field[] neighbours = myField.getNeighbours();
		return computeFieldWithSmallestDistance(neighbours, targetField);
	}
	
	private Field computeFieldWithSmallestDistance(Field[] field, Field target){
		int minDistance = 100;
		int numberOfMinDistanceField = -1;
		for (int i = 0; i<field.length;i++){
			int dist = computeDistance(field[i], target);
			if(dist < minDistance){
				minDistance = dist;
				numberOfMinDistanceField = i;
			};
		}
		
		return field[numberOfMinDistanceField];
	}
	
	private int computeDistance(Field f1, Field f2){
		int result = 0;
		result += Math.abs(f1.getColumn() - f2.getColumn());
		result += Math.abs(f1.getRow() - f2.getRow());
		return result;
	}
	
	private boolean ballIsInsideMyCourtSide(){
		if (isTeamA){
			if (fBall.isInsideA())
				return true;
			else 
				return false;
		}else{
			if (fBall.isInsideB())
				return true;
			else 
				return false;
		}
		
	}
	
	public String toString(){
		if (pi_h.length != 2){
			if (pi_h[0].length != 4 || pi_h[1].length != 4){
				System.err.println("Strategy: " + this.name + " hat nicht die richtige Länge bei pi_h!");
			}
		}
		return this.name + ": " + isTeamA + ", " + pi_b + ", {{" + pi_h[0][0] +", " + pi_h[0][1] + ", " + pi_h[0][2] + ", " +  pi_h[0][3] + "},"
				+ "{" + pi_h[1][0] +", " + pi_h[1][1] + ", " + pi_h[1][2] + ", " +  pi_h[1][3] + "}"	+ "}, " + String.format("%.4f", pi_o1) + ";";
	}

	public String toStringShort(){
		return this.name;
	}
}
