package de.ubt.wm.sb.probs;

import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.sim.MainSim;

public class Prob_Hit {
	
	public static int minObs = 11;

	public static int l_set_succ = 3, l_set_fault = 2, l_set_dev = 2;
	public static int l_def_succ = 3, l_def_fault = 2, l_def_dev = 2;
	
	private HitTechnique tech;
	
	private int numberOfObservations;
	private Prob_Hit predProb;
	
	/* Probs*/
	private double succProb;
	private double faultProb;
	private double neighbourProb;
	/* modified Probs */
	private double succProb_mod;
	private double faultProb_mod;
	private double neighbourProb_mod;
	
	
	private double fracSuccFault;
	private double fracSuccNeighbour;
	private double fracFaultNeighbour;
	
	public Prob_Hit(HitTechnique tech, int numberOfObservations, Prob_Hit predecessor, double succProb, double faultProb) {
		super();
		if (numberOfObservations == 0){
			this.succProb = -1;
			this.faultProb = -1;
			this.neighbourProb = -1;
		}else{
			this.succProb = succProb;
			this.faultProb = faultProb;
			this.neighbourProb = 1-succProb-faultProb;
			// Achtung erst müssen succProb & faultProb & neighbourProb gesetzt sein!
			if (MainSim.modSet && tech == HitTechnique.s){
				// ist dafür zuständigt, das p_quer_field größer ist als q_quer_field
				this.weightOriginalProbs(l_set_succ, l_set_fault, l_set_succ);
			
			}
			if(MainSim.modDef && (tech == HitTechnique.d || tech == HitTechnique.d_m)){
				// hebt das Level der direkten Punkte nach Feldangriffen
				this.weightOriginalProbs(l_def_succ, l_def_fault, l_def_succ);
			
			}
		}
		this.tech = tech;
		this.numberOfObservations = numberOfObservations;
		this.predProb = predecessor;
		this.fracSuccFault = succProb/faultProb; 
		this.fracFaultNeighbour = faultProb/neighbourProb;
		this.fracSuccNeighbour = succProb/neighbourProb;
	}
	
	public void weightOriginalProbs(int l_succ, int l_fault, int l_dev){
		if (this.succProb + this.faultProb + this.neighbourProb <= 0.999){
			System.err.println("Achtung, weighted Prob darf nur auf bereits bestehenden gültigen Probs aufgerufen werden!");
		}
		
		double oldSucc = this.succProb;
		double oldFault = this.faultProb;
		double oldDev = this.neighbourProb;
		
		double divisor = l_succ * oldSucc + l_fault*oldFault + l_dev*oldDev;
		
		this.succProb = (l_succ*oldSucc)/divisor;
		this.faultProb = (l_fault * oldFault)/divisor;
		this.neighbourProb = (l_dev * oldDev)/divisor;		
	}
	
	public void updateProbHit(HitTechnique tech, int numberOfObservations, Prob_Hit predecessor, double succProb, double faultProb){
		if (succProb < -0.1 || faultProb < -0.1){
			this.succProb = -1;
			this.faultProb = -1;
			this.neighbourProb = -1;
		}else{
			this.succProb = succProb;
			this.faultProb = faultProb;
			this.neighbourProb = 1-succProb-faultProb;
		}
		this.tech = tech;
		this.numberOfObservations = numberOfObservations;
		this.predProb = predecessor;
		this.fracSuccFault = succProb/faultProb; 
		this.fracFaultNeighbour = faultProb/neighbourProb;
		this.fracSuccNeighbour = succProb/neighbourProb;
	}
	
	
	public Prob_Hit(HitTechnique tech, Prob_Hit predecessor) {
		super();
		this.succProb = -1;
		this.faultProb = -1;
		this.neighbourProb = -1;
		
		this.tech = tech;
		this.predProb = predecessor;
		this.fracSuccFault = succProb/faultProb; 
		this.fracFaultNeighbour = faultProb/neighbourProb;
		this.fracSuccNeighbour = succProb/neighbourProb;
	}
	
	public Prob_Hit(Prob_Hit prob){
		this.tech = prob.tech;
		this.numberOfObservations = prob.numberOfObservations;
		this.succProb = prob.succProb;
		this.faultProb = prob.faultProb;
		this.neighbourProb = prob.neighbourProb;
		this.fracSuccFault = prob.fracSuccFault;
		this.fracFaultNeighbour = prob.fracSuccNeighbour;
		this.fracSuccNeighbour = prob.fracSuccNeighbour;
		this.succProb_mod = prob.succProb_mod;
		this.faultProb_mod = prob.faultProb_mod;
		this.neighbourProb_mod = prob.neighbourProb_mod;
	}

	@Override
	public String toString() {
		return tech + "=["+ numberOfObservations + ", " + succProb + ", " + faultProb + ", " + neighbourProb + "]";
	}

	public String toStringModified() {
		return tech + "=["+ numberOfObservations + ", " + succProb_mod + ", " + faultProb_mod + ", " + neighbourProb_mod + "]";
	}
	
	public String toStringLatex() {
		String string_succProb, string_faultProb;
		
		if (succProb < -0.1){
			string_succProb = "\\textover[c]{--}{0.00}";
			string_faultProb = "\\textover[c]{--}{0.00}";
		}else{
			string_succProb = String.format("%.2f", succProb);
			string_faultProb = String.format("%.2f",faultProb);
		}
		
		return " & $"+ numberOfObservations + "$ & $" + String.format("%.2f",succProb_mod) + "$ ($" + string_succProb + "$)"
				+ " & $" + String.format("%.2f",faultProb_mod) + "$ ($" + string_faultProb + "$)";
	}
	
	public void setAllThreeProbs(double succProb, double faultProb){
		this.succProb = succProb;
		this.faultProb = faultProb;
		this.neighbourProb = 1-succProb-faultProb;		
	}
	
	public void setModifiedProbs() {				
		int observation_counter = this.numberOfObservations;
		succProb_mod = succProb;
		faultProb_mod = faultProb;
		Prob_Hit pAgg = predProb;
		while(observation_counter < minObs){
			if (Math.min(minObs, observation_counter+pAgg.numberOfObservations) > 0){
				succProb_mod = (observation_counter*succProb_mod + Math.min(pAgg.numberOfObservations, minObs - observation_counter)*pAgg.succProb)
						/Math.min(minObs, observation_counter+pAgg.numberOfObservations);
				faultProb_mod = (observation_counter*faultProb_mod + Math.min(pAgg.numberOfObservations, minObs - observation_counter)*pAgg.faultProb)
						/Math.min(minObs, observation_counter+pAgg.numberOfObservations);
			}
			observation_counter += pAgg.numberOfObservations;
			pAgg = pAgg.predProb;
		}
		neighbourProb_mod = 1 - succProb_mod - faultProb_mod;	
	}
	
	public HitTechnique getTech() {
		return tech;
	}

	public void setTech(HitTechnique tech) {
		this.tech = tech;
	}

	public int getNumberOfObservations() {
		return numberOfObservations;
	}

	public void setNumberOfObservations(int numberOfObservations) {
		this.numberOfObservations = numberOfObservations;
	}

	public Prob_Hit getPredProb() {
		return predProb;
	}

	public void setPredProb(Prob_Hit predProb) {
		this.predProb = predProb;
	}

	public double getSuccProb() {
		return succProb;
	}

	public void setSuccProb(double succProb) {
		if (succProb < -0.1){
			this.succProb = succProb;
			this.neighbourProb = -1.0;
			this.faultProb = -1.0;
		}else{
			this.succProb = succProb;
			this.neighbourProb = (1 - succProb)/(1+this.fracFaultNeighbour);
			this.faultProb = 1 - this.succProb - this.neighbourProb;
		}
				
	}

	public double getFaultProb() {
		return faultProb;
	}

	public void setFaultProb(double faultProb) {
		if (faultProb < -0.1){
			this.succProb = -1.0;
			this.neighbourProb = -1.0;
			this.faultProb = -1.0;
		}else{
			this.faultProb = faultProb;
			this.neighbourProb = (1-faultProb)/(1+this.fracSuccNeighbour);
			this.succProb = 1 - this.faultProb - this.neighbourProb;
		}
	}

	public double getNeighbourProb() {
		return neighbourProb;
	}

	public void setNeighbourProb(double neighbourProb) {
		if (neighbourProb < -0.1){
			this.succProb = -1.0;
			this.neighbourProb = -1.0;
			this.faultProb = -1.0;
		}else{
			this.neighbourProb = neighbourProb;
			this.faultProb = (1-neighbourProb)/(1+this.fracSuccFault);
			this.succProb = 1 - this.neighbourProb - this.faultProb;
		}
	}
	
	public double getSuccProb_mod() {
		return succProb_mod;
	}

	public void setSuccProb_mod(double succProb_mod) {
		this.succProb_mod = succProb_mod;
	}

	public double getFaultProb_mod() {
		return faultProb_mod;
	}

	public void setFaultProb_mod(double faultProb_mod) {
		this.faultProb_mod = faultProb_mod;
	}

	public double getNeighbourProb_mod() {
		return neighbourProb_mod;
	}

	public void setNeighbourProb_mod(double neighbourProb_mod) {
		this.neighbourProb_mod = neighbourProb_mod;
	}

	public double getFracSuccFault() {
		return fracSuccFault;
	}

	public double getFracSuccNeighbour() {
		return fracSuccNeighbour;
	}

	public double getFracFaultNeighbour() {
		return fracFaultNeighbour;
	}

	public String printName() {
		return tech.toString();
	}
}

