package de.ubt.wm.sb.probs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.model.BallProperty;
import de.ubt.wm.sb.model.BlockTechnique;
import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.TechniqueTest;
import jxl.Cell;
import jxl.CellType;
import jxl.NumberCell;
import jxl.Sheet;
import jxl.Workbook;
import jxl.read.biff.BiffException;

public class Prob_IndPlayer {

	static final Logger log4j = LogManager.getLogger(Game.class.getName());
	
	private HashMap<HitTechnique, HashMap <Field, Prob_Hit>> servingProbs; // Technique -> Target Field -> ProbHit
	private HashMap<HitTechnique, HashMap <Field, HashMap <Field, Prob_Hit>>> attackingProbs; // Technique -> Player Field -> Target Field -> Prob_Hit
	private HashMap<HitTechnique, HashMap <BallProperty, Prob_Hit>> receivingProbs;
	private HashMap<HitTechnique, HashMap <BallProperty, Prob_Hit>> defendingProbs;
	private Prob_Hit settingProb;
	private Prob_Block blockingProb;
	
	/* aggregated Probs */
	public Prob_Hit hit;
	
	public Prob_Hit crossingNet;
	
	public Prob_Hit general_serve;
	public Prob_Hit S_F;
	public Prob_Hit S_J;
	
	public Prob_Hit F;
	public Prob_Hit F_SM;
	public Prob_Hit F_SH;
	public Prob_Hit F_P;
	
	public Prob_Hit compositionHit;
	public Prob_Hit recDef;
	public Prob_Hit def;
	public Prob_Hit d;
	public Prob_Hit d_m;
	public Prob_Hit rec;
	public Prob_Hit r;
	public Prob_Hit r_m;
	
	
	public Prob_IndPlayer() {
		super();
		this.servingProbs = new HashMap<HitTechnique, HashMap <Field, Prob_Hit>>();
		this.attackingProbs = new HashMap<HitTechnique, HashMap <Field, HashMap <Field, Prob_Hit>>>();
		this.receivingProbs = new HashMap<HitTechnique, HashMap <BallProperty, Prob_Hit>>();
		this.defendingProbs = new HashMap<HitTechnique, HashMap <BallProperty, Prob_Hit>>();
		this.blockingProb = null;
		
		this.hit = new Prob_Hit(HitTechnique.h, null);
		this.crossingNet = new Prob_Hit(HitTechnique.cross, hit);
		this.general_serve = new Prob_Hit(HitTechnique.S, crossingNet);
		this.S_F = new Prob_Hit(HitTechnique.S_F, general_serve);
		this.S_J = new Prob_Hit(HitTechnique.S_J, general_serve);
		this.F = new Prob_Hit(HitTechnique.F, crossingNet);
		this.F_SM = new Prob_Hit(HitTechnique.F_SM, F);
		this.F_SH = new Prob_Hit(HitTechnique.F_E, F_SH);
		this.F_P = new Prob_Hit(HitTechnique.F_P, F_P);
		this.compositionHit = new Prob_Hit(HitTechnique.compose, hit);
		this.recDef = new Prob_Hit(HitTechnique.recdef, compositionHit);
		this.def = new Prob_Hit(HitTechnique.def, recDef);
		this.rec = new Prob_Hit(HitTechnique.rec, recDef);
		this.d = new Prob_Hit(HitTechnique.d, def);
		this.d_m = new Prob_Hit(HitTechnique.d_m, def);
		this.r = new Prob_Hit(HitTechnique.r, rec);
		this.r_m = new Prob_Hit(HitTechnique.r_m, rec);
	}


	public Prob_IndPlayer(Prob_IndPlayer indProbs) {
		this.servingProbs = new HashMap<HitTechnique, HashMap <Field, Prob_Hit>>();
		for (HitTechnique t : indProbs.servingProbs.keySet()){
			HashMap <Field, Prob_Hit> fieldProbMap = new HashMap<Field, Prob_Hit>();
			for (Field f : indProbs.servingProbs.get(t).keySet()){
				fieldProbMap.put(f, new Prob_Hit(indProbs.servingProbs.get(t).get(f)));
			}
			this.servingProbs.put(t, fieldProbMap);
		}
		this.attackingProbs = new HashMap<HitTechnique, HashMap <Field, HashMap <Field, Prob_Hit>>>();
		for (HitTechnique t : indProbs.attackingProbs.keySet()){
			HashMap <Field, HashMap <Field, Prob_Hit>> fieldfieldProb = new HashMap <Field, HashMap <Field, Prob_Hit>>();
			for (Field f1 : indProbs.attackingProbs.get(t).keySet()){
				HashMap<Field, Prob_Hit> fieldProb = new HashMap<Field, Prob_Hit>();
				for (Field f2 :  indProbs.attackingProbs.get(t).get(f1).keySet()){
					fieldProb.put(f2, new Prob_Hit(indProbs.attackingProbs.get(t).get(f1).get(f2)));
				}
				fieldfieldProb.put(f1, fieldProb);
			}
			this.attackingProbs.put(t, fieldfieldProb);
		}
		
		this.receivingProbs = new HashMap<HitTechnique, HashMap <BallProperty, Prob_Hit>>();
		for (HitTechnique t: indProbs.receivingProbs.keySet()){
			HashMap<BallProperty, Prob_Hit> ballProb = new HashMap<BallProperty, Prob_Hit>();
			for (BallProperty b : indProbs.receivingProbs.get(t).keySet()){
				ballProb.put(b, new Prob_Hit(indProbs.receivingProbs.get(t).get(b)));
			}
			this.receivingProbs.put(t, ballProb);
		}
		
		this.defendingProbs = new HashMap<HitTechnique, HashMap <BallProperty, Prob_Hit>>();
		for (HitTechnique t: indProbs.defendingProbs.keySet()){
			HashMap<BallProperty, Prob_Hit> ballProb = new HashMap<BallProperty, Prob_Hit>();
			for (BallProperty b : indProbs.defendingProbs.get(t).keySet()){
				ballProb.put(b, new Prob_Hit(indProbs.defendingProbs.get(t).get(b)));
			}
			this.defendingProbs.put(t, ballProb);
		}
		
		this.settingProb = indProbs.settingProb;

		this.blockingProb = indProbs.blockingProb;
		
		this.hit = indProbs.hit;
		this.crossingNet = indProbs.crossingNet;
		this.compositionHit = indProbs.compositionHit;
		this.recDef = indProbs.recDef;
		this.general_serve = indProbs.general_serve;
		this.S_F = indProbs.S_F;
		this.S_J = indProbs.S_J;
		this.F = indProbs.F;
		this.F_P = indProbs.F_P;
		this.F_SM = indProbs.F_SM;
		this.F_SH = indProbs.F_SH;
		this.rec = indProbs.rec;
		this.r_m = indProbs.r_m;
		this.r = indProbs.r;
		this.def = indProbs.def;
		this.d = indProbs.d;
		this.d_m = indProbs.d_m;
	}
	

	public HashMap<HitTechnique, HashMap<Field, Prob_Hit>> getServingProbs() {
		return servingProbs;
	}


	public void setServingProbs(HashMap<HitTechnique, HashMap<Field, Prob_Hit>> servingProbs) {
		this.servingProbs = servingProbs;
	}


	public HashMap<HitTechnique, HashMap<Field, HashMap<Field, Prob_Hit>>> getAttackingProbs() {
		return attackingProbs;
	}


	public void setAttackingProbs(HashMap<HitTechnique, HashMap<Field, HashMap<Field, Prob_Hit>>> attackingProbs) {
		this.attackingProbs = attackingProbs;
	}


	public HashMap<HitTechnique, HashMap<BallProperty, Prob_Hit>> getReceivingProbs() {
		return receivingProbs;
	}


	public void setReceivingProbs(HashMap<HitTechnique, HashMap<BallProperty, Prob_Hit>> receivingProbs) {
		this.receivingProbs = receivingProbs;
	}


	public HashMap<HitTechnique, HashMap<BallProperty, Prob_Hit>> getDefendingProbs() {
		return defendingProbs;
	}


	public void setDefendingProbs(HashMap<HitTechnique, HashMap<BallProperty, Prob_Hit>> defendingProbs) {
		this.defendingProbs = defendingProbs;
	}


	public Prob_Block getBlockingProb() {
		return blockingProb;
	}


	public void setBlockingProb(Prob_Block blockingProb) {
		this.blockingProb = blockingProb;
	}


	public Prob_Hit getCrossingNet() {
		return crossingNet;
	}


	public void setCrossingNet(Prob_Hit crossingNet) {
		this.crossingNet = crossingNet;
	}


	public Prob_Hit getCompositionHit() {
		return compositionHit;
	}


	public void setCompositionHit(Prob_Hit compositionHit) {
		this.compositionHit = compositionHit;
	}


	public void setSettingProb(Prob_Hit settingProb) {
		this.settingProb = settingProb;
	}


	public void computeModifiedProbs() {
		for (HitTechnique t : this.servingProbs.keySet()){
			for (Field f : this.servingProbs.get(t).keySet()){
				this.servingProbs.get(t).get(f).setModifiedProbs();
			}
		}
		for (HitTechnique t : this.attackingProbs.keySet()){
			for (Field f1 : this.attackingProbs.get(t).keySet()){
				for (Field f2 :  this.attackingProbs.get(t).get(f1).keySet()){
					this.attackingProbs.get(t).get(f1).get(f2).setModifiedProbs();
				}
			}
		}
		for (HitTechnique t: this.receivingProbs.keySet()){
			for (BallProperty b : this.receivingProbs.get(t).keySet()){
				this.receivingProbs.get(t).get(b).setModifiedProbs();
			}
		}
		for (HitTechnique t: this.defendingProbs.keySet()){
			for (BallProperty b : this.defendingProbs.get(t).keySet()){
				this.defendingProbs.get(t).get(b).setModifiedProbs();
			}
		}
		
		this.settingProb.setModifiedProbs();

	}
	
	@SuppressWarnings("serial")
	public void readFrom_BVEVAL_CSV(Player player, boolean isPlayer1) throws BiffException, IOException {
		boolean isTeamA = false;
		if (player.getTeam() == Game.getInstance().getTeamA()){
			isTeamA = true;
		}else if (player.getTeam() == Game.getInstance().getTeamB()){
			isTeamA = false;
		}else{
			try {
				throw new Exception("can't readFromExcelTemplate player " + player.getName() + " is not in the game!");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		String csvFile = player.getDataFile();
        BufferedReader br = null;
        String line = "";
        String cvsSplitBy = ";";
        ArrayList<String[]> input = new ArrayList<String[]>(){};
		try {
		
		    br = new BufferedReader(new FileReader(csvFile));
		    while ((line = br.readLine()) != null) {
		
		        // use cvsSplitBy as separator
		        String[] myline = line.split(cvsSplitBy);
		        if (myline!= null && myline[0].length() > 0 && myline[0].charAt(0)!='#'){
		        	input.add(myline);
		        }
		    }
		
		} catch (FileNotFoundException e) {
		    e.printStackTrace();
		} catch (IOException e) {
		    e.printStackTrace();
		} finally {
		    if (br != null) {
		        try {
		            br.close();
		        } catch (IOException e) {
		            e.printStackTrace();
		        }
		    }
		}
		
		ArrayList<String[]> inputPlay = new ArrayList<String[]>(){};
		if (input.size()!=114){
			System.err.println("Einleselänge der InputProbs stimmt nicht!");
		}else{
			if (isPlayer1){
				for(int i=0;i<57;i++){
					inputPlay.add(input.get(i));
				}
			}else{
				for(int i=57;i<114;i++){
					inputPlay.add(input.get(i));
				}
			}
		}
		
//		for (String[] s : inputPlay){
//			String res = "";
//			for(String ss : s){
//				res+=ss+",";
//			}
//			System.out.println(res);
//		}
//		
		
		// Serves
		Field[] targetFields;
		Field[] playerFields;
		if (isTeamA){
			targetFields = Field.LAST_ROW_B;
			}else{
			targetFields = Field.LAST_ROW_A;
		}
	
		Prob_Serve probServe;
		for(Field targetField : targetFields){
			int number = Integer.parseInt(inputPlay.get(1)[2]);
			Double succ = Double.parseDouble(inputPlay.get(1)[3]);
			Double fault = Double.parseDouble(inputPlay.get(1)[4]);
			probServe = (Prob_Serve) initializeProb(HitTechnique.S_F, number, this.S_F, succ,fault,targetField,null, null);
			this.addProbHit(probServe);
			
			number = Integer.parseInt(inputPlay.get(5)[2]);
			succ = Double.parseDouble(inputPlay.get(5)[3]);
			fault = Double.parseDouble(inputPlay.get(5)[4]);
			probServe = (Prob_Serve) initializeProb(HitTechnique.S_J, number, this.S_J, succ,fault,targetField, null, null);
			this.addProbHit(probServe);
		}
		
		if (isTeamA){
			targetFields = Field.MIDDLE_ROW_B;
			}else{
			targetFields = Field.MIDDLE_ROW_A;
		}			
		for(Field targetField : targetFields){
			int number = Integer.parseInt(inputPlay.get(2)[2]);
			Double succ = Double.parseDouble(inputPlay.get(2)[3]);
			Double fault = Double.parseDouble(inputPlay.get(2)[4]);
			probServe = (Prob_Serve) initializeProb(HitTechnique.S_F, number, this.S_F, succ,fault,targetField,null, null);
			this.addProbHit(probServe);
			
			number = Integer.parseInt(inputPlay.get(6)[2]);
			succ = Double.parseDouble(inputPlay.get(6)[3]);
			fault = Double.parseDouble(inputPlay.get(6)[4]);
			probServe = (Prob_Serve) initializeProb(HitTechnique.S_J, number, this.S_J, succ,fault,targetField,null, null);
			this.addProbHit(probServe);
		}
		
		
		// Smash
		if (isTeamA){
			playerFields = Field.OUT_A;
			}else{
			playerFields = Field.OUT_B;
		}
		Prob_Attack probAttack;
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(9)[2]);
				Double succ = Double.parseDouble(inputPlay.get(9)[3]);
				Double fault = Double.parseDouble(inputPlay.get(9)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(10)[2]);
				Double succ = Double.parseDouble(inputPlay.get(10)[3]);
				Double fault = Double.parseDouble(inputPlay.get(10)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}			
		}
		if (isTeamA){
			playerFields = Field.LAST_ROW_A;
			}else{
			playerFields = Field.LAST_ROW_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(12)[2]);
				Double succ = Double.parseDouble(inputPlay.get(12)[3]);
				Double fault = Double.parseDouble(inputPlay.get(12)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(13)[2]);
				Double succ = Double.parseDouble(inputPlay.get(13)[3]);
				Double fault = Double.parseDouble(inputPlay.get(13)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}			
		}
		if (isTeamA){
			playerFields = Field.MIDDLE_ROW_A;
			}else{
			playerFields = Field.MIDDLE_ROW_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(15)[2]);
				Double succ = Double.parseDouble(inputPlay.get(15)[3]);
				Double fault = Double.parseDouble(inputPlay.get(15)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(16)[2]);
				Double succ = Double.parseDouble(inputPlay.get(16)[3]);
				Double fault = Double.parseDouble(inputPlay.get(16)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}			
		}
		if (isTeamA){
			playerFields = Field.AT_THE_NET_A;
			}else{
			playerFields = Field.AT_THE_NET_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(18)[2]);
				Double succ = Double.parseDouble(inputPlay.get(18)[3]);
				Double fault = Double.parseDouble(inputPlay.get(18)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(19)[2]);
				Double succ = Double.parseDouble(inputPlay.get(19)[3]);
				Double fault = Double.parseDouble(inputPlay.get(19)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}			
		}
		
		
		// Shot
		if (isTeamA){
			playerFields = Field.OUT_A;
			}else{
			playerFields = Field.OUT_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(22)[2]);
				Double succ = Double.parseDouble(inputPlay.get(22)[3]);
				Double fault = Double.parseDouble(inputPlay.get(22)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(23)[2]);
				Double succ = Double.parseDouble(inputPlay.get(23)[3]);
				Double fault = Double.parseDouble(inputPlay.get(23)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}			
		}
		if (isTeamA){
			playerFields = Field.LAST_ROW_A;
			}else{
			playerFields = Field.LAST_ROW_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(25)[2]);
				Double succ = Double.parseDouble(inputPlay.get(25)[3]);
				Double fault = Double.parseDouble(inputPlay.get(25)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(26)[2]);
				Double succ = Double.parseDouble(inputPlay.get(26)[3]);
				Double fault = Double.parseDouble(inputPlay.get(26)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}			
		}
		if (isTeamA){
			playerFields = Field.MIDDLE_ROW_A;
			}else{
			playerFields = Field.MIDDLE_ROW_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(28)[2]);
				Double succ = Double.parseDouble(inputPlay.get(28)[3]);
				Double fault = Double.parseDouble(inputPlay.get(28)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(29)[2]);
				Double succ = Double.parseDouble(inputPlay.get(29)[3]);
				Double fault = Double.parseDouble(inputPlay.get(29)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}		
		}
		if (isTeamA){
			playerFields = Field.AT_THE_NET_A;
			}else{
			playerFields = Field.AT_THE_NET_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(31)[2]);
				Double succ = Double.parseDouble(inputPlay.get(31)[3]);
				Double fault = Double.parseDouble(inputPlay.get(31)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(32)[2]);
				Double succ = Double.parseDouble(inputPlay.get(32)[3]);
				Double fault = Double.parseDouble(inputPlay.get(32)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}			
		}

		
		// Poke
		if (isTeamA){
			playerFields = Field.OUT_A;
			}else{
			playerFields = Field.OUT_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(35)[2]);
				Double succ = Double.parseDouble(inputPlay.get(35)[3]);
				Double fault = Double.parseDouble(inputPlay.get(35)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(36)[2]);
				Double succ = Double.parseDouble(inputPlay.get(36)[3]);
				Double fault = Double.parseDouble(inputPlay.get(36)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.AT_THE_NET_B;
				}else{
				targetFields = Field.AT_THE_NET_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(37)[2]);
				Double succ = Double.parseDouble(inputPlay.get(37)[3]);
				Double fault = Double.parseDouble(inputPlay.get(37)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}		
		}
		if (isTeamA){
			playerFields = Field.LAST_ROW_A;
			}else{
			playerFields = Field.LAST_ROW_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(38)[2]);
				Double succ = Double.parseDouble(inputPlay.get(38)[3]);
				Double fault = Double.parseDouble(inputPlay.get(38)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(39)[2]);
				Double succ = Double.parseDouble(inputPlay.get(39)[3]);
				Double fault = Double.parseDouble(inputPlay.get(39)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.AT_THE_NET_B;
				}else{
				targetFields = Field.AT_THE_NET_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(40)[2]);
				Double succ = Double.parseDouble(inputPlay.get(40)[3]);
				Double fault = Double.parseDouble(inputPlay.get(40)[4]);
				probAttack =(Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}		
		}
		if (isTeamA){
			playerFields = Field.MIDDLE_ROW_A;
			}else{
			playerFields = Field.MIDDLE_ROW_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(41)[2]);
				Double succ = Double.parseDouble(inputPlay.get(41)[3]);
				Double fault = Double.parseDouble(inputPlay.get(41)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(42)[2]);
				Double succ = Double.parseDouble(inputPlay.get(42)[3]);
				Double fault = Double.parseDouble(inputPlay.get(42)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.AT_THE_NET_B;
				}else{
				targetFields = Field.AT_THE_NET_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(43)[2]);
				Double succ = Double.parseDouble(inputPlay.get(43)[3]);
				Double fault = Double.parseDouble(inputPlay.get(43)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}	
		}
		if (isTeamA){
			playerFields = Field.AT_THE_NET_A;
			}else{
			playerFields = Field.AT_THE_NET_B;
		}
		for (Field playerField : playerFields){
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(44)[2]);
				Double succ = Double.parseDouble(inputPlay.get(44)[3]);
				Double fault = Double.parseDouble(inputPlay.get(44)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(45)[2]);
				Double succ = Double.parseDouble(inputPlay.get(45)[3]);
				Double fault = Double.parseDouble(inputPlay.get(45)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}
			if (isTeamA){
				targetFields = Field.AT_THE_NET_B;
				}else{
				targetFields = Field.AT_THE_NET_A;
			}
			for (Field targetField : targetFields){
				int number = Integer.parseInt(inputPlay.get(46)[2]);
				Double succ = Double.parseDouble(inputPlay.get(46)[3]);
				Double fault = Double.parseDouble(inputPlay.get(46)[4]);
				probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
				this.addProbHit(probAttack);
			}
		}
		
		// Defence
		int number = Integer.parseInt(inputPlay.get(47)[2]);
		Double succ = Double.parseDouble(inputPlay.get(47)[3]);
		Double fault = Double.parseDouble(inputPlay.get(47)[4]);
		Prob_DefRec probDef = (Prob_DefRec) initializeProb(HitTechnique.d, number, this.d, succ,fault,null,null,BallProperty.NORMAL);
		this.addProbHit(probDef);
		
		number = Integer.parseInt(inputPlay.get(48)[2]);
		succ = Double.parseDouble(inputPlay.get(48)[3]);
		fault = Double.parseDouble(inputPlay.get(48)[4]);
		probDef = (Prob_DefRec)initializeProb(HitTechnique.d,number, this.d, succ,fault,null,null,BallProperty.HARD);
		this.addProbHit(probDef);
		
		number = Integer.parseInt(inputPlay.get(49)[2]);
		succ = Double.parseDouble(inputPlay.get(49)[3]);
		fault = Double.parseDouble(inputPlay.get(49)[4]);
		probDef = (Prob_DefRec)initializeProb(HitTechnique.d_m,number, this.d_m, succ,fault,null,null,BallProperty.NORMAL);
		this.addProbHit(probDef);
		
		number = Integer.parseInt(inputPlay.get(50)[2]);
		succ = Double.parseDouble(inputPlay.get(50)[3]);
		fault = Double.parseDouble(inputPlay.get(50)[4]);
		probDef = (Prob_DefRec)initializeProb(HitTechnique.d_m,number, this.d_m,succ,fault,null,null,BallProperty.HARD);
		this.addProbHit(probDef);
		
		
		// Receive
		number = Integer.parseInt(inputPlay.get(51)[2]);
		succ = Double.parseDouble(inputPlay.get(51)[3]);
		fault = Double.parseDouble(inputPlay.get(51)[4]);
		probDef = (Prob_DefRec)initializeProb(HitTechnique.r,number,this.r,succ,fault,null,null,BallProperty.NORMAL);
		this.addProbHit(probDef);
		
		number = Integer.parseInt(inputPlay.get(52)[2]);
		succ = Double.parseDouble(inputPlay.get(52)[3]);
		fault = Double.parseDouble(inputPlay.get(52)[4]);
		probDef = (Prob_DefRec)initializeProb(HitTechnique.r,number,this.r,succ,fault,null,null,BallProperty.HARD);
		this.addProbHit(probDef);
		
		number = Integer.parseInt(inputPlay.get(53)[2]);
		succ = Double.parseDouble(inputPlay.get(53)[3]);
		fault = Double.parseDouble(inputPlay.get(53)[4]);			
		probDef = (Prob_DefRec)initializeProb(HitTechnique.r_m,number,this.r_m,succ,fault,null,null,BallProperty.NORMAL);
		this.addProbHit(probDef);
		
		number = Integer.parseInt(inputPlay.get(54)[2]);
		succ = Double.parseDouble(inputPlay.get(54)[3]);
		fault = Double.parseDouble(inputPlay.get(54)[4]);
		probDef = (Prob_DefRec)initializeProb(HitTechnique.r_m,number,this.r_m,succ,fault,null,null,BallProperty.HARD);
		this.addProbHit(probDef);
		
		// set
		number = Integer.parseInt(inputPlay.get(55)[2]);
		succ = Double.parseDouble(inputPlay.get(55)[3]);
		fault = Double.parseDouble(inputPlay.get(55)[4]);
		Prob_Hit probSet = new Prob_Hit(HitTechnique.s,number,this.compositionHit,succ,fault);
		this.addProbHit(probSet);
		
		
		// Block
		number = Integer.parseInt(inputPlay.get(56)[2]);
		Double prob1 = Double.parseDouble(inputPlay.get(56)[3]);
		Double prob2 = Double.parseDouble(inputPlay.get(56)[4]);
		Double prob3 = Double.parseDouble(inputPlay.get(56)[5]);
		if (prob1!=null && prob2 != null & prob3 != null){
			Prob_Block probBlock = new Prob_Block(BlockTechnique.b,number,prob1,prob2,prob3);
			this.setProbBlock(probBlock);
		}
	}
	
	public void readFromExcelTemplate(Player player) throws BiffException, IOException {
		boolean isTeamA = false;
		if (player.getTeam() == Game.getInstance().getTeamA()){
			isTeamA = true;
		}else if (player.getTeam() == Game.getInstance().getTeamB()){
			isTeamA = false;
		}else{
			try {
				throw new Exception("can't readFromExcelTemplate player " + player.getName() + " is not in the game!");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		// Einleseroutine angepasst auf Excel Sheet PaperInputData_allMatches.xls		
		Workbook workbook = Workbook.getWorkbook(new File(player.getDataFile()));
		Sheet sheet = workbook.getSheet(2);
		
//		// Blocking Player Value
//		NumberCell cellIsBlockPlayer = (NumberCell) sheet.getCell("M5");
//		isBlockingPlayer = cellIsBlockPlayer.getValue();
		
//		cellIsBlockPlayer = (NumberCell) sheet.getCell("F40");
//		System.out.println(cellIsBlockPlayer.getValue());
		
//		if (player.getTeam() == Game.getInstance().getTeamA()){
			// Serves
		
			Field[] targetFields;
			Field[] playerFields;
			if (isTeamA){
				targetFields = Field.LAST_ROW_B;
				}else{
				targetFields = Field.LAST_ROW_A;
			}
		
			Prob_Serve probServe;
			for(Field targetField : targetFields){
				int number = (int) ((NumberCell) sheet.getCell("F10")).getValue();
				Double succ = helper_getValueOfCell(sheet.getCell("G10"));
				Double fault = helper_getValueOfCell(sheet.getCell("H10"));
				probServe = (Prob_Serve) initializeProb(HitTechnique.S_F, number, this.S_F, succ,fault,targetField,null, null);
				this.addProbHit(probServe);
				
				number = (int) ((NumberCell) sheet.getCell("F11")).getValue();
				succ = helper_getValueOfCell(sheet.getCell("G11"));
				fault = helper_getValueOfCell(sheet.getCell("H11"));
				probServe = (Prob_Serve) initializeProb(HitTechnique.S_J, number, this.S_J, succ,fault,targetField, null, null);
				this.addProbHit(probServe);
			}
			
			if (isTeamA){
				targetFields = Field.MIDDLE_ROW_B;
				}else{
				targetFields = Field.MIDDLE_ROW_A;
			}			
			for(Field targetField : targetFields){
				int number = (int) ((NumberCell) sheet.getCell("J10")).getValue();
				Double succ = helper_getValueOfCell(sheet.getCell("K10"));
				Double fault = helper_getValueOfCell(sheet.getCell("L10"));
				probServe = (Prob_Serve) initializeProb(HitTechnique.S_F, number, this.S_F, succ,fault,targetField,null, null);
				this.addProbHit(probServe);
				
				number = (int) ((NumberCell) sheet.getCell("J11")).getValue();
				succ = helper_getValueOfCell(sheet.getCell("K11"));
				fault = helper_getValueOfCell(sheet.getCell("L11"));
				probServe = (Prob_Serve) initializeProb(HitTechnique.S_J, number, this.S_J, succ,fault,targetField,null, null);
				this.addProbHit(probServe);
			}
			
			
			// Smash
			if (isTeamA){
				playerFields = Field.OUT_A;
				}else{
				playerFields = Field.OUT_B;
			}
			Prob_Attack probAttack;
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F14")).getValue();
					Double succ = helper_getValueOfCell(sheet.getCell("G14"));
					Double fault = helper_getValueOfCell(sheet.getCell("H14"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J14")).getValue();
					Double succ = helper_getValueOfCell(sheet.getCell("K14"));
					Double fault = helper_getValueOfCell(sheet.getCell("L14"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}			
			}
			if (isTeamA){
				playerFields = Field.LAST_ROW_A;
				}else{
				playerFields = Field.LAST_ROW_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F15")).getValue();
					Double succ = helper_getValueOfCell(sheet.getCell("G15"));
					Double fault = helper_getValueOfCell(sheet.getCell("H15"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J15")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K15"));
					Double fault = helper_getValueOfCell(sheet.getCell("L15"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}			
			}
			if (isTeamA){
				playerFields = Field.MIDDLE_ROW_A;
				}else{
				playerFields = Field.MIDDLE_ROW_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F16")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G16"));
					Double fault = helper_getValueOfCell(sheet.getCell("H16"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J16")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K16"));
					Double fault = helper_getValueOfCell(sheet.getCell("L16"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}			
			}
			if (isTeamA){
				playerFields = Field.AT_THE_NET_A;
				}else{
				playerFields = Field.AT_THE_NET_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F17")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G17"));
					Double fault = helper_getValueOfCell(sheet.getCell("H17"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J17")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K17"));
					Double fault = helper_getValueOfCell(sheet.getCell("L17"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_SM, number, this.F_SM, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}			
			}
			
			
			// Shot
			if (isTeamA){
				playerFields = Field.OUT_A;
				}else{
				playerFields = Field.OUT_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F19")).getValue();
					Double succ = helper_getValueOfCell(sheet.getCell("G19"));
					Double fault = helper_getValueOfCell(sheet.getCell("H19"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J19")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K19"));
					Double fault = helper_getValueOfCell(sheet.getCell("L19"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}			
			}
			if (isTeamA){
				playerFields = Field.LAST_ROW_A;
				}else{
				playerFields = Field.LAST_ROW_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F20")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G20"));
					Double fault = helper_getValueOfCell(sheet.getCell("H20"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J20")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K20"));
					Double fault = helper_getValueOfCell(sheet.getCell("L20"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}			
			}
			if (isTeamA){
				playerFields = Field.MIDDLE_ROW_A;
				}else{
				playerFields = Field.MIDDLE_ROW_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F21")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G21"));
					Double fault = helper_getValueOfCell(sheet.getCell("H21"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J21")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K21"));
					Double fault = helper_getValueOfCell(sheet.getCell("L21"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}		
			}
			if (isTeamA){
				playerFields = Field.AT_THE_NET_A;
				}else{
				playerFields = Field.AT_THE_NET_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F22")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G22"));
					Double fault = helper_getValueOfCell(sheet.getCell("H22"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J22")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K22"));
					Double fault = helper_getValueOfCell(sheet.getCell("L22"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_E, number, this.F_SH, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}			
			}

			
			// Poke
			if (isTeamA){
				playerFields = Field.OUT_A;
				}else{
				playerFields = Field.OUT_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F24")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G24"));
					Double fault = helper_getValueOfCell(sheet.getCell("H24"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J24")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K24"));
					Double fault = helper_getValueOfCell(sheet.getCell("L24"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.AT_THE_NET_B;
					}else{
					targetFields = Field.AT_THE_NET_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("N24")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("O24"));
					Double fault = helper_getValueOfCell(sheet.getCell("P24"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}		
			}
			if (isTeamA){
				playerFields = Field.LAST_ROW_A;
				}else{
				playerFields = Field.LAST_ROW_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F25")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G25"));
					Double fault = helper_getValueOfCell(sheet.getCell("H25"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField,null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J25")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K25"));
					Double fault = helper_getValueOfCell(sheet.getCell("L25"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.AT_THE_NET_B;
					}else{
					targetFields = Field.AT_THE_NET_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("N25")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("O25"));
					Double fault = helper_getValueOfCell(sheet.getCell("P25"));
					probAttack =(Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}		
			}
			if (isTeamA){
				playerFields = Field.MIDDLE_ROW_A;
				}else{
				playerFields = Field.MIDDLE_ROW_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F26")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G26"));
					Double fault = helper_getValueOfCell(sheet.getCell("H26"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J26")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("K26"));
					Double fault = helper_getValueOfCell(sheet.getCell("L26"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.AT_THE_NET_B;
					}else{
					targetFields = Field.AT_THE_NET_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("N26")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("O26"));
					Double fault = helper_getValueOfCell(sheet.getCell("P26"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}	
			}
			if (isTeamA){
				playerFields = Field.AT_THE_NET_A;
				}else{
				playerFields = Field.AT_THE_NET_B;
			}
			for (Field playerField : playerFields){
				if (isTeamA){
					targetFields = Field.LAST_ROW_B;
					}else{
					targetFields = Field.LAST_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("F27")).getValue(); 
					Double succ = helper_getValueOfCell(sheet.getCell("G27"));
					Double fault = helper_getValueOfCell(sheet.getCell("H27"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.MIDDLE_ROW_B;
					}else{
					targetFields = Field.MIDDLE_ROW_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("J27")).getValue();
					Double succ = helper_getValueOfCell(sheet.getCell("K27"));
					Double fault = helper_getValueOfCell(sheet.getCell("L27"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}
				if (isTeamA){
					targetFields = Field.AT_THE_NET_B;
					}else{
					targetFields = Field.AT_THE_NET_A;
				}
				for (Field targetField : targetFields){
					int number = (int) ((NumberCell) sheet.getCell("N27")).getValue();
					Double succ = helper_getValueOfCell(sheet.getCell("O27"));
					Double fault = helper_getValueOfCell(sheet.getCell("P27"));
					probAttack = (Prob_Attack) initializeProb(HitTechnique.F_P, number, this.F_P, succ, fault, targetField, playerField, null);
					this.addProbHit(probAttack);
				}
			}
			
			// Defence
			int number = (int) ((NumberCell) sheet.getCell("F32")).getValue();
			Double succ = helper_getValueOfCell(sheet.getCell("G32"));
			Double fault = helper_getValueOfCell(sheet.getCell("H32"));
			Prob_DefRec probDef = (Prob_DefRec) initializeProb(HitTechnique.d, number, this.d, succ,fault,null,null,BallProperty.NORMAL);
			this.addProbHit(probDef);
			
			number = (int) ((NumberCell) sheet.getCell("J32")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("K32"));
			fault = helper_getValueOfCell(sheet.getCell("L32"));
			probDef = (Prob_DefRec)initializeProb(HitTechnique.d,number, this.d, succ,fault,null,null,BallProperty.HARD);
			this.addProbHit(probDef);
			
			number = (int) ((NumberCell) sheet.getCell("F33")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("G33"));
			fault = helper_getValueOfCell(sheet.getCell("H33"));
			probDef = (Prob_DefRec)initializeProb(HitTechnique.d_m,number, this.d_m, succ,fault,null,null,BallProperty.NORMAL);
			this.addProbHit(probDef);
			
			number = (int) ((NumberCell) sheet.getCell("J33")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("K33"));
			fault = helper_getValueOfCell(sheet.getCell("L33"));
			probDef = (Prob_DefRec)initializeProb(HitTechnique.d_m,number, this.d_m,succ,fault,null,null,BallProperty.HARD);
			this.addProbHit(probDef);
			
			
			// Receive
			number = (int) ((NumberCell) sheet.getCell("F34")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("G34"));
			fault = helper_getValueOfCell(sheet.getCell("H34"));
			probDef = (Prob_DefRec)initializeProb(HitTechnique.r,number,this.r,succ,fault,null,null,BallProperty.NORMAL);
			this.addProbHit(probDef);
			
			number = (int) ((NumberCell) sheet.getCell("J34")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("K34"));
			fault = helper_getValueOfCell(sheet.getCell("L34"));
			probDef = (Prob_DefRec)initializeProb(HitTechnique.r,number,this.r,succ,fault,null,null,BallProperty.HARD);
			this.addProbHit(probDef);
			
			number = (int) ((NumberCell) sheet.getCell("F35")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("G35"));
			fault = helper_getValueOfCell(sheet.getCell("H35"));
			
			probDef = (Prob_DefRec)initializeProb(HitTechnique.r_m,number,this.r_m,succ,fault,null,null,BallProperty.NORMAL);
			this.addProbHit(probDef);
			
			number = (int) ((NumberCell) sheet.getCell("J35")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("K35"));
			fault = helper_getValueOfCell(sheet.getCell("L35"));
			probDef = (Prob_DefRec)initializeProb(HitTechnique.r_m,number,this.r_m,succ,fault,null,null,BallProperty.HARD);
			this.addProbHit(probDef);
			
			// set
			number = (int) ((NumberCell) sheet.getCell("F36")).getValue();
			succ = helper_getValueOfCell(sheet.getCell("G36"));
			fault = helper_getValueOfCell(sheet.getCell("H36"));
			Prob_Hit probSet = new Prob_Hit(HitTechnique.s,number,this.compositionHit,succ,fault);
			this.addProbHit(probSet);
			
			
			// Block
			number = (int) ((NumberCell) sheet.getCell("F40")).getValue();
			Double prob1 = helper_getValueOfCell(sheet.getCell("G40"));
			Double prob2 = helper_getValueOfCell(sheet.getCell("H40"));
			Double prob3 = helper_getValueOfCell(sheet.getCell("I40"));
			if (prob1!=null && prob2 != null & prob3 != null){
				Prob_Block probBlock = new Prob_Block(BlockTechnique.b,number,prob1,prob2,prob3);
				this.setProbBlock(probBlock);
			}
	}
	
	public Double helper_getValueOfCell(Cell cell){				
		if(cell.getType()==CellType.NUMBER || cell.getType() == CellType.NUMBER_FORMULA){
			return ((NumberCell) cell).getValue();
		}else{
			return null;
		}
	}
	
	private Prob_Hit initializeProb(HitTechnique t, int n, Prob_Hit pred, Double succ, Double fault, Field tf, Field pf, BallProperty prop){
		if (t == HitTechnique.S_F || t == HitTechnique.S_J){
			if (succ != null && fault != null){
				return new Prob_Serve(t, n, pred, succ, fault, tf);
			}else{
				return new Prob_Serve(t, n, pred, -1, -1, tf);
			}
		}
		if (t == HitTechnique.F_SM || t == HitTechnique.F_E || t == HitTechnique.F_P){
			if (succ != null && fault != null){
				return new Prob_Attack(t, n, pred, succ, fault, tf, pf);
			}else{
				return new Prob_Attack(t, n, pred, -1, -1, tf, pf);
			}
		}
		if (t == HitTechnique.d || t == HitTechnique.d_m || t == HitTechnique.r || t == HitTechnique.r_m){
			if (succ != null && fault != null){
				return new Prob_DefRec(t, n, pred, succ, fault, prop);
			}else{
				return new Prob_DefRec(t, n, pred, -1, -1, prop);
			}
		}
		if (t == HitTechnique.s){
			if (succ != null && fault != null){
				return new Prob_Hit(t, n, pred, succ, fault);
			}else{
				return new Prob_Hit(t, n, pred, -1, -1);
			}
		}
		
		System.err.println("initialize Prob Fehler - Technik konnte nicht zugeordnet werden");
		return null;
	};
	
	public void computeAggregatedProbs(Player player){
		boolean isTeamA;
		if (player.isInTeamA()){
			isTeamA = true;
		}else{
			isTeamA = false;
		}
		
		// Hit
		int totalNumberHit;
		double succProbHit;
		double faultProbHit;
		{
		//CrossingNet
		int totalNumberCrossingNet;
		double succProbCrossingNet;
		double faultProbCrossingNet;
		{
			// Serves
			int totalNumberServes;
			double succProbServes;
			double faultProbServes;
			{
				int totalNumberFloatServes = 0;
				double succProbFloat = 0.0;
				double faultProbFloat = 0.0;
				
				int totalNumberJumpServes = 0;
				double succProbJump = 0.0;
				double faultProbJump = 0.0;
				
				for (int i = 1; i <= 2; i++){
					totalNumberFloatServes += this.getProbHit(HitTechnique.S_F, helper_getField(i,1,isTeamA, false), null, null).getNumberOfObservations();
					succProbFloat += this.getProbHit(HitTechnique.S_F, helper_getField(i,1,isTeamA, false), null, null).getNumberOfObservations()*
							this.getProbHit(HitTechnique.S_F, helper_getField(i,1,isTeamA, false), null, null).getSuccProb();
					faultProbFloat += this.getProbHit(HitTechnique.S_F, helper_getField(i,1,isTeamA, false), null, null).getNumberOfObservations()*
							this.getProbHit(HitTechnique.S_F, helper_getField(i,1,isTeamA, false), null, null).getFaultProb();
					
					totalNumberJumpServes += this.getProbHit(HitTechnique.S_J, helper_getField(i,1,isTeamA, false), null, null).getNumberOfObservations();
					succProbJump += this.getProbHit(HitTechnique.S_J, helper_getField(i,1,isTeamA, false), null, null).getNumberOfObservations()*
							this.getProbHit(HitTechnique.S_J, helper_getField(i,1,isTeamA, false), null, null).getSuccProb();
					faultProbJump += this.getProbHit(HitTechnique.S_J, helper_getField(i,1,isTeamA, false), null, null).getNumberOfObservations()*
							this.getProbHit(HitTechnique.S_J, helper_getField(i,1,isTeamA, false), null, null).getFaultProb();	
				}
				
				succProbFloat = succProbFloat/totalNumberFloatServes;
				faultProbFloat = faultProbFloat/totalNumberFloatServes;
				this.S_F.updateProbHit(HitTechnique.S_F, totalNumberFloatServes, this.general_serve, succProbFloat, faultProbFloat);
				log4j.debug(this.S_F);
				
				succProbJump = succProbJump/totalNumberJumpServes;
				faultProbJump = faultProbJump/totalNumberJumpServes;
				this.S_J.updateProbHit(HitTechnique.S_J, totalNumberJumpServes, this.general_serve, succProbJump, faultProbJump);
				log4j.debug(this.S_J);
				
				totalNumberServes = totalNumberFloatServes+totalNumberJumpServes;
				succProbServes = (succProbFloat*totalNumberFloatServes+succProbJump*totalNumberJumpServes)/totalNumberServes;
				faultProbServes = (faultProbFloat*totalNumberFloatServes+faultProbJump*totalNumberJumpServes)/totalNumberServes;
				this.general_serve.updateProbHit(HitTechnique.S, totalNumberServes, this.crossingNet, 
						succProbServes, faultProbServes);
				log4j.debug(this.general_serve);
			}
			
			
			// Attack
			int totalNumberF;
			double succProbF;
			double faultProbF;
			{
				int totalNumberSM = 0;
				double succProbSM = 0.0;
				double faultProbSM = 0.0;
				int totalNumberSH = 0;
				double succProbSH = 0.0;
				double faultProbSH = 0.0;
				int totalNumberP = 0;
				double succProbP = 0.0;
				double faultProbP = 0.0;
				
				for (int playerfieldi = 0; playerfieldi < 8; playerfieldi++){
					for (int i = 1; i <= 3; i++){
						if ( i < 3){
							if(this.getProbHit(HitTechnique.F_SM, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null)!=null){
								totalNumberSM += this.getProbHit(HitTechnique.F_SM, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations();
								succProbSM += this.getProbHit(HitTechnique.F_SM, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations()*
										this.getProbHit(HitTechnique.F_SM, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getSuccProb();
								faultProbSM += this.getProbHit(HitTechnique.F_SM, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations()*
										this.getProbHit(HitTechnique.F_SM, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getFaultProb();
							}
							if(this.getProbHit(HitTechnique.F_E, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null)!=null){
								totalNumberSH += this.getProbHit(HitTechnique.F_E, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations();
								succProbSH += this.getProbHit(HitTechnique.F_E, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations()*
										this.getProbHit(HitTechnique.F_E, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getSuccProb();
								faultProbSH += this.getProbHit(HitTechnique.F_E, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations()*
										this.getProbHit(HitTechnique.F_E, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getFaultProb();	
							}
						}
						if (this.getProbHit(HitTechnique.F_P, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null)!=null){
							totalNumberP += this.getProbHit(HitTechnique.F_P, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations();
							succProbP += this.getProbHit(HitTechnique.F_P, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations()*
									this.getProbHit(HitTechnique.F_P, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getSuccProb();
							faultProbP += this.getProbHit(HitTechnique.F_P, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getNumberOfObservations()*
									this.getProbHit(HitTechnique.F_P, helper_getField(i,1,isTeamA, false), helper_getField(playerfieldi,1,isTeamA, true), null).getFaultProb();	
						}
					}
				}
				if(totalNumberSM!=0){
					succProbSM = succProbSM/totalNumberSM;
					faultProbSM = faultProbSM/totalNumberSM;
				}
				this.F_SM.updateProbHit(HitTechnique.F_SM, totalNumberSM, this.F, succProbSM, faultProbSM);
				log4j.debug(this.F_SM);
				if(totalNumberSH!=0){
					succProbSH = succProbSH/totalNumberSH;
					faultProbSH = faultProbSH/totalNumberSH;
				}
				this.F_SH.updateProbHit(HitTechnique.F_E, totalNumberSH, this.F, succProbSH, faultProbSH);
				log4j.debug(this.F_SH);
				if(totalNumberP!=0){
					succProbP = succProbP/totalNumberP;
					faultProbP = faultProbP/totalNumberP;
				}
				this.F_P.updateProbHit(HitTechnique.F_P, totalNumberP, this.F, succProbP, faultProbP);
				log4j.debug(this.F_P);
				
				totalNumberF = totalNumberSM + totalNumberSH + totalNumberP;
				succProbF = (succProbSM*totalNumberSM+succProbSH*totalNumberSH+succProbP*totalNumberP)/totalNumberF;
				faultProbF = (faultProbSM*totalNumberSM+faultProbSH*totalNumberSH+faultProbP*totalNumberP)/totalNumberF;
				this.F.updateProbHit(HitTechnique.F, totalNumberF, this.crossingNet, succProbF, faultProbF);
				log4j.debug(this.F);
			}
			
			totalNumberCrossingNet = totalNumberF + totalNumberServes;
			succProbCrossingNet = (succProbF*totalNumberF+succProbServes*totalNumberServes)/totalNumberCrossingNet;
			faultProbCrossingNet = (faultProbF*totalNumberF+faultProbServes*totalNumberServes)/totalNumberCrossingNet;
			this.crossingNet.updateProbHit(HitTechnique.cross, totalNumberCrossingNet, this.hit, succProbCrossingNet, faultProbCrossingNet);
			log4j.debug(this.crossingNet);
		}
		
		
		// composition hit
		int totalNumberCompos;
		double succProbCompos;
		double faultProbCompos;
		{
		
			// recdef
			int totalNumberRecDef;
			double succProbRecDef;
			double faultProbRecDef;
			
			{	
			
				//defence
				int totalNumberdef;
				double succProbdef;
				double faultProbdef;
				{
					int totalNumberd = 0;
					double succProbd = 0.0;
					double faultProbd = 0.0;
					
					if (this.getProbHit(HitTechnique.d, null, null, BallProperty.HARD)!=null){
						totalNumberd += this.getProbHit(HitTechnique.d, null, null, BallProperty.HARD).getNumberOfObservations();
						succProbd += this.getProbHit(HitTechnique.d, null, null, BallProperty.HARD).getSuccProb()*this.getProbHit(HitTechnique.d, null, null, BallProperty.HARD).getNumberOfObservations();
						faultProbd += this.getProbHit(HitTechnique.d, null, null, BallProperty.HARD).getFaultProb()*this.getProbHit(HitTechnique.d, null, null, BallProperty.HARD).getNumberOfObservations();
					}
					if (this.getProbHit(HitTechnique.d, null, null, BallProperty.NORMAL)!=null){
						totalNumberd += this.getProbHit(HitTechnique.d, null, null, BallProperty.NORMAL).getNumberOfObservations();
						succProbd += this.getProbHit(HitTechnique.d, null, null, BallProperty.NORMAL).getSuccProb()*this.getProbHit(HitTechnique.d, null, null, BallProperty.NORMAL).getNumberOfObservations();
						faultProbd += this.getProbHit(HitTechnique.d, null, null, BallProperty.NORMAL).getFaultProb()*this.getProbHit(HitTechnique.d, null, null, BallProperty.NORMAL).getNumberOfObservations();
					}
					succProbd = succProbd/totalNumberd;
					faultProbd = faultProbd/totalNumberd;
					this.d.updateProbHit(HitTechnique.d,totalNumberd, this.def, succProbd, faultProbd);
					log4j.debug(this.d);
					
					int totalNumberdm = 0;
					double succProbdm = 0.0;
					double faultProbdm = 0.0;
					
					if (this.getProbHit(HitTechnique.d_m, null, null, BallProperty.HARD)!=null){
						totalNumberdm += this.getProbHit(HitTechnique.d_m, null, null, BallProperty.HARD).getNumberOfObservations();
						succProbdm += this.getProbHit(HitTechnique.d_m, null, null, BallProperty.HARD).getSuccProb()*this.getProbHit(HitTechnique.d_m, null, null, BallProperty.HARD).getNumberOfObservations();
						faultProbdm += this.getProbHit(HitTechnique.d_m, null, null, BallProperty.HARD).getFaultProb()*this.getProbHit(HitTechnique.d_m, null, null, BallProperty.HARD).getNumberOfObservations();
					}
					if (this.getProbHit(HitTechnique.d_m, null, null, BallProperty.NORMAL)!=null){
						totalNumberdm += this.getProbHit(HitTechnique.d_m, null, null, BallProperty.NORMAL).getNumberOfObservations();
						succProbdm += this.getProbHit(HitTechnique.d_m, null, null, BallProperty.NORMAL).getSuccProb()*this.getProbHit(HitTechnique.d_m, null, null, BallProperty.NORMAL).getNumberOfObservations();
						faultProbdm += this.getProbHit(HitTechnique.d_m, null, null, BallProperty.NORMAL).getFaultProb()*this.getProbHit(HitTechnique.d_m, null, null, BallProperty.NORMAL).getNumberOfObservations();
					}
					succProbdm = succProbdm/totalNumberdm;
					faultProbdm = faultProbdm/totalNumberdm;
					this.d_m.updateProbHit(HitTechnique.d_m,totalNumberdm, this.def, succProbdm, faultProbdm);
					log4j.debug(this.d_m);
				
					totalNumberdef = totalNumberd + totalNumberdm;
					succProbdef = (succProbd*totalNumberd + succProbdm*totalNumberdm)/totalNumberdef;
					faultProbdef = (faultProbd*totalNumberd + faultProbdm*totalNumberdm)/totalNumberdef;
					this.def.updateProbHit(HitTechnique.def, totalNumberdef, this.recDef, succProbdef, faultProbdef);
					log4j.debug(this.def);
				}
				
				//reception
				int totalNumberrec = 0;
				double succProbrec = 0.0;
				double faultProbrec = 0.0;
				{
					int totalNumberr = 0;
					double succProbr = 0.0;
					double faultProbr = 0.0;
					
					if (this.getProbHit(HitTechnique.r, null, null, BallProperty.HARD)!=null){
						totalNumberr += this.getProbHit(HitTechnique.r, null, null, BallProperty.HARD).getNumberOfObservations();
						succProbr += this.getProbHit(HitTechnique.r, null, null, BallProperty.HARD).getSuccProb()*this.getProbHit(HitTechnique.r, null, null, BallProperty.HARD).getNumberOfObservations();
						faultProbr += this.getProbHit(HitTechnique.r, null, null, BallProperty.HARD).getFaultProb()*this.getProbHit(HitTechnique.r, null, null, BallProperty.HARD).getNumberOfObservations();
					}
					if (this.getProbHit(HitTechnique.r, null, null, BallProperty.NORMAL)!=null){
						totalNumberr += this.getProbHit(HitTechnique.r, null, null, BallProperty.NORMAL).getNumberOfObservations();
						succProbr += this.getProbHit(HitTechnique.r, null, null, BallProperty.NORMAL).getSuccProb()*this.getProbHit(HitTechnique.r, null, null, BallProperty.NORMAL).getNumberOfObservations();
						faultProbr += this.getProbHit(HitTechnique.r, null, null, BallProperty.NORMAL).getFaultProb()*this.getProbHit(HitTechnique.r, null, null, BallProperty.NORMAL).getNumberOfObservations();
					}
					succProbr = succProbr/totalNumberr;
					faultProbr = faultProbr/totalNumberr;
					this.r.updateProbHit(HitTechnique.r,totalNumberr, this.rec, succProbr, faultProbr);
					log4j.debug(this.r);
					
					int totalNumberrm = 0;
					double succProbrm = 0.0;
					double faultProbrm = 0.0;
					
					if (this.getProbHit(HitTechnique.r_m, null, null, BallProperty.HARD)!=null){
						totalNumberrm += this.getProbHit(HitTechnique.r_m, null, null, BallProperty.HARD).getNumberOfObservations();
						succProbrm += this.getProbHit(HitTechnique.r_m, null, null, BallProperty.HARD).getSuccProb()*this.getProbHit(HitTechnique.r_m, null, null, BallProperty.HARD).getNumberOfObservations();
						faultProbrm += this.getProbHit(HitTechnique.r_m, null, null, BallProperty.HARD).getFaultProb()*this.getProbHit(HitTechnique.r_m, null, null, BallProperty.HARD).getNumberOfObservations();
					}
					if (this.getProbHit(HitTechnique.r_m, null, null, BallProperty.NORMAL)!=null){
						totalNumberrm += this.getProbHit(HitTechnique.r_m, null, null, BallProperty.NORMAL).getNumberOfObservations();
						succProbrm += this.getProbHit(HitTechnique.r_m, null, null, BallProperty.NORMAL).getSuccProb()*this.getProbHit(HitTechnique.r_m, null, null, BallProperty.NORMAL).getNumberOfObservations();
						faultProbrm += this.getProbHit(HitTechnique.r_m, null, null, BallProperty.NORMAL).getFaultProb()*this.getProbHit(HitTechnique.r_m, null, null, BallProperty.NORMAL).getNumberOfObservations();
					}
					succProbrm = succProbrm/totalNumberrm;
					faultProbrm = faultProbrm/totalNumberrm;
					this.r_m.updateProbHit(HitTechnique.r_m,totalNumberrm, this.rec, succProbrm, faultProbrm);
					log4j.debug(this.r_m);
				
					totalNumberrec = totalNumberr + totalNumberrm;
					succProbrec = (succProbr*totalNumberr + succProbrm*totalNumberrm)/totalNumberrec;
					faultProbrec = (faultProbr*totalNumberr + faultProbrm*totalNumberrm)/totalNumberrec;
					this.rec.updateProbHit(HitTechnique.rec, totalNumberrec, this.recDef, succProbrec, faultProbrec);
					log4j.debug(this.rec);
				}
				
				totalNumberRecDef = totalNumberrec + totalNumberdef;
				succProbRecDef = (succProbrec*totalNumberrec+succProbdef*totalNumberdef)/totalNumberRecDef;
				faultProbRecDef = (faultProbrec*totalNumberrec + faultProbdef*totalNumberdef)/totalNumberRecDef;
				this.recDef.updateProbHit(HitTechnique.recdef, totalNumberRecDef, this.compositionHit, succProbRecDef, faultProbRecDef);
				log4j.debug(this.recDef);
			}
		
			int totalNumberSet = 0;
			double succProbSet = 0.0;
			double faultProbSet = 0.0;
			
			totalNumberSet = this.getProbHit(HitTechnique.s, null, null, null).getNumberOfObservations();
			succProbSet = this.getProbHit(HitTechnique.s, null, null, null).getSuccProb();
			faultProbSet = this.getProbHit(HitTechnique.s, null, null, null).getFaultProb();
			
			totalNumberCompos = totalNumberRecDef + totalNumberSet;
			succProbCompos = (succProbSet*totalNumberSet+succProbRecDef*totalNumberRecDef)/totalNumberCompos;
			faultProbCompos = (faultProbSet*totalNumberSet+faultProbRecDef*totalNumberRecDef)/totalNumberCompos;
			this.compositionHit.updateProbHit(HitTechnique.compose, totalNumberCompos, this.hit, succProbCompos, faultProbCompos);
			log4j.debug(this.compositionHit);
		}
			
		totalNumberHit = totalNumberCompos + totalNumberCrossingNet;
		succProbHit = (succProbCompos*totalNumberCompos+succProbCrossingNet*totalNumberCrossingNet)/totalNumberHit;
		faultProbHit = (faultProbCompos*totalNumberCompos+faultProbCrossingNet*totalNumberCrossingNet)/totalNumberHit;
		this.hit.updateProbHit(HitTechnique.h, totalNumberHit, null, succProbHit, faultProbHit);
		log4j.debug(hit);
		}
	}
	
	private Field helper_getField(int r, int c, boolean isTeamA, boolean ownSide){
		if ((isTeamA && ownSide) || (!isTeamA && !ownSide)){
			return Field.fieldMatrix[r][c];
		}else{
			return Field.fieldMatrix[7-r][5-c];
		}
	}
	
	public HashSet<Prob_Hit> getAllHitProbs(HitTechnique tech){
		TechniqueTest techTest = new TechniqueTest(tech);
		HashSet<Prob_Hit> returnSet = new HashSet<Prob_Hit>();
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			returnSet.addAll(servingProbs.get(tech).values());
			return returnSet;
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){		
			for (Field f : attackingProbs.get(tech).keySet()){
				returnSet.addAll(attackingProbs.get(tech).get(f).values());
			}
			return returnSet;	
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			returnSet.addAll(receivingProbs.get(tech).values());
			return returnSet;
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			returnSet.addAll(defendingProbs.get(tech).values());
		}
		
		// Setting
		if (techTest.isSettingTechnique()){		
			returnSet.add(settingProb);
			return returnSet;
		}
		
		System.err.println("No Prob_Hit for " + tech + " gefunden!");
		return returnSet;
	}
		
	public Prob_Hit getProbHit(HitTechnique tech, Field targetfield, Field playerfield, BallProperty strength){
		TechniqueTest techTest = new TechniqueTest(tech);
		Prob_Hit prob_Hit = null;
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			if (servingProbs.get(tech)==null){
				log4j.debug("ProbHit for Technique " + tech.toString() + " does not exist at all!");
			}else if (servingProbs.get(tech).get(targetfield) != null){
				prob_Hit = servingProbs.get(tech).get(targetfield);
			}
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){
			if (attackingProbs.get(tech)==null){
				log4j.debug("ProbHit for Technique " + tech.toString() + " does not exist at all!");
			}else if (attackingProbs.get(tech).get(playerfield) == null){
				log4j.debug("ProbHit for Technique " + tech.toString() + " with playerfield " + playerfield + "does not exist.");
			}else if (attackingProbs.get(tech).get(playerfield).get(targetfield) != null){
				prob_Hit = attackingProbs.get(tech).get(playerfield).get(targetfield);	
			}
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			if (receivingProbs.get(tech)==null){
				log4j.debug("ProbHit for Technique " + tech.toString() + " does not exist at all!");
			}else if (receivingProbs.get(tech).get(strength)==null){
				log4j.debug("ProbHit for Technique " + tech.toString() + " with strength " + strength.toString() + "does not exist.");
			}else{
				prob_Hit = receivingProbs.get(tech).get(strength);
			}
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			if (defendingProbs.get(tech)==null){
				log4j.debug("ProbHit for Technique " + tech.toString() + " does not exist at all!");
			}else if (defendingProbs.get(tech).get(strength)==null){
				log4j.debug("ProbHit for Technique " + tech.toString() + " with strength " + strength.toString() + "does not exist.");
			}else{
				prob_Hit = defendingProbs.get(tech).get(strength);
			}
		}
		
		// Setting
		if (techTest.isSettingTechnique()){
			if (settingProb==null){
				log4j.debug("ProbHit for Technique sets does not exist at all!");
			}else{
				prob_Hit = settingProb;
			}
		}
		
		return prob_Hit;
	}
	
	public Prob_Block getProbBlock(){
		return blockingProb;
	}
	
	public void overrideProbHit(Prob_Hit prob){
		TechniqueTest techTest = new TechniqueTest(prob.getTech());
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			if (prob instanceof Prob_Serve){
				Prob_Serve probServe = (Prob_Serve) prob;
				if (servingProbs.get(probServe.getTech()) == null){
					HashMap <Field, Prob_Hit> map = new HashMap <Field, Prob_Hit>();
					map.put(probServe.getTargetField(), probServe);
					servingProbs.put(prob.getTech(), map);
				}else if (servingProbs.get(prob.getTech()).get(probServe.getTargetField())!=null){
					servingProbs.get(prob.getTech()).put(probServe.getTargetField(), probServe);
				}else{
					log4j.warn("Achtung Prob serving Prob for tech " + prob.getTech() + " and target field " + probServe.getTargetField() + " existiert noch nicht und wurde NICHT ueberschrieben.");
//					servingProbs.get(prob.getTech()).put(probServe.getTargetField(), probServe);
				}
			}else{
				System.err.println("Serving Technique " + prob.getTech().toString() + " but not instance of Prob_Serve!");
			}
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){
			if (prob instanceof Prob_Attack){
				Prob_Attack probAttack = (Prob_Attack) prob;
				if (attackingProbs.get(prob.getTech()) == null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					HashMap<Field, HashMap<Field, Prob_Hit>> mapPlayerField = new HashMap<Field, HashMap<Field, Prob_Hit>>();
					mapPlayerField.put(probAttack.getPlayerField(), mapTargetField);
					attackingProbs.put(probAttack.getTech(), mapPlayerField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField())==null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					attackingProbs.get(probAttack.getTech()).put(probAttack.getPlayerField(), mapTargetField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).get(probAttack.getTargetField()) != null){
					attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).put(probAttack.getTargetField(), prob);				
				}else{
					log4j.warn("Achtung Prob attacking Prob for tech " +  probAttack.getTech().toString() + " and playerfield " 
							+ probAttack.getPlayerField().toString() + " and target field " + probAttack.getTargetField().toString() 
							+ " existiert noch nicht und wurde NICHT ueberschrieben.");
//					attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).put(probAttack.getTargetField(), prob);
				}
			}else{
				System.err.println("Attacking Technique " + prob.getTech().toString() + " but not instance of Prob_Attack!");
			}			
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (receivingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					receivingProbs.put(prob_DefRec.getTech(), map);
				}else if (receivingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) != null){
					receivingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				}else{
					log4j.warn("Achtung receving Prob with tech " + prob_DefRec.getTech() + " and ball property " + prob_DefRec.getStrength() + " existierte noch nicht und wurde ueberschrieben.");
//					receivingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				}
			}else{
				System.err.println("Reception Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (defendingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					defendingProbs.put(prob_DefRec.getTech(), map);
				}else if(defendingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) != null){
					defendingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				} else {
					log4j.warn("Achtung defending Prob with tech " + prob_DefRec.getTech() + " and ball property " + prob_DefRec.getStrength() 
					+ " existierte noch nicht und wurde NICHT ueberschrieben.");
//					defendingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				}
			}else{
				System.err.println("Defending Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Setting
		if (techTest.isSettingTechnique()){
			if (this.settingProb != null){
				this.settingProb = prob;
			}else{
				this.settingProb = prob;
				log4j.warn("Achtung Prob " + this.settingProb.toString() + " existierte noch nicht und wurde NICHT ueberschrieben.");
			}
		}
	}
	
	public void addProbHit(Prob_Hit prob){
		TechniqueTest techTest = new TechniqueTest(prob.getTech());
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			if (prob instanceof Prob_Serve){
				Prob_Serve probServe = (Prob_Serve) prob;
				if (servingProbs.get(probServe.getTech()) == null){
					HashMap <Field, Prob_Hit> map = new HashMap <Field, Prob_Hit>();
					map.put(probServe.getTargetField(), probServe);
					servingProbs.put(prob.getTech(), map);
				}else if (servingProbs.get(prob.getTech()).get(probServe.getTargetField())==null){
					servingProbs.get(prob.getTech()).put(probServe.getTargetField(), probServe);
				}else{
					System.err.println("Achtung Prob " + servingProbs.get(prob.getTech()).get(probServe.getTargetField()).toString() + " existiert schon! Kann nicht hinzgef�gt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Serving Technique " + prob.getTech().toString() + " but not instance of Prob_Serve!");
			}
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){
			if (prob instanceof Prob_Attack){
				Prob_Attack probAttack = (Prob_Attack) prob;
				if (attackingProbs.get(prob.getTech()) == null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					HashMap<Field, HashMap<Field, Prob_Hit>> mapPlayerField = new HashMap<Field, HashMap<Field, Prob_Hit>>();
					mapPlayerField.put(probAttack.getPlayerField(), mapTargetField);
					attackingProbs.put(probAttack.getTech(), mapPlayerField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField())==null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					attackingProbs.get(probAttack.getTech()).put(probAttack.getPlayerField(), mapTargetField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).get(probAttack.getTargetField()) == null){
					attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).put(probAttack.getTargetField(), prob);				
				}else{
					System.err.println("Achtung Prob " + attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).get(probAttack.getTargetField()).toString() 
							+ " existiert schon! Kann nicht hinzgef�gt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Attacking Technique " + prob.getTech().toString() + " but not instance of Prob_Attack!");
			}			
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (receivingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					receivingProbs.put(prob_DefRec.getTech(), map);
				}else if (receivingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) == null){
					receivingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				}else{
					System.err.println("Achtung Prob " + receivingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()).toString() 
						+ " existiert schon! Kann nicht hinzgefuegt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Reception Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (defendingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					defendingProbs.put(prob_DefRec.getTech(), map);
				}else if(defendingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) == null){
					defendingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				} else {
					System.err.println("Achtung Prob " + defendingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()).toString() 
							+ " existiert schon! Kann nicht hinzgefuegt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Defending Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Setting
		if (techTest.isSettingTechnique()){
			if (this.settingProb == null){
				this.settingProb = prob;
			}else{
				System.err.println("Achtung Prob " + this.settingProb.toString() 
						+ " existiert schon! Kann nicht hinzgefuegt werden!");
				System.exit(1);
			}
		}
	}
	
	public void setProbBlock(Prob_Block probBlock){
		// Blocking
		this.blockingProb = probBlock;			
	}

	public Prob_Hit getSettingProb() {
		return settingProb;
	}

	public String printAggProbs(){
		StringBuffer s = new StringBuffer();
		s.append(this.hit.toString() + "\n");
		s.append(this.crossingNet.toString() + "\n");
		s.append(this.F + "\n");
		s.append(this.F_SM + "\n");
		s.append(this.F_SH + "\n");
		s.append(this.F_P + "\n");
		s.append(this.general_serve + "\n");
		s.append(this.S_J + "\n");
		s.append(this.S_F + "\n");
		s.append(this.compositionHit + "\n");
		s.append(this.recDef + "\n");
		s.append(this.def + "\n");
		s.append(this.d + "\n");
		s.append(this.d_m + "\n");
		s.append(this.rec + "\n");
		s.append(this.r + "\n");
		s.append(this.r_m + "\n");
		
		return s.toString();
	}
	
	/**
	 * Prints IndPlayer Probs as on the PaperInputData Excel Sheet
	 * @return String with formatted Probs
	 */
	public String printAnalogToExcelInputFormat(Player player){	
		StringBuffer string = new StringBuffer();
		
		string.append(player.getName() + "\t" + player.getTeam().name); // + "\t blocking Player " + this.isBlockingPlayer + "\n \n");
		
		if (player.getTeam() == Game.getInstance().getTeamA()){
			string.append("SERVES: \n");
			string.append(helper_ProbHitToString(HitTechnique.S_F, Field.B11, null, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.S_F, Field.B21, null, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.S_J, Field.B11, null, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.S_J, Field.B21, null, null) + "\n");
	
			string.append("ATTTACKS: \n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.B11, Field.A01, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.B21, Field.A01, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.B11, Field.A11, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.B21, Field.A11, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.B11, Field.A21, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.B21, Field.A21, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.B11, Field.A31, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.B21, Field.A31, null) + "\n");
			

			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.B11, Field.A01, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.B21, Field.A01, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.B11, Field.A11, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.B21, Field.A11, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.B11, Field.A21, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.B21, Field.A21, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.B11, Field.A31, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.B21, Field.A31, null) + "\n");
			
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.B11, Field.A01, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B21, Field.A01, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B31, Field.A01, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.B11, Field.A11, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B21, Field.A11, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B31, Field.A11, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.B11, Field.A21, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B21, Field.A21, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B31, Field.A21, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.B11, Field.A31, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B21, Field.A31, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.B31, Field.A31, null) + "\n");
			
		}else if(player.getTeam() == Game.getInstance().getTeamB()){
			string.append("SERVES: \n");
			string.append(helper_ProbHitToString(HitTechnique.S_F, Field.A11, null, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.S_F, Field.A21, null, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.S_J, Field.A11, null, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.S_J, Field.A21, null, null) + "\n");
	
			string.append("ATTTACKS: \n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.A11, Field.B01, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.A21, Field.B01, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.A11, Field.B11, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.A21, Field.B11, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.A11, Field.B21, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.A21, Field.B21, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_SM, Field.A11, Field.B31, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_SM, Field.A21, Field.B31, null) + "\n");
			

			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.A11, Field.B01, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.A21, Field.B01, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.A11, Field.B11, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.A21, Field.B11, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.A11, Field.B21, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.A21, Field.B21, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_E, Field.A11, Field.B31, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_E, Field.A21, Field.B31, null) + "\n");
			
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.A11, Field.B01, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A21, Field.B01, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A31, Field.B01, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.A11, Field.B11, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A21, Field.B11, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A31, Field.B11, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.A11, Field.B21, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A21, Field.B21, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A31, Field.B21, null) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.F_P, Field.A11, Field.B31, null)  + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A21, Field.B31, null) + "\t"
					+ helper_ProbHitToString(HitTechnique.F_P, Field.A31, Field.B31, null) + "\n");
		}else{
			System.err.println("Team of Player " + player.toString() + " does not belong to the game!");
		}
			
			
			string.append("DEFENCE:\n");
			string.append(helper_ProbHitToString(HitTechnique.d, null, null, BallProperty.NORMAL) + "\t"
					+ helper_ProbHitToString(HitTechnique.d, null, null, BallProperty.HARD) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.d_m, null, null, BallProperty.NORMAL) + "\t"
					+ helper_ProbHitToString(HitTechnique.d_m, null, null, BallProperty.HARD) + "\n");
			
			string.append("RECEIVE:\n");
			string.append(helper_ProbHitToString(HitTechnique.r, null, null, BallProperty.NORMAL) + "\t"
					+ helper_ProbHitToString(HitTechnique.r, null, null, BallProperty.HARD) + "\n");
			string.append(helper_ProbHitToString(HitTechnique.r_m, null, null, BallProperty.NORMAL) + "\t"
					+ helper_ProbHitToString(HitTechnique.r_m, null, null, BallProperty.HARD) + "\n");
			
			string.append("SET:\n");
			string.append(helper_ProbHitToString(HitTechnique.s, null, null, null) + "\n");
			
			string.append("BLOCK:\n");
			string.append(this.getProbBlock() + "\n");
	
		return string.toString();
	}	

	/**
	 * Prints IndPlayer Probs as in latex Format for Paper needed
	 * @return String with formatted Probs
	 */
	public String printAnalogToLatexFormat(Player player){	
		StringBuffer string = new StringBuffer();
		
		string.append(player.getName() + "\t" + player.getTeam().name 
				+ "\n"
				+ player.getDataFile()
				+ "\n\n"); // + "\t blocking Player " + this.isBlockingPlayer + "\n \n");
		
		if (player.getTeam() == Game.getInstance().getTeamA()){
			string.append("\\begin{table}[ht]\n"
					+ "\\scriptsize\n"
					+ "\\captionsetup{font=scriptsize}"
					+ "\\begin{center}\n"
					+ "\\begin{tabular}{ccccccccccc}\n"
					+ "\\toprule\n"
					+ "& target fields & \\multicolumn{3}{c}{\\TeamB11-\\TeamB14} & \\multicolumn{3}{c}{\\TeamB21-\\TeamB24} & \\multicolumn{3}{c}{\\TeamB31-\\TeamB34}\\\\\n"
					+ "& performance & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$\\\\\n"
					+ "\\midrule\n"
					+ "\\multicolumn{11}{c}{Serve}\\\\\n"
					+ "\\midrule\n"
					+ "$S_F$ & \\multirow{2}{*}{\\TeamA01 - \\TeamA04}"
			 		+ helper_ProbHitToLatexString(HitTechnique.S_F, Field.B11, null, null) 
					+ helper_ProbHitToLatexString(HitTechnique.S_F, Field.B21, null, null) 
					+ "& - & - & -\\\\ \n"
					+ "$S_J$ & " 
					+ helper_ProbHitToLatexString(HitTechnique.S_J, Field.B11, null, null)
					+ helper_ProbHitToLatexString(HitTechnique.S_J, Field.B21, null, null) 
					+ "& - & - & -\\\\ \n");
	
			string.append("\\midrule\n"
					+ "\\multicolumn{11}{c}{Attack-Hit}\\\\\n"
					+ "\\midrule\n");
			string.append("\\multirow{4}{*}{$\\Smash$}& out "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B11, Field.A01, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B21, Field.A01, null) 
					+ "& - & - & -\\\\ \n"
			        + "& \\TeamA11-\\TeamA14 "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B11, Field.A11, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B21, Field.A11, null) 
					+ "& - & - & -\\\\ \n"
					+ "& \\TeamA21-\\TeamA24 "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B11, Field.A21, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B21, Field.A21, null) 
					+ "& - & - & -\\\\ \n"
					+ "& \\TeamA31-\\TeamA34 "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B11, Field.A31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.B21, Field.A31, null) 
					+ "& - & - & -\\\\ \n");
			
			string.append("\\midrule\n"
					+ "\\multirow{4}{*}{$\\EmergencyShot$}& out "
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B11, Field.A01, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B21, Field.A01, null) 
					+ "& - & - & -\\\\ \n"
					+ "& \\TeamA11-\\TeamA14 "
			 		+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B11, Field.A11, null)  
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B21, Field.A11, null) 
					+ "& - & - & -\\\\ \n"
					+ "& \\TeamA21-\\TeamA24 "
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B11, Field.A21, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B21, Field.A21, null) 
					+ "& - & - & -\\\\ \n"
					+ "& \\TeamA31-\\TeamA34 "
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B11, Field.A31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.B21, Field.A31, null) 
					+ "& - & - & -\\\\ \n");
			
			string.append("\\midrule\n"
					+ "\\multirow{4}{*}{$\\PlannedShot$}& out"
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B11, Field.A01, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B21, Field.A01, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B31, Field.A01, null) 
					+ "\\\\ \n"
					+ "& \\TeamA11-\\TeamA14 "
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B11, Field.A11, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B21, Field.A11, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B31, Field.A11, null) 
					+ "\\\\ \n"
					+ "& \\TeamA21-\\TeamA24 "
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B11, Field.A21, null)  
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B21, Field.A21, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B31, Field.A21, null) 
					+ "\\\\ \n"
					+ "& \\TeamA31-\\TeamA34 "
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B11, Field.A31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B21, Field.A31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.B31, Field.A31, null)
					+ "\\\\ \n"
					+ "\\bottomrule\n"
					+ "\\end{tabular}\n"
					+ "\\end{center}\n"
					+ "\\caption{Input data from all matches except final: "+ player.getName() +  " -- Serves and Attack-Hits}\n"
					+ "\\label{tab:playerInputDataAttacks-"+player.getName().toLowerCase()+ "-mod}\n"
					+ "\\end{table}\n");
			
		}else if(player.getTeam() == Game.getInstance().getTeamB()){
			string.append("\\begin{table}[ht]\n"
					+ "\\scriptsize\n"
					+ "\\captionsetup{font=scriptsize}"
					+ "\\begin{center}\n"
					+ "\\begin{tabular}{ccccccccccc}\n"
					+ "\\toprule\n"
					+ "& target fields & \\multicolumn{3}{c}{\\TeamA11-\\TeamA14} & \\multicolumn{3}{c}{\\TeamA21-\\TeamA24} & \\multicolumn{3}{c}{\\TeamA31-\\TeamA34}\\\\\n"
					+ "& performance & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$\\\\\n"
					+ "\\midrule\n"
					+ "\\multicolumn{11}{c}{Serve}\\\\\n"
					+ "\\midrule\n"
					+ "$S_F$ & \\multirow{2}{*}{\\TeamB01 - \\TeamB04} "
			 		+ helper_ProbHitToLatexString(HitTechnique.S_F, Field.A11, null, null) 
					+ helper_ProbHitToLatexString(HitTechnique.S_F, Field.A21, null, null) 
					+ "& - & - & -\\\\\n"
					+ "$S_J$ & " 
					+ helper_ProbHitToLatexString(HitTechnique.S_J, Field.A11, null, null)
					+ helper_ProbHitToLatexString(HitTechnique.S_J, Field.A21, null, null) 
					+ "& - & - & -\\\\\n");
	
			string.append("\\midrule\n"
					+ "\\multicolumn{11}{c}{Attack-Hit}\\\\\n"
					+ "\\midrule\n");
			string.append("\\multirow{4}{*}{$\\Smash$}& out "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A11, Field.B01, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A21, Field.B01, null) 
					+ "& - & - & -\\\\\n"
			        + "& \\TeamB11-\\TeamB14 "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A11, Field.B11, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A21, Field.B11, null) 
					+ "& - & - & -\\\\\n"
					+ "& \\TeamB21-\\TeamB24 "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A11, Field.B21, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A21, Field.B21, null) 
					+ "& - & - & -\\\\\n"
					+ "& \\TeamB31-\\TeamB34 "
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A11, Field.B31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_SM, Field.A21, Field.B31, null) 
					+ "& - & - & -\\\\\n");
			
			string.append("\\midrule\n"
					+ "\\multirow{4}{*}{$\\EmergencyShot$}& out "
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A11, Field.B01, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A21, Field.B01, null) 
					+ "& - & - & -\\\\\n"
					+ "& \\TeamB11-\\TeamB14 "
			 		+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A11, Field.B11, null)  
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A21, Field.B11, null) 
					+ "& - & - & -\\\\\n"
					+ "& \\TeamB21-\\TeamB24 "
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A11, Field.B21, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A21, Field.B21, null) 
					+ "& - & - & -\\\\\n"
					+ "& \\TeamB31-\\TeamB34 "
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A11, Field.B31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_E, Field.A21, Field.B31, null) 
					+ "& - & - & -\\\\\n");
			
			string.append("\\midrule\n"
					+ "\\multirow{4}{*}{$\\PlannedShot$}& out"
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A11, Field.B01, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A21, Field.B01, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A31, Field.B01, null) 
					+ "\\\\\n"
					+ "& \\TeamB11-\\TeamB14 "
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A11, Field.B11, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A21, Field.B11, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A31, Field.B11, null) 
					+ "\\\\\n"
					+ "& \\TeamB21-\\TeamB24 "
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A11, Field.B21, null)  
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A21, Field.B21, null) 
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A31, Field.B21, null) 
					+ "\\\\\n"
					+ "& \\TeamB31-\\TeamB34 "
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A11, Field.B31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A21, Field.B31, null)
					+ helper_ProbHitToLatexString(HitTechnique.F_P, Field.A31, Field.B31, null)
					+ "\\\\\n"
					+ "\\bottomrule\n"
					+ "\\end{tabular}\n"
					+ "\\end{center}\n"
					+ "\\caption{Input data from all matches except final: "+ player.getName() + " ["+ player.getTeam().name +"]" + " -- Serves and Attack-Hits}\n"
					+ "\\label{tab:playerInputDataAttacks-"+player.getName().toLowerCase()+ "-mod}\n"
					+ "\\end{table}\n");
		}else{
			System.err.println("Team of Player " + player.toString() + " does not belong to the game!");
		}
			
		string.append("\n\n");
			string.append("\\begin{table}[ht]\n"
					+"\\captionsetup{font=scriptsize}\n"
					+"\\scriptsize\n"
					+"\\begin{center}\n"
					+"\\begin{tabular}{cccccccc}\n"
					+"\\toprule& attack strength & \\multicolumn{3}{c}{\\normal} & \\multicolumn{3}{c}{\\hard}\\\\\n"
					+"\\midrule\n"
					+"& performance & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$\\\\\n");
			string.append("\\toprule\n"
					+ "\\multirow{2}{*}{Defence} & $d$ "
					+ helper_ProbHitToLatexString(HitTechnique.d, null, null, BallProperty.NORMAL)
					+ helper_ProbHitToLatexString(HitTechnique.d, null, null, BallProperty.HARD)
					+ "\\\\\n "
					+ "& $d_m$ "
					+ helper_ProbHitToLatexString(HitTechnique.d_m, null, null, BallProperty.NORMAL)
					+ helper_ProbHitToLatexString(HitTechnique.d_m, null, null, BallProperty.HARD) 
					+ "\\\\\n");
			string.append("\\midrule\n"
					+ "\\multirow{2}{*}{Reception} & $r$ "
					+ helper_ProbHitToLatexString(HitTechnique.r, null, null, BallProperty.NORMAL)
					+ helper_ProbHitToLatexString(HitTechnique.r, null, null, BallProperty.HARD) 
					+ "\\\\ \n "
					+ "& $r_m$ "
					+ helper_ProbHitToLatexString(HitTechnique.r_m, null, null, BallProperty.NORMAL)
					+ helper_ProbHitToLatexString(HitTechnique.r_m, null, null, BallProperty.HARD)
					+ "\\\\\n");
			string.append("\\midrule\n"
					+ "Set & $s$ "
					+ helper_ProbHitToLatexString(HitTechnique.s, null, null, null)
					+ "& - & - & - \\\\ \n"
					+ "\\midrule\n");
			string.append("& performance & \\# & \\goodBlockDesc & \\okBlockDesc & \\badBlockDesc & \\noBlockDesc & \\\\\n"
					+"\\midrule\n"
					+this.getProbBlock().toStringLatexFormat() + "\\\\\n"
					+"\\bottomrule\n"
					+"\\end{tabular}\n"
					+"\\end{center}\n"
					+"\\caption{Input data from all matches except final: "+ player.getName() + " ["+ player.getTeam().name +"]" +" -- Defence, Reception, Set, Block}\n"
					+"\\label{tab:playerInputDataDefRecSet-"+player.getName().toLowerCase()+"-mod}\n"
					+"\\end{table}\n");
	
		return string.toString();
	}
	
	private String helper_ProbHitToString(HitTechnique tech, Field targetField, Field playerField, BallProperty prob){
			if (this.getProbHit(tech, targetField, playerField, prob)==null){
				if (targetField != null && playerField != null){
					return tech+"^"+targetField.toString()+"_"+playerField.toString()+ "=[-, -, -]";
				}
				if (targetField != null){
					return tech+"^"+targetField.toString()+ "=[-, -, -]";
				}
				if (prob != null){
					return tech + "^"+prob.toString() + "=[-, -, -]";
				}
				return tech + "=[-, -, -]";
			}else{
				return this.getProbHit(tech, targetField, playerField, prob).toStringModified();
			}	
	}

	private String helper_ProbHitToLatexString(HitTechnique tech, Field targetField, Field playerField, BallProperty prob){
			return this.getProbHit(tech, targetField, playerField, prob).toStringLatex();
	}
	
}
