package de.ubt.wm.sb.probs;

import java.util.Arrays;

import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.sim.MainSim;

public class Prob_Serve extends Prob_Hit{
	
	public static double border_factor = 1.7;
	public static double nonborder_dev_factor = 1.3;
	
	private Field targetField;

	public Prob_Serve(HitTechnique tech, int observations, Prob_Hit pred, double succProb, double faultProb, Field field) {
		super(tech, observations, pred, succProb, faultProb);
		this.targetField = field;
		
		if (MainSim.devFromRealMatch && observations > 0){
			if(Arrays.asList(Field.BORDER_LEFT_RIGHT_A).contains(field) || Arrays.asList(Field.BORDER_LEFT_RIGHT_B).contains(field)){
				double newDevProb = Math.min(1, 2*(1-succProb-faultProb));
				double newSuccProb = Math.max(0, succProb-newDevProb/2);
				this.setAllThreeProbs(newSuccProb, faultProb);
			}
		}
		if (MainSim.modServes && observations > 0){
			if(Arrays.asList(Field.BORDER_LEFT_RIGHT_A).contains(field) || Arrays.asList(Field.BORDER_LEFT_RIGHT_B).contains(field)){
				double oldDevProb = 1 - succProb - faultProb;
				double newFaultProb = Math.min(1, faultProb + border_factor*oldDevProb);
				double newSuccProb = Math.max(0, succProb - (border_factor-1)*oldDevProb);
				this.setAllThreeProbs(newSuccProb, newFaultProb);
			}else{
				double oldDevProb = 1 - succProb - faultProb;
				double newDevProb = Math.min(1, nonborder_dev_factor*oldDevProb);
				double newSuccProb = Math.max(0, succProb-(newDevProb-oldDevProb));
				this.setAllThreeProbs(newSuccProb, faultProb);
			}
		}
	}

	public Field getTargetField() {
		return targetField;
	}

	public void setTargetField(Field targetField) {
		this.targetField = targetField;
	}

	@Override
	public String toString() {
		return super.getTech() + "^"+targetField.toString()+ "=["+ super.getNumberOfObservations() + ", " + super.getSuccProb() + ", " + super.getFaultProb() + ", " + super.getNeighbourProb() + "]";
	}
	
	public String printName() {
		return super.getTech() + "^" + targetField;
	}
}
