package de.ubt.wm.sb.sim;

import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Locale;

import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import jxl.write.biff.RowsExceededException;

public class CalibrationSeries {

	private String skillDesc;
	private String[] skillFiles;
	private int numberOfSets;
	
	private static double[] theTruthP = new double[]{0.0185, 0.037, 0.9445, 0.4722, 0.2407, 0.2871};
	private static double[] theTruthQ = new double[]{0.0351, 0.1404, 0.8245, 0.5200, 0.2105, 0.2695};
	
	private HashMap<Integer, double[]> computedPValues;
	private HashMap<Integer, double[]> computedQValues;
	
	private HashMap<Integer, Double> deviationPValues;
	private HashMap<Integer, Double> deviationQValues;
	
	public CalibrationSeries(String skillDesc, String[] skillFiles, int n) {
		super();
		this.skillDesc = skillDesc;
		this.skillFiles = skillFiles;
		this.computedPValues = new HashMap<Integer, double[]>();
		this.computedQValues = new HashMap<Integer, double[]>();
		this.numberOfSets = n;
	}

	public static double[] getTheTruthP() {
		return theTruthP;
	}

	public static void setTheTruthP(double[] theTruthP) {
		CalibrationSeries.theTruthP = theTruthP;
	}

	public static double[] getTheTruthQ() {
		return theTruthQ;
	}

	public static void setTheTruthQ(double[] theTruthQ) {
		CalibrationSeries.theTruthQ = theTruthQ;
	}
			
	public HashMap<Integer, double[]> getComputedPValues() {
		return computedPValues;
	}

	public void setComputedPValues(HashMap<Integer, double[]> computedPValues) {
		this.computedPValues = computedPValues;
	}

	public HashMap<Integer, double[]> getComputedQValues() {
		return computedQValues;
	}

	public void setComputedQValues(HashMap<Integer, double[]> computedQValues) {
		this.computedQValues = computedQValues;
	}

	public HashMap<Integer, Double> getDeviationPValues() {
		return deviationPValues;
	}

	public void setDeviationPValues(HashMap<Integer, Double> deviationPValues) {
		this.deviationPValues = deviationPValues;
	}

	public HashMap<Integer, Double> getDeviationQValues() {
		return deviationQValues;
	}

	public void setDeviationQValues(HashMap<Integer, Double> deviationQValues) {
		this.deviationQValues = deviationQValues;
	}

	public void addComputedPValues(int k, double[] pValues){
		this.computedPValues.put(k, pValues);
	}

	public void addComputedQValues(int k, double[] qValues){
		this.computedQValues.put(k, qValues);
	}
	
	public void computeDeviation(){
		deviationPValues = new HashMap<Integer, Double>();
		for (Integer k : this.computedPValues.keySet()){
			double dev = 0.0;
			for (int i = 0; i < 6; i++){
				double diff = theTruthP[i]-computedPValues.get(k)[i];
				dev += Math.abs(diff);
			}
			deviationPValues.put(k, dev/6);
		}
		
		deviationQValues = new HashMap<Integer, Double>();
		for (Integer k : this.computedPValues.keySet()){
			double dev = 0.0;
			for (int i = 0; i < 6; i++){
				double diff = theTruthQ[i]-computedQValues.get(k)[i];
				dev += Math.abs(diff);
			}
			deviationQValues.put(k, dev/6);
		}
	}

	public void writeToExcelFile(){
	
		Calendar calendar = Calendar.getInstance();
		Timestamp currentTimestamp = new java.sql.Timestamp(calendar.getTime().getTime());

		File file = new File("calibrationSeries/caliSeries_"+ skillDesc + "_" + currentTimestamp+".xls");
	    WorkbookSettings wbSettings = new WorkbookSettings();

	    wbSettings.setLocale(new Locale("en", "EN"));

	    WritableWorkbook workbook;
		try {
			workbook = Workbook.createWorkbook(file, wbSettings);
			workbook.createSheet("Overview", 0);
		    WritableSheet excelSheetOverview = workbook.getSheet(0);
		    excelSheetOverview.addCell(new Label(0,2,"Skills:"));
		    for (int k = 0; k < skillFiles.length; k++){
		    	excelSheetOverview.addCell(new Label(1,2+k,skillFiles[k]));
		    }
		    
		    excelSheetOverview.addCell(new Label(0,6,"n:"));
		    excelSheetOverview.addCell(new Number(1,6,numberOfSets));
		    excelSheetOverview.addCell(new Label(0,7,"Seed:"));
		    excelSheetOverview.addCell(new Number(1,7,MainSim.l));
		    
		    int iO = 1;
		    excelSheetOverview.addCell(new Label(0,iO+6,"devToOutsideFieldOnly:"));
		    excelSheetOverview.addCell(new Label(1,iO+6,MainSim.M1_devToOutsideFieldOnlyForServesAndAttacks + ""));
		    excelSheetOverview.addCell(new Label(0,iO+7,"devFromRealMatch:"));
		    excelSheetOverview.addCell(new Label(1,iO+7,MainSim.devFromRealMatch + ""));
		    excelSheetOverview.addCell(new Label(0,iO+8,"Modify Serves:"));
		    excelSheetOverview.addCell(new Label(1,iO+8,MainSim.modServes + ""));
		    excelSheetOverview.addCell(new Label(0,iO+9,"Modify Sets:"));
		    excelSheetOverview.addCell(new Label(1,iO+9,MainSim.modSet + ""));
		    excelSheetOverview.addCell(new Label(0,iO+10,"Modify Defs:"));
		    excelSheetOverview.addCell(new Label(1,iO+10,MainSim.modDef + ""));
		    excelSheetOverview.addCell(new Label(0,iO+11,"Modify Attacks:"));
		    excelSheetOverview.addCell(new Label(1,iO+11,MainSim.modAttacks + ""));
		    // TODO hier Werte der Mods rein
		    
			{
			    workbook.createSheet("Team A", 1);
			    WritableSheet excelSheet = workbook.getSheet(1);
		    
			    int i = 0;
			    excelSheet.addCell(new Label(1,i,"p^serve"));
			    excelSheet.addCell(new Label(2,i,"p_quer^serve"));
			    excelSheet.addCell(new Label(3,i,"p_hat^serve"));
			    excelSheet.addCell(new Label(4,i,"p^field"));
			    excelSheet.addCell(new Label(5,i,"p_quer^field"));
			    excelSheet.addCell(new Label(6,i,"p_hat^field"));
			    excelSheet.addCell(new Label(8,i,"Dev"));
			    i++;
			    excelSheet.addCell(new Label(0,i,"the Truth"));
			    for (int j = 0; j < 6; j++){
			    	excelSheet.addCell(new Number(j+1,i,theTruthP[j]));
			    }
			    i++;
			    for (Integer k : computedPValues.keySet()){
			    	excelSheet.addCell(new Label(0,i,"k = " + k));
			    	for (int j = 0; j < 6; j++){
			    		excelSheet.addCell(new Number(j+1,i,computedPValues.get(k)[j]));
			    	}
			    	excelSheet.addCell(new Number(8,i,deviationPValues.get(k)));
			    	i++;
			    }
			}
		    
			{
			    workbook.createSheet("Team B", 2);
			    WritableSheet excelSheet = workbook.getSheet(2);
		    
			    int i = 0;
			    excelSheet.addCell(new Label(1,i,"q^serve"));
			    excelSheet.addCell(new Label(2,i,"q_quer^serve"));
			    excelSheet.addCell(new Label(3,i,"q_hat^serve"));
			    excelSheet.addCell(new Label(4,i,"q^field"));
			    excelSheet.addCell(new Label(5,i,"q_quer^field"));
			    excelSheet.addCell(new Label(6,i,"q_hat^field"));
			    excelSheet.addCell(new Label(8,i,"Dev"));
			    i++;
			    excelSheet.addCell(new Label(0,i,"the Truth"));
			    for (int j = 0; j < 6; j++){
			    	excelSheet.addCell(new Number(j+1,i,theTruthQ[j]));
			    }
			    i++;
			    for (Integer k : computedQValues.keySet()){
			    	excelSheet.addCell(new Label(0,i,"k = " + k));
			    	for (int j = 0; j < 6; j++){
			    		excelSheet.addCell(new Number(j+1,i,computedQValues.get(k)[j]));
			    	}
			    	excelSheet.addCell(new Number(8,i,deviationQValues.get(k)));
			    	i++;
			    }
			}
		    workbook.write();
		    workbook.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RowsExceededException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WriteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
