package de.ubt.wm.sb.sim;

import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Random;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.exceptions.StateResetException;
import de.ubt.wm.sb.gui.AttackType;
import de.ubt.wm.sb.gui.SimSwingWorker;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.strategy.RealStrat;
import de.ubt.wm.sb.model.strategy.Strategy;
import de.ubt.wm.sb.probs.ProbType;
import de.ubt.wm.sb.probs.Prob_Attack;
import de.ubt.wm.sb.probs.Prob_Hit;
import de.ubt.wm.sb.probs.Prob_IndPlayer;
import jxl.JXLException;
import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.read.biff.BiffException;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;


public class MainSim {

	static final Logger log4j = LogManager.getLogger(MainSim.class.getName());
	
	public static int numBlocksA2;
	public static int numSuccBlocksA2;
	
	public static long l = 1521562573648L;
//	public static long l = System.currentTimeMillis();
	public static Random r = new Random(l);
	public static int calls = 0;
	
	public static boolean M1_devToOutsideFieldOnlyForServesAndAttacks = true;
	
	// hat sich nicht durchgesetzt... da Wahrscheinlichkeiten manipuliert werden
	public static boolean devFromRealMatch = false;
	public static boolean modServes = false;
	public static boolean modAttacks = false;
	public static boolean modSet = false;
	public static boolean modDef = false;
	
	public static double countRandomCalls(){
		calls++;
		double res = r.nextDouble();
		log4j.debug("r = " + res);
//		System.out.println(res);
		return res;
	}
	
	public static final String[][] skillDataFiles = MPC_TableRuns_Part1.skillDataFiles;
	public static String[] skillDescription = MPC_TableRuns_Part1.skillDescription;
	
	public static void main(String[] args) throws BiffException, IOException {
		System.out.println("Seed: " + l);
		// fuer die Ausgabe, dass Dezimalzahlen mit Punkt getrennt werden
		Locale.setDefault(Locale.US);

		System.out.println("------------------------------------------------");
		System.out.println("loaded Strategies from *.gdata.csv files:");
		System.out.println(Strategy.cali_allMatchesExceptFinal_TeamA_read);
		System.out.println(Strategy.cali_allMatchesExceptFinal_TeamB_read);
		System.out.println(Strategy.cali_allMatches_TeamA_read);
		System.out.println(Strategy.cali_allMatches_TeamB_read);
		System.out.println(Strategy.cali_finalMatch_TeamA_read);
		System.out.println(Strategy.cali_finalMatch_TeamB_read);
		System.out.println("------------------------------------------------");
		System.out.println("real direct point Probs of final:");
		String s = "Team A: [p^serve, pquer^serve, p^field, pquer^field] = ";
		s+= "[";
		for (double d : MPC_TableRuns_Part1.real_sprobs_final_A){
			s+= String.format("%.2f", d*100) + "\\%, ";
		}
		s=(String) s.subSequence(0, s.length()-2);
		s+= "]";
		System.out.println(s);
		s = "Team B: [q^serve, qquer^serve, q^field, qquer^field] = ";
		s+= "[";
		for (double d :  MPC_TableRuns_Part1.real_sprobs_final_B){
			s+= String.format("%.2f", d*100) + "\\%, ";
		}
		s=(String) s.subSequence(0, s.length()-2);
		s+= "]";
		System.out.println(s);
		System.out.println("------------------------------------------------");
		System.out.println("Detected skill description: " + Arrays.toString(skillDescription));
		System.out.println("------------------------------------------------");
		
		/**
		 * Routines for BV-SSO Paper
		 */
//		calibrationRun();
//		printProbsForPaper();
		
//		recommendationTableRun();
		setUpSimulationSeries();
//		try {
//			largeRecommendationTableRun();
//		} catch (JXLException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
		
		// fuer SSSC, Sensitivitätsanalyse
//		setUpSimulationSeries("data/allMatchesExceptFinal/GER_Brink_InputData_ExceptFinal.xls","data/allMatchesExceptFinal/GER_Reckermann_InputData_ExceptFinal.xls",
//		"data/allMatchesExceptFinal/BRA_Alison_InputData_ExceptFinal.xls","data/allMatchesExceptFinal/BRA_Emanuel_InputData_ExceptFinal.xls");
		
		/**
		 * nicht sortierte Sachen
		 * 
		 */		
//		Game.getInstance().initGameFromEvalCSV("Germany", "Brink", "Reckerman","data/GER_allMatchesExceptFinal.gprobs.csv", Strategy.cali_allMatchesExceptFinal_TeamA,
//				"Brasilien", "Alison", "Emanuel", "data/GER_allMatchesExceptFinal.gprobs.csv", Strategy.cali_allMatchesExceptFinal_TeamB);
//		System.out.println(Game.getInstance().getTeamA().getPlayer1().printIndProbs());
//		System.out.println(Game.getInstance().getTeamA().getPlayer2().printIndProbs());
//		System.out.println(Game.getInstance().getTeamB().getPlayer1().printIndProbs());
//		System.out.println(Game.getInstance().getTeamB().getPlayer2().printIndProbs());
		
//		Game.getInstance().initGame("Germany", "Brink", "Reckerman",skillDataFiles[0][0],skillDataFiles[0][1], Strategy.cali_allMatchesExceptFinal_TeamA,
//				"Brasilien", "Alison", "Emanuel", skillDataFiles[0][2], skillDataFiles[0][3], Strategy.cali_allMatchesExceptFinal_TeamB);
//		Player A2 = Game.getInstance().getTeamA().getPlayer2();
//		Prob_IndPlayer indProbsA2 = new Prob_IndPlayer();
//		A2.setDataFile("data/1521722882950allMatchesExceptFinal.eval.csv");
//		indProbsA2.readFrom_BVEVAL_CSV(A2, false);
//		indProbsA2.computeAggregatedProbs(A2);
//		indProbsA2.computeModifiedProbs();
//		System.out.println(indProbsA2.printAnalogToLatexFormat(A2));
		
//		System.out.println(Game.getInstance().getTeamA().getAggregatedSkillsForSSC());
		
//		System.out.println(Game.getInstance().getTeamA().getPlayer1().printIndProbs());
//		game.getTeamA().getPlayer1().readCalibrationData("data/allMatches/GER_Brink_InputData_allMatches.xls");
//		System.out.println(game.getTeamA().getPlayer1().prob_FSH_MIDDLE_ROW);
//		
//		System.out.println(game.getTeamA().getPlayer1().printIndProbs());
//		System.out.println(game.getTeamA().getPlayer1().printIndProbs());
		
//		game.getTeamA().getPlayer1().getIndProbs().computeAggregatedProbs(game.getTeamA().getPlayer1());
		
//		System.out.println(game.getTeamB().getPlayer2().printIndProbs());
	
		
//		SimulationManager sim = startSim(100, null);
//		System.out.println(sim.printResultsSummary());
	
//		System.out.println(MainSim.l);
//		MainSim.r =  new Random(l);
//		

		
//		int n = Integer.parseInt(args[0]);
//		int minNumber = Integer.parseInt(args[1]);
//		int maxNumber = Integer.parseInt(args[2]);

		
//		MainSim.r.setSeed(l);
//		modProbWeightsSeries(n, 2, 4);
	}
	
	/**
	 * gibt einfach nur die eingelesenen Skills im Latex Tabellenformat aus
	 */
	public static void printProbsForPaper(){
		for (int i = 0; i <skillDataFiles.length; i++){
			Game.getInstance().initGameFromEvalCSV("Germany", "Brink", "Reckerman",skillDataFiles[i][0], Strategy.cali_allMatchesExceptFinal_TeamA_read,
					"Brasilien", "Alison", "Emanuel", skillDataFiles[i][1], Strategy.cali_allMatchesExceptFinal_TeamB_read);
			System.out.println(Game.getInstance().getTeamA().getPlayer1().printIndProbs());
			System.out.println(Game.getInstance().getTeamA().getPlayer2().printIndProbs());
			System.out.println(Game.getInstance().getTeamB().getPlayer1().printIndProbs());
			System.out.println(Game.getInstance().getTeamB().getPlayer2().printIndProbs());
		}
	}
	
	/**
	 * validiert die aus dem g-MDP simulierten s-Probs
	 * simuliert Finale mit Skills basierend auf allen drei Varianten
	 * vergleicht anschließend mit echten sProbs und gibt Fehler aus
	 */
	public static void validationRun(){
		
		double[][] serveProbs = new double[3][4];
		double[][] fieldProbs = new double[3][4];
		
		for (int i=0; i<skillDataFiles.length; i++){
			Game.getInstance().initGameFromEvalCSV("Germany", "Brink", "Reckerman", skillDataFiles[i][0], Strategy.cali_finalMatch_TeamA_read,
					"Brasilien", "Alison", "Emanuel", skillDataFiles[i][1], Strategy.cali_finalMatch_TeamB_read);
			SimulationManager sim_allMatchesExceptFinal = startSim(1000, false, 100, false, null, skillDescription[i]);
//			printSimManagerForCalibrationRun(sim_allMatchesExceptFinal);
			sim_allMatchesExceptFinal.writeResultsToFile("calibrationRun_"+skillDescription[i]);
			
			serveProbs[i][0] = sim_allMatchesExceptFinal.getServeP();
			serveProbs[i][1] = sim_allMatchesExceptFinal.getServePOverline();
			serveProbs[i][2] = sim_allMatchesExceptFinal.getServeQ();
			serveProbs[i][3] = sim_allMatchesExceptFinal.getServeQOverline();
			
			fieldProbs[i][0] = sim_allMatchesExceptFinal.getfAP();
			fieldProbs[i][1] = sim_allMatchesExceptFinal.getfAPOverline();
			fieldProbs[i][2] = sim_allMatchesExceptFinal.getfAQ();
			fieldProbs[i][3] = sim_allMatchesExceptFinal.getfAQOverline();
		}
		
		printSimManagerForValidationRun(serveProbs, fieldProbs, skillDescription);
		
	}
	
	private static void printSimManagerForValidationRun(double[][] serveProbs, double[][] fieldProbs, String[] skillDescription){			
		StringBuffer s = new StringBuffer();
		double[] realServeProbsOfFinal = new double[]{MPC_TableRuns_Part1.real_sprobs_final_A[0],MPC_TableRuns_Part1.real_sprobs_final_A[1],
				MPC_TableRuns_Part1.real_sprobs_final_B[0],MPC_TableRuns_Part1.real_sprobs_final_B[1]};
		double[] realFieldProbsOfFinal = new double[]{MPC_TableRuns_Part1.real_sprobs_final_A[2],MPC_TableRuns_Part1.real_sprobs_final_A[3],
				MPC_TableRuns_Part1.real_sprobs_final_B[2],MPC_TableRuns_Part1.real_sprobs_final_B[3]};
		
		for (int i = 0; i < serveProbs.length; i++){
			s.append(skillDescription[i]);
			
			for (int j = 0; j < serveProbs[i].length; j++){
				s.append(" & $");
				s.append(String.format("%.2f", serveProbs[i][j]*100));
				s.append("\\%$");
			}
			s.append(" & $");
			s.append(String.format("%.4f",computeL1Error(realServeProbsOfFinal, serveProbs[i])));
			s.append("$");
			s.append("\\\\\n");
		}
		s.append("------------------------------------\n");
		for (int i = 0; i < fieldProbs.length; i++){
			s.append(skillDescription[i]);
			
			for (int j = 0; j < fieldProbs[i].length; j++){
				s.append(" & $");
				s.append(String.format("%.2f", fieldProbs[i][j]*100));
				s.append("\\%$");
			}
			s.append(" & $");
			s.append(String.format("%.4f",computeL1Error(realFieldProbsOfFinal, fieldProbs[i])));
			s.append("$");
			s.append("\\\\\n");
		}
		
		System.out.println(s.toString());
	}
	
	public static double computeL1Error(double[] first, double[] second){
		double result = 0;
		for (int i = 0; i < first.length; i++){
			result = result + Math.abs(first[i]-second[i]);
		}
		return result/first.length;
	}
	
	@SuppressWarnings("unused")
	private static void printSimManagerForCalibrationRun(SimulationManager sim){
		System.out.println("Skills: " + sim.skillsDescr);
		System.out.println("Team A:\n"
				+ " Strategie: " + Game.getInstance().getTeamA().getStrat().toStringShort());
		System.out.println(" p^serve\t p_quer^serve\t p^field\t p_quer^field");
		System.out.println(" "+ String.format("%.4f", sim.getServeP()) + "       \t " + String.format("%.4f",sim.getServePOverline()) + "   \t " 
							  + String.format("%.4f",sim.getfAP()) + "     \t " + String.format("%.4f",sim.getfAPOverline()) + ";");
		System.out.println("Team B;\n"
				+ " Strategie: " + Game.getInstance().getTeamB().getStrat().toStringShort());
		System.out.println(" q^serve\t q_quer^serve\t q^field\t q_quer^field");
		System.out.println(" "+String.format("%.4f", sim.getServeQ()) + "       \t " + String.format("%.4f", sim.getServeQOverline()) + "       \t " 
							+ String.format("%.4f", sim.getfAQ()) + "       \t " + String.format("%.4f", sim.getfAQOverline()) + ";");
	}
	
	public static void largeRecommendationTableRun() throws IOException, JXLException {
			Game game = Game.getInstance();			
			
			// all möglichen extremen Strategien erzeugen
			List<Strategy> allStratsA = new ArrayList<Strategy>();
			List<Strategy> allStratsB = new ArrayList<Strategy>();
			for (int pi_b = 0; pi_b<=1; pi_b++){
				for (int pi_h_serve_tech = 0; pi_h_serve_tech<=1; pi_h_serve_tech++){
					for (int pi_h_serve_field = 0; pi_h_serve_field<=1; pi_h_serve_field++){
						for (int pi_h_field_tech = 0; pi_h_field_tech<=0; pi_h_field_tech++){
							for (int pi_h_field_field = 0; pi_h_field_field<=0; pi_h_field_field++){
								// Spieler 2
								for (int pi_h_serve_tech2 = 0; pi_h_serve_tech2<=0; pi_h_serve_tech2++){
									for (int pi_h_serve_field2 = 0; pi_h_serve_field2<=0; pi_h_serve_field2++){
										for (int pi_h_field_tech2 = 0; pi_h_field_tech2<=0; pi_h_field_tech2++){
											for (int pi_h_field_field2 = 0; pi_h_field_field2<=0; pi_h_field_field2++){
												for (int pi_s = 0; pi_s<=0; pi_s++){
													allStratsA.add(new Strategy("A:"+pi_b+"|"+pi_h_serve_tech+pi_h_serve_field+pi_h_field_tech+pi_h_field_field+"|"+pi_s, true,
															pi_b, new double[][]{{pi_h_serve_tech,pi_h_serve_field,pi_h_field_tech,pi_h_field_field},{pi_h_serve_tech2,pi_h_serve_field2,pi_h_field_tech2,pi_h_field_field2}},pi_s));
													allStratsB.add(new Strategy("B:"+pi_b+"|"+pi_h_serve_tech+pi_h_serve_field+pi_h_field_tech+pi_h_field_field+"|"+pi_s, false,
															pi_b, new double[][]{{pi_h_serve_tech,pi_h_serve_field,pi_h_field_tech,pi_h_field_field},{pi_h_serve_tech2,pi_h_serve_field2,pi_h_field_tech2,pi_h_field_field2}},pi_s));												
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			
//			List<Strategy> allStratsA = Arrays.asList(
//					Strategy.prefinalRiskyA,
//					Strategy.prefinalRiskySafeA,
//					Strategy.prefinalSafeRiskyA,
//					Strategy.prefinalSafeA,
//					Strategy.cali_allMatchesExceptFinal_TeamA,
//					Strategy.cali_finalMatchBuck_TeamA);
//			
//			List<Strategy> allStratsB = Arrays.asList(
//					Strategy.prefinalRiskyB,
//					Strategy.prefinalRiskySafeB,
//					Strategy.prefinalSafeRiskyB,
//					Strategy.prefinalSafeB,
//					Strategy.cali_allMatchesExceptFinal_TeamB,
//					Strategy.cali_finalMatchBuck_TeamB);
			
			/**
			 * mit allExceptFinalSkills
			 */
			{
			String[] skillDataFiles = new String[] {
					"data/allMatchesExceptFinal/GER_Brink_InputData_ExceptFinal.xls",
					"data/allMatchesExceptFinal/GER_Reckermann_InputData_ExceptFinal.xls",
					"data/allMatchesExceptFinal/BRA_Alison_InputData_ExceptFinal.xls",
					"data/allMatchesExceptFinal/BRA_Emanuel_InputData_ExceptFinal.xls" 
			};
			
			String skillDescription = "allExceptFinal";
			List<String[]> resultDesc = new ArrayList<String[]>();
			resultDesc.add(new String[]{"skillsA", "skillsB", "stratA", "stratB"});
			List<Double[]> resultValues = new ArrayList<Double[]>();
			resultValues.add(new Double[]{0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0});
			for (Strategy stratA : allStratsA) {
				for (Strategy stratB : allStratsB) {
					game.initGame("Germany", "Brink", "Reckerman", skillDataFiles[0], skillDataFiles[1],
							stratA, "Brasilien", "Alison", "Emanuel", skillDataFiles[2], skillDataFiles[3], stratB);
				
					SimulationManager simManager = startSim(100, false, 100, false, null, skillDescription);
					// abspeichern der Ergebnisse
					String[] current_result = new String[]{skillDescription,skillDescription,
							stratA.getName(), stratB.getName()};
					resultDesc.add(current_result);
					Double[] current_values = new Double[]{simManager.getServeP(), simManager.getServePOverline(),simManager.getfAP(), simManager.getfAPOverline(),
							simManager.getServeQ(), simManager.getServeQOverline(), simManager.getfAQ(), simManager.getfAQOverline()};
					resultValues.add(current_values);
					}
			}
			
			writeResultToFile(resultDesc,resultValues);
			}
			
			
//			allStratsA = Arrays.asList(
//					Strategy.finalRiskyA,
//					Strategy.finalRiskySafeA,
//					Strategy.finalSafeRiskyA,
//					Strategy.finalSafeA,
//					Strategy.cali_allMatchesExceptFinal_TeamA,
//					Strategy.cali_finalMatchBuck_TeamA);
//			
//			allStratsB = Arrays.asList(
//					Strategy.finalRiskyB,
//					Strategy.finalRiskySafeB,
//					Strategy.finalSafeRiskyB,
//					Strategy.finalSafeB,
//					Strategy.cali_allMatchesExceptFinal_TeamB,
//					Strategy.cali_finalMatchBuck_TeamB);	
			/**
			 * das gleiche mit final Skills
			 */
			{
			String[] skillDataFilesFinal = new String[] {
				 "data/finalMatch/GER_Brink_InputData_final.xls", 
				  "data/finalMatch/GER_Reckermann_InputData_final.xls",
				  "data/finalMatch/BRA_Alison_InputData_final.xls",
				  "data/finalMatch/BRA_Emanuel_InputData_final.xls"
				};
			
			String skillDescriptionFinal = "finalMatch";
			List<String[]> resultDesc = new ArrayList<String[]>();
			resultDesc.add(new String[]{"skillsA", "skillsB", "stratA", "stratB"});
			List<Double[]> resultValues = new ArrayList<Double[]>();
			resultValues.add(new Double[]{0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0});
			
			for (Strategy stratA : allStratsA) {
				for (Strategy stratB : allStratsB) {
					game.initGame("Germany", "Brink", "Reckerman", skillDataFilesFinal[0], skillDataFilesFinal[1],
							stratA, "Brasilien", "Alison", "Emanuel", skillDataFilesFinal[2], skillDataFilesFinal[3], stratB);
				
					SimulationManager simManager = startSim(100, false, 100, false, null, skillDescriptionFinal);
					// abspeichern der Ergebnisse
					String[] current_result = new String[]{skillDescriptionFinal,skillDescriptionFinal,
							stratA.getName(), stratB.getName()};
					resultDesc.add(current_result);
					Double[] current_values = new Double[]{simManager.getServeP(), simManager.getServePOverline(),simManager.getfAP(), simManager.getfAPOverline(),
							simManager.getServeQ(), simManager.getServeQOverline(), simManager.getfAQ(), simManager.getfAQOverline()};
					resultValues.add(current_values);
					}
			}
			
			writeResultToFile(resultDesc,resultValues);
			}
			
	}
	
	public static void writeResultToFile(List<String[]> result, List<Double[]> resultValues) throws JXLException, IOException {
		Calendar calendar = Calendar.getInstance();
		Timestamp currentTimestamp = new java.sql.Timestamp(calendar.getTime().getTime());
		
		File folder = new File("largeRecommendationsTable");
		if (!folder.exists() || !folder.isDirectory()) {
			folder.mkdir();
		}
					
		File file = new File("largeRecommendationsTable/largeRecommendationsTable_"+ currentTimestamp.toString().replaceAll(" ", "_")+ ".xls");
		
		WorkbookSettings wbSettings = new WorkbookSettings();
	    wbSettings.setLocale(new Locale("en", "EN"));
	    WritableWorkbook workbook = Workbook.createWorkbook(file, wbSettings);
		
	    /*------------------------------------------------------------------------------
	     * Sheet Setting
	     * ----------------------------------------------------------------------------- 	    
	     */
	    workbook.createSheet("Results", 0);
	    WritableSheet excelSheetSettings = workbook.getSheet(0);
	    
	    for (int i=0; i<result.size(); i++){
	    	for (int j = 0; j< result.get(i).length; j++){
	    		excelSheetSettings.addCell(new Label(j,i,result.get(i)[j]));
	    	}
	    	for (int j = 0; j< resultValues.get(i).length; j++){
	    		excelSheetSettings.addCell(new Number(result.get(i).length+j,i,resultValues.get(i)[j]));
	    	}
	    }
	    
	    workbook.write();
	    workbook.close();
	}

	/**	
	 * alte Tabelle fuer 2 Personen Constant Sum Game
	 */
	public static void recommendationTableRun() {
		
		String[] resultString = new String[]{"Data Lines:\n", "Description\n"};
		
		List<Strategy> normalStratsA = Arrays.asList(
				Strategy.prefinalRiskyA,
				Strategy.prefinalRiskySafeA,
				Strategy.prefinalSafeRiskyA,
				Strategy.prefinalSafeA,
				Strategy.cali_allMatchesExceptFinal_TeamA_read,
				Strategy.cali_finalMatch_TeamA_read
				);
		
		List<Strategy> normalStratsB = Arrays.asList(
				Strategy.prefinalRiskyB,
				Strategy.prefinalRiskySafeB,
				Strategy.prefinalSafeRiskyB,
				Strategy.prefinalSafeB,
				Strategy.cali_allMatchesExceptFinal_TeamB_read,
				Strategy.cali_finalMatch_TeamB_read);
		
		for (RealStrat stratTeamA : normalStratsA) {
			for (RealStrat stratTeamB : normalStratsB) {
				{	
					if (skillDescription[0].contains("Except")){
						Game.getInstance().initGameFromEvalCSV("Germany", "Brink", "Reckerman",skillDataFiles[0][0], stratTeamA,
								"Brasilien", "Alison", "Emanuel", skillDataFiles[0][1], stratTeamB);
						SimulationManager simManager = startSim(500, false, 100, false, null, skillDescription[0]);
						resultString = simManager.printResultsSummaryShortForInputSMDP(resultString);
					}else{
						System.err.println("skill Description passt nicht zum exceptFinal recommendations run");
					}
				}
			}
		}
		
		normalStratsA = Arrays.asList(
				Strategy.finalRiskyA,
				Strategy.finalRiskySafeA,
				Strategy.finalSafeRiskyA,
				Strategy.finalSafeA,
				Strategy.cali_allMatchesExceptFinal_TeamA_read,
				Strategy.cali_finalMatch_TeamA_read);
		
		normalStratsB = Arrays.asList(
				Strategy.finalRiskyB,
				Strategy.finalRiskySafeB,
				Strategy.finalSafeRiskyB,
				Strategy.finalSafeB,
				Strategy.cali_allMatchesExceptFinal_TeamB_read,
				Strategy.cali_finalMatch_TeamB_read);
		
		for (RealStrat stratTeamA : normalStratsA) {
			for (RealStrat stratTeamB : normalStratsB) {
				{	
					if (skillDescription[2].contains("final")){
						Game.getInstance().initGameFromEvalCSV("Germany", "Brink", "Reckerman",skillDataFiles[2][0], stratTeamA,
								"Brasilien", "Alison", "Emanuel", skillDataFiles[2][1], stratTeamB);
						SimulationManager simManager = startSim(500, false, 100, false, null, skillDescription[2]);
						resultString = simManager.printResultsSummaryShortForInputSMDP(resultString);
					}else{
						System.err.println("skill Description passt nicht zum final recommendations run");
					}
				}
			}
		}

		System.out.println("Result from simple Simulation: \n");
		System.out.println(resultString[0]);
		System.out.println(resultString[1]);
	}
	
	/**
	 * wird nicht mehr gebraucht?
	 * @param n
	 * @param minNumber
	 * @param maxNumber
	 */
	public static void modProbWeightsSeries(int n, int minNumber, int maxNumber) {
		Game game = Game.getInstance();
		
		for (int i = 2; i < skillDataFiles.length; i++) {
			Prob_Hit.minObs = 11;
			ProbModificationSeries probSeries = new ProbModificationSeries(skillDescription[i], skillDataFiles[i], n);
			for (int l_set_succ = minNumber; l_set_succ < maxNumber; l_set_succ++) {
				Prob_Hit.l_set_succ = l_set_succ;
				System.out.println("l_set_succ = " + l_set_succ);
				for (int l_set_fault = minNumber; l_set_fault < maxNumber; l_set_fault++) {
					Prob_Hit.l_set_fault = l_set_fault;
					System.out.println("l_set_fault = " + l_set_fault);
					for (int l_set_dev = minNumber; l_set_dev < maxNumber; l_set_dev++) {
						Prob_Hit.l_set_dev = l_set_dev;
						System.out.println("l_set_dev = " + l_set_dev);
						for (int l_def_succ = minNumber; l_def_succ < maxNumber; l_def_succ++) {
							Prob_Hit.l_def_succ = l_def_succ;
							System.out.println("l_def_succ = " + l_def_succ);
							for (int l_def_fault = minNumber; l_def_fault < maxNumber; l_def_fault++) {
								Prob_Hit.l_def_fault = l_def_fault;
								System.out.println("l_def_fault = " + l_def_fault);
								for (int l_def_dev = minNumber; l_def_dev < maxNumber; l_def_dev++) {
									Prob_Hit.l_def_dev = l_def_dev;
									System.out.println("l_def_dev = " + l_def_dev);
									for (int l_attack_succ = minNumber; l_attack_succ < maxNumber; l_attack_succ++) {
										Prob_Attack.l_attack_succ = l_attack_succ;
										System.out.println("l_attack_succ = " + l_attack_succ);
										for (int l_attack_fault = minNumber; l_attack_fault < maxNumber; l_attack_fault++) {
											Prob_Attack.l_attack_fault = l_attack_fault;
											System.out.println("l_attack_fault = " + l_attack_fault);
											for (int l_attack_dev = minNumber; l_attack_dev < maxNumber; l_attack_dev++) {
												Prob_Attack.l_attack_dev = l_attack_dev;
												System.out.println("l_attack_dev = " + l_attack_dev);
			
			
												game.initGame("Germany", "Brink", "Reckerman",skillDataFiles[i][0],skillDataFiles[i][1], Strategy.cali_finalMatch_TeamA_read,
														"Brasilien", "Alison", "Emanuel", skillDataFiles[i][2],skillDataFiles[i][3], Strategy.cali_finalMatch_TeamB_read);
												
												SimulationManager simManager = startSim(n, true, 0, true, null, skillDescription[i]);
												
												double devA = probSeries.computeDeviationA(new double[] { simManager.getServeP(),
														simManager.getServePOverline(), simManager.getServePHat(),
														simManager.getfAP(), simManager.getfAPOverline(), simManager.getfAPHat() });
			
												double devB = probSeries.computeDeviationB(new double[] { simManager.getServeQ(),
														simManager.getServeQOverline(), simManager.getServeQHat(),
														simManager.getfAQ(), simManager.getfAQOverline(), simManager.getfAQHat() });
			
												double dev = probSeries.computeDeviation(
														new double[] { simManager.getServeP(), simManager.getServePOverline(),
																simManager.getServePHat(), simManager.getfAP(),
																simManager.getfAPOverline(), simManager.getfAPHat() },
														new double[] { simManager.getServeQ(), simManager.getServeQOverline(),
																simManager.getServeQHat(), simManager.getfAQ(),
																simManager.getfAQOverline(), simManager.getfAQHat() });
												probSeries.addValues(
														new double[] { Prob_Hit.l_set_succ, Prob_Hit.l_set_fault, Prob_Hit.l_set_dev, Prob_Hit.l_def_succ, Prob_Hit.l_def_fault, Prob_Hit.l_def_dev,
																Prob_Attack.l_attack_succ, Prob_Attack.l_attack_fault, Prob_Attack.l_attack_dev, devA, devB, dev });
												probSeries.addValues(
														new double[] { l_set_succ, l_set_fault, l_set_dev, l_def_succ, l_def_fault, l_def_dev,
																l_attack_succ, l_attack_fault, l_attack_dev, devA, devB, dev });
											}
										}
									}
								}
							}
						}
					}
				}
			}

			probSeries.writeToExcelFile();
		}
	}
	
	public static void calibrationSimulationSeries(int n){
		Game game = Game.getInstance();
		
		for (int i = 2; i<skillDataFiles.length; i++){
			CalibrationSeries caliSeries = new CalibrationSeries(skillDescription[i], skillDataFiles[i], n);			
			for (int k = 11; k < 12; k++){
				Prob_Hit.minObs = k;
				game.initGame("Germany", "Brink", "Reckerman",skillDataFiles[i][0],skillDataFiles[i][1], Strategy.cali_finalMatch_TeamA_read,
									"Brasilien", "Alison", "Emanuel", skillDataFiles[i][2],skillDataFiles[i][3], Strategy.cali_finalMatch_TeamB_read);
				
//				System.out.println(game.getTeamB().getPlayer1().getIndProbs().printAggProbs());
//				System.out.println(game.getTeamB().getPlayer1().getIndProbs().printAnalogToExcelInputFormat(game.getTeamB().getPlayer1()));

				
				SimulationManager simManager = startSim(n, true, 0, true, null, skillDescription[i]);
			
				
				caliSeries.addComputedPValues(k, new double[]{simManager.getServeP(), simManager.getServePOverline(), simManager.getServePHat(),
				                                            simManager.getfAP(), simManager.getfAPOverline(), simManager.getfAPHat()});
				caliSeries.addComputedQValues(k, new double[]{simManager.getServeQ(), simManager.getServeQOverline(), simManager.getServeQHat(),
                        simManager.getfAQ(), simManager.getfAQOverline(), simManager.getfAQHat()});
			}
			
			caliSeries.computeDeviation();
			caliSeries.writeToExcelFile();
		}
		
	}
	
	/** 
	 * Setzt eine Simulationsreihe mit Parametern auf	
	 */
	public static void setUpSimulationSeries(){
//		Game.getInstance().initGame("Germany", "Brink", "Reckerman",inputDataFileA1,inputDataFileA2, Strategy.cali_allMatchesExceptFinal_TeamAOLD,
//				"Brasilien", "Alison", "Emanuel", inputDataFileB1, inputDataFileB2, Strategy.cali_allMatchesExceptFinal_TeamBOLD);
		Game.getInstance().initGameFromEvalCSV("Germany", "Brink", "Reckerman",MPC_TableRuns_Part1.skillDataFiles[0][0], Strategy.cali_allMatchesExceptFinal_TeamA_read,
				"Brasilien", "Alison", "Emanuel", MPC_TableRuns_Part1.skillDataFiles[0][1], Strategy.cali_allMatchesExceptFinal_TeamB_read);
		
		int numberOfSets = 10; // Anzahl der Saetze die simuliert werden sollen		
		HitTechnique tech = HitTechnique.F_P; // Technik die modifiziert werden sollen
		ArrayList<RealStrat> strats = new ArrayList<RealStrat>(Arrays.asList(Strategy.prefinalRiskyA, Strategy.prefinalSafeA, Strategy.cali_allMatchesExceptFinal_TeamA_read)); // Strategien die gegenuebergestellt werden sollen
		ArrayList<Player> parameterizedPlayers = new ArrayList<Player>(Arrays.asList(Game.getInstance().getTeamA().getPlayer1(), Game.getInstance().getTeamA().getPlayer2())); // beide Spieler werden variiert
		
		ResultSimSeries_new resultSimSeries = simulationSeriesSMDP_new(tech, parameterizedPlayers, numberOfSets, 0, 0.1, 1, strats);
		
		try {
			resultSimSeries.writeToFile();
		} catch (WriteException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static ResultSimSeries_new simulationSeriesSMDP_new(HitTechnique tech, ArrayList<Player> players, int numberOfSets, double startValue, double stepSize, double endValue, ArrayList<RealStrat> strats){
		
		Game game = Game.getInstance();

		HashMap<Player, Prob_IndPlayer> inputProbs = new HashMap<Player, Prob_IndPlayer>();
		// ACHTUNG: Kopie ist wichtig, damit die "Ausgangsdaten" am Ende ausgegeben werden koennen!
		inputProbs.put(game.getTeamA().getPlayer1(), new Prob_IndPlayer(game.getTeamA().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamA().getPlayer2(), new Prob_IndPlayer(game.getTeamA().getPlayer2().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer1(), new Prob_IndPlayer(game.getTeamB().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer2(), new Prob_IndPlayer(game.getTeamB().getPlayer2().getIndProbs()));

		// alle zur Technik gehoerenden WKeiten raussuchen 
		ArrayList<Prob_Hit> parameterizedProbs = new ArrayList<Prob_Hit>();
		parameterizedProbs.addAll(game.getTeamA().getPlayer1().getIndProbs().getAllHitProbs(tech));
		
		ResultSimSeries_new resultSimSeries = new ResultSimSeries_new(tech, startValue, stepSize, endValue, numberOfSets, strats, players, parameterizedProbs, inputProbs);
		
		// alle Kombinationen der Erfolgs und Fehlerwkeit durchgehen
		for (double p_ind = startValue; p_ind<=endValue + 0.00000000001; p_ind=p_ind+stepSize){
				
			for (double p_ind_fault = startValue; p_ind+p_ind_fault < endValue + 0.000000001; p_ind_fault=p_ind_fault+stepSize){
			
				// fuer alle Strategien, die simuliert werden sollen
				for (int i = 0; i < strats.size(); i++){
				
					ResultSim_new result = new ResultSim_new();
					result.setP_ind(p_ind);
					result.setP_ind_fault(p_ind_fault);
					
					// modify all Serving Probabilities of Team A
					for (Player player : players){
						for (Prob_Hit prob : parameterizedProbs){
							prob.setAllThreeProbs(p_ind, p_ind_fault);
							player.getIndProbs().overrideProbHit(prob);								
						}
						// dann müssen modifizierte Probs neu berechnet werden
						if (Game.useModProbs){
							player.getIndProbs().computeAggregatedProbs(player);
							player.getIndProbs().computeModifiedProbs();
						}
					}
										
					game.getTeamA().setStrat(strats.get(i));			
					result.setStratTeamA(game.getTeamA().getStrat());
					result.setStratTeamB(game.getTeamB().getStrat());
					
					SimulationManager simManager = startSim(numberOfSets, false, 100, false, null, "simSeries");
					result.setNumberOfSets(numberOfSets);
					
					result.setP_serve(simManager.getServeP());
					result.setpOverline_serve(simManager.getServePOverline());
					result.setQ_serve(simManager.getServeQ());
					result.setqOverline_serve(simManager.getServeQOverline());
					
					result.setP_fA(simManager.getfAP());
					result.setpOverline_fA(simManager.getfAPOverline());
					result.setQ_fA(simManager.getfAQ());
					result.setqOverline_fA(simManager.getfAQOverline());
			
					resultSimSeries.addResult(result);
					System.out.println(result.toString());					
				}
			}
		}
		
		return resultSimSeries;
	}
	
	public static ResultSimSeries simulationSeriesSMDP(HashSet<HitTechnique> techs, ArrayList<Player> parameterizedPlayers, int numberOfSets, double startValue, double stepSize, double endValue, ArrayList<RealStrat> strats){
		
		Game game = Game.getInstance();
				
		HashMap<Player, Prob_IndPlayer> inputProbs = new HashMap<Player, Prob_IndPlayer>();
		// ACHTUNG: Kopie ist wichtig, damit die "Ausgangsdaten" am Ende ausgegeben werden kuennen!
		inputProbs.put(game.getTeamA().getPlayer1(), new Prob_IndPlayer(game.getTeamA().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamA().getPlayer2(), new Prob_IndPlayer(game.getTeamA().getPlayer2().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer1(), new Prob_IndPlayer(game.getTeamB().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer2(), new Prob_IndPlayer(game.getTeamB().getPlayer2().getIndProbs()));

		ArrayList<Prob_Hit> parameterizedProbs = new ArrayList<Prob_Hit>();
		for (HitTechnique tech : techs){
			parameterizedProbs.addAll(game.getTeamA().getPlayer1().getIndProbs().getAllHitProbs(tech));
		}
		
		ResultSimSeries resultSimSeries = new ResultSimSeries(techs,  numberOfSets, startValue, stepSize, endValue, parameterizedProbs, parameterizedPlayers, inputProbs, strats);
		
		// es wird von der vorgegebenen Technik nacheinander alle Wkeiten (succ, fault, neighbour) parameterisiert.
		for (ProbType probType : ProbType.values()){
		
			// fuer alle Parameterwerte die simuliert werden sollen
			for (double d = startValue; d<=endValue + 0.00000000001; d=d+stepSize){
				
				// fuer alle Strategien, die simuliert werden sollen
				for (int i = 0; i < strats.size(); i++){
				
					ResultSim result = new ResultSim();
					result.setProbType(probType);
					
					// modify all Serving Probabilities of Team A
					result.setParameterizedValue(d);
					for (Prob_Hit prob : parameterizedProbs){
						for (Player player : parameterizedPlayers){
							if (probType == ProbType.succ){
								prob.setSuccProb(d);
								try {
									player.getIndProbs().overrideProbHit(prob);
								} catch (Exception e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}								
							}
							if (probType == ProbType.fault){
								prob.setFaultProb(d);
								try {
									player.getIndProbs().overrideProbHit(prob);
								} catch (Exception e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}								
							}
							if (probType == ProbType.neighbour){
								prob.setNeighbourProb(d);
								try {
									player.getIndProbs().overrideProbHit(prob);
								} catch (Exception e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}								
							}
						}
					}
					
					game.getTeamA().setStrat(strats.get(i));			
					result.setStratTeamA(game.getTeamA().getStrat());
					result.setStratTeamB(game.getTeamB().getStrat());
					
					SimulationManager simManager = startSim(numberOfSets, true, 0, true, null, "simSeries");
					result.setNumberOfSets(numberOfSets);
					
					result.setP_serve(simManager.getServeP());
					result.setpOverline_serve(simManager.getServePOverline());
					result.setQ_serve(simManager.getServeQ());
					result.setqOverline_serve(simManager.getServeQOverline());
					
					result.setP_fA(simManager.getfAP());
					result.setpOverline_fA(simManager.getfAPOverline());
					result.setQ_fA(simManager.getfAQ());
					result.setqOverline_fA(simManager.getfAQOverline());
			
					resultSimSeries.addResult(result);
					log4j.info(result.toString());					
				}
			}
		}
		
		return resultSimSeries;
	}
	
	/**
	 * Hier findet die tatsächliche Simulation statt. Sowohl für GUI als auch für SimSeries
	 * @param Anzahl der zu spielenden Sets
	 */
	public static SimulationManager startSim(int numberOfBatches, boolean simulateInSets, int numberOfRalliesPerBatch, boolean followServingRules, SimSwingWorker worker, String skillDesc){		

		boolean withGui = false;
		if (worker != null){
			withGui = true;
		}
				
		// Objekte besorgen die man braucht fuer das Spiel
		Game game = Game.getInstance();
		// legt ein neues Simulationobjekt an
		SimulationManager simManager = new SimulationManager(skillDesc);
		if (withGui){
			worker.simManager = simManager;
		}
		
		// Angriffstyp
		AttackType attackType =  null;	
				
		// Spiel
		while (simManager.getWonSetsTeamA()+simManager.getWonSetsTeamB() < numberOfBatches && simManager.numberOfResets <= SimulationManager.maxNumberOfResets){
			
			log4j.debug("neuer Satz; gespielte Saetze bis hier " + (simManager.getWonSetsTeamA()+simManager.getWonSetsTeamB()));
					
			// zaehlt fuer einen Satz die Angriffsstatistik
			AttackCounter setAttackCounter = new AttackCounter();
			game.resetGame();		
			
			// Satz
			while ( 
					// Simulation in Sätze, Abbruchkriterium = gewonnener Satz
					((simulateInSets && (game.getPointsTeamA()<21 && game.getPointsTeamB() < 21 || Math.abs(game.getPointsTeamA()-game.getPointsTeamB())<2))
					||
					// Simulation in Batches, solange Anzahl der Rallies noch nicht erreicht
					(!simulateInSets && game.getPointsTeamA()+game.getPointsTeamB() < numberOfRalliesPerBatch))
					// Anzahl der Resets noch nicht überschritten
					&& simManager.numberOfResets <= SimulationManager.maxNumberOfResets) {		
						
				log4j.debug("neuer Ballwechsel; aktueller Punktestand " + game.getPointsTeamA() + " : " + game.getPointsTeamB());
				AttackCounter rallyAttackCounter = new AttackCounter();
						
				// Ballwechsel - d.h. solange bis ein Punkt erzielt wurde
				while (!State.POINT_A.contains(game.getCurrentState()) 
						&& !State.POINT_B.contains(game.getCurrentState())
						&& simManager.numberOfResets <= SimulationManager.maxNumberOfResets) {
							
					if (withGui){
						if (worker.interruped){
							continue;
						}
					}
					
					// Anzahl der Angriffe zaehlen
					// Angriff: Serve
					if (Arrays.asList(State.START_STATES_A).contains(game.getCurrentState())){
						rallyAttackCounter.newAttackTeamA();
						rallyAttackCounter.newServeTeamA();
						attackType = AttackType.SERVE_A;
						log4j.debug("Neuer Angriff A - Typ: Serve");
						if (withGui){
							worker.attackInfoText = "serve attack A";
						}
					}
					if (Arrays.asList(State.START_STATES_B).contains(game.getCurrentState())){
						rallyAttackCounter.newAttackTeamB();
						rallyAttackCounter.newServeTeamB();
						attackType = AttackType.SERVE_B;
						log4j.debug("Neuer Angriff B - Typ: Serve");
						if (withGui){
							worker.attackInfoText = "serve attack B";
						}
					}
							
					game.chooseTeamActions();
					try {
						log4j.debug("Team Action A: " + game.getTeamA().getTeamAction().toString());
						log4j.debug("Team Action B: " + game.getTeamB().getTeamAction().toString());
					} catch (Exception e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					
					// Angriff: aus dem Spiel
					// ACHTUNG: beginnt erst, wenn Ball erfolgreich angenommen wurde, d.h. Counter = 1
					// und das Team einen weiteren Schlag zum Stellen spielen kann
					if (game.getCurrentState().getCounter() == 1) {
						if (game.getCurrentState().getFieldBall().belongsToSideA() && 
								game.getTeamA().makesHit()){ 
							rallyAttackCounter.newAttackTeamA();
							rallyAttackCounter.newFieldAttackTeamA();
							attackType = AttackType.FIELD_ATTACK_A;
							log4j.debug("Neuer Angriff A - Typ: aus dem Spiel");
							if (withGui){
								worker.attackInfoText = "field attack A";
							}
						}
						if (game.getCurrentState().getFieldBall().belongsToSideB()&& 
								game.getTeamB().makesHit()) { 
							rallyAttackCounter.newAttackTeamB();
							rallyAttackCounter.newFieldAttackTeamB();
							attackType = AttackType.FIELD_ATTACK_B;
							log4j.debug("Neuer Angriff B - Typ: aus dem Spiel");
							if (withGui){
								worker.attackInfoText = "field attack B";
							}
						}
					}
									
					if(withGui){
						// PUBLISH
						worker.myPublish(simManager.getWonSetsTeamA()+simManager.getWonSetsTeamB(), false, game.getCurrentState());
						
						if (worker.interruped){
							continue;
						};						
					}
					
					try {
						game.computeNextState();
					} catch (StateResetException e){
						/* in der Funktion computeNextState wurde der Zustand schon auf den letzten Aufschlag zurückgesetzt */
						simManager.numberOfResets++;
						simManager.addReset(e.getHittingPlayer(), e.getPlayerAction());
						rallyAttackCounter = new AttackCounter();
					}					
					catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					log4j.debug("Following state: " + game.getCurrentState());
					log4j.debug(MainSim.calls);
							
					if (withGui){
						worker.myPublish(simManager.getWonSetsTeamA()+simManager.getWonSetsTeamB(), true, game.getCurrentState());
					}

				} // end while loop - Ballwechsel
				
			
				log4j.debug(game.printScore());
	
				if (State.POINT_A.contains(game.getCurrentState())) {
					switch (attackType){
						case SERVE_A:
							log4j.debug("point after serve A");
							rallyAttackCounter.directPointTeamA();
							rallyAttackCounter.directPointServesTeamA();
							if (withGui){
								worker.attackInfoText = "point after serve A";
							}
							break;
						
						case FIELD_ATTACK_A:
							log4j.debug("point after field attack A");
							rallyAttackCounter.directPointTeamA();
							rallyAttackCounter.directPointFieldAttackTeamA();
							if (withGui){
								worker.attackInfoText = "point after field attack A";
							}
							break;
							
						case SERVE_B:
							log4j.debug("fault after serve B");
							rallyAttackCounter.attackFaultTeamB();
							rallyAttackCounter.attackFaultServesTeamB();
							if (withGui){
								worker.attackInfoText = "fault after serve B";
							}
							break;
							
						case FIELD_ATTACK_B:
							log4j.debug("fault after field attack B");
							rallyAttackCounter.attackFaultTeamB();
							rallyAttackCounter.attackFaultFieldAttackTeamB();
							if (withGui){
								worker.attackInfoText = "fault after field attack B";
							}
							break;
							
						default:
							log4j.warn("AttackType nicht gekannt!");
							break;
					}
				}
				
				if (State.POINT_B.contains(game.getCurrentState())) {
					
					switch (attackType){
						case SERVE_A:
							log4j.debug("fault after serve A");
							rallyAttackCounter.attackFaultTeamA();
							rallyAttackCounter.attackFaultServesTeamA();
							if (withGui){
								worker.attackInfoText = "fault after serve A";
							}
							break;
							
						case FIELD_ATTACK_A:
							log4j.debug("fault after field attack A");
							rallyAttackCounter.attackFaultTeamA();
							rallyAttackCounter.attackFaultFieldAttackTeamA();
							if (withGui){
								worker.attackInfoText = "fault after field attack A";
							}
							break;
							
						case SERVE_B:
							log4j.debug("point after serve B");
							rallyAttackCounter.directPointTeamB();
							rallyAttackCounter.directPointServesTeamB();
							if (withGui){
								worker.attackInfoText = "point after serve B";
							}
							break;
							
						case FIELD_ATTACK_B:
							log4j.debug("point after field attack B");
							rallyAttackCounter.directPointTeamB();
							rallyAttackCounter.directPointFieldAttackTeamB();
							if (withGui){
								worker.attackInfoText = "point after field attack B";
							}
							
							break;		
							
						default:
							log4j.warn("AttackType nicht gekannt!");
							break;
					}
				}
			
				// rally Attack Counter zu Set Attack Counter hinzuzählen
				setAttackCounter.add(rallyAttackCounter);
				if (simManager.numberOfResets < SimulationManager.maxNumberOfResets){
					game.setEmptyTeamActions();
					if(withGui){
						worker.hitInfoText = "-";
						worker.blockInfoText = "-";
					}
					if (followServingRules){
						game.setCurrentState(game.getNextServingState(game.getCurrentState()));
					}else{
						game.setCurrentState(game.getRandomServingState());
					}
					log4j.debug(game.getCurrentState().toStringLong());
					
					if(withGui){
						// PUBLISH
						worker.myPublish(simManager.getWonSetsTeamA()+simManager.getWonSetsTeamB(), true, game.getCurrentState());
					}
				}
			} // end while loop - ein Set / Batch
					
			if (game.getPointsTeamA()>game.getPointsTeamB()){
				simManager.increaseWonSetsTeamA();
			}else
			{				
				simManager.increaseWonSetsTeamB();
			}
				
			simManager.addCounterResultToSimManager(setAttackCounter);
				
		} // end while loop - number of Sets / Batches

		if (simManager.numberOfResets < SimulationManager.maxNumberOfResets){
			simManager.computeSummary();
		}else{
			log4j.warn("Simulation abgebrochen, number of Resets: " + simManager.numberOfResets);
		}
				
		return simManager;		
	}
	
}
