package de.ubt.wm.sb.sim;

import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Locale;

import de.ubt.wm.sb.probs.Prob_Hit;
import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import jxl.write.biff.RowsExceededException;

public class ProbModificationSeries {

	private String skillDesc;
	private String[] skillFiles;
	private int numberOfSets;
	
	private static double[] theTruthP = new double[]{0.0185, 0.037, 0.9445, 0.4722, 0.2407, 0.2871};
	private static double[] theTruthQ = new double[]{0.0351, 0.1404, 0.8245, 0.5200, 0.2105, 0.2695};

	private ArrayList<double[]> computedValues;
	
	public String[] getSkillFiles() {
		return skillFiles;
	}

	public void setSkillFiles(String[] skillFiles) {
		this.skillFiles = skillFiles;
	}

	public ProbModificationSeries(String skillDesc, String[] skillFiles, int n) {
		super();
		this.skillDesc = skillDesc;
		this.setSkillFiles(skillFiles);
		this.computedValues = new  ArrayList<double[]>();
		this.numberOfSets = n;
	}
	
	public void addValues(double[] values){
		this.computedValues.add(values);
	}
	
	public double computeDeviation(double[] p, double q[]){
		double dev = 0.0;
		for (int i = 0; i<p.length; i++){
			dev += Math.abs(theTruthP[i]-p[i]);
		}
		for (int i = 0; i<q.length; i++){
			dev += Math.abs(theTruthQ[i]-q[i]);
		}

		dev = dev/12;
		
		return dev;
	}
	
	public double computeDeviationA(double[] p) {
		double dev = 0.0;
		for (int i = 0; i<p.length; i++){
			dev += Math.abs(theTruthP[i]-p[i]);
		}
		dev = dev/6;
		
		return dev;
	}
	
	public double computeDeviationB(double[] q) {
		double dev = 0.0;
		for (int i = 0; i<q.length; i++){
			dev += Math.abs(theTruthQ[i]-q[i]);
		}
		dev = dev/6;
		
		return dev;
	}
	
	public void writeToExcelFile(){
		Calendar calendar = Calendar.getInstance();
		Timestamp currentTimestamp = new java.sql.Timestamp(calendar.getTime().getTime());

		File file = new File("ProbModificationSeries/probModSeries_"+ skillDesc + "_" + currentTimestamp+".xls");
	    WorkbookSettings wbSettings = new WorkbookSettings();
	    wbSettings.setLocale(new Locale("en", "EN"));
	    WritableWorkbook workbook;
	    
	    
	  
	    try {
			workbook = Workbook.createWorkbook(file, wbSettings);
			workbook.createSheet("Overview", 0);
		    WritableSheet excelSheetOverview = workbook.getSheet(0);
		    
		    excelSheetOverview.addCell(new Label(0,1,"n:"));
		    excelSheetOverview.addCell(new Number(1,1,numberOfSets));
		    excelSheetOverview.addCell(new Label(0,2,"Seed:"));
		    excelSheetOverview.addCell(new Number(1,2,MainSim.l));
		    
		    int iO = 2;
		    excelSheetOverview.addCell(new Label(0,iO+1,"devToOutsideFieldOnly:"));
		    excelSheetOverview.addCell(new Label(1,iO+1,MainSim.M1_devToOutsideFieldOnlyForServesAndAttacks + ""));
		    excelSheetOverview.addCell(new Label(0,iO+2,"devFromRealMatch:"));
		    excelSheetOverview.addCell(new Label(1,iO+2,MainSim.devFromRealMatch + ""));
		    excelSheetOverview.addCell(new Label(0,iO+3,"Modify Serves:"));
		    excelSheetOverview.addCell(new Label(1,iO+3,MainSim.modServes + ""));
		    excelSheetOverview.addCell(new Label(0,iO+4,"Modify Sets:"));
		    excelSheetOverview.addCell(new Label(1,iO+4,MainSim.modSet + ""));
		    excelSheetOverview.addCell(new Label(0,iO+5,"Modify Defs:"));
		    excelSheetOverview.addCell(new Label(1,iO+5,MainSim.modDef + ""));
		    excelSheetOverview.addCell(new Label(0,iO+6,"Modify Attacks:"));
		    excelSheetOverview.addCell(new Label(1,iO+6,MainSim.modAttacks + ""));
		    excelSheetOverview.addCell(new Label(0,iO+7,"Auffüllgröße:"));
		    excelSheetOverview.addCell(new Number(1,iO+7, Prob_Hit.minObs));
		    
		    // SHEET		    
		    workbook.createSheet("ProbModificationSeries", 1);
		    WritableSheet excelSheet = workbook.getSheet(1);
		    excelSheet.addCell(new Label(0,0,"l_set_succ"));
		    excelSheet.addCell(new Label(1,0,"l_set_fault"));
		    excelSheet.addCell(new Label(2,0,"l_set_dev"));
		    excelSheet.addCell(new Label(3,0,"l_def_succ"));
		    excelSheet.addCell(new Label(4,0,"l_def_fault"));
		    excelSheet.addCell(new Label(5,0,"l_def_dev"));
		    excelSheet.addCell(new Label(6,0,"l_attack_succ"));
		    excelSheet.addCell(new Label(7,0,"l_attack_fault"));
		    excelSheet.addCell(new Label(8,0,"l_attack_dev"));
		    excelSheet.addCell(new Label(9,0,"DevA"));
		    excelSheet.addCell(new Label(10,0,"DevB"));
		    excelSheet.addCell(new Label(11,0,"DevTotal"));

		    int k = 1;
		    for (int i = 0; i<computedValues.size(); i++){
		    	for (int j = 0; j < computedValues.get(i).length; j++){
		    		excelSheet.addCell(new Number(j,k+i,computedValues.get(i)[j]));
		    	}	    	
		    }
		    
		    workbook.write();
		    workbook.close();
		    
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RowsExceededException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WriteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
