package de.ubt.wm.sb.sim;

import java.io.File;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.model.HitTechnique;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.strategy.RealStrat;
import de.ubt.wm.sb.probs.ProbType;
import de.ubt.wm.sb.probs.Prob_Hit;
import de.ubt.wm.sb.probs.Prob_IndPlayer;
import jxl.CellView;
import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.format.Alignment;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.WritableCellFormat;
import jxl.write.WritableFont;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import jxl.write.biff.RowsExceededException;

public class ResultSimSeries {
	
	static final Logger log4j = LogManager.getLogger(ResultSimSeries.class.getName());

	private HashSet<HitTechnique> techs;
	
	private double startValue;
	private double stepSize;
	private double endValue;
	
	private int numberOfSetsPerSim;
	
	private ArrayList<RealStrat> simulatedStrategies;
	private ArrayList<Player> parameterizedPlayers;
		
	private ArrayList<Prob_Hit> changedProb;
	private HashMap<Player, Prob_IndPlayer> playerInputProbs;
	private HashMap<ProbType, HashMap<Double, HashMap<RealStrat, ResultSim>>> computedResults;
	
	public ResultSimSeries(HashSet<HitTechnique> techs, int numberOfSetsPerSim, double startValue, double stepSize, double endValue,
			ArrayList<Prob_Hit> changedProb, ArrayList<Player> parameterizedPlayer, HashMap<Player, Prob_IndPlayer> playerInputProbs, ArrayList<RealStrat> strats){
		this.techs = techs;
		this.numberOfSetsPerSim = numberOfSetsPerSim;
		this.changedProb = changedProb;
		this.startValue = startValue;
		this.stepSize = stepSize;
		this.endValue = endValue;
		this.playerInputProbs = playerInputProbs;
		this.computedResults = new HashMap<ProbType, HashMap<Double, HashMap<RealStrat, ResultSim>>>();
		this.simulatedStrategies = strats;
		this.parameterizedPlayers = parameterizedPlayer;    
	}
	
	
	public void addResult(ResultSim result){		
		if (result == null){
			System.err.println("ResultSim ist leer!");
			return;
		}		
		if (computedResults.get(result.getProbType()) == null){
			HashMap<RealStrat, ResultSim> mapStratRes = new HashMap<RealStrat, ResultSim>();
			mapStratRes.put(result.getStratTeamA(), result);
			HashMap<Double, HashMap<RealStrat, ResultSim>> mapDoubleMap = new HashMap<Double, HashMap<RealStrat, ResultSim>>();
			mapDoubleMap.put(result.getParameterizedValue(), mapStratRes);
			computedResults.put(result.getProbType(), mapDoubleMap);
		}else if(computedResults.get(result.getProbType()).get(result.getParameterizedValue()) == null){
			HashMap<RealStrat, ResultSim> mapStratRes = new HashMap<RealStrat, ResultSim>();
			mapStratRes.put(result.getStratTeamA(), result);
			computedResults.get(result.getProbType()).put(result.getParameterizedValue(), mapStratRes);
		}else {
			computedResults.get(result.getProbType()).get(result.getParameterizedValue()).put(result.getStratTeamA(), result);
		}
		
	}
	
	

	
	public HashMap<Player, Prob_IndPlayer> getPlayerInputProbs() {
		return playerInputProbs;
	}


	public void writeToFile() throws Exception{
				
		Calendar calendar = Calendar.getInstance();
		Timestamp currentTimestamp = new java.sql.Timestamp(calendar.getTime().getTime());
		File file = new File("results_SimSeries/SimSeries_"+ currentTimestamp +"_" + techs.toString() + ".xls");
	    WorkbookSettings wbSettings = new WorkbookSettings();
	    wbSettings.setLocale(new Locale("en", "EN"));
	    WritableWorkbook workbook = Workbook.createWorkbook(file, wbSettings);
	    
	    /*
	     * Format 
	     */	    
	    // Create cell font and format
	    WritableFont cellBigFont = new WritableFont(WritableFont.ARIAL, 14);
	    cellBigFont.setBoldStyle(WritableFont.BOLD);
	    
	    WritableCellFormat cellBigHeadingFormat = new WritableCellFormat(cellBigFont);
	    
	    WritableFont cellFont = new WritableFont(WritableFont.ARIAL, 10);
	    cellFont.setBoldStyle(WritableFont.BOLD);
	    WritableCellFormat cellHeadingFormat = new WritableCellFormat(cellFont);
	    cellHeadingFormat.setAlignment(Alignment.CENTRE);
	    
	    WritableCellFormat cellHeadingFormatLeft = new WritableCellFormat(cellFont);
	    cellHeadingFormatLeft.setAlignment(Alignment.LEFT);
	    
	    WritableCellFormat cellHeadingRight = new WritableCellFormat(cellFont);
	    cellHeadingRight.setAlignment(Alignment.RIGHT);
	    	    
	    WritableCellFormat cellCentered = new WritableCellFormat();
	    cellCentered.setAlignment(Alignment.CENTRE);
	    
	    WritableCellFormat cellLeft = new WritableCellFormat();
	    cellLeft.setAlignment(Alignment.LEFT);
	    
	    /*------------------------------------------------------------------------------
	     * Sheet Setting
	     * ----------------------------------------------------------------------------- 	    
	     */
	    workbook.createSheet("Settings", 0);
	    WritableSheet excelSheetSettings = workbook.getSheet(0);
	        	    
	    excelSheetSettings.addCell(new Label(1,1,"Simulation Series", cellBigHeadingFormat));
	    excelSheetSettings.addCell(new Label(1,3,"number of Sets per Sim:", cellHeadingRight));
	    excelSheetSettings.addCell(new Number(3,3, numberOfSetsPerSim));
	    excelSheetSettings.addCell(new Label(1,4,"Deviation only to Outside Field:", cellHeadingRight));
	    excelSheetSettings.addCell(new Label(3,4, Boolean.toString(MainSim.M1_devToOutsideFieldOnlyForServesAndAttacks)));
	    
	    excelSheetSettings.addCell(new Label(1,5,"Techniques:", cellHeadingRight));
	    excelSheetSettings.addCell(new Label(3,5,techs.toString()));
	    excelSheetSettings.addCell(new Label(1,6,"parameterized Probs:", cellHeadingRight));
	    for (int i = 0; i<changedProb.size(); i++){
	    	excelSheetSettings.addCell(new Label(3+i,6,changedProb.get(i).printName()));
	    }
	    excelSheetSettings.addCell(new Label(1,7,"parameterized Players:", cellHeadingRight));
	    for (int i = 0; i<parameterizedPlayers.size(); i++){
	    	excelSheetSettings.addCell(new Label(3+i,7,parameterizedPlayers.get(i).getName()));
	    }
	    excelSheetSettings.addCell(new Label(1,8,"simulated Strategies:", cellHeadingRight));
	    for (int i = 0; i<simulatedStrategies.size(); i++){
	    	excelSheetSettings.addCell(new Label(3+i,8,simulatedStrategies.get(i).toString()));
	    }	
	    excelSheetSettings.addCell(new Label(1,9,"Player Input Probs:", cellHeadingRight));
	    int k = 0;
	    for (Player player : playerInputProbs.keySet()){
	    	excelSheetSettings.addCell(new Label(3,9+k,player.getName()));
	    	excelSheetSettings.addCell(new Label(4,9+k,playerInputProbs.get(player).printAnalogToExcelInputFormat(player)));
//	    	System.out.println(playerInputProbs.get(player).printAnalogToExcelInputFormat(player));
//	    	System.out.println(playerInputProbs.get(player).toString());
	    	k++;
	    }	 

	    CellView cellView = excelSheetSettings.getColumnView(1);
	    cellView.setAutosize(true);
	    excelSheetSettings.setColumnView(1, cellView);
	    	
	    
	    for (ProbType probType : ProbType.values()){
	    
	    /*------------------------------------------------------------------------------
	     * Results Serving Statistic
	     * ----------------------------------------------------------------------------- 	    
	     */
	    workbook.createSheet("Results Serves " + probType.toString(), 1);
	    WritableSheet excelSheetServes = workbook.getSheet(1);
		
	    excelSheetServes.addCell(new Label(0,0,techs.toString()+probType, cellHeadingFormatLeft));
	    excelSheetServes.addCell(new Label(0,1,"value", cellHeadingFormatLeft));
	    
	    for (int i = 0; i<simulatedStrategies.size(); i++){
	    	excelSheetServes.addCell(new Label(4*i+1,0,simulatedStrategies.get(i).toString(), cellHeadingFormatLeft));
	    	excelSheetServes.addCell(new Label(4*i+1,1,"p_serve", cellHeadingFormatLeft));
		    excelSheetServes.addCell(new Label(4*i+2,1,"pOverline_serve", cellHeadingFormatLeft));
		    excelSheetServes.addCell(new Label(4*i+3,1,"q_serve", cellHeadingFormatLeft));
		    excelSheetServes.addCell(new Label(4*i+4,1,"q_Overline_serve", cellHeadingFormatLeft));	    	
	    }
	    
	    int row = 2;
	    for (double d = startValue; d<= endValue + 0.00000000001; d=d+stepSize){
	    	writeNumberToExcelSheet(excelSheetServes,row,0,d);
	    	int column = 0;
	    	for (RealStrat strat : simulatedStrategies){
	    		writeNumberToExcelSheet(excelSheetServes,row,column+1, computedResults.get(probType).get(d).get(strat).getP_serve());
		    	writeNumberToExcelSheet(excelSheetServes,row,column+2, computedResults.get(probType).get(d).get(strat).getpOverline_serve());
		    	writeNumberToExcelSheet(excelSheetServes,row,column+3, computedResults.get(probType).get(d).get(strat).getQ_serve());
		    	writeNumberToExcelSheet(excelSheetServes,row,column+4, computedResults.get(probType).get(d).get(strat).getqOverline_serve());
		    	column = column + 4;
	    	} 	
	    	row++;
	    }
	    
	    cellView = excelSheetServes.getColumnView(0);
	    cellView.setAutosize(true);
	    cellView.setFormat(cellLeft);
	    excelSheetServes.setColumnView(0, cellView);
	    for (int i = 1; i<=simulatedStrategies.size()*4; i++){
		    cellView = excelSheetServes.getColumnView(i);
		    cellView.setSize(14*256);
		    cellView.setFormat(cellLeft);
		    excelSheetServes.setColumnView(i, cellView);	    	
	    }
	    
	    /*------------------------------------------------------------------------------
	     * Results Field Attack Statistic
	     * ----------------------------------------------------------------------------- 	    
	     */
	    workbook.createSheet("Results fA " + probType, 1);
	    WritableSheet excelSheetfA = workbook.getSheet(1);
		
	    excelSheetfA.addCell(new Label(0,0,techs.toString()+probType, cellHeadingFormatLeft));
	    excelSheetfA.addCell(new Label(0,1,"value", cellHeadingFormatLeft));
	    
	    for (int i = 0; i<simulatedStrategies.size(); i++){
	    	excelSheetfA.addCell(new Label(4*i+1,0,simulatedStrategies.get(i).toString(), cellHeadingFormatLeft));
	    	excelSheetfA.addCell(new Label(4*i+1,1,"p_fA", cellHeadingFormatLeft));
		    excelSheetfA.addCell(new Label(4*i+2,1,"pOverline_fA", cellHeadingFormatLeft));
		    excelSheetfA.addCell(new Label(4*i+3,1,"q_fA", cellHeadingFormatLeft));
		    excelSheetfA.addCell(new Label(4*i+4,1,"q_Overline_fA", cellHeadingFormatLeft));	    	
	    }
	    
	    row = 2;
	    for (double d = startValue; d<= endValue + 0.00000000001; d=d+stepSize){
	    	writeNumberToExcelSheet(excelSheetfA,row,0,d);
	    	
	    	int column = 0;
	    	for (RealStrat strat : simulatedStrategies){
	    		writeNumberToExcelSheet(excelSheetfA,row,column+1, computedResults.get(probType).get(d).get(strat).getP_fA());
		    	writeNumberToExcelSheet(excelSheetfA,row,column+2, computedResults.get(probType).get(d).get(strat).getpOverline_fA());
		    	writeNumberToExcelSheet(excelSheetfA,row,column+3, computedResults.get(probType).get(d).get(strat).getQ_fA());
		    	writeNumberToExcelSheet(excelSheetfA,row,column+4, computedResults.get(probType).get(d).get(strat).getqOverline_fA());
		    	column = column + 4;
	    	} 	
	    	row++;
	    }
	    
	    
	    cellView = excelSheetfA.getColumnView(0);
	    cellView.setAutosize(true);
	    cellView.setFormat(cellLeft);
	    excelSheetfA.setColumnView(0, cellView);
	    for (int i = 1; i<=simulatedStrategies.size()*4; i++){
		    cellView = excelSheetfA.getColumnView(i);
		    cellView.setSize(14*256);
		    cellView.setFormat(cellLeft);
		    excelSheetfA.setColumnView(i, cellView);	    	
	    }
	    
	    }// end for all ProbTypes
	    
	    workbook.write();
	    workbook.close();
	}
	
	private static void writeNumberToExcelSheet(WritableSheet sheet, int row, int column, double number) throws RowsExceededException, WriteException{
		if (Double.isNaN(number)){
			sheet.addCell(new Label(column,row,"#NV"));
		}else{
		sheet.addCell(new Number(column,row,number));
		}
	}
	
}
