package de.ubt.wm.sb.sim;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * AttackCounter zaehlt die Angriffe, direkten Punkte und direkten Fehler in einem Satz.
 * Speichert diese in drei Zhlern.
 * @author bt300874
 */
public class AttackCounter {
	
	static final Logger log4j = LogManager.getLogger(AttackCounter.class.getName());
	
	private int attacksTeamA;
	private int failedAttacksTeamA;
	private int directPointAttacksTeamA;
	
	private int attacksTeamB;
	private int failedAttacksTeamB;
	private int directPointAttacksTeamB;
	
	private int servesTeamA;
	private int failedServesTeamA;
	private int directPointServesTeamA;
	
	private int fieldAttacksTeamA;
	private int failedFieldAttacksTeamA;
	private int directPointFieldAttacksTeamA;
	
	private int servesTeamB;
	private int failedServesTeamB;
	private int directPointServesTeamB;
	
	private int fieldAttacksTeamB;
	private int failedFieldAttacksTeamB;
	private int directPointFieldAttacksTeamB;
	
	/**
	 * Konstruktor. Alle Zaehler auf 0 gesetzt.
	 */
	public AttackCounter(){
		this.attacksTeamA = 0;
		this.failedAttacksTeamA = 0;
		this.directPointAttacksTeamA = 0;
		this.attacksTeamB = 0;
		this.failedAttacksTeamB = 0;
		this.directPointAttacksTeamB = 0;
		
		this.servesTeamA = 0;
		this.failedServesTeamA = 0;
		this.directPointServesTeamA = 0;
		
		this.fieldAttacksTeamA = 0;
		this.failedFieldAttacksTeamA = 0;
		this.directPointFieldAttacksTeamA = 0;
		
		this.servesTeamB = 0;
		this.failedServesTeamB = 0;
		this.directPointServesTeamB = 0;
		
		this.fieldAttacksTeamB = 0;
		this.failedFieldAttacksTeamB = 0;
		this.directPointFieldAttacksTeamB = 0;
	}
	

	// Field Attack
	
	/**
	 * Anzahl der Angriffe von Team A um 1 erhhen
	 */
	public void newFieldAttackTeamA(){
		this.fieldAttacksTeamA++;
		log4j.debug("new field attack A");
	}
	
	/**
	 * Anzahl der Angriffe von Team B um 1 erhhen
	 */
	public void newFieldAttackTeamB(){
		this.fieldAttacksTeamB++;
		log4j.debug("new field attack B");
	}
	
	/**
	 * Anzahl der direkten Punkte von Team A um 1 erhhen
	 */
	public void directPointFieldAttackTeamA(){
		this.directPointFieldAttacksTeamA++;
		log4j.debug("direct point A after field attack");
	}
	
	/**
	 * Anzahl der direkten Punkte von Team B um 1 erhhen
	 */
	public void directPointFieldAttackTeamB(){
		this.directPointFieldAttacksTeamB++;
		log4j.debug("direct point B after field attack");
	}
	
	/**
	 * Anzahl der direkten Fehler von Team A um 1 erhhen
	 */
	public void attackFaultFieldAttackTeamA(){
		this.failedFieldAttacksTeamA++;
		log4j.debug("attack fault A after field attack");
	}
	
	/**
	 * Anzahl der direkten Fehler von Team B um 1 erhhen
	 */
	public void attackFaultFieldAttackTeamB(){
		this.failedFieldAttacksTeamB++;
		log4j.debug("attack fault B after field attack");
	}
	
	
	// Serving Attacks
	
	/**
	 * Anzahl der Angriffe von Team A um 1 erhhen
	 */
	public void newServeTeamA(){
		this.servesTeamA++;
		log4j.debug("new serve A");
	}
	
	/**
	 * Anzahl der Angriffe von Team B um 1 erhhen
	 */
	public void newServeTeamB(){
		this.servesTeamB++;
		log4j.debug("new serve B");
	}
	
	/**
	 * Anzahl der direkten Punkte von Team A um 1 erhhen
	 */
	public void directPointServesTeamA(){
		this.directPointServesTeamA++;
		log4j.debug("direct point A after serve");
	}
	
	/**
	 * Anzahl der direkten Punkte von Team B um 1 erhhen
	 */
	public void directPointServesTeamB(){
		this.directPointServesTeamB++;
		log4j.debug("direct point B after serve");
	}
	
	/**
	 * Anzahl der direkten Fehler von Team A um 1 erhhen
	 */
	public void attackFaultServesTeamA(){
		this.failedServesTeamA++;
		log4j.debug("attack fault A after serve");
	}
	
	/**
	 * Anzahl der direkten Fehler von Team B um 1 erhhen
	 */
	public void attackFaultServesTeamB(){
		this.failedServesTeamB++;
		log4j.debug("attack fault B after serve");
	}
	
	
	// Allgemeine Angriffe
	
	/**
	 * Anzahl der Angriffe von Team A um 1 erhhen
	 */
	public void newAttackTeamA(){
		this.attacksTeamA++;
		log4j.debug("new attack A");
	}
	
	/**
	 * Anzahl der Angriffe von Team B um 1 erhhen
	 */
	public void newAttackTeamB(){
		this.attacksTeamB++;
		log4j.debug("new attack B");
	}
	
	/**
	 * Anzahl der direkten Punkte von Team A um 1 erhhen
	 */
	public void directPointTeamA(){
		this.directPointAttacksTeamA++;
		log4j.debug("direct point A");
	}
	
	/**
	 * Anzahl der direkten Punkte von Team B um 1 erhhen
	 */
	public void directPointTeamB(){
		this.directPointAttacksTeamB++;
		log4j.debug("direct point B");
	}
	
	/**
	 * Anzahl der direkten Fehler von Team A um 1 erhhen
	 */
	public void attackFaultTeamA(){
		this.failedAttacksTeamA++;
		log4j.debug("attack fault A");
	}
	
	/**
	 * Anzahl der direkten Fehler von Team B um 1 erhhen
	 */
	public void attackFaultTeamB(){
		this.failedAttacksTeamB++;
		log4j.debug("attack fault B");
	}
	
	
	
	public int getAttacksTeamA(){
		return this.attacksTeamA;
	}
	
	public int getDirectPointsAttacksTeamA(){
		return this.directPointAttacksTeamA;
	}
	
	public int getFailedAttacksTeamA(){
		return this.failedAttacksTeamA;
	}

	public int getAttacksTeamB(){
		return this.attacksTeamB;
	}
	
	public int getDirectPointsAttacksTeamB(){
		return this.directPointAttacksTeamB;
	}
	
	public int getFailedAttacksTeamB(){
		return this.failedAttacksTeamB;
	}


	public static Logger getLog4j() {
		return log4j;
	}


	public int getDirectPointAttacksTeamA() {
		return directPointAttacksTeamA;
	}


	public int getDirectPointAttacksTeamB() {
		return directPointAttacksTeamB;
	}


	public int getServesTeamA() {
		return servesTeamA;
	}


	public int getFailedServesTeamA() {
		return failedServesTeamA;
	}


	public int getDirectPointServesTeamA() {
		return directPointServesTeamA;
	}


	public int getFieldAttacksTeamA() {
		return fieldAttacksTeamA;
	}


	public int getFailedFieldAttacksTeamA() {
		return failedFieldAttacksTeamA;
	}


	public int getDirectPointFieldAttacksTeamA() {
		return directPointFieldAttacksTeamA;
	}


	public int getServesTeamB() {
		return servesTeamB;
	}


	public int getFailedServesTeamB() {
		return failedServesTeamB;
	}


	public int getDirectPointServesTeamB() {
		return directPointServesTeamB;
	}


	public int getFieldAttacksTeamB() {
		return fieldAttacksTeamB;
	}


	public int getFailedFieldAttacksTeamB() {
		return failedFieldAttacksTeamB;
	}


	public int getDirectPointFieldAttacksTeamB() {
		return directPointFieldAttacksTeamB;
	}
	
}
