package de.ubt.wm.sb.sim;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;

import jxl.read.biff.BiffException;
import jxl.write.WriteException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.gui.AttackType;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.RealStrat;
import de.ubt.wm.sb.model.State;
import de.ubt.wm.sb.model.Technique;
import de.ubt.wm.sb.model.strategy.Strat_AGG_AGG;
import de.ubt.wm.sb.model.strategy.Strat_BASIC_BASIC;
import de.ubt.wm.sb.model.strategy.Strat_DEF_DEF;
import de.ubt.wm.sb.probs.ProbType;
import de.ubt.wm.sb.probs.Prob_Hit;
import de.ubt.wm.sb.probs.Prob_IndPlayer;

public class MainSim {

	static final Logger log4j = LogManager.getLogger(MainSim.class.getName());
	
	public static void main(String[] args) throws BiffException, IOException {
		
//		threeSingleSimulations(1000);

		setUpSimulationSeries(args[0]);
					
	}
	
	public static void threeSingleSimulations(int n){
		Game game = Game.getInstance();
		
		for (RealStrat stratTeamA : Arrays.asList(Strat_AGG_AGG.getInstanceTeamA(), Strat_DEF_DEF.getInstanceTeamA(), Strat_BASIC_BASIC.getInstanceTeamA())){
			game.initGame("Germany", "Brink", "Reckerman","data/allMatches/SpielerA1_PaperInputData_allMatches.xls","data/allMatches/SpielerA1_PaperInputData_allMatches.xls", stratTeamA,
					"France", "Alison", "Emanuel", "data/allMatches/SpielerA1_PaperInputData_allMatches.xls","data/allMatches/SpielerA1_PaperInputData_allMatches.xls");
		
			SimulationManager simManager = startSim(n);
			
			// Nur falls Ausgabe der einzelnen attackStatistics erwuenscht ist
			try {
				simManager.writeResultsToFile("attackStatistics");
			} catch (Exception e) {
				log4j.error("Could not write attack Statistics to Excel File");
				e.printStackTrace();
			}
			System.out.println("Results Summary:\n" + simManager.printResultsSummary());
		}	
	}
	
	/**
	 * 
	 * 
	 * Setzt eine Simulationsreihe mit Parametern auf	
	 */
	public static void setUpSimulationSeries(String inputDataFile){
		Game.getInstance().initGame("Germany", "Brink", "Reckerman",inputDataFile,inputDataFile, Strat_BASIC_BASIC.getInstanceTeamA(),
				"France", "Alison", "Emanuel", inputDataFile,inputDataFile);
		
		int numberOfSets = 5000; // Anzahl der Saetze die simuliert werden sollen		
		Technique tech = Technique.S_JF; // Technik die modifiziert werden sollen
		ArrayList<RealStrat> strats = new ArrayList<RealStrat>(Arrays.asList(Strat_AGG_AGG.getInstanceTeamA(), Strat_DEF_DEF.getInstanceTeamA(), Strat_BASIC_BASIC.getInstanceTeamA())); // Strategien die gegen�bergestellt werden sollen
		ArrayList<Player> parameterizedPlayers = new ArrayList<Player>(Arrays.asList(Game.getInstance().getTeamA().getPlayer1(), Game.getInstance().getTeamA().getPlayer2()));
		
		ResultSimSeries_new resultSimSeries = simulationSeriesSMDP_new(tech, parameterizedPlayers, numberOfSets, 0, 0.1, 1, strats);
		
		try {
			resultSimSeries.writeToFile();
		} catch (WriteException e) {  
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public static ResultSimSeries_new simulationSeriesSMDP_new(Technique tech, ArrayList<Player> players, int numberOfSets, double startValue, double stepSize, double endValue, ArrayList<RealStrat> strats){
		
		Game game = Game.getInstance();

		HashMap<Player, Prob_IndPlayer> inputProbs = new HashMap<Player, Prob_IndPlayer>();
		// ACHTUNG: Kopie ist wichtig, damit die "Ausgangsdaten" am Ende ausgegeben werden k�nnen!
		inputProbs.put(game.getTeamA().getPlayer1(), new Prob_IndPlayer(game.getTeamA().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamA().getPlayer2(), new Prob_IndPlayer(game.getTeamA().getPlayer2().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer1(), new Prob_IndPlayer(game.getTeamB().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer2(), new Prob_IndPlayer(game.getTeamB().getPlayer2().getIndProbs()));

		// alle zur Technik gehoerenden WKeiten raussuchen 
		ArrayList<Prob_Hit> parameterizedProbs = new ArrayList<Prob_Hit>();
		parameterizedProbs.addAll(game.getTeamA().getPlayer1().getIndProbs().getAllProbs(tech));
		
		ResultSimSeries_new resultSimSeries = new ResultSimSeries_new(tech, startValue, stepSize, endValue, numberOfSets, strats, players, parameterizedProbs, inputProbs);
		
		// alle Kombinationen der Erfolgs und Fehlerwkeit durchgehen
		for (double p_ind = startValue; p_ind<=endValue + 0.00000000001; p_ind=p_ind+stepSize){
				
			for (double p_ind_fault = startValue; p_ind+p_ind_fault < endValue + 0.000000001; p_ind_fault=p_ind_fault+stepSize){
			
				// f�r alle Strategien, die simuliert werden sollen
				for (int i = 0; i < strats.size(); i++){
				
					ResultSim_new result = new ResultSim_new();
					result.setP_ind(p_ind);
					result.setP_ind_fault(p_ind_fault);
					
					// modify all Serving Probabilities of Team A
					for (Prob_Hit prob : parameterizedProbs){
						for (Player player : players){
							prob.setAllThreeProbs(p_ind, p_ind_fault);
							player.getIndProbs().overrideProbHit(prob);								
						}
					}
										
					game.getTeamA().setStrat(strats.get(i));			
					result.setStratTeamA(game.getTeamA().getStrat());
					result.setStratTeamB(game.getTeamB().getStrat());
					
					SimulationManager simManager = startSim(numberOfSets);
					result.setNumberOfSets(numberOfSets);
					
					result.setP_serve(simManager.getServeP());
					result.setpOverline_serve(simManager.getServePOverline());
					result.setQ_serve(simManager.getServeQ());
					result.setqOverline_serve(simManager.getServeQOverline());
					
					result.setP_fA(simManager.getfAP());
					result.setpOverline_fA(simManager.getfAPOverline());
					result.setQ_fA(simManager.getfAQ());
					result.setqOverline_fA(simManager.getfAQOverline());
			
					resultSimSeries.addResult(result);
					System.out.println(result.toString());					
				}
			}
		}
		
		return resultSimSeries;
	}
	

	public static ResultSimSeries simulationSeriesSMDP(HashSet<Technique> techs, ArrayList<Player> parameterizedPlayers, int numberOfSets, double startValue, double stepSize, double endValue, ArrayList<RealStrat> strats){
		
		Game game = Game.getInstance();
				
		HashMap<Player, Prob_IndPlayer> inputProbs = new HashMap<Player, Prob_IndPlayer>();
		// ACHTUNG: Kopie ist wichtig, damit die "Ausgangsdaten" am Ende ausgegeben werden k�nnen!
		inputProbs.put(game.getTeamA().getPlayer1(), new Prob_IndPlayer(game.getTeamA().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamA().getPlayer2(), new Prob_IndPlayer(game.getTeamA().getPlayer2().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer1(), new Prob_IndPlayer(game.getTeamB().getPlayer1().getIndProbs()));
		inputProbs.put(game.getTeamB().getPlayer2(), new Prob_IndPlayer(game.getTeamB().getPlayer2().getIndProbs()));
		
		ArrayList<Prob_Hit> parameterizedProbs = new ArrayList<Prob_Hit>();
		for (Technique tech : techs){
			parameterizedProbs.addAll(game.getTeamA().getPlayer1().getIndProbs().getAllProbs(tech));
		}
		
		ResultSimSeries resultSimSeries = new ResultSimSeries(techs,  numberOfSets, startValue, stepSize, endValue, parameterizedProbs, parameterizedPlayers, inputProbs, strats);
		
		// es wird von der vorgegebenen Technik nacheinander alle Wkeiten (succ, fault, neighbour) parameterisiert.
		for (ProbType probType : ProbType.values()){
		
			// f�r alle Parameterwerte die simuliert werden sollen
			for (double d = startValue; d<=endValue + 0.00000000001; d=d+stepSize){
				
				// f�r alle Strategien, die simuliert werden sollen
				for (int i = 0; i < strats.size(); i++){
				
					ResultSim result = new ResultSim();
					result.setProbType(probType);
					
					// modify all Serving Probabilities of Team A
					result.setParameterizedValue(d);
					for (Prob_Hit prob : parameterizedProbs){
						for (Player player : parameterizedPlayers){
							if (probType == ProbType.succ){
								prob.setSuccProb(d);
								player.getIndProbs().overrideProbHit(prob);								
							}
							if (probType == ProbType.fault){
								prob.setFaultProb(d);
								player.getIndProbs().overrideProbHit(prob);								
							}
							if (probType == ProbType.neighbour){
								prob.setNeighbourProb(d);
								player.getIndProbs().overrideProbHit(prob);								
							}
						}
					}
					log4j.debug(game.getTeamA().getPlayer1().getIndProbs().printAnalogToExcelInputFormat(game.getTeamA().getPlayer1()));
					
					game.getTeamA().setStrat(strats.get(i));			
					result.setStratTeamA(game.getTeamA().getStrat());
					result.setStratTeamB(game.getTeamB().getStrat());
					
					SimulationManager simManager = startSim(numberOfSets);
					result.setNumberOfSets(numberOfSets);
					
					result.setP_serve(simManager.getServeP());
					result.setpOverline_serve(simManager.getServePOverline());
					result.setQ_serve(simManager.getServeQ());
					result.setqOverline_serve(simManager.getServeQOverline());
					
					result.setP_fA(simManager.getfAP());
					result.setpOverline_fA(simManager.getfAPOverline());
					result.setQ_fA(simManager.getfAQ());
					result.setqOverline_fA(simManager.getfAQOverline());
			
					resultSimSeries.addResult(result);
					log4j.info(result.toString());					
				}
			}
		}
		
		return resultSimSeries;
	}
	
	/**
	 * @param Anzahl der zu spielenden Sets
	 */
	public static SimulationManager startSim(int numberOfSets){		
		
		int fA_B = 0;
		int fA_point_B = 0;
		
		int total_dm_A = 0;
		int fault_dm_A = 0;
		
		int total_d_A = 0;
		int fault_d_A = 0;		
		
		int total_block_A = 0;
		
		int total_am_B = 0;
		
		// Objekte besorgen die man braucht f�r das Spiel
		Game game = Game.getInstance();
		// legt ein neues Simulationobjekt an
		SimulationManager simManager = new SimulationManager();
		
		// Angriffstyp
		AttackType attackType =  null;	
				
		// Spiel
		while (simManager.getWonSetsTeamA()+simManager.getWonSetsTeamB() < numberOfSets){		
			log4j.debug("neuer Satz; gespielte Saetze bis hier " + (simManager.getWonSetsTeamA()+simManager.getWonSetsTeamB()));
					
			// zaehlt fuer einen Satz die Angriffsstatistik
			AttackCounter counter = new AttackCounter();
			game.resetGame();		
			
			// Satz
			while ((game.getPointsTeamA()<21 && game.getPointsTeamB() < 21) || (Math.abs(game.getPointsTeamA()-game.getPointsTeamB())<2) ) {		
						
				log4j.debug("neuer Ballwechsel; aktueller Punktestand " + game.getPointsTeamA() + " : " + game.getPointsTeamB());
						
				// Ballwechsel - d.h. solange bis ein Punkt erzielt wurde
				while (game.getCurrentState() != State.POINT_A && game.getCurrentState() != State.POINT_B) {
							
					// Anzahl der Angriffe z�hlen
					// Angriff: Serve
					if (game.getCurrentState() == State.START_STATE_A){
						counter.newAttackTeamA();
						counter.newServeTeamA();
						attackType = AttackType.SERVE_A;
						log4j.debug("Neuer Angriff A - Typ: Serve");		
					}
					if (game.getCurrentState() == State.START_STATE_B){
						counter.newAttackTeamB();
						counter.newServeTeamB();
						attackType = AttackType.SERVE_B;
						log4j.debug("Neuer Angriff B - Typ: Serve");
					}
							
					game.chooseTeamActions();
					log4j.debug("Team Action A: " + game.getTeamA().getTeamAction().toString());
					log4j.debug("Team Action B: " + game.getTeamB().getTeamAction().toString());
					
					if (game.getTeamA().getTeamAction().getActionPlayer1().getHit().getTechnique() == Technique.d_m 
							|| game.getTeamA().getTeamAction().getActionPlayer2().getHit().getTechnique() == Technique.d_m)
						total_dm_A++;
					
					if (game.getTeamA().getTeamAction().getActionPlayer1().getHit().getTechnique() == Technique.d 
							|| game.getTeamA().getTeamAction().getActionPlayer2().getHit().getTechnique() == Technique.d)
						total_d_A++;
					
					if (game.getTeamA().getPlayer1().getPlayerAction().getMove().isBlockingAction() 
							|| game.getTeamA().getPlayer2().getPlayerAction().getMove().isBlockingAction())
						total_block_A++;
					
					if (game.getTeamB().getTeamAction().getActionPlayer1().getHit().getTechnique() == Technique.a_m 
							|| game.getTeamB().getTeamAction().getActionPlayer2().getHit().getTechnique() == Technique.a_m)
						total_am_B++;					
					
					
					// Angriff: aus dem Spiel
					// ACHTUNG: beginnt erst, wenn Ball erfolgreich angenommen wurde, d.h. Counter = 1
					// und das Team einen weiteren Schlag zum Stellen spielen kann
					if (game.getCurrentState().getCounter() == 1) {
						if (game.getCurrentState().getFieldBall().belongsToSideA() && 
								game.getTeamA().makesHit()){ 
							counter.newAttackTeamA();
							counter.newFieldAttackTeamA();
							attackType = AttackType.FIELD_ATTACK_A;
							log4j.debug("Neuer Angriff A - Typ: aus dem Spiel");
						}
						if (game.getCurrentState().getFieldBall().belongsToSideB()&& 
								game.getTeamB().makesHit()) { 
							counter.newAttackTeamB();
							counter.newFieldAttackTeamB();
							attackType = AttackType.FIELD_ATTACK_B;
							log4j.debug("Neuer Angriff B - Typ: aus dem Spiel");
							fA_B++;
						}
					}
																		
					game.computeNextState();
					log4j.debug("Following state: " + game.getCurrentState());

				} // end while loop - Ballwechsel
			
				log4j.debug(game.printScore());
	
				if (game.getCurrentState() == State.POINT_A) {
					switch (attackType){
						case SERVE_A:
							log4j.debug("point after serve A");
							counter.directPointTeamA();
							counter.directPointServesTeamA();
							break;
						
						case FIELD_ATTACK_A:
							log4j.debug("point after field attack A");
							counter.directPointTeamA();
							counter.directPointFieldAttackTeamA();
							break;
							
						case SERVE_B:
							log4j.debug("fault after serve B");
							counter.attackFaultTeamB();
							counter.attackFaultServesTeamB();
							break;
							
						case FIELD_ATTACK_B:
							log4j.debug("fault after field attack B");
							counter.attackFaultTeamB();
							counter.attackFaultFieldAttackTeamB();
							break;
							
						default:
							log4j.warn("AttackType nicht gekannt!");
							break;
					}
						

					game.setEmptyTeamActions();
					game.setCurrentState(State.START_STATE_A);
				}
				
				if (game.getCurrentState() == State.POINT_B) {
					
					switch (attackType){
						case SERVE_A:
							log4j.debug("fault after serve A");
							counter.attackFaultTeamA();
							counter.attackFaultServesTeamA();
							break;
							
						case FIELD_ATTACK_A:
							log4j.debug("fault after field attack A");
							counter.attackFaultTeamA();
							counter.attackFaultFieldAttackTeamA();
							break;
							
						case SERVE_B:
							log4j.debug("point after serve B");
							counter.directPointTeamB();
							counter.directPointServesTeamB();
							break;
							
						case FIELD_ATTACK_B:
							log4j.debug("point after field attack B");
							counter.directPointTeamB();
							counter.directPointFieldAttackTeamB();
							
							fA_point_B++;
							
							if (game.getTeamA().getTeamAction().getActionPlayer1().getHit().getTechnique() == Technique.d_m 
									|| game.getTeamA().getTeamAction().getActionPlayer2().getHit().getTechnique() == Technique.d_m)
								fault_dm_A++;
							
							if (game.getTeamA().getTeamAction().getActionPlayer1().getHit().getTechnique() == Technique.d 
									|| game.getTeamA().getTeamAction().getActionPlayer2().getHit().getTechnique() == Technique.d)
								fault_d_A++;
							break;		
							
						default:
							log4j.warn("AttackType nicht gekannt!");
							break;
					}
					game.setEmptyTeamActions();
					game.setCurrentState(State.START_STATE_B);
				}

			
			} // end while loop - ein Set
					
					
			if (game.getPointsTeamA()>game.getPointsTeamB()){
				simManager.increaseWonSetsTeamA();
			}else
			{				
				simManager.increaseWonSetsTeamB();
			}
				
			simManager.addCounterResultToSimManager(counter);
				
		} // end while loop - number of Sets

		simManager.computeSummary();
		
		log4j.debug(game.getTeamA().getStrat().toString());
		log4j.debug("Number of FA Team B:" + fA_B);
		log4j.debug("Points after FA Team B:" + fA_point_B);
		log4j.debug("Number of attacks middle Team B" + total_am_B);
		log4j.debug("Number of used d_m Team A" + total_dm_A);
		log4j.debug("Faults of used d_m Team A" + fault_dm_A);
		log4j.debug("Number of used d Team A" + total_d_A);
		log4j.debug("Faults of used d Team A" + fault_d_A);
		log4j.debug("Number of blocks Team A" + total_block_A);
		
		return simManager;		
	}

}
