package de.ubt.wm.sb.probs;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Random;

import jxl.NumberCell;
import jxl.Sheet;
import jxl.Workbook;
import jxl.read.biff.BiffException;
import de.ubt.wm.sb.model.BallProperty;
import de.ubt.wm.sb.model.Field;
import de.ubt.wm.sb.model.Game;
import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.Technique;
import de.ubt.wm.sb.model.TechniqueTest;

public class Prob_IndPlayer {

	private HashMap<Technique, HashMap <Field, Prob_Hit>> servingProbs; // Technique -> Target Field -> ProbHit
	private HashMap<Technique, HashMap <Field, HashMap <Field, Prob_Hit>>> attackingProbs; // Technique -> Player Field -> Target Field -> Prob_Hit
	private HashMap<Technique, HashMap <BallProperty, Prob_Hit>> receivingProbs;
	private HashMap<Technique, HashMap <BallProperty, Prob_Hit>> defendingProbs;
	private Prob_Hit settingProb;
	private HashMap<Technique, Prob_Hit> blockingProb;
	
	
	public Prob_IndPlayer() {
		super();
		this.servingProbs = new HashMap<Technique, HashMap <Field, Prob_Hit>>();
		this.attackingProbs = new HashMap<Technique, HashMap <Field, HashMap <Field, Prob_Hit>>>();
		this.receivingProbs = new HashMap<Technique, HashMap <BallProperty, Prob_Hit>>();
		this.defendingProbs = new HashMap<Technique, HashMap <BallProperty, Prob_Hit>>();
		this.blockingProb = new HashMap<Technique, Prob_Hit>();
	}


	public Prob_IndPlayer(Prob_IndPlayer indProbs) {
		this.servingProbs = new HashMap<Technique, HashMap <Field, Prob_Hit>>();
		for (Technique t : indProbs.servingProbs.keySet()){
			HashMap <Field, Prob_Hit> fieldProbMap = new HashMap<Field, Prob_Hit>();
			for (Field f : indProbs.servingProbs.get(t).keySet()){
				fieldProbMap.put(f, new Prob_Hit(indProbs.servingProbs.get(t).get(f)));
			}
			this.servingProbs.put(t, fieldProbMap);
		}
		this.attackingProbs = new HashMap<Technique, HashMap <Field, HashMap <Field, Prob_Hit>>>();
		for (Technique t : indProbs.attackingProbs.keySet()){
			HashMap <Field, HashMap <Field, Prob_Hit>> fieldfieldProb = new HashMap <Field, HashMap <Field, Prob_Hit>>();
			for (Field f1 : indProbs.attackingProbs.get(t).keySet()){
				HashMap<Field, Prob_Hit> fieldProb = new HashMap<Field, Prob_Hit>();
				for (Field f2 :  indProbs.attackingProbs.get(t).get(f1).keySet()){
					fieldProb.put(f2, new Prob_Hit(indProbs.attackingProbs.get(t).get(f1).get(f2)));
				}
				fieldfieldProb.put(f1, fieldProb);
			}
			this.attackingProbs.put(t, fieldfieldProb);
		}
		
		this.receivingProbs = new HashMap<Technique, HashMap <BallProperty, Prob_Hit>>();
		for (Technique t: indProbs.receivingProbs.keySet()){
			HashMap<BallProperty, Prob_Hit> ballProb = new HashMap<BallProperty, Prob_Hit>();
			for (BallProperty b : indProbs.receivingProbs.get(t).keySet()){
				ballProb.put(b, new Prob_Hit(indProbs.receivingProbs.get(t).get(b)));
			}
			this.receivingProbs.put(t, ballProb);
		}
		
		this.defendingProbs = new HashMap<Technique, HashMap <BallProperty, Prob_Hit>>();
		for (Technique t: indProbs.defendingProbs.keySet()){
			HashMap<BallProperty, Prob_Hit> ballProb = new HashMap<BallProperty, Prob_Hit>();
			for (BallProperty b : indProbs.defendingProbs.get(t).keySet()){
				ballProb.put(b, new Prob_Hit(indProbs.defendingProbs.get(t).get(b)));
			}
			this.defendingProbs.put(t, ballProb);
		}
		this.blockingProb = new HashMap<Technique, Prob_Hit>();
		for (Technique t : indProbs.blockingProb.keySet()){
			blockingProb.put(t, new Prob_Hit(indProbs.blockingProb.get(t)));
		}
	}

	public void readFromExcelTemplate(String filename, long seed, Player player) throws BiffException, IOException {
		// Einleseroutine angepasst auf Excel Sheet PaperInputData_allMatches.xls
				
		Workbook workbook = Workbook.getWorkbook(new File(filename));
		Sheet sheet = workbook.getSheet(0);
		
		NumberCell cellEps = (NumberCell) sheet.getCell("N5");
		double eps = cellEps.getValue();
		
		Random r = new Random(seed);
		
		if (player.getTeam() == Game.getInstance().getTeamA()){
			// Serves
			Prob_Serve probServe;
			for(Field targetField : Field.LAST_ROW_B){
				probServe = new Prob_Serve(Technique.S_T,
						((NumberCell) sheet.getCell("F10")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G10")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_F,
						((NumberCell) sheet.getCell("F11")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G11")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_J,
						((NumberCell) sheet.getCell("F12")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G12")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_JF,
						((NumberCell) sheet.getCell("F13")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G13")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
			}
			
			for(Field targetField : Field.MIDDLE_ROW_B){
				probServe = new Prob_Serve(Technique.S_T,
						((NumberCell) sheet.getCell("I10")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J10")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_F,
						((NumberCell) sheet.getCell("I11")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J11")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_J,
						((NumberCell) sheet.getCell("I12")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J12")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_JF,
						((NumberCell) sheet.getCell("I13")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J13")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
			}
			
			
			// Smash
			Prob_Attack probAttack;
			for (Field playerField : Field.LAST_ROW_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F17")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G17")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I17")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J17")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.MIDDLE_ROW_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F18")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G18")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I18")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J18")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.AT_THE_NET_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F19")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G19")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I19")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J19")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.OUT_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F20")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G20")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I20")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J20")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			
			
			// Shot
			for (Field playerField : Field.LAST_ROW_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F21")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G21")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I21")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J21")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.MIDDLE_ROW_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F22")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G22")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I22")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J22")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.AT_THE_NET_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F23")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G23")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I23")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J23")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.OUT_A){
				for (Field targetField : Field.LAST_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F24")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G24")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_B){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I24")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J24")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			
			// Poke
			for (Field playerField : Field.LAST_ROW_A){
				for (Field targetField : Field.AT_THE_NET_B){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L25")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M25")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}		
			}
			for (Field playerField : Field.MIDDLE_ROW_A){
				for (Field targetField : Field.AT_THE_NET_B){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L26")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M26")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}		
			}
			for (Field playerField : Field.AT_THE_NET_A){
				for (Field targetField : Field.AT_THE_NET_B){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L27")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M27")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
			}
			for (Field playerField : Field.OUT_A){
				for (Field targetField : Field.AT_THE_NET_B){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L28")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M28")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}	
			}
			
			// Defence
			Prob_DefRec probDef = new Prob_DefRec(Technique.d,
					((NumberCell) sheet.getCell("F34")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G34")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probDef);
			probDef = new Prob_DefRec(Technique.d,
					((NumberCell) sheet.getCell("I34")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J34")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probDef);
			probDef = new Prob_DefRec(Technique.d_m,
					((NumberCell) sheet.getCell("F35")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G35")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probDef);
			probDef = new Prob_DefRec(Technique.d_m,
					((NumberCell) sheet.getCell("I35")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J35")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probDef);
			
			// Receive
			Prob_DefRec probRec = new Prob_DefRec(Technique.r,
					((NumberCell) sheet.getCell("F36")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G36")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probRec);
			probRec = new Prob_DefRec(Technique.r,
					((NumberCell) sheet.getCell("I36")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J36")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probRec);
			probRec = new Prob_DefRec(Technique.r_m,
					((NumberCell) sheet.getCell("F37")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G37")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probRec);
			probRec = new Prob_DefRec(Technique.r_m,
					((NumberCell) sheet.getCell("I37")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J37")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probRec);
			
			// set
			Prob_Hit probSet = new Prob_Hit(Technique.s,
					((NumberCell) sheet.getCell("F38")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G38")).getValue()+r.nextGaussian()*eps);
			this.addProbHit(probSet);
			
			// Block
			Prob_Hit probBlock = new Prob_Hit(Technique.b_l,
					((NumberCell) sheet.getCell("F44")).getValue()+r.nextGaussian()*eps,
					1-((NumberCell) sheet.getCell("F44")).getValue()+r.nextGaussian()*eps);
			this.addProbHit(probBlock);
			probBlock = new Prob_Hit(Technique.b_d,
					((NumberCell) sheet.getCell("F45")).getValue()+r.nextGaussian()*eps,
					1-((NumberCell) sheet.getCell("F45")).getValue()+r.nextGaussian()*eps);
			this.addProbHit(probBlock);	
		
		}else if (player.getTeam() == Game.getInstance().getTeamB()){
			// Serves
			Prob_Serve probServe;
			for(Field targetField : Field.LAST_ROW_A){
				probServe = new Prob_Serve(Technique.S_T,
						((NumberCell) sheet.getCell("F10")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G10")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_F,
						((NumberCell) sheet.getCell("F11")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G11")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_J,
						((NumberCell) sheet.getCell("F12")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G12")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_JF,
						((NumberCell) sheet.getCell("F13")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("G13")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
			}
			
			for(Field targetField : Field.MIDDLE_ROW_A){
				probServe = new Prob_Serve(Technique.S_T,
						((NumberCell) sheet.getCell("I10")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J10")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_F,
						((NumberCell) sheet.getCell("I11")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J11")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_J,
						((NumberCell) sheet.getCell("I12")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J12")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
				probServe = new Prob_Serve(Technique.S_JF,
						((NumberCell) sheet.getCell("I13")).getValue()+r.nextGaussian()*eps,
						((NumberCell) sheet.getCell("J13")).getValue()+r.nextGaussian()*eps,
						targetField);
				this.addProbHit(probServe);
			}
			
			
			// Smash
			Prob_Attack probAttack;
			for (Field playerField : Field.LAST_ROW_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F17")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G17")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I17")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J17")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.MIDDLE_ROW_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F18")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G18")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I18")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J18")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.AT_THE_NET_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F19")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G19")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I19")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J19")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.OUT_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("F20")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G20")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_h, 
							((NumberCell) sheet.getCell("I20")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J20")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			
			
			// Shot
			for (Field playerField : Field.LAST_ROW_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F21")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G21")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I21")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J21")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.MIDDLE_ROW_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F22")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G22")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I22")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J22")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.AT_THE_NET_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F23")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G23")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I23")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J23")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			for (Field playerField : Field.OUT_B){
				for (Field targetField : Field.LAST_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("F24")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("G24")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
				for (Field targetField : Field.MIDDLE_ROW_A){
					probAttack = new Prob_Attack(Technique.a_m, 
							((NumberCell) sheet.getCell("I24")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("J24")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}			
			}
			
			// Poke
			for (Field playerField : Field.LAST_ROW_B){
				for (Field targetField : Field.AT_THE_NET_A){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L25")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M25")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}		
			}
			for (Field playerField : Field.MIDDLE_ROW_B){
				for (Field targetField : Field.AT_THE_NET_A){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L26")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M26")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}		
			}
			for (Field playerField : Field.AT_THE_NET_B){
				for (Field targetField : Field.AT_THE_NET_A){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L27")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M27")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}
			}
			for (Field playerField : Field.OUT_B){
				for (Field targetField : Field.AT_THE_NET_A){
					probAttack = new Prob_Attack(Technique.a_s, 
							((NumberCell) sheet.getCell("L28")).getValue()+r.nextGaussian()*eps,
							((NumberCell) sheet.getCell("M28")).getValue()+r.nextGaussian()*eps, targetField, playerField);
					this.addProbHit(probAttack);
				}	
			}
			
			// Defence
			Prob_DefRec probDef = new Prob_DefRec(Technique.d,
					((NumberCell) sheet.getCell("F34")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G34")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probDef);
			probDef = new Prob_DefRec(Technique.d,
					((NumberCell) sheet.getCell("I34")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J34")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probDef);
			probDef = new Prob_DefRec(Technique.d_m,
					((NumberCell) sheet.getCell("F35")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G35")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probDef);
			probDef = new Prob_DefRec(Technique.d_m,
					((NumberCell) sheet.getCell("I35")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J35")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probDef);
			
			// Receive
			Prob_DefRec probRec = new Prob_DefRec(Technique.r,
					((NumberCell) sheet.getCell("F36")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G36")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probRec);
			probRec = new Prob_DefRec(Technique.r,
					((NumberCell) sheet.getCell("I36")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J36")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probRec);
			probRec = new Prob_DefRec(Technique.r_m,
					((NumberCell) sheet.getCell("F37")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G37")).getValue()+r.nextGaussian()*eps, 
					BallProperty.NORMAL);
			this.addProbHit(probRec);
			probRec = new Prob_DefRec(Technique.r_m,
					((NumberCell) sheet.getCell("I37")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("J37")).getValue()+r.nextGaussian()*eps, 
					BallProperty.HARD);
			this.addProbHit(probRec);
			
			// set
			Prob_Hit probSet = new Prob_Hit(Technique.s,
					((NumberCell) sheet.getCell("F38")).getValue()+r.nextGaussian()*eps,
					((NumberCell) sheet.getCell("G38")).getValue()+r.nextGaussian()*eps);
			this.addProbHit(probSet);
			
			// Block
			Prob_Hit probBlock = new Prob_Hit(Technique.b_l,
					((NumberCell) sheet.getCell("F44")).getValue()+r.nextGaussian()*eps,
					1-((NumberCell) sheet.getCell("F44")).getValue()+r.nextGaussian()*eps);
			this.addProbHit(probBlock);
			probBlock = new Prob_Hit(Technique.b_d,
					((NumberCell) sheet.getCell("F45")).getValue()+r.nextGaussian()*eps,
					1-((NumberCell) sheet.getCell("F45")).getValue()+r.nextGaussian()*eps);
			this.addProbHit(probBlock);	
			
		}else{
			System.err.println("Team not known in the game! Can't read Input Probs.");
			System.exit(1);
		}
	}
	
	public HashSet<Prob_Hit> getAllProbs(Technique tech){
		TechniqueTest techTest = new TechniqueTest(tech);
		HashSet<Prob_Hit> returnSet = new HashSet<Prob_Hit>();
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			returnSet.addAll(servingProbs.get(tech).values());
			return returnSet;
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){		
			for (Field f : attackingProbs.get(tech).keySet()){
				returnSet.addAll(attackingProbs.get(tech).get(f).values());
			}
			return returnSet;	
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			returnSet.addAll(receivingProbs.get(tech).values());
			return returnSet;
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			returnSet.addAll(defendingProbs.get(tech).values());
		}
		
		// Setting
		if (techTest.isSettingTechnique()){		
			returnSet.add(settingProb);
			return returnSet;
		}
		
		// Blocking
		if (techTest.isBlockingTechnique()){
			returnSet.add(blockingProb.get(tech));
			return returnSet;
		}
		
		System.err.println("No Prob_Hit for " + tech + " gefunden!");
		return returnSet;
	}
	
	public Prob_Hit getProbHit(Technique tech, Field targetfield, Field playerfield, BallProperty strength){
		TechniqueTest techTest = new TechniqueTest(tech);
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			return servingProbs.get(tech).get(targetfield);
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){
			return attackingProbs.get(tech).get(playerfield).get(targetfield);	
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			return receivingProbs.get(tech).get(strength);
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			return defendingProbs.get(tech).get(strength);
		}
		
		// Setting
		if (techTest.isSettingTechnique()){
			return settingProb;
		}
		
		// Blocking
		if (techTest.isBlockingTechnique()){
			return blockingProb.get(tech);
		}
		
		System.err.println("No Prob_Hit for " + tech + " target: " + targetfield + " player: " + playerfield + " strength " + strength.toString() + " gefunden!");
		return null;
	}
	
	public void overrideProbHit(Prob_Hit prob){
		TechniqueTest techTest = new TechniqueTest(prob.getTech());
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			if (prob instanceof Prob_Serve){
				Prob_Serve probServe = (Prob_Serve) prob;
				if (servingProbs.get(probServe.getTech()) == null){
					HashMap <Field, Prob_Hit> map = new HashMap <Field, Prob_Hit>();
					map.put(probServe.getTargetField(), probServe);
					servingProbs.put(prob.getTech(), map);
				}else if (servingProbs.get(prob.getTech()).get(probServe.getTargetField())!=null){
					servingProbs.get(prob.getTech()).put(probServe.getTargetField(), probServe);
				}else{
					System.err.println("Achtung Prob " + servingProbs.get(prob.getTech()).get(probServe.getTargetField()).toString() + " existiert noch nicht! Kann nicht �berschrieben werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Serving Technique " + prob.getTech().toString() + " but not instance of Prob_Serve!");
			}
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){
			if (prob instanceof Prob_Attack){
				Prob_Attack probAttack = (Prob_Attack) prob;
				if (attackingProbs.get(prob.getTech()) == null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					HashMap<Field, HashMap<Field, Prob_Hit>> mapPlayerField = new HashMap<Field, HashMap<Field, Prob_Hit>>();
					mapPlayerField.put(probAttack.getPlayerField(), mapTargetField);
					attackingProbs.put(probAttack.getTech(), mapPlayerField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField())==null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					attackingProbs.get(probAttack.getTech()).put(probAttack.getPlayerField(), mapTargetField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).get(probAttack.getTargetField()) != null){
					attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).put(probAttack.getTargetField(), prob);				
				}else{
					System.err.println("Achtung Prob " + attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).get(probAttack.getTargetField()).toString() 
							+ " existiert noch nicht! Kann nicht �berschrieben werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Attacking Technique " + prob.getTech().toString() + " but not instance of Prob_Attack!");
			}			
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (receivingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					receivingProbs.put(prob_DefRec.getTech(), map);
				}else if (receivingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) != null){
					receivingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				}else{
					System.err.println("Achtung Prob " + receivingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()).toString() 
						+ " existiert noch nicht! Kann nicht �berschrieben werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Reception Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (defendingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					defendingProbs.put(prob_DefRec.getTech(), map);
				}else if(defendingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) != null){
					defendingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				} else {
					System.err.println("Achtung Prob " + defendingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()).toString() 
							+ " existiert noch nicht! Kann nicht �berschrieben werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Defending Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Setting
		if (techTest.isSettingTechnique()){
			if (this.settingProb != null){
				this.settingProb = prob;
			}else{
				System.err.println("Achtung Prob " + this.settingProb.toString() 
						+ " existiert noch nicht! Kann nicht �berschrieben werden!");
				System.exit(1);
			}
		}
		
		// Blocking
		if (techTest.isBlockingTechnique()){
			if (this.blockingProb.get(prob.getTech()) != null){
			this.blockingProb.put(prob.getTech(), prob);				
			}else{
				System.err.println("Achtung Prob " + this.blockingProb.toString() 
						+ " existiert noch nicht! Kann nicht �berschrieben werden!");
				System.exit(1);
			}
		}
		
		
	}
	
	public void addProbHit(Prob_Hit prob){
		TechniqueTest techTest = new TechniqueTest(prob.getTech());
		
		// Aufschlagswkeit
		if (techTest.isServingTechnique()){
			if (prob instanceof Prob_Serve){
				Prob_Serve probServe = (Prob_Serve) prob;
				if (servingProbs.get(probServe.getTech()) == null){
					HashMap <Field, Prob_Hit> map = new HashMap <Field, Prob_Hit>();
					map.put(probServe.getTargetField(), probServe);
					servingProbs.put(prob.getTech(), map);
				}else if (servingProbs.get(prob.getTech()).get(probServe.getTargetField())==null){
					servingProbs.get(prob.getTech()).put(probServe.getTargetField(), probServe);
				}else{
					System.err.println("Achtung Prob " + servingProbs.get(prob.getTech()).get(probServe.getTargetField()).toString() + " existiert schon! Kann nicht hinzgef�gt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Serving Technique " + prob.getTech().toString() + " but not instance of Prob_Serve!");
			}
		}
		
		// Angriffswkeit
		if (techTest.isAttackingTechnique()){
			if (prob instanceof Prob_Attack){
				Prob_Attack probAttack = (Prob_Attack) prob;
				if (attackingProbs.get(prob.getTech()) == null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					HashMap<Field, HashMap<Field, Prob_Hit>> mapPlayerField = new HashMap<Field, HashMap<Field, Prob_Hit>>();
					mapPlayerField.put(probAttack.getPlayerField(), mapTargetField);
					attackingProbs.put(probAttack.getTech(), mapPlayerField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField())==null){
					HashMap<Field, Prob_Hit> mapTargetField = new HashMap<Field, Prob_Hit>();
					mapTargetField.put(probAttack.getTargetField(), probAttack);
					attackingProbs.get(probAttack.getTech()).put(probAttack.getPlayerField(), mapTargetField);
				}else if (attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).get(probAttack.getTargetField()) == null){
					attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).put(probAttack.getTargetField(), prob);				
				}else{
					System.err.println("Achtung Prob " + attackingProbs.get(probAttack.getTech()).get(probAttack.getPlayerField()).get(probAttack.getTargetField()).toString() 
							+ " existiert schon! Kann nicht hinzgef�gt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Attacking Technique " + prob.getTech().toString() + " but not instance of Prob_Attack!");
			}			
		}
		
		// Reception
		if (techTest.isReceptionTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (receivingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					receivingProbs.put(prob_DefRec.getTech(), map);
				}else if (receivingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) == null){
					receivingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				}else{
					System.err.println("Achtung Prob " + receivingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()).toString() 
						+ " existiert schon! Kann nicht hinzgef�gt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Reception Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Defence
		if (techTest.isDefendingTechnique()){
			if (prob instanceof Prob_DefRec){
				Prob_DefRec prob_DefRec = (Prob_DefRec) prob;
				if (defendingProbs.get(prob_DefRec.getTech()) == null){
					HashMap<BallProperty, Prob_Hit> map = new HashMap<BallProperty, Prob_Hit>();
					map.put(prob_DefRec.getStrength(), prob_DefRec);
					defendingProbs.put(prob_DefRec.getTech(), map);
				}else if(defendingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()) == null){
					defendingProbs.get(prob_DefRec.getTech()).put(prob_DefRec.getStrength(),prob_DefRec);
				} else {
					System.err.println("Achtung Prob " + defendingProbs.get(prob_DefRec.getTech()).get(prob_DefRec.getStrength()).toString() 
							+ " existiert schon! Kann nicht hinzgef�gt werden!");
					System.exit(1);
				}
			}else{
				System.err.println("Defending Technique " + prob.getTech().toString() + " but not instance of Prob_DefRec!");
			}	
		}
		
		// Setting
		if (techTest.isSettingTechnique()){
			if (this.settingProb == null){
				this.settingProb = prob;
			}else{
				System.err.println("Achtung Prob " + this.settingProb.toString() 
						+ " existiert schon! Kann nicht hinzgef�gt werden!");
				System.exit(1);
			}
		}
		
		// Blocking
		if (techTest.isBlockingTechnique()){
			if (this.blockingProb.get(prob.getTech()) == null){
			this.blockingProb.put(prob.getTech(), prob);				
			}else{
				System.err.println("Achtung Prob " + this.blockingProb.toString() 
						+ " existiert schon! Kann nicht hinzgef�gt werden!");
				System.exit(1);
			}
		}
	}
	

	public Prob_Hit getSettingProb() {
		return settingProb;
	}

	/**
	 * Prints IndPlayer Probs as on the PaperInputData Excel Sheet
	 * @return
	 */
	public String printAnalogToExcelInputFormat(Player player){		
		StringBuffer string = new StringBuffer();
		if (player.getTeam() == Game.getInstance().getTeamA()){
			string.append("SERVES: \n");
			string.append(this.getProbHit(Technique.S_T, Field.B11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_T, Field.B21, null, null).toString() + "\n");
			string.append(this.getProbHit(Technique.S_F, Field.B11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_F, Field.B21, null, null).toString() + "\n");
			string.append(this.getProbHit(Technique.S_J, Field.B11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_J, Field.B21, null, null).toString() + "\n");
			string.append(this.getProbHit(Technique.S_JF, Field.B11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_JF, Field.B21, null, null).toString() + "\n");
	
			string.append("ATTTACKS: \n");
			string.append(this.getProbHit(Technique.a_h, Field.B11, Field.A11, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.B21, Field.A11, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_h, Field.B11, Field.A21, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.B21, Field.A21, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_h, Field.B11, Field.A31, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.B21, Field.A31, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_h, Field.B11, Field.A01, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.B21, Field.A01, null).toString() + "\n");
			
			string.append(this.getProbHit(Technique.a_m, Field.B11, Field.A11, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.B21, Field.A11, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_m, Field.B11, Field.A21, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.B21, Field.A21, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_m, Field.B11, Field.A31, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.B21, Field.A31, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_m, Field.B11, Field.A01, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.B21, Field.A01, null).toString() + "\n");
			
			string.append(this.getProbHit(Technique.a_s, Field.B31, Field.A11, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_s, Field.B31, Field.A21, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_s, Field.B31, Field.A31, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_s, Field.B31, Field.A01, null).toString() + "\n");
			
		}else if(player.getTeam() == Game.getInstance().getTeamB()){
			string.append("SERVES: \n");
			string.append(this.getProbHit(Technique.S_T, Field.A11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_T, Field.A21, null, null).toString() + "\n");
			string.append(this.getProbHit(Technique.S_F, Field.A11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_F, Field.A21, null, null).toString() + "\n");
			string.append(this.getProbHit(Technique.S_J, Field.A11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_J, Field.A21, null, null).toString() + "\n");
			string.append(this.getProbHit(Technique.S_JF, Field.A11, null, null).toString() + "\t"
					+ this.getProbHit(Technique.S_JF, Field.A21, null, null).toString() + "\n");
	
			string.append("ATTTACKS: \n");
			string.append(this.getProbHit(Technique.a_h, Field.A11, Field.B11, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.A21, Field.B11, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_h, Field.A11, Field.B21, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.A21, Field.B21, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_h, Field.A11, Field.B31, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.A21, Field.B31, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_h, Field.A11, Field.B01, null).toString() + "\t"
					+ this.getProbHit(Technique.a_h, Field.A21, Field.B01, null).toString() + "\n");
			
			string.append(this.getProbHit(Technique.a_m, Field.A11, Field.B11, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.A21, Field.B11, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_m, Field.A11, Field.B21, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.A21, Field.B21, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_m, Field.A11, Field.B31, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.A21, Field.B31, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_m, Field.A11, Field.B01, null).toString() + "\t"
					+ this.getProbHit(Technique.a_m, Field.A21, Field.B01, null).toString() + "\n");
			
			string.append(this.getProbHit(Technique.a_s, Field.A31, Field.B11, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_s, Field.A31, Field.B21, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_s, Field.A31, Field.B31, null).toString() + "\n");
			string.append(this.getProbHit(Technique.a_s, Field.A31, Field.B01, null).toString() + "\n");
		}else{
			System.err.println("Team of Player " + player.toString() + " does not belong to the game!");
		}
			
			
			string.append("DEFENCE:\n");
			string.append(this.getProbHit(Technique.d, null, null, BallProperty.NORMAL) + "\t"
					+ this.getProbHit(Technique.d, null, null, BallProperty.HARD) + "\n");
			string.append(this.getProbHit(Technique.d_m, null, null, BallProperty.NORMAL) + "\t"
					+ this.getProbHit(Technique.d_m, null, null, BallProperty.HARD) + "\n");
			
			string.append("RECEIVE:\n");
			string.append(this.getProbHit(Technique.r, null, null, BallProperty.NORMAL) + "\t"
					+ this.getProbHit(Technique.r, null, null, BallProperty.HARD) + "\n");
			string.append(this.getProbHit(Technique.r_m, null, null, BallProperty.NORMAL) + "\t"
					+ this.getProbHit(Technique.r_m, null, null, BallProperty.HARD) + "\n");
			
			string.append("SET:\n");
			string.append(this.getProbHit(Technique.s, null, null, null) + "\n");
			
			string.append("BLOCK:\n");
			string.append(this.getProbHit(Technique.b_l, null, null, null) + "\n");
			string.append(this.getProbHit(Technique.b_d, null, null, null) + "\n");
	
		return string.toString();
	}	
	
//	@Override
//	public String toString() {
//		return "Prob_IndPlayer [servingProbs=" + servingProbs
//				+ ", attackingProbs=" + attackingProbs + ", receivingProbs="
//				+ receivingProbs + ", defendingProbs=" + defendingProbs
//				+ ", settingProb=" + settingProb + ", blockingProb="
//				+ blockingProb + "]";
//	}
	
	
}
