package de.ubt.wm.sb.sim;

import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Locale;

import jxl.CellView;
import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.format.Alignment;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.WritableCellFormat;
import jxl.write.WritableFont;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import jxl.write.biff.RowsExceededException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.model.Player;
import de.ubt.wm.sb.model.RealStrat;
import de.ubt.wm.sb.model.Technique;
import de.ubt.wm.sb.probs.Prob_Hit;
import de.ubt.wm.sb.probs.Prob_IndPlayer;

public class ResultSimSeries_new {
	
	static final Logger log4j = LogManager.getLogger(ResultSimSeries_new.class.getName());

	private Technique tech;
	
	private double startValue;
	private double stepSize;
	private double endValue;
	
	private int numberOfSetsPerSim;
	
	private ArrayList<RealStrat> simulatedStrategies;
	private ArrayList<Player> parameterizedPlayers;
		
	private ArrayList<Prob_Hit> changedProb;
	private HashMap<Player, Prob_IndPlayer> playerInputProbs;
	private HashMap<Double, HashMap<Double, HashMap<RealStrat, ResultSim_new>>> computedResults; // Wert p_ind -> Wert p_ind_fault -> Strat -> ResultSim_new
	
	public ResultSimSeries_new(Technique tech, double startValue,
			double stepSize, double endValue, int numberOfSetsPerSim,
			ArrayList<RealStrat> simulatedStrategies,
			ArrayList<Player> parameterizedPlayers,
			ArrayList<Prob_Hit> changedProb,
			HashMap<Player, Prob_IndPlayer> playerInputProbs) {
		super();
		this.tech = tech;
		this.startValue = startValue;
		this.stepSize = stepSize;
		this.endValue = endValue;
		this.numberOfSetsPerSim = numberOfSetsPerSim;
		this.simulatedStrategies = simulatedStrategies;
		this.parameterizedPlayers = parameterizedPlayers;
		this.changedProb = changedProb;
		this.playerInputProbs = playerInputProbs;
		this.computedResults = new HashMap<Double,HashMap<Double,HashMap<RealStrat,ResultSim_new>>>();
	}




	public void addResult(ResultSim_new result){		
		if (result == null){
			System.err.println("ResultSim ist leer!");
			return;
		}		
		if (computedResults.get(result.getP_ind()) == null){
			HashMap<RealStrat, ResultSim_new> mapStratRes = new HashMap<RealStrat, ResultSim_new>();
			mapStratRes.put(result.getStratTeamA(), result);
			HashMap<Double, HashMap<RealStrat, ResultSim_new>> mapDoubleMap = new HashMap<Double, HashMap<RealStrat, ResultSim_new>>();
			mapDoubleMap.put(result.getP_ind_fault(), mapStratRes);
			computedResults.put(result.getP_ind(), mapDoubleMap);
		}else if(computedResults.get(result.getP_ind()).get(result.getP_ind_fault()) == null){
			HashMap<RealStrat, ResultSim_new> mapStratRes = new HashMap<RealStrat, ResultSim_new>();
			mapStratRes.put(result.getStratTeamA(), result);
			computedResults.get(result.getP_ind()).put(result.getP_ind_fault(), mapStratRes);
		}else {
			computedResults.get(result.getP_ind()).get(result.getP_ind_fault()).put(result.getStratTeamA(), result);
		}
		
	}
	
	

	
	public HashMap<Player, Prob_IndPlayer> getPlayerInputProbs() {
		return playerInputProbs;
	}


	public void writeToFile() throws IOException, WriteException{
				
		Calendar calendar = Calendar.getInstance();
		Timestamp currentTimestamp = new java.sql.Timestamp(calendar.getTime().getTime());
		
		File folder = new File("results_SimSeries");
		if (!folder.exists() || !folder.isDirectory()) {
			folder.mkdir();
		}
					
		File file = new File("results_SimSeries/SimSeries_"+ currentTimestamp.toString().replaceAll(" ", "_") +"_" + tech.toString() + "_allMatches.xls");
		
		WorkbookSettings wbSettings = new WorkbookSettings();
	    wbSettings.setLocale(new Locale("en", "EN"));
	    WritableWorkbook workbook = Workbook.createWorkbook(file, wbSettings);
	    
	    /*
	     * Format 
	     */	    
	    // Create cell font and format
	    WritableFont cellBigFont = new WritableFont(WritableFont.ARIAL, 14);
	    cellBigFont.setBoldStyle(WritableFont.BOLD);
	    
	    WritableCellFormat cellBigHeadingFormat = new WritableCellFormat(cellBigFont);
	    
	    WritableFont cellFont = new WritableFont(WritableFont.ARIAL, 10);
	    cellFont.setBoldStyle(WritableFont.BOLD);
	    WritableCellFormat cellHeadingFormat = new WritableCellFormat(cellFont);
	    cellHeadingFormat.setAlignment(Alignment.CENTRE);
	    
	    WritableCellFormat cellHeadingFormatLeft = new WritableCellFormat(cellFont);
	    cellHeadingFormatLeft.setAlignment(Alignment.LEFT);
	    
	    WritableCellFormat cellHeadingRight = new WritableCellFormat(cellFont);
	    cellHeadingRight.setAlignment(Alignment.RIGHT);
	    	    
	    WritableCellFormat cellCentered = new WritableCellFormat();
	    cellCentered.setAlignment(Alignment.CENTRE);
	    
	    WritableCellFormat cellLeft = new WritableCellFormat();
	    cellLeft.setAlignment(Alignment.LEFT);
	    
	    /*------------------------------------------------------------------------------
	     * Sheet Setting
	     * ----------------------------------------------------------------------------- 	    
	     */
	    workbook.createSheet("Settings", 0);
	    WritableSheet excelSheetSettings = workbook.getSheet(0);
	        	    
	    excelSheetSettings.addCell(new Label(1,1,"Simulation Series", cellBigHeadingFormat));
	    excelSheetSettings.addCell(new Label(1,3,"number of Sets per Sim:", cellHeadingRight));
	    excelSheetSettings.addCell(new Number(3,3, numberOfSetsPerSim));
	    excelSheetSettings.addCell(new Label(1,4,"Technique:", cellHeadingRight));
	    excelSheetSettings.addCell(new Label(3,4,tech.toString()));
	    excelSheetSettings.addCell(new Label(1,5,"parameterized Probs:", cellHeadingRight));
	    for (int i = 0; i<changedProb.size(); i++){
	    	excelSheetSettings.addCell(new Label(3+i,5,changedProb.get(i).printName()));
	    }
	    excelSheetSettings.addCell(new Label(1,6,"parameterized Players:", cellHeadingRight));
	    for (int i = 0; i<parameterizedPlayers.size(); i++){
	    	excelSheetSettings.addCell(new Label(3+i,6,parameterizedPlayers.get(i).getName()));
	    }
	    excelSheetSettings.addCell(new Label(1,7,"simulated Strategies:", cellHeadingRight));
	    for (int i = 0; i<simulatedStrategies.size(); i++){
	    	excelSheetSettings.addCell(new Label(3+i,7,simulatedStrategies.get(i).toString()));
	    }	
	    excelSheetSettings.addCell(new Label(1,8,"Player Input Probs:", cellHeadingRight));
	    int k = 0;
	    for (Player player : playerInputProbs.keySet()){
	    	excelSheetSettings.addCell(new Label(3,8+k,player.getName()));
	    	excelSheetSettings.addCell(new Label(4,8+k,playerInputProbs.get(player).printAnalogToExcelInputFormat(player)));
//	    	System.out.println(playerInputProbs.get(player).printAnalogToExcelInputFormat(player));
//	    	System.out.println(playerInputProbs.get(player).toString());
	    	k++;
	    }	 

	    CellView cellView = excelSheetSettings.getColumnView(1);
	    cellView.setAutosize(true);
	    excelSheetSettings.setColumnView(1, cellView);
	    	
	        
	    /*------------------------------------------------------------------------------
	     * Results Serving Statistic
	     * ----------------------------------------------------------------------------- 	    
	     */
	    workbook.createSheet("Results Serves", 1);
	    WritableSheet excelSheetServes = workbook.getSheet(1);
		
	    excelSheetServes.addCell(new Label(0,0,tech.toString(), cellHeadingFormatLeft));
	    excelSheetServes.addCell(new Label(0,1,"p_ind", cellHeadingFormatLeft));
	    excelSheetServes.addCell(new Label(1,1,"p_ind_fault", cellHeadingFormatLeft));
	    
	    for (int i = 0; i<simulatedStrategies.size(); i++){
	    	excelSheetServes.addCell(new Label(4*i+2,0,simulatedStrategies.get(i).toString(), cellHeadingFormatLeft));
	    	excelSheetServes.addCell(new Label(4*i+2,1,"p_serve", cellHeadingFormatLeft));
		    excelSheetServes.addCell(new Label(4*i+3,1,"pOverline_serve", cellHeadingFormatLeft));
		    excelSheetServes.addCell(new Label(4*i+4,1,"q_serve", cellHeadingFormatLeft));
		    excelSheetServes.addCell(new Label(4*i+5,1,"q_Overline_serve", cellHeadingFormatLeft));	    	
	    }
	    
	    int row = 2;
	    for (double p_ind = startValue; p_ind <= endValue + 0.00000000001; p_ind=p_ind+stepSize){	    	
	    	for (double p_ind_fault = startValue; p_ind_fault + p_ind <= endValue + 0.00000001; p_ind_fault = p_ind_fault+stepSize){
	    		writeNumberToExcelSheet(excelSheetServes,row,0,p_ind);
	    		writeNumberToExcelSheet(excelSheetServes,row,1,p_ind_fault);
		    	int column = 1;
		    	for (RealStrat strat : simulatedStrategies){
		    		writeNumberToExcelSheet(excelSheetServes,row,column+1, computedResults.get(p_ind).get(p_ind_fault).get(strat).getP_serve());
			    	writeNumberToExcelSheet(excelSheetServes,row,column+2, computedResults.get(p_ind).get(p_ind_fault).get(strat).getpOverline_serve());
			    	writeNumberToExcelSheet(excelSheetServes,row,column+3, computedResults.get(p_ind).get(p_ind_fault).get(strat).getQ_serve());
			    	writeNumberToExcelSheet(excelSheetServes,row,column+4, computedResults.get(p_ind).get(p_ind_fault).get(strat).getqOverline_serve());
			    	column = column + 4;
		    	} 	
		    	row++;
	    	
	    	}
	    }
	    
	    cellView = excelSheetServes.getColumnView(0);
	    cellView.setAutosize(true);
	    cellView.setFormat(cellLeft);
	    excelSheetServes.setColumnView(0, cellView);
	    for (int i = 1; i<=simulatedStrategies.size()*4; i++){
		    cellView = excelSheetServes.getColumnView(i);
		    cellView.setSize(14*256);
		    cellView.setFormat(cellLeft);
		    excelSheetServes.setColumnView(i, cellView);	    	
	    }
	    
	    /*------------------------------------------------------------------------------
	     * Results Field Attack Statistic
	     * ----------------------------------------------------------------------------- 	    
	     */
	    workbook.createSheet("Results fieldAttack", 1);
	    WritableSheet excelSheetfA = workbook.getSheet(1);
		
	    excelSheetfA.addCell(new Label(0,0,tech.toString(), cellHeadingFormatLeft));
	    excelSheetfA.addCell(new Label(0,1,"p_ind", cellHeadingFormatLeft));
	    excelSheetfA.addCell(new Label(1,1,"p_ind_fault", cellHeadingFormatLeft));
	    
	    for (int i = 0; i<simulatedStrategies.size(); i++){
	    	excelSheetfA.addCell(new Label(4*i+2,0,simulatedStrategies.get(i).toString(), cellHeadingFormatLeft));
	    	excelSheetfA.addCell(new Label(4*i+2,1,"p_fA", cellHeadingFormatLeft));
		    excelSheetfA.addCell(new Label(4*i+3,1,"pOverline_fA", cellHeadingFormatLeft));
		    excelSheetfA.addCell(new Label(4*i+4,1,"q_fA", cellHeadingFormatLeft));
		    excelSheetfA.addCell(new Label(4*i+5,1,"q_Overline_fA", cellHeadingFormatLeft));	    	
	    }
	    
	    row = 2;
	    for (double p_ind = startValue; p_ind <= endValue + 0.00000000001; p_ind=p_ind+stepSize){	    	
	    	for (double p_ind_fault = startValue; p_ind_fault + p_ind <= endValue + 0.00000001; p_ind_fault = p_ind_fault+stepSize){
	    		writeNumberToExcelSheet(excelSheetfA,row,0,p_ind);
	    		writeNumberToExcelSheet(excelSheetfA,row,1,p_ind_fault);
		    	int column = 1;
		    	for (RealStrat strat : simulatedStrategies){
		    		writeNumberToExcelSheet(excelSheetfA,row,column+1, computedResults.get(p_ind).get(p_ind_fault).get(strat).getP_fA());
			    	writeNumberToExcelSheet(excelSheetfA,row,column+2, computedResults.get(p_ind).get(p_ind_fault).get(strat).getpOverline_fA());
			    	writeNumberToExcelSheet(excelSheetfA,row,column+3, computedResults.get(p_ind).get(p_ind_fault).get(strat).getQ_fA());
			    	writeNumberToExcelSheet(excelSheetfA,row,column+4, computedResults.get(p_ind).get(p_ind_fault).get(strat).getqOverline_fA());
			    	column = column + 4;
		    	} 	
		    	row++;
	    	
	    	}
	    }
	    
	    
	    cellView = excelSheetfA.getColumnView(0);
	    cellView.setAutosize(true);
	    cellView.setFormat(cellLeft);
	    excelSheetfA.setColumnView(0, cellView);
	    for (int i = 2; i<=simulatedStrategies.size()*4; i++){
		    cellView = excelSheetfA.getColumnView(i);
		    cellView.setSize(14*256);
		    cellView.setFormat(cellLeft);
		    excelSheetfA.setColumnView(i, cellView);	    	
	    }
	   
	    
	    workbook.write();
	    workbook.close();
	}
	
	private static void writeNumberToExcelSheet(WritableSheet sheet, int row, int column, double number) throws RowsExceededException, WriteException{
		if (Double.isNaN(number)){
			sheet.addCell(new Label(column,row,"#NV"));
		}else{
		sheet.addCell(new Number(column,row,number));
		}
	}
	
}
