package de.ubt.wm.sb.sim;

import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;

import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.write.Label;
import jxl.write.Number;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.ubt.wm.sb.model.Game;

public class SimulationManager {
	
	static final Logger log4j = LogManager.getLogger(SimulationManager.class.getName());
	
	private int wonSetsTeamA;
	private int wonSetsTeamB;
	public float teamA_SetWinningProb;
	
	// serve attack
	public List<Integer> teamA_serveStatistics;
	public List<Integer> teamA_pointServeStatistics;
	public List<Integer> teamA_faultServeStatistics;

	public List<Integer> teamB_serveStatistics;
	public List<Integer> teamB_pointServeStatistics;
	public List<Integer> teamB_faultServeStatistics;

	// field attack
	public List<Integer> teamA_fieldAttackStatistics;
	public List<Integer> teamA_pointFieldAttackStatistics;
	public List<Integer> teamA_faultFieldAttackStatistics;
	
	public List<Integer> teamB_fieldAttackStatistics;
	public List<Integer> teamB_pointFieldAttackStatistics;
	public List<Integer> teamB_faultFieldAttackStatistics;
	
	// serving attacks
	public double[] teamA_servesampleMean; // mean of all three counters (#attacks, #points, #faults)
	public double[] teamA_servesampleSD; // standard deviation of all three counters
	public double[] teamA_servecoVar; // coVar (attacks&points, attacks& faults)
	public double[] teamA_servecorr; // correlation (attacks&points, attacks& faults)
	public double[] teamA_serveR; // mean of Ratio (mean points/mean attacks, mean faults/mean attacks)
	public double[] teamA_serveSDR; // standard deviation of Ratio
	public double[][] teamA_serveCI95; // 95 % confidence interval
	
	public double[] teamB_servesampleMean; // mean of all three counters (#attacks, #points, #faults)
	public double[] teamB_servesampleSD; // standard deviation of all three counters
	public double[] teamB_servecoVar; // coVar (attacks&points, attacks& faults)
	public double[] teamB_servecorr; // correlation (attacks&points, attacks& faults)
	public double[] teamB_serveR; // mean of Ratio (mean points/mean attacks, mean faults/mean attacks)
	public double[] teamB_serveSDR; // standard deviation of Ratio
	public double[][] teamB_serveCI95; // 95 % confidence interval
	
	// field attacks
	public double[] teamA_fieldAttacksampleMean; // mean of all three counters (#attacks, #points, #faults)
	public double[] teamA_fieldAttacksampleSD; // standard deviation of all three counters
	public double[] teamA_fieldAttackcoVar; // coVar (attacks&points, attacks& faults)
	public double[] teamA_fieldAttackcorr; // correlation (attacks&points, attacks& faults)
	public double[] teamA_fieldAttackR; // mean of Ratio (mean points/mean attacks, mean faults/mean attacks)
	public double[] teamA_fieldAttackSDR; // standard deviation of Ratio
	public double[][] teamA_fieldAttackCI95; // 95 % confidence interval
	
	public double[] teamB_fieldAttacksampleMean; // mean of all three counters (#attacks, #points, #faults)
	public double[] teamB_fieldAttacksampleSD; // standard deviation of all three counters
	public double[] teamB_fieldAttackcoVar; // coVar (attacks&points, attacks& faults)
	public double[] teamB_fieldAttackcorr; // correlation (attacks&points, attacks& faults)
	public double[] teamB_fieldAttackR; // mean of Ratio (mean points/mean attacks, mean faults/mean attacks)
	public double[] teamB_fieldAttackSDR; // standard deviation of Ratio
	public double[][] teamB_fieldAttackCI95; // 95 % confidence interval
	
	
	public SimulationManager() {
		this.wonSetsTeamA = 0;
		this.wonSetsTeamB = 0;
		this.teamA_SetWinningProb = -1;
		
		this.teamA_serveStatistics = new ArrayList<Integer>();
		this.teamA_pointServeStatistics = new ArrayList<Integer>();
		this.teamA_faultServeStatistics = new ArrayList<Integer>();

		this.teamB_serveStatistics = new ArrayList<Integer>();
		this.teamB_pointServeStatistics = new ArrayList<Integer>();
		this.teamB_faultServeStatistics = new ArrayList<Integer>();

		this.teamA_fieldAttackStatistics = new ArrayList<Integer>();
		this.teamA_pointFieldAttackStatistics = new ArrayList<Integer>();
		this.teamA_faultFieldAttackStatistics = new ArrayList<Integer>();

		this.teamB_pointFieldAttackStatistics = new ArrayList<Integer>();
		this.teamB_fieldAttackStatistics = new ArrayList<Integer>();
		this.teamB_faultFieldAttackStatistics = new ArrayList<Integer>();
		
		this.teamA_servesampleMean = new double[3];
		this.teamA_servesampleSD = new double[3];
		this.teamA_servecoVar = new double[2];
		this.teamA_servecorr = new double[2];
		this.teamA_serveR = new double[2];
		this.teamA_serveSDR = new double[2];
		this.teamA_serveCI95 = new double[2][2];
		
		this.teamB_servesampleMean = new double[3];
		this.teamB_servesampleSD = new double[3];
		this.teamB_servecoVar = new double[2];
		this.teamB_servecorr = new double[2];
		this.teamB_serveR = new double[2];
		this.teamB_serveSDR = new double[2];
		this.teamB_serveCI95 = new double[2][2];
		
		this.teamA_fieldAttacksampleMean = new double[3];
		this.teamA_fieldAttacksampleSD = new double[3];
		this.teamA_fieldAttackcoVar = new double[2];
		this.teamA_fieldAttackcorr = new double[2];
		this.teamA_fieldAttackR = new double[2];
		this.teamA_fieldAttackSDR = new double[2];
		this.teamA_fieldAttackCI95 = new double[2][2];
		
		this.teamB_fieldAttacksampleMean = new double[3];
		this.teamB_fieldAttacksampleSD = new double[3];
		this.teamB_fieldAttackcoVar = new double[2];
		this.teamB_fieldAttackcorr = new double[2];
		this.teamB_fieldAttackR = new double[2];
		this.teamB_fieldAttackSDR = new double[2];
		this.teamB_fieldAttackCI95 = new double[2][2];
	}
		
	public void reset(){
		this.wonSetsTeamA = 0;
		this.wonSetsTeamB = 0;
		this.teamA_SetWinningProb = -1;
		
		this.teamA_serveStatistics = new ArrayList<Integer>();
		this.teamA_pointServeStatistics = new ArrayList<Integer>();
		this.teamA_faultServeStatistics = new ArrayList<Integer>();

		this.teamB_serveStatistics = new ArrayList<Integer>();
		this.teamB_pointServeStatistics = new ArrayList<Integer>();
		this.teamB_faultServeStatistics = new ArrayList<Integer>();

		this.teamA_fieldAttackStatistics = new ArrayList<Integer>();
		this.teamA_pointFieldAttackStatistics = new ArrayList<Integer>();
		this.teamA_faultFieldAttackStatistics = new ArrayList<Integer>();

		this.teamB_pointFieldAttackStatistics = new ArrayList<Integer>();
		this.teamB_fieldAttackStatistics = new ArrayList<Integer>();
		this.teamB_faultFieldAttackStatistics = new ArrayList<Integer>();
		
		this.teamA_servesampleMean = new double[3];
		this.teamA_servesampleSD = new double[3];
		this.teamA_servecoVar = new double[2];
		this.teamA_servecorr = new double[2];
		this.teamA_serveR = new double[2];
		this.teamA_serveSDR = new double[2];
		this.teamA_serveCI95 = new double[2][2];
		
		this.teamB_servesampleMean = new double[3];
		this.teamB_servesampleSD = new double[3];
		this.teamB_servecoVar = new double[2];
		this.teamB_servecorr = new double[2];
		this.teamB_serveR = new double[2];
		this.teamB_serveSDR = new double[2];
		this.teamB_serveCI95 = new double[2][2];
		
		this.teamA_fieldAttacksampleMean = new double[3];
		this.teamA_fieldAttacksampleSD = new double[3];
		this.teamA_fieldAttackcoVar = new double[2];
		this.teamA_fieldAttackcorr = new double[2];
		this.teamA_fieldAttackR = new double[2];
		this.teamA_fieldAttackSDR = new double[2];
		this.teamA_fieldAttackCI95 = new double[2][2];
		
		this.teamB_fieldAttacksampleMean = new double[3];
		this.teamB_fieldAttacksampleSD = new double[3];
		this.teamB_fieldAttackcoVar = new double[2];
		this.teamB_fieldAttackcorr = new double[2];
		this.teamB_fieldAttackR = new double[2];
		this.teamB_fieldAttackSDR = new double[2];
		this.teamB_fieldAttackCI95 = new double[2][2];
	}
	
	/**
	 * Fuegt die neue attack statistic des letzten Satzes in die Listen ein #attacks, #directPoints, #faults ein
	 * 
	 * @param attacks
	 * @param directPoints
	 * @param directFaults
	 */
	public void addCounterResultToSimManager(AttackCounter counter){
	
		// serves
		teamA_serveStatistics.add(counter.getServesTeamA());
		teamA_pointServeStatistics.add(counter.getDirectPointServesTeamA());
		teamA_faultServeStatistics.add(counter.getFailedServesTeamA());
		
		teamB_serveStatistics.add(counter.getServesTeamB());
		teamB_pointServeStatistics.add(counter.getDirectPointServesTeamB());
		teamB_faultServeStatistics.add(counter.getFailedServesTeamB());
		
		// field attacks
		teamA_fieldAttackStatistics.add(counter.getFieldAttacksTeamA());
		teamA_pointFieldAttackStatistics.add(counter.getDirectPointFieldAttacksTeamA());
		teamA_faultFieldAttackStatistics.add(counter.getFailedFieldAttacksTeamA());
		
		teamB_fieldAttackStatistics.add(counter.getFieldAttacksTeamB());
		teamB_pointFieldAttackStatistics.add(counter.getDirectPointFieldAttacksTeamB());
		teamB_faultFieldAttackStatistics.add(counter.getFailedFieldAttacksTeamB());
	}
	
	/**
	 * Schreibt die Attack Statistik in ein Excel Sheet in dem Ordner results mit angehaengten timestamp an den Dateinamen.
	 * @param filename
	 * @throws IOException
	 * @throws WriteException
	 */
	public void writeResultsToFile(String filename) throws IOException, WriteException{
		Calendar calendar = Calendar.getInstance();
		Timestamp currentTimestamp = new java.sql.Timestamp(calendar.getTime().getTime());
		
		String stratShort = Game.getInstance().getTeamA().getStrat().toStringShort();
		File file = new File("results/"+filename+currentTimestamp+stratShort+"_allMatches.xls");
	    WorkbookSettings wbSettings = new WorkbookSettings();

	    wbSettings.setLocale(new Locale("en", "EN"));

	    WritableWorkbook workbook = Workbook.createWorkbook(file, wbSettings);
	    workbook.createSheet("Team A", 0);
	    WritableSheet excelSheet = workbook.getSheet(0);
	    
	    excelSheet.addCell(new Label(0,0,"Simulation with n = " + teamA_serveStatistics.size()));
	    excelSheet.addCell(new Label(0,1,"Team A"));
	    excelSheet.addCell(new Label(0,2,"Strategie: "));
	    
	    excelSheet.addCell(new Label(1,2,Game.getInstance().getTeamA().getStrat().toString()));
		
		// Field Attacks
		int i = 2;
		excelSheet.addCell(new Label(1,i+2,"FIELD ATTACK statistic"));
		 excelSheet.addCell(new Label(1,i+4,"mean"));excelSheet.addCell(new Label(2,i+4,"sd"));
		    excelSheet.addCell(new Label(0,i+5,"field attacks: "));
		    	excelSheet.addCell(new Number(1,i+5,teamA_fieldAttacksampleMean[0]));
		    	excelSheet.addCell(new Number(2,i+5,teamA_fieldAttacksampleSD[0]));
		    excelSheet.addCell(new Label(0,i+6,"direct points after field attacks: "));
			    excelSheet.addCell(new Number(1,i+6,teamA_fieldAttacksampleMean[1]));
				excelSheet.addCell(new Number(2,i+6,teamA_fieldAttacksampleSD[1]));
		    excelSheet.addCell(new Label(0,i+7,"attack faults after field attacks: "));
			    excelSheet.addCell(new Number(1,i+7,teamA_fieldAttacksampleMean[2]));
				excelSheet.addCell(new Number(2,i+7,teamA_fieldAttacksampleSD[2]));	
				
			excelSheet.addCell(new Label(1,i+9,"points/attacks"));excelSheet.addCell(new Label(2,i+9,"faults/attacks"));
		    excelSheet.addCell(new Label(0,i+10,"R: "));
		    	excelSheet.addCell(new Number(1,i+10,teamA_fieldAttackR[0]));
		    	excelSheet.addCell(new Number(2,i+10,teamA_fieldAttackR[1]));
		    excelSheet.addCell(new Label(0,i+11,"covariance: "));
			    excelSheet.addCell(new Number(1,i+11,teamA_fieldAttackcoVar[0]));
				excelSheet.addCell(new Number(2,i+11,teamA_fieldAttackcoVar[1]));
		    excelSheet.addCell(new Label(0,i+12,"correlation: "));
			    excelSheet.addCell(new Number(1,i+12,teamA_fieldAttackcorr[0]));
				excelSheet.addCell(new Number(2,i+12,teamA_fieldAttackcorr[1]));	
			excelSheet.addCell(new Label(0,i+13,"SDR: "));
			    excelSheet.addCell(new Number(1,i+13,teamA_fieldAttackSDR[0]));
				excelSheet.addCell(new Number(2,i+13,teamA_fieldAttackSDR[1]));	
				
			// Serves
			i = 17;
			excelSheet.addCell(new Label(1,i+2,"SERVE statistic"));
			excelSheet.addCell(new Label(1,i+4,"mean"));excelSheet.addCell(new Label(2,i+4,"sd"));
			    excelSheet.addCell(new Label(0,i+5,"serves: "));
			    	excelSheet.addCell(new Number(1,i+5,teamA_servesampleMean[0]));
			    	excelSheet.addCell(new Number(2,i+5,teamA_servesampleSD[0]));
			    excelSheet.addCell(new Label(0,i+6,"direct points after serve: "));
				    excelSheet.addCell(new Number(1,i+6,teamA_servesampleMean[1]));
					excelSheet.addCell(new Number(2,i+6,teamA_servesampleSD[1]));
			    excelSheet.addCell(new Label(0,i+7,"attack faults after serve: "));
				    excelSheet.addCell(new Number(1,i+7,teamA_servesampleMean[2]));
					excelSheet.addCell(new Number(2,i+7,teamA_servesampleSD[2]));	
					
				excelSheet.addCell(new Label(1,i+9,"points/attacks"));excelSheet.addCell(new Label(2,i+9,"faults/attacks"));
			    excelSheet.addCell(new Label(0,i+10,"R: "));
			    	excelSheet.addCell(new Number(1,i+10,teamA_serveR[0]));
			    	excelSheet.addCell(new Number(2,i+10,teamA_serveR[1]));
			    excelSheet.addCell(new Label(0,i+11,"covariance: "));
				    excelSheet.addCell(new Number(1,i+11,teamA_servecoVar[0]));
					excelSheet.addCell(new Number(2,i+11,teamA_servecoVar[1]));
			    excelSheet.addCell(new Label(0,i+12,"correlation: "));
				    excelSheet.addCell(new Number(1,i+12,teamA_servecorr[0]));
					excelSheet.addCell(new Number(2,i+12,teamA_servecorr[1]));	
				excelSheet.addCell(new Label(0,i+13,"SDR: "));
				    excelSheet.addCell(new Number(1,i+13,teamA_serveSDR[0]));
					excelSheet.addCell(new Number(2,i+13,teamA_serveSDR[1]));	

		
		workbook.createSheet("Team B", 1);
	    excelSheet = workbook.getSheet(1);
		
	    excelSheet.addCell(new Label(0,0,"Simulation with n = " + teamB_serveStatistics.size()));
		excelSheet.addCell(new Label(0,1,"Team B"));
	    excelSheet.addCell(new Label(0,2,"Strategie: "));
	    
	    excelSheet.addCell(new Label(1,2,Game.getInstance().getTeamB().getStrat().toString()));
	    
		// Field Attacks
		i = 2;
		excelSheet.addCell(new Label(1,i+2,"FIELD ATTACK statistic"));
		 excelSheet.addCell(new Label(1,i+4,"mean"));excelSheet.addCell(new Label(2,i+4,"sd"));
		    excelSheet.addCell(new Label(0,i+5,"field attacks: "));
		    	excelSheet.addCell(new Number(1,i+5,teamB_fieldAttacksampleMean[0]));
		    	excelSheet.addCell(new Number(2,i+5,teamB_fieldAttacksampleSD[0]));
		    excelSheet.addCell(new Label(0,i+6,"direct points after field attacks: "));
			    excelSheet.addCell(new Number(1,i+6,teamB_fieldAttacksampleMean[1]));
				excelSheet.addCell(new Number(2,i+6,teamB_fieldAttacksampleSD[1]));
		    excelSheet.addCell(new Label(0,i+7,"attack faults after field attacks: "));
			    excelSheet.addCell(new Number(1,i+7,teamB_fieldAttacksampleMean[2]));
				excelSheet.addCell(new Number(2,i+7,teamB_fieldAttacksampleSD[2]));	
				
			excelSheet.addCell(new Label(1,i+9,"points/attacks"));excelSheet.addCell(new Label(2,i+9,"faults/attacks"));
		    excelSheet.addCell(new Label(0,i+10,"R: "));
		    	excelSheet.addCell(new Number(1,i+10,teamB_fieldAttackR[0]));
		    	excelSheet.addCell(new Number(2,i+10,teamB_fieldAttackR[1]));
		    excelSheet.addCell(new Label(0,i+11,"covariance: "));
			    excelSheet.addCell(new Number(1,i+11,teamB_fieldAttackcoVar[0]));
				excelSheet.addCell(new Number(2,i+11,teamB_fieldAttackcoVar[1]));
		    excelSheet.addCell(new Label(0,i+12,"correlation: "));
			    excelSheet.addCell(new Number(1,i+12,teamB_fieldAttackcorr[0]));
				excelSheet.addCell(new Number(2,i+12,teamB_fieldAttackcorr[1]));	
			excelSheet.addCell(new Label(0,i+13,"SDR: "));
			    excelSheet.addCell(new Number(1,i+13,teamB_fieldAttackSDR[0]));
				excelSheet.addCell(new Number(2,i+13,teamB_fieldAttackSDR[1]));	
				
			// Serves
			i = 17;
			excelSheet.addCell(new Label(1,i+2,"SERVE statistic"));
			excelSheet.addCell(new Label(1,i+4,"mean"));excelSheet.addCell(new Label(2,i+4,"sd"));
			    excelSheet.addCell(new Label(0,i+5,"serves: "));
			    	excelSheet.addCell(new Number(1,i+5,teamB_servesampleMean[0]));
			    	excelSheet.addCell(new Number(2,i+5,teamB_servesampleSD[0]));
			    excelSheet.addCell(new Label(0,i+6,"direct points after serves: "));
				    excelSheet.addCell(new Number(1,i+6,teamB_servesampleMean[1]));
					excelSheet.addCell(new Number(2,i+6,teamB_servesampleSD[1]));
			    excelSheet.addCell(new Label(0,i+7,"attack faults after serves: "));
				    excelSheet.addCell(new Number(1,i+7,teamB_servesampleMean[2]));
					excelSheet.addCell(new Number(2,i+7,teamB_servesampleSD[2]));	
					
				excelSheet.addCell(new Label(1,i+9,"points/attacks"));excelSheet.addCell(new Label(2,i+9,"faults/attacks"));
			    excelSheet.addCell(new Label(0,i+10,"R: "));
			    	excelSheet.addCell(new Number(1,i+10,teamB_serveR[0]));
			    	excelSheet.addCell(new Number(2,i+10,teamB_serveR[1]));
			    excelSheet.addCell(new Label(0,i+11,"covariance: "));
				    excelSheet.addCell(new Number(1,i+11,teamB_servecoVar[0]));
					excelSheet.addCell(new Number(2,i+11,teamB_servecoVar[1]));
			    excelSheet.addCell(new Label(0,i+12,"correlation: "));
				    excelSheet.addCell(new Number(1,i+12,teamB_servecorr[0]));
					excelSheet.addCell(new Number(2,i+12,teamB_servecorr[1]));	
				excelSheet.addCell(new Label(0,i+13,"SDR: "));
				    excelSheet.addCell(new Number(1,i+13,teamB_serveSDR[0]));
					excelSheet.addCell(new Number(2,i+13,teamB_serveSDR[1]));
			
	    workbook.write();
	    workbook.close();
	}
	
	
	/**
	 * nur f�r die Ausgabe/ Stringformatierung zust�ndig, nicht f�r die Berechnung
	 * @return
	 */
	public String printResultsSummary(){		
		StringBuilder s = new StringBuilder();
		
		s.append("Team A Attack statistic n = " + teamA_serveStatistics.size() + "\n");
		s.append("Strategie: " + Game.getInstance().getTeamA().getStrat().toString());
		s.append("\n");
		s.append("Gewonnene Saetze Team A: " + this.wonSetsTeamA);
		s.append("Satz Gewinnwahrscheinlichkeit: ");
		s.append(String.format("%10.10f", teamA_SetWinningProb));
		s.append("\n");
		s.append("\n");
		
		s.append("\n \n ---------------Team A---------------\n");
		s.append("Team A Field Attack statistic n = " + teamA_fieldAttackStatistics.size() + "\n");
		s.append(String.format("%14s %8s %8s %n", "","mean:","sd:"));
		s.append(String.format("%14s %6.6f %6.6f %n", "field attacks", teamA_fieldAttacksampleMean[0], teamA_fieldAttacksampleSD[0]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct points", teamA_fieldAttacksampleMean[1], teamA_fieldAttacksampleSD[1]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct faults", teamA_fieldAttacksampleMean[2], teamA_fieldAttacksampleSD[2]));
		s.append("\n");
		s.append(String.format("%14s %18s %18s %n", "","points/attacks FA:","faults/attacks FA:"));
		s.append(String.format("%14s %10.15f %10.10f %n", "R",teamA_fieldAttackR[0], teamA_fieldAttackR[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "covariances", teamA_fieldAttackcoVar[0], teamA_fieldAttackcoVar[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "correlation", teamA_fieldAttackcorr[0], teamA_fieldAttackcorr[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "SDR", teamA_fieldAttackSDR[0], teamA_fieldAttackSDR[1]));
		s.append("\n");
		s.append("\n");		
		s.append("Team A Serve statistic n = " + teamA_serveStatistics.size() + "\n");
		s.append(String.format("%14s %8s %8s %n", "","mean:","sd:"));
		s.append(String.format("%14s %6.6f %6.6f %n", "serves", teamA_servesampleMean[0], teamA_servesampleSD[0]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct points", teamA_servesampleMean[1], teamA_servesampleSD[1]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct faults", teamA_servesampleMean[2], teamA_servesampleSD[2]));
		s.append("\n");
		s.append(String.format("%14s %18s %18s %n", "","points/serves:","faults/serves:"));
		s.append(String.format("%14s %10.15f %10.10f %n", "R",teamA_serveR[0], teamA_serveR[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "covariances", teamA_servecoVar[0], teamA_servecoVar[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "correlation", teamA_servecorr[0], teamA_servecorr[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "SDR", teamA_serveSDR[0], teamA_serveSDR[1]));
		s.append("\n");
		
		// Serves
		s.append("\n \n ---------------Team B--------------\n");
		s.append("Team B Field Attack statistic n = " + teamB_fieldAttackStatistics.size() + "\n");
		s.append(String.format("%14s %8s %8s %n", "","mean:","sd:"));
		s.append(String.format("%14s %6.6f %6.6f %n", "field attacks", teamB_fieldAttacksampleMean[0], teamB_fieldAttacksampleSD[0]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct points", teamB_fieldAttacksampleMean[1], teamB_fieldAttacksampleSD[1]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct faults", teamB_fieldAttacksampleMean[2], teamB_fieldAttacksampleSD[2]));
		s.append("\n");
		s.append(String.format("%14s %18s %18s %n", "","points/attacks FA:","faults/attacks FA:"));
		s.append(String.format("%14s %10.15f %10.10f %n", "R",teamB_fieldAttackR[0], teamB_fieldAttackR[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "covariances", teamB_fieldAttackcoVar[0], teamB_fieldAttackcoVar[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "correlation", teamB_fieldAttackcorr[0], teamB_fieldAttackcorr[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "SDR", teamB_fieldAttackSDR[0], teamB_fieldAttackSDR[1]));
		s.append("\n");
		s.append("\n");
		s.append("Team B  Serve statistic n = " + teamB_serveStatistics.size() + "\n");
		s.append(String.format("%14s %8s %8s %n", "","mean:","sd:"));
		s.append(String.format("%14s %6.6f %6.6f %n", "serves", teamB_servesampleMean[0], teamB_servesampleSD[0]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct points", teamB_servesampleMean[1], teamB_servesampleSD[1]));
		s.append(String.format("%14s %6.6f %6.6f %n", "direct faults", teamB_servesampleMean[2], teamB_servesampleSD[2]));
		s.append("\n");
		s.append(String.format("%14s %18s %18s %n", "","points/serves:","faults/serves:"));
		s.append(String.format("%14s %10.15f %10.10f %n", "R",teamB_serveR[0], teamB_serveR[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "covariances", teamB_servecoVar[0], teamB_servecoVar[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "correlation", teamB_servecorr[0], teamB_servecorr[1]));
		s.append(String.format("%14s %10.15f %10.10f %n", "SDR", teamB_serveSDR[0], teamB_serveSDR[1]));
		s.append("\n");
 		
		return s.toString();
	}
	
	/**
	 * berechnet die Zusammenfassung der Simulation. Ruft daf�r in der richtigen Reihenfolge die einzelnen Teilrechnungen auf.
	 */	
	public void computeSummary(){
		computeSetWinningProb();
		
		computeServeMean();
		computeServeSD();
		computeServeCovariance();
		computeServeCorr();
		computeServeR();
		computeServeSDR();
		computeServeCI95();
		
		computeFieldAttackMean();
		computeFieldAttackSD();
		computeFieldAttackCovariance();
		computeFieldAttackCorr();
		computeFieldAttackR();
		computeFieldAttackSDR();
		computeFieldAttackCI95();
	}
		
	public double getServeP(){
		return teamA_serveR[0];
	}
	public double getServePOverline(){
		return teamA_serveR[1];
	}
	public double getServeQ(){
		return teamB_serveR[0];
	}
	public double getServeQOverline(){
		return teamB_serveR[1];
	}
	
	public double getfAP(){
		return teamA_fieldAttackR[0];
	}
	public double getfAPOverline(){
		return teamA_fieldAttackR[1];
	}
	public double getfAQ(){
		return teamB_fieldAttackR[0];
	}
	public double getfAQOverline(){
		return teamB_fieldAttackR[1];
	}
	
	private void computeServeMean(){
		// Team A
		teamA_servesampleMean = new double[3];
		for (int i : teamA_serveStatistics){
			teamA_servesampleMean[0] = teamA_servesampleMean[0]+i;
		}
		teamA_servesampleMean[0] = teamA_servesampleMean[0]/teamA_serveStatistics.size();
		
		for (int i : teamA_pointServeStatistics){
			teamA_servesampleMean[1] = teamA_servesampleMean[1]+i;
		}
		teamA_servesampleMean[1] = teamA_servesampleMean[1]/teamA_pointServeStatistics.size();
		
		for (int i : teamA_faultServeStatistics){
			teamA_servesampleMean[2] = teamA_servesampleMean[2]+i;
		}
		teamA_servesampleMean[2] = teamA_servesampleMean[2]/teamA_faultServeStatistics.size();
		
		// Team B
		teamB_servesampleMean = new double[3];
		for (int i : teamB_serveStatistics){
			teamB_servesampleMean[0] = teamB_servesampleMean[0]+i;
		}
		teamB_servesampleMean[0] = teamB_servesampleMean[0]/teamB_serveStatistics.size();
		
		for (int i : teamB_pointServeStatistics){
			teamB_servesampleMean[1] = teamB_servesampleMean[1]+i;
		}
		teamB_servesampleMean[1] = teamB_servesampleMean[1]/teamB_pointServeStatistics.size();
		
		for (int i : teamB_faultServeStatistics){
			teamB_servesampleMean[2] = teamB_servesampleMean[2]+i;
		}
		teamB_servesampleMean[2] = teamB_servesampleMean[2]/teamB_faultServeStatistics.size();
	}
	
	private void computeFieldAttackMean(){
		// Team A
		teamA_fieldAttacksampleMean = new double[3];
		for (int i : teamA_fieldAttackStatistics){
			teamA_fieldAttacksampleMean[0] = teamA_fieldAttacksampleMean[0]+i;
		}
		teamA_fieldAttacksampleMean[0] = teamA_fieldAttacksampleMean[0]/teamA_fieldAttackStatistics.size();
		
		for (int i : teamA_pointFieldAttackStatistics){
			teamA_fieldAttacksampleMean[1] = teamA_fieldAttacksampleMean[1]+i;
		}
		teamA_fieldAttacksampleMean[1] = teamA_fieldAttacksampleMean[1]/teamA_pointFieldAttackStatistics.size();
		
		for (int i : teamA_faultFieldAttackStatistics){
			teamA_fieldAttacksampleMean[2] = teamA_fieldAttacksampleMean[2]+i;
		}
		teamA_fieldAttacksampleMean[2] = teamA_fieldAttacksampleMean[2]/teamA_faultFieldAttackStatistics.size();
		
		// Team B
		teamB_fieldAttacksampleMean = new double[3];
		for (int i : teamB_fieldAttackStatistics){
			teamB_fieldAttacksampleMean[0] = teamB_fieldAttacksampleMean[0]+i;
		}
		teamB_fieldAttacksampleMean[0] = teamB_fieldAttacksampleMean[0]/teamB_fieldAttackStatistics.size();
		
		for (int i : teamB_pointFieldAttackStatistics){
			teamB_fieldAttacksampleMean[1] = teamB_fieldAttacksampleMean[1]+i;
		}
		teamB_fieldAttacksampleMean[1] = teamB_fieldAttacksampleMean[1]/teamB_pointFieldAttackStatistics.size();
		
		for (int i : teamB_faultFieldAttackStatistics){
			teamB_fieldAttacksampleMean[2] = teamB_fieldAttacksampleMean[2]+i;
		}
		teamB_fieldAttacksampleMean[2] = teamB_fieldAttacksampleMean[2]/teamB_faultFieldAttackStatistics.size();
	}
	
	private void computeServeSD(){
		// Team A
		teamA_servesampleSD = new double[3];
		
		for (int i : teamA_serveStatistics){
			teamA_servesampleSD[0] = teamA_servesampleSD[0] + (i - teamA_servesampleMean[0])*(i - teamA_servesampleMean[0]);
		}
		teamA_servesampleSD[0] = Math.sqrt(teamA_servesampleSD[0]/(teamA_serveStatistics.size()-1));
		
		for (int i : teamA_pointServeStatistics){
			teamA_servesampleSD[1] = teamA_servesampleSD[1] + (i - teamA_servesampleMean[1])*(i - teamA_servesampleMean[1]);
		}
		teamA_servesampleSD[1] = Math.sqrt(teamA_servesampleSD[1]/(teamA_pointServeStatistics.size()-1));
		
		for (int i : teamA_faultServeStatistics){
			teamA_servesampleSD[2] = teamA_servesampleSD[2] + (i - teamA_servesampleMean[2])*(i - teamA_servesampleMean[2]);
		}
		teamA_servesampleSD[2] = Math.sqrt(teamA_servesampleSD[2]/(teamA_faultServeStatistics.size()-1));
		
		// Team B
		teamB_servesampleSD = new double[3];
		
		for (int i : teamB_serveStatistics){
			teamB_servesampleSD[0] = teamB_servesampleSD[0] + (i - teamB_servesampleMean[0])*(i - teamB_servesampleMean[0]);
		}
		teamB_servesampleSD[0] = Math.sqrt(teamB_servesampleSD[0]/(teamB_serveStatistics.size()-1));
		
		for (int i : teamB_pointServeStatistics){
			teamB_servesampleSD[1] = teamB_servesampleSD[1] + (i - teamB_servesampleMean[1])*(i - teamB_servesampleMean[1]);
		}
		teamB_servesampleSD[1] = Math.sqrt(teamB_servesampleSD[1]/(teamB_pointServeStatistics.size()-1));
		
		for (int i : teamB_faultServeStatistics){
			teamB_servesampleSD[2] = teamB_servesampleSD[2] + (i - teamB_servesampleMean[2])*(i - teamB_servesampleMean[2]);
		}
		teamB_servesampleSD[2] = Math.sqrt(teamB_servesampleSD[2]/(teamB_faultServeStatistics.size()-1));
	}
	
	private void computeFieldAttackSD(){
		// Team A
		teamA_fieldAttacksampleSD = new double[3];
		
		for (int i : teamA_fieldAttackStatistics){
			teamA_fieldAttacksampleSD[0] = teamA_fieldAttacksampleSD[0] + (i - teamA_fieldAttacksampleMean[0])*(i - teamA_fieldAttacksampleMean[0]);
		}
		teamA_fieldAttacksampleSD[0] = Math.sqrt(teamA_fieldAttacksampleSD[0]/(teamA_fieldAttackStatistics.size()-1));
		
		for (int i : teamA_pointFieldAttackStatistics){
			teamA_fieldAttacksampleSD[1] = teamA_fieldAttacksampleSD[1] + (i - teamA_fieldAttacksampleMean[1])*(i - teamA_fieldAttacksampleMean[1]);
		}
		teamA_fieldAttacksampleSD[1] = Math.sqrt(teamA_fieldAttacksampleSD[1]/(teamA_fieldAttackStatistics.size()-1));
		
		for (int i : teamA_faultFieldAttackStatistics){
			teamA_fieldAttacksampleSD[2] = teamA_fieldAttacksampleSD[2] + (i - teamA_fieldAttacksampleMean[2])*(i - teamA_fieldAttacksampleMean[2]);
		}
		teamA_fieldAttacksampleSD[2] = Math.sqrt(teamA_fieldAttacksampleSD[2]/(teamA_fieldAttackStatistics.size()-1));
		
		// Team B
		teamB_fieldAttacksampleSD = new double[3];
		
		for (int i : teamB_fieldAttackStatistics){
			teamB_fieldAttacksampleSD[0] = teamB_fieldAttacksampleSD[0] + (i - teamB_fieldAttacksampleMean[0])*(i - teamB_fieldAttacksampleMean[0]);
		}
		teamB_fieldAttacksampleSD[0] = Math.sqrt(teamB_fieldAttacksampleSD[0]/(teamB_fieldAttackStatistics.size()-1));
		
		for (int i : teamB_pointFieldAttackStatistics){
			teamB_fieldAttacksampleSD[1] = teamB_fieldAttacksampleSD[1] + (i - teamB_fieldAttacksampleMean[1])*(i - teamB_fieldAttacksampleMean[1]);
		}
		teamB_fieldAttacksampleSD[1] = Math.sqrt(teamB_fieldAttacksampleSD[1]/(teamB_fieldAttackStatistics.size()-1));
		
		for (int i : teamB_faultFieldAttackStatistics){
			teamB_fieldAttacksampleSD[2] = teamB_fieldAttacksampleSD[2] + (i - teamB_fieldAttacksampleMean[2])*(i - teamB_fieldAttacksampleMean[2]);
		}
		teamB_fieldAttacksampleSD[2] = Math.sqrt(teamB_fieldAttacksampleSD[2]/(teamB_fieldAttackStatistics.size()-1));
	}


	private void computeFieldAttackCovariance(){
		// Team A
		teamA_fieldAttackcoVar = new double[2];
		int nA = teamA_fieldAttackStatistics.size();
		for (int i = 0; i<nA; i++){
			teamA_fieldAttackcoVar[0] = teamA_fieldAttackcoVar[0] + teamA_fieldAttackStatistics.get(i)*teamA_pointFieldAttackStatistics.get(i);
			teamA_fieldAttackcoVar[1] = teamA_fieldAttackcoVar[1] + teamA_fieldAttackStatistics.get(i)*teamA_faultFieldAttackStatistics.get(i);
		}
		
		teamA_fieldAttackcoVar[0]=teamA_fieldAttackcoVar[0]-nA*teamA_fieldAttacksampleMean[0]*teamA_fieldAttacksampleMean[1];
		teamA_fieldAttackcoVar[0]=teamA_fieldAttackcoVar[0]/(nA-1);
		teamA_fieldAttackcoVar[1]=teamA_fieldAttackcoVar[1]-nA*teamA_fieldAttacksampleMean[0]*teamA_fieldAttacksampleMean[2];
		teamA_fieldAttackcoVar[1]=teamA_fieldAttackcoVar[1]/(nA-1);
		
		// Team B
		teamB_fieldAttackcoVar = new double[2];
		int nB = teamB_fieldAttackStatistics.size();
		for (int i = 0; i<nB; i++){
			teamB_fieldAttackcoVar[0] = teamB_fieldAttackcoVar[0] + teamB_fieldAttackStatistics.get(i)*teamB_pointFieldAttackStatistics.get(i);
			teamB_fieldAttackcoVar[1] = teamB_fieldAttackcoVar[1] + teamB_fieldAttackStatistics.get(i)*teamB_faultFieldAttackStatistics.get(i);
		}
		
		teamB_fieldAttackcoVar[0]=teamB_fieldAttackcoVar[0]-nB*teamB_fieldAttacksampleMean[0]*teamB_fieldAttacksampleMean[1];
		teamB_fieldAttackcoVar[0]=teamB_fieldAttackcoVar[0]/(nB-1);
		teamB_fieldAttackcoVar[1]=teamB_fieldAttackcoVar[1]-nB*teamB_fieldAttacksampleMean[0]*teamB_fieldAttacksampleMean[2];
		teamB_fieldAttackcoVar[1]=teamB_fieldAttackcoVar[1]/(nB-1);
	}
	
	private void computeServeCovariance(){
		// Team A
		teamA_servecoVar = new double[2];
		int nA = teamA_serveStatistics.size();
		for (int i = 0; i<nA; i++){
			teamA_servecoVar[0] = teamA_servecoVar[0] + teamA_serveStatistics.get(i)*teamA_pointServeStatistics.get(i);
			teamA_servecoVar[1] = teamA_servecoVar[1] + teamA_serveStatistics.get(i)*teamA_faultServeStatistics.get(i);
		}
		
		teamA_servecoVar[0]=teamA_servecoVar[0]-nA*teamA_servesampleMean[0]*teamA_servesampleMean[1];
		teamA_servecoVar[0]=teamA_servecoVar[0]/(nA-1);
		teamA_servecoVar[1]=teamA_servecoVar[1]-nA*teamA_servesampleMean[0]*teamA_servesampleMean[2];
		teamA_servecoVar[1]=teamA_servecoVar[1]/(nA-1);
		
		// Team B
		teamB_servecoVar = new double[2];
		int nB = teamB_serveStatistics.size();
		for (int i = 0; i<nB; i++){
			teamB_servecoVar[0] = teamB_servecoVar[0] + teamB_serveStatistics.get(i)*teamB_pointServeStatistics.get(i);
			teamB_servecoVar[1] = teamB_servecoVar[1] + teamB_serveStatistics.get(i)*teamB_faultServeStatistics.get(i);
		}
		
		teamB_servecoVar[0]=teamB_servecoVar[0]-nB*teamB_servesampleMean[0]*teamB_servesampleMean[1];
		teamB_servecoVar[0]=teamB_servecoVar[0]/(nB-1);
		teamB_servecoVar[1]=teamB_servecoVar[1]-nB*teamB_servesampleMean[0]*teamB_servesampleMean[2];
		teamB_servecoVar[1]=teamB_servecoVar[1]/(nB-1);
	}


	private void computeFieldAttackCorr(){
		// Team A
		teamA_fieldAttackcorr = new double[2];
		
		teamA_fieldAttackcorr[0]=teamA_fieldAttackcoVar[0]/(teamA_fieldAttacksampleSD[0]*teamA_fieldAttacksampleSD[1]);
		teamA_fieldAttackcorr[1]=teamA_fieldAttackcoVar[1]/(teamA_fieldAttacksampleSD[0]*teamA_fieldAttacksampleSD[2]);
		
		// Team B
		teamB_fieldAttackcorr = new double[2];
		
		teamB_fieldAttackcorr[0]=teamB_fieldAttackcoVar[0]/(teamB_fieldAttacksampleSD[0]*teamB_fieldAttacksampleSD[1]);
		teamB_fieldAttackcorr[1]=teamB_fieldAttackcoVar[1]/(teamB_fieldAttacksampleSD[0]*teamB_fieldAttacksampleSD[2]);
	}
	
	private void computeServeCorr(){
		// Team A
		teamA_servecorr = new double[2];
		
		teamA_servecorr[0]=teamA_servecoVar[0]/(teamA_servesampleSD[0]*teamA_servesampleSD[1]);
		teamA_servecorr[1]=teamA_servecoVar[1]/(teamA_servesampleSD[0]*teamA_servesampleSD[2]);
		
		// Team B
		teamB_servecorr = new double[2];
		
		teamB_servecorr[0]=teamB_servecoVar[0]/(teamB_servesampleSD[0]*teamB_servesampleSD[1]);
		teamB_servecorr[1]=teamB_servecoVar[1]/(teamB_servesampleSD[0]*teamB_servesampleSD[2]);
	}
	


	private void computeFieldAttackR(){
		// Team A
		teamA_fieldAttackR = new double[2];
		if (teamA_fieldAttacksampleMean[0] == 0){
			log4j.debug("no field Attacks of Team A observed");
			teamA_fieldAttackR[0]=Double.NaN;
			teamA_fieldAttackR[1]=Double.NaN;
		}else{
			teamA_fieldAttackR[0]=teamA_fieldAttacksampleMean[1]/teamA_fieldAttacksampleMean[0];
			teamA_fieldAttackR[1]=teamA_fieldAttacksampleMean[2]/teamA_fieldAttacksampleMean[0];			
		}
		
		// Team B
		teamB_fieldAttackR = new double[2];
		if (teamA_fieldAttacksampleMean[0] == 0){
			log4j.debug("no field Attacks of Team B observed");
			teamB_fieldAttackR[0]=Double.NaN;
			teamB_fieldAttackR[1]=Double.NaN;
		}else{
			teamB_fieldAttackR[0]=teamB_fieldAttacksampleMean[1]/teamB_fieldAttacksampleMean[0];
			teamB_fieldAttackR[1]=teamB_fieldAttacksampleMean[2]/teamB_fieldAttacksampleMean[0];
		}
	}
	
	
	private void computeServeR(){
		// Team A
		teamA_serveR = new double[2];
		
		if (teamA_servesampleMean[0] == 0){
			log4j.debug("no serves of Team A observed");
			teamA_serveR[0]=Double.NaN;
			teamA_serveR[1]=Double.NaN;
		}else{
			teamA_serveR[0]=teamA_servesampleMean[1]/teamA_servesampleMean[0];
			teamA_serveR[1]=teamA_servesampleMean[2]/teamA_servesampleMean[0];
		}

		
		// Team B
		teamB_serveR = new double[2];
		
		if (teamB_servesampleMean[0] == 0){
			log4j.debug("no serves of Team B observed");
			teamB_serveR[0]=Double.NaN;
			teamB_serveR[1]=Double.NaN;
		}else{
			teamB_serveR[0]=teamB_servesampleMean[1]/teamB_servesampleMean[0];
			teamB_serveR[1]=teamB_servesampleMean[2]/teamB_servesampleMean[0];
		}
	}


	private void computeFieldAttackSDR(){
		// Team A
		teamA_fieldAttackSDR = new double[2];
		
		int n = teamA_fieldAttackStatistics.size();
		
		if(n*teamA_fieldAttacksampleMean[0]*teamA_fieldAttacksampleMean[0]==0){
			log4j.debug("team A SDR Field Attacks kann nicht berechnet werden, da keine Beobachtungen!");
			teamA_fieldAttackSDR[0] = Double.NaN;
			teamA_fieldAttackSDR[1] = Double.NaN;
		}else{
			teamA_fieldAttackSDR[0] = (teamA_fieldAttackR[0]*teamA_fieldAttackR[0]*teamA_fieldAttacksampleSD[0]*teamA_fieldAttacksampleSD[0] + teamA_fieldAttacksampleSD[1]*teamA_fieldAttacksampleSD[1] - 2*teamA_fieldAttackR[0]*teamA_fieldAttackcoVar[0])/(n*teamA_fieldAttacksampleMean[0]*teamA_fieldAttacksampleMean[0]);
			teamA_fieldAttackSDR[0] = Math.sqrt(teamA_fieldAttackSDR[0]);
	
			teamA_fieldAttackSDR[1] = (teamA_fieldAttackR[1]*teamA_fieldAttackR[1]*teamA_fieldAttacksampleSD[0]*teamA_fieldAttacksampleSD[0] + teamA_fieldAttacksampleSD[2]*teamA_fieldAttacksampleSD[2] - 2*teamA_fieldAttackR[1]*teamA_fieldAttackcoVar[1])/(n*teamA_fieldAttacksampleMean[0]*teamA_fieldAttacksampleMean[0]);
			teamA_fieldAttackSDR[1] = Math.sqrt(teamA_fieldAttackSDR[1]);
		}
		
		// Team B
		teamB_fieldAttackSDR = new double[2];
		
		n = teamB_fieldAttackStatistics.size();
		if(n*teamB_fieldAttacksampleMean[0]*teamB_fieldAttacksampleMean[0]==0){
			log4j.debug("team B SDR Field Attacks kann nicht berechnet werden, da keine Beobachtungen!");
			teamB_fieldAttackSDR[0] = Double.NaN;
			teamB_fieldAttackSDR[1] = Double.NaN;
		}else{
			teamB_fieldAttackSDR[0] = (teamB_fieldAttackR[0]*teamB_fieldAttackR[0]*teamB_fieldAttacksampleSD[0]*teamB_fieldAttacksampleSD[0] + teamB_fieldAttacksampleSD[1]*teamB_fieldAttacksampleSD[1] - 2*teamB_fieldAttackR[0]*teamB_fieldAttackcoVar[0])/(n*teamB_fieldAttacksampleMean[0]*teamB_fieldAttacksampleMean[0]);
			teamB_fieldAttackSDR[0] = Math.sqrt(teamB_fieldAttackSDR[0]);
		
			teamB_fieldAttackSDR[1] = (teamB_fieldAttackR[1]*teamB_fieldAttackR[1]*teamB_fieldAttacksampleSD[0]*teamB_fieldAttacksampleSD[0] + teamB_fieldAttacksampleSD[2]*teamB_fieldAttacksampleSD[2] - 2*teamB_fieldAttackR[1]*teamB_fieldAttackcoVar[1])/(n*teamB_fieldAttacksampleMean[0]*teamB_fieldAttacksampleMean[0]);
			teamB_fieldAttackSDR[1] = Math.sqrt(teamB_fieldAttackSDR[1]);
		}
	}
	
	private void computeServeSDR(){
		// Team A
		teamA_serveSDR = new double[2];
		
		int n = teamA_serveStatistics.size();
		if(n*teamA_servesampleMean[0]*teamA_servesampleMean[0]==0){
			log4j.debug("team A SDR Serves kann nicht berechnet werden, da keine Beobachtungen!");
			teamA_serveSDR[0] = -1;
			teamA_serveSDR[1] = -1;
		}else{
			teamA_serveSDR[0] = (teamA_serveR[0]*teamA_serveR[0]*teamA_servesampleSD[0]*teamA_servesampleSD[0] + teamA_servesampleSD[1]*teamA_servesampleSD[1] - 2*teamA_serveR[0]*teamA_servecoVar[0])/(n*teamA_servesampleMean[0]*teamA_servesampleMean[0]);
			teamA_serveSDR[0] = Math.sqrt(teamA_serveSDR[0]);
		
			teamA_serveSDR[1] = (teamA_serveR[1]*teamA_serveR[1]*teamA_servesampleSD[0]*teamA_servesampleSD[0] + teamA_servesampleSD[2]*teamA_servesampleSD[2] - 2*teamA_serveR[1]*teamA_servecoVar[1])/(n*teamA_servesampleMean[0]*teamA_servesampleMean[0]);
			teamA_serveSDR[1] = Math.sqrt(teamA_serveSDR[1]);
		}
		// Team B
		teamB_serveSDR = new double[2];
		
		n = teamB_serveStatistics.size();
		if(n*teamB_servesampleMean[0]*teamB_servesampleMean[0]==0){
			log4j.debug("team B SDR Serves kann nicht berechnet werden, da keine Beobachtungen!");
			teamB_serveSDR[0] = -1;
			teamB_serveSDR[1] = -1;
		}else{
			teamB_serveSDR[0] = (teamB_serveR[0]*teamB_serveR[0]*teamB_servesampleSD[0]*teamB_servesampleSD[0] + teamB_servesampleSD[1]*teamB_servesampleSD[1] - 2*teamB_serveR[0]*teamB_servecoVar[0])/(n*teamB_servesampleMean[0]*teamB_servesampleMean[0]);
			teamB_serveSDR[0] = Math.sqrt(teamB_serveSDR[0]);
		
			teamB_serveSDR[1] = (teamB_serveR[1]*teamB_serveR[1]*teamB_servesampleSD[0]*teamB_servesampleSD[0] + teamB_servesampleSD[2]*teamB_servesampleSD[2] - 2*teamB_serveR[1]*teamB_servecoVar[1])/(n*teamB_servesampleMean[0]*teamB_servesampleMean[0]);
			teamB_serveSDR[1] = Math.sqrt(teamB_serveSDR[1]);
		}
	}


	private void computeFieldAttackCI95(){
		// Team A
		teamA_fieldAttackCI95 = new double[2][2];
		
		teamA_fieldAttackCI95[0][0]=teamA_fieldAttackR[0]-1.96*teamA_fieldAttackSDR[0];
		teamA_fieldAttackCI95[0][1]=teamA_fieldAttackR[0]+1.96*teamA_fieldAttackSDR[0];
		
		teamA_fieldAttackCI95[1][0]=teamA_fieldAttackR[1]-1.96*teamA_fieldAttackSDR[1];
		teamA_fieldAttackCI95[1][1]=teamA_fieldAttackR[1]+1.96*teamA_fieldAttackSDR[1];
		
		
		// Team B
		teamB_fieldAttackCI95 = new double[2][2];
		
		teamB_fieldAttackCI95[0][0]=teamB_fieldAttackR[0]-1.96*teamB_fieldAttackSDR[0];
		teamB_fieldAttackCI95[0][1]=teamB_fieldAttackR[0]+1.96*teamB_fieldAttackSDR[0];
		
		teamB_fieldAttackCI95[1][0]=teamB_fieldAttackR[1]-1.96*teamB_fieldAttackSDR[1];
		teamB_fieldAttackCI95[1][1]=teamB_fieldAttackR[1]+1.96*teamB_fieldAttackSDR[1];
	}
	
	private void computeServeCI95(){
		// Team A
		teamA_serveCI95 = new double[2][2];
		
		teamA_serveCI95[0][0]=teamA_serveR[0]-1.96*teamA_serveSDR[0];
		teamA_serveCI95[0][1]=teamA_serveR[0]+1.96*teamA_serveSDR[0];
		
		teamA_serveCI95[1][0]=teamA_serveR[1]-1.96*teamA_serveSDR[1];
		teamA_serveCI95[1][1]=teamA_serveR[1]+1.96*teamA_serveSDR[1];
		
		
		// Team B
		teamB_serveCI95 = new double[2][2];
		
		teamB_serveCI95[0][0]=teamB_serveR[0]-1.96*teamB_serveSDR[0];
		teamB_serveCI95[0][1]=teamB_serveR[0]+1.96*teamB_serveSDR[0];
		
		teamB_serveCI95[1][0]=teamB_serveR[1]-1.96*teamB_serveSDR[1];
		teamB_serveCI95[1][1]=teamB_serveR[1]+1.96*teamB_serveSDR[1];
	}
	
	public void computeSetWinningProb(){
		this.teamA_SetWinningProb = wonSetsTeamA;
		int numberOfSets = this.wonSetsTeamA + this.wonSetsTeamB;
		teamA_SetWinningProb = teamA_SetWinningProb/numberOfSets;		
	}
	
	public int getWonSetsTeamA() {
		return wonSetsTeamA;
	}

	public int getWonSetsTeamB() {
		return wonSetsTeamB;
	}

	public void increaseWonSetsTeamA() {
		this.wonSetsTeamA++;
	}	
	
	public void increaseWonSetsTeamB() {
		this.wonSetsTeamB++;
	}
	
}
