//////////////////////////////////////
// author: Susanne Hoffmeister
// created: 21.03.18
// gprobsuation of BV Tracker V3 as independent programm
//////////////////////////////////////

if (process.argv.length != 3){
	console.log('########################################\n'
               +'# evaluates gdata to .gprobs.csv which is input for Java Simulation\n'
			   +'# an option for a readable output in latex format is provided if latex variable is set to true in script file\n'
			   +'# must be called with an argument; \n'
			   +'#\t gMDP.gdata\n'
               +'########################################\n');
	console.log('# Example runs for Diss;');
	console.log('\t node evaluate-gdata.js BRA/allMatchesExceptFinal/BRA_allMatchesExceptFinal.gdata\n');
	process.exit(1);
}else{
	var gMDPData_path = process.argv[2];
	var header = '# This file was automatically generated by evaluate-gdata.js\n'
			+ '# evaluates gdata to .gprobs.csv which is input for Java Simulation\n'
			+ '# an option (not default) for a readable output in latex format is provided if latex variable is set to true in script file\n'
			+ '# ------------------------------------\n'
			+ '# gMDP data: ' + gMDPData_path + "\n"
			+ '# ------------------------------------\n';
	console.log(header);
}

var gMDPData_filename = gMDPData_path.substring(gMDPData_path.lastIndexOf("/")+1,gMDPData_path.lastIndexOf("."))

var fs = require('fs');
var output_file = "temp_result/" + Date.now() + gMDPData_filename + ".gprobs.csv";
fs.writeFileSync(output_file, header);


//-------------------
// einlesen
//-------------------

var gMDPData = fs.readFileSync(gMDPData_path).toString().split('\n');

var gMDPDataJSON = new Array();
for (i=0;i<gMDPData.length;i++){
	var entryhelper = gMDPData[i].split(';');
	if (entryhelper.length > 1){
		var e = {
			time: Number(entryhelper[0]),
			posP1: {
				x: Number(entryhelper[1]),
				y: Number(entryhelper[2])
			},
			posP2: {
				x: Number(entryhelper[3]),
				y: Number(entryhelper[4])
			},
			player: Number(entryhelper[5]),
			action: entryhelper[6],
			hardness: (entryhelper[7] == 'true'),
			from: {
				x: Number(entryhelper[8]),
				y: Number(entryhelper[9])
			},
			to: {
				x: Number(entryhelper[10]),
				y: Number(entryhelper[11])
			},
			result: entryhelper[12],
		};
		gMDPDataJSON.push(e);
	}
}

//-----------------------
// Auswertungs settings
//-----------------------
// Standard für Java Input
var estimateStrat = true; // Standard für Java Input: true
var latex = false;		  // Standard für Java Input: false
var renameFields = true;  // Standard für Java Input: true

//-------------------------
// Feldeinteilung
//-------------------------
var vLines = new Array(0,0.21875,0.46875,0.5,0.53125,0.78125,1);
var hLines = new Array(0,0.125,0.5,0.875,1);
var output = "# vLines: ";
output += vLines.toString() +"\n";
output += "# hLines: ";
output += hLines.toString()+"\n";
output += '# ------------------------------------\n';
fs.appendFile(output_file, output,function(err){
	if(err) {
        return console.log(err);
    }

    console.log("Lines saved to file!");
});

//-------------------------
// Schlaege
//-------------------------
const attacks = Array("FSM","FSH","FP");
const serves = Array("ST","SF","SJ","SJF");

//-------------------------
// globale Variablen fuer Strategie
//-------------------------
var Strat_P1_pi_h={
	serve: {
		tech: -1,
		field: -1,
	},
	field: {
		tech: -1,
		field: -1,
	}
};
var Strat_P2_pi_h={
	serve: {
		tech: -1,
		field: -1,
	},
	field: {
		tech: -1,
		field: -1,
	}
};
var pi_b = -1;
var pi_s = -1;
// counter der absoluten Werte
var Strat_SJ_total = new Array(0,0);
var Strat_SF_total = new Array(0,0);
var Strat_Serve_riskyField = new Array(0,0);
var Strat_Serve_safeField = new Array(0,0);

var Strat_FSM_total = new Array(0,0);
var Strat_FP_total = new Array(0,0);
var Strat_field_riskyField = new Array(0,0);
var Strat_field_safeField = new Array(0,0);

var Strat_block_total = new Array(0,0);
var Strat_serve_on_Player = new Array(0,0);
//-------------------------------


//---------------------------------
// Programm Ablauf
//---------------------------------
// erst die StringsEvaluieren, dann calculateStrat! (Beim StringsEvaluieren werden die absoluten Werte für die Strat gezählt!)
var result = stringEval();
var strat_output = calculateStrat(false);

console.log(result);
console.log(strat_output);

// in File schrieben
fs.appendFile(output_file, output,function(err){
	if(err) {
        return console.log(err);
    }

    console.log("Strat saved to file!");
});

fs.appendFile(output_file, result,function(err){
	if(err) {
        return console.log(err);
    }

    console.log("Evaluation result saved to file!");
});
//---------------------------------



//---------------------------------
// Funktionen
//---------------------------------
function calculateStrat(logToConsole) {
	if (logToConsole){
		console.log(Strat_SJ_total);
		console.log(Strat_SF_total);
		console.log(Strat_Serve_riskyField);
		console.log(Strat_Serve_safeField);
		console.log('---------------');
		console.log(Strat_FSM_total);
		console.log(Strat_FP_total);
		console.log(Strat_field_riskyField);
		console.log(Strat_field_safeField);
		console.log('------block-----');
		console.log(Strat_block_total[0] + ', ' + Strat_block_total[1]);
	}

	pi_b = myRoundN(Strat_block_total[0]/(Strat_block_total[0]+Strat_block_total[1]),4);
	pi_s = myRoundN(Strat_serve_on_Player[0]/(Strat_serve_on_Player[0]+Strat_serve_on_Player[1]),4);
	Strat_P1_pi_h.serve.tech = myRoundN(Strat_SJ_total[0]/(Strat_SJ_total[0]+Strat_SF_total[0]),4);
	Strat_P2_pi_h.serve.tech = myRoundN(Strat_SJ_total[1]/(Strat_SJ_total[1]+Strat_SF_total[1]),4);
	Strat_P1_pi_h.serve.field = myRoundN(Strat_Serve_riskyField[0]/(Strat_Serve_riskyField[0]+Strat_Serve_safeField[0]),4);
	Strat_P2_pi_h.serve.field = myRoundN(Strat_Serve_riskyField[1]/(Strat_Serve_riskyField[1]+Strat_Serve_safeField[1]),4);
	Strat_P1_pi_h.field.tech = myRoundN(Strat_FSM_total[0]/(Strat_FSM_total[0]+Strat_FP_total[0]),4);
	Strat_P2_pi_h.field.tech = myRoundN(Strat_FSM_total[1]/(Strat_FSM_total[1]+Strat_FP_total[1]),4);
	Strat_P1_pi_h.field.field = myRoundN(Strat_field_riskyField[0]/(Strat_field_riskyField[0]+Strat_field_safeField[0]),4);
	Strat_P2_pi_h.field.field = myRoundN(Strat_field_riskyField[1]/(Strat_field_riskyField[1]+Strat_field_safeField[1]),4);

	output = '# ------------------------------------\n';
	output +='# estimated Strat:\n'
	output +='# Player: [pi^serve_tech, pi^serve_field,pi^field_tech, pi^field_field]\n'
	output +='# pi_b: ' + pi_b + '\n'
	output +='# P1: [ ' +Strat_P1_pi_h.serve.tech+", " + Strat_P1_pi_h.serve.field+', '+Strat_P1_pi_h.field.tech+', '+ Strat_P1_pi_h.field.field+' ]\n'
	output +='# P2: [ ' +Strat_P2_pi_h.serve.tech+", " + Strat_P2_pi_h.serve.field+', '+Strat_P2_pi_h.field.tech+', '+ Strat_P2_pi_h.field.field+' ]\n'
	output +='# pi_s[on player in Q*1-Q*2]: ' + pi_s + '\n';
	output += '# ------------------------------------\n';

	return output;
}

// übernommen aus BVTracker V3
function stringEval() {
    var resultString = "";
    resultString += '# ------------------------------------\n';
    resultString += "# Total number of records: "+(gMDPDataJSON.length)+"\n";
    // resultString += "#Used lines;\n";
    // resultString += linesToString("#");
    resultString += '# ------------------------------------\n';
    for(var i=1; i<=2 ;i++) {
        if(!latex){
            resultString += "########## Player "+i+" ##########\n";
            resultString += "SF;\n";
            resultString += stringEvalFieldAttack(i,["ST","SF","SJF"]);
            resultString += "SJ;\n";
            resultString += stringEvalFieldAttack(i,["SJ"]);
            resultString += "FSM;\n";
            resultString += stringEvalFieldAttack(i,"FSM");
            resultString += "FSH;\n";
            resultString += stringEvalFieldAttack(i,"FSH");
            resultString += "FP;\n";
            resultString += stringEvalFieldAttack(i,"FP");
            resultString += stringEvalDefense(i);
            resultString += stringEvalBlock(i);
            resultString += "\n";
        }
        else
        {
                resultString += "########## Player "+i+" ##########\n\n\n";
                resultString+= "\\begin{table}[H]\n"
                            + "\\footnotesize\n"
                            + "\\begin{center}\n"
                            + "\\begin{tabular}{ccccccccccc}\n"
                            + "\\toprule\n"
                            + "& target fields & \\multicolumn{3}{c}{\\TeamB11-\\TeamB14} & \\multicolumn{3}{c}{\\TeamB21-\\TeamB24} & \\multicolumn{3}{c}{\\TeamB31-\\TeamB34}\\\\\n"
                            + "& performance & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$\\\\\n"
                            + "\\midrule\n"
                            + "\\multicolumn{11}{c}{Serve}\\\\\n"
                            + "\\midrule\n"
                            + "$S_F$ & \\multirow{2}{*}{\\TeamB01 - \\TeamB04}"
                resultString += stringEvalFieldAttack(i,["ST","SF","SJF"]);
                resultString += "$S_J$ & ";
                resultString += stringEvalFieldAttack(i,["SJ"]);

                resultString += "\\midrule\n"
                             + "\\multicolumn{11}{c}{Field Attack}\\\\\n"
                             + "\\midrule\n"
                             + "\\multirow{4}{*}{$\\Smash$}";
                resultString += stringEvalFieldAttack(i,"FSM");

                resultString += "\\midrule\n"
                                + "\\multirow{4}{*}{$\\EmergencyShot$}";
                resultString += stringEvalFieldAttack(i,"FSH");

                resultString += "\\midrule\n"
                                + "\\multirow{4}{*}{$\\PlannedShot$}";
                resultString += stringEvalFieldAttack(i,"FP");
                resultString += "\\bottomrule"+
                            "\n"+"\\end{tabular}"+
                            "\n"+"\\end{center}"+
                            "\n"+"\\caption{}"+
                            "\n"+"\\label{tab;}"+
                            +"\\end{table}\n\n\n";

                resultString += stringEvalDefense(i);
                resultString += stringEvalBlock(i);
        }
    }
    return resultString;
}

function stringEvalDefense(tPlayer) {
    var dNum = 0.0;
    var dNumSuc = 0.0;
    var dNumDev = 0.0;
    var dNumFault = 0.0;

    var dmNum = 0.0;
    var dmNumSuc = 0.0;
    var dmNumDev = 0.0;
    var dmNumFault = 0.0;

    var dhNum = 0.0;
    var dhNumSuc = 0.0;
    var dhNumDev = 0.0;
    var dhNumFault = 0.0;

    var dmhNum = 0.0;
    var dmhNumSuc = 0.0;
    var dmhNumDev = 0.0;
    var dmhNumFault = 0.0;

    var rNum = 0.0;
    var rNumSuc = 0.0;
    var rNumDev = 0.0;
    var rNumFault = 0.0;

    var rmNum = 0.0;
    var rmNumSuc = 0.0;
    var rmNumDev = 0.0;
    var rmNumFault = 0.0;

    var rhNum = 0.0;
    var rhNumSuc = 0.0;
    var rhNumDev = 0.0;
    var rhNumFault = 0.0;

    var rmhNum = 0.0;
    var rmhNumSuc = 0.0;
    var rmhNumDev = 0.0;
    var rmhNumFault = 0.0;

    var sNum = 0.0;
    var sNumSuc = 0.0;
    var sNumDev = 0.0;
    var sNumFault = 0.0;

    for(var i=0; i<gMDPDataJSON.length; i++) {
        var tData = gMDPDataJSON[i];
        if(tData.player === tPlayer) {
            if(tData.action == "d") {
                if(isMovingAction(tData)) {
                    if(tData.hardness) {
                        dmhNum += 1.0;
                        if(tData.result == "success")
                            dmhNumSuc += 1.0;
                        if(tData.result == "deviation")
                            dmhNumDev += 1.0;
                        if(tData.result == "fault")
                            dmhNumFault += 1.0;
                    }
                    else {
                        dmNum += 1.0;
                        if(tData.result == "success")
                            dmNumSuc += 1.0;
                        if(tData.result == "deviation")
                            dmNumDev += 1.0;
                        if(tData.result == "fault")
                            dmNumFault += 1.0;
                    }
                }
                else {
                    if(tData.hardness) {
                        dhNum += 1.0;
                        if(tData.result == "success")
                            dhNumSuc += 1.0;
                        if(tData.result == "deviation")
                            dhNumDev += 1.0;
                        if(tData.result == "fault")
                            dhNumFault += 1.0;
                    }
                    else {
                        dNum += 1.0;
                        if(tData.result == "success")
                            dNumSuc += 1.0;
                        if(tData.result == "deviation")
                            dNumDev += 1.0;
                        if(tData.result == "fault")
                            dNumFault += 1.0;
                    }
                }
            }
            if(tData.action == "r") {
                if(isMovingAction(tData)) {
                    if(tData.hardness) {
                        rmhNum += 1.0;
                        if(tData.result == "success")
                            rmhNumSuc += 1.0;
                        if(tData.result == "deviation")
                            rmhNumDev += 1.0;
                        if(tData.result == "fault")
                            rmhNumFault += 1.0;
                    }
                    else {
                        rmNum += 1.0;
                        if(tData.result == "success")
                            rmNumSuc += 1.0;
                        if(tData.result == "deviation")
                            rmNumDev += 1.0;
                        if(tData.result == "fault")
                            rmNumFault += 1.0;
                    }
                }
                else {
                    if(tData.hardness) {
                        rhNum += 1.0;
                        if(tData.result == "success")
                            rhNumSuc += 1.0;
                        if(tData.result == "deviation")
                            rhNumDev += 1.0;
                        if(tData.result == "fault")
                            rhNumFault += 1.0;
                    }
                    else {
                        rNum += 1.0;
                        if(tData.result == "success")
                            rNumSuc += 1.0;
                        if(tData.result == "deviation")
                            rNumDev += 1.0;
                        if(tData.result == "fault")
                            rNumFault += 1.0;
                    }
                }
            }
            if(tData.action == "s") {
                sNum += 1.0;
                if(tData.result == "success")
                    sNumSuc += 1.0;
                if(tData.result == "deviation")
                    sNumDev += 1.0;
                if(tData.result == "fault")
                    sNumFault += 1.0;
            }
        }
    }
    var resultString = "";
    if (!latex){
    if(dNum === 0.0)
        resultString += "d;normal;0;-1;-1;\n";
    else
        resultString += "d;normal;"+ dNum + ";" +writeNumberToExcel(dNumSuc/dNum)+";"+writeNumberToExcel(dNumFault/dNum)+"\n";
    if(dhNum === 0.0)
        resultString += "d;hard;0;-1;-1;\n";
    else
        resultString += "d;hard;"+ dhNum + ";" + writeNumberToExcel(dhNumSuc/dhNum)+";"+writeNumberToExcel(dhNumFault/dhNum)+"\n";
    if(dmNum === 0.0)
        resultString += "dm;normal;0;-1;-1;\n";
    else
        resultString += "dm;normal;"+dmNum + ";" + writeNumberToExcel(dmNumSuc/dmNum)+";"+writeNumberToExcel(dmNumFault/dmNum)+"\n";
    if(dmhNum === 0.0)
        resultString += "dm;hard;0;-1;-1;\n";
    else
        resultString += "dm;hard;"+ dmhNum + ";" +writeNumberToExcel(dmhNumSuc/dmhNum)+";"+writeNumberToExcel(dmhNumFault/dmhNum)+"\n";

    if(rNum === 0.0)
        resultString += "r;normal;0;-1;-1;\n";
    else
        resultString += "r;normal;"+rNum+";"+writeNumberToExcel(rNumSuc/rNum)+";"+writeNumberToExcel(rNumFault/rNum)+"\n";
    if(rhNum === 0.0)
        resultString += "r;hard;0;-1;-1;\n";
    else
        resultString += "r;hard;"+rhNum+";"+writeNumberToExcel(rhNumSuc/rhNum)+";"+writeNumberToExcel(rhNumFault/rhNum)+"\n";
    if(rmNum === 0.0)
        resultString += "rm;normal;0;-1;-1;\n";
    else
        resultString += "rm;normal;"+rmNum+";"+writeNumberToExcel(rmNumSuc/rmNum)+";"+writeNumberToExcel(rmNumFault/rmNum)+"\n";
    if(rmhNum === 0.0)
        resultString += "rm;hard;0;-1;-1;\n";
    else
        resultString += "rm;hard;"+rmhNum+ ";" + writeNumberToExcel(rmhNumSuc/rmhNum)+";"+writeNumberToExcel(rmhNumFault/rmhNum)+"\n";

    if(sNum === 0.0)
        resultString += "s;;0;-1;-1;\n";
    else
        resultString += "s;;"+sNum+";"+writeNumberToExcel(sNumSuc/sNum)+";"+writeNumberToExcel(sNumFault/sNum)+"\n";
    } else {
        // Latex Format für Tabelle in Paper
        resultString += "\n\n\n\\begin{table}[htbp]\n\\footnotesize\n\\begin{center}\n\\begin{tabular}{cccccccc}\n\\toprule";
        resultString += "& attack strength & \\multicolumn{3}{c}{\\normal} & \\multicolumn{3}{c}{\\hard}\\\\\n";
        resultString += "\\toprule\n& performance & \\# & $\\success$ & $\\fault$ & \\# & $\\success$ & $\\fault$\\\\ \n";
        resultString += "\\toprule\n";
        if(dNum === 0.0)
            resultString += "\\multirow{2}{*}{Defence} & $d$ & - & - & - &";
        else
            resultString += "\\multirow{2}{*}{Defence} & $d$ & "+dNum+" & "+myRound(dNumSuc/dNum)+" & "+myRound(dNumFault/dNum)+" & ";
        if(dhNum === 0.0)
            resultString += "- & - & - \\\\ \n";
        else
            resultString += dhNum+" & "+myRound(dhNumSuc/dhNum)+" & "+myRound(dhNumFault/dhNum)+"\\\\ \n";
        if(dmNum === 0.0)
            resultString += " & $d_m$ & - & - & - &";
        else
            resultString += " & $d_m$ &"+dmNum+" & "+myRound(dmNumSuc/dmNum)+" & "+myRound(dmNumFault/dmNum)+" & ";
        if(dmhNum === 0.0)
            resultString += "- & - & - \\\\ \n";
        else
            resultString += dmhNum+" & "+myRound(dmhNumSuc/dmhNum)+" & "+myRound(dmhNumFault/dmhNum)+"\\\\ \n";

        resultString += "\\toprule \n";
        if(rNum === 0.0)
            resultString += "\\multirow{2}{*}{Reception} & $r$ & - & - & - &";
        else
            resultString += "\\multirow{2}{*}{Reception} & $r$ & "+rNum+" & "+myRound(rNumSuc/rNum)+" & "+myRound(rNumFault/rNum)+" & ";
        if(rhNum === 0.0)
            resultString += "- & - & - \\\\ \n";
        else
            resultString += rhNum+" & "+myRound(rhNumSuc/rhNum)+" & "+myRound(rhNumFault/rhNum)+"\\\\ \n";
        if(rmNum === 0.0)
            resultString += " & $r_m$ & - & - & - &";
        else
            resultString += " & $r_m$ &"+rmNum+" & "+myRound(rmNumSuc/rmNum)+" & "+myRound(rmNumFault/rmNum)+" & ";
        if(rmhNum === 0.0)
            resultString += "- & - & - \\\\ \n";
        else
            resultString += rmhNum+" & "+myRound(rmhNumSuc/rmhNum)+" & "+myRound(rmhNumFault/rmhNum)+"\\\\ \n";

        resultString += "\\toprule \n";
        if(sNum === 0.0)
            resultString += "Set & $s$ & - & - & - & - & - & - \\\\ \n";
        else
            resultString += "Set & $s$ & "+sNum+" & "+myRound(sNumSuc/sNum)+" & "+myRound(sNumFault/sNum)+" & - & - & - \\\\ \n";
        resultString += "\\toprule\n";
        resultString += "\\end{tabular}\n\\end{center}\n\\caption{ }\\label{}\n\\end{table}\n\n\n";
    }
    return resultString;
}

// gprobsuiert ob sich Spieler weit entfernt vom Ball befindet
function isMovingAction(tData) {
    if(tData.player === 1) {
    	// horizontale Bewegung
        for(var i=0; i<hLines.length; i++) {
            var border = hLines[i];
            if(tData.posP1.y >= border) {
                if(tData.from.y < border) {
                    return true;
                }
            }
            else {
                if(tData.from.y >= border) {
                    return true;
                }
            }
        }
        // vertikale Bewegung
        for(var i=0; i<vLines.length; i++) {
            var border = vLines[i];
            if(tData.posP1.x >= border) {
                if(tData.from.x < border) {
                    return true;
                }
            }
            else {
                if(tData.from.x >= border) {
                    return true;
                }
            }
        }
    }
    else {
        for(var i=0; i<hLines.length; i++) {
            var border = hLines[i];
            if(tData.posP2.y >= border) {
                if(tData.from.y < border) {
                    return true;
                }
            }
            else {
                if(tData.from.y >= border) {
                    return true;
                }
            }
        }
        for(var i=0; i<vLines.length; i++) {
            var border = vLines[i];
            if(tData.posP2.x >= border) {
                if(tData.from.x < border) {
                    return true;
                }
            }
            else {
                if(tData.from.x >= border) {
                    return true;
                }
            }
        }
    }
    return false;
}

function stringEvalBlock(tPlayer) {
    var bNum = 0.0;
    var bNumGood = 0.0;
    var bNumOk = 0.0;
    var bNumBad = 0.0;
    var bNumNo = 0.0;
    for(var i=0; i<gMDPDataJSON.length; i++) {
        var tData = gMDPDataJSON[i];
        if(tData.player === tPlayer) {
            if(tData.action == "b" || tData.action == "bl" || tData.action == "bd") {
                bNum += 1.0;
                if(tData.result == "goodBlock")
                    bNumGood += 1.0;
                if(tData.result == "okBlock")
                    bNumOk += 1.0;
                if(tData.result == "badBlock")
                    bNumBad += 1.0;
                if(tData.result == "noBlock")
                    bNumNo += 1.0;
            }
        }
    }
    var resultString = "";
    if (!latex){
        if(bNum === 0.0)
            resultString += "b;;0;-1;-1;\n";
        else
            resultString += "b;;"+bNum+";"+writeNumberToExcel(bNumGood/bNum)+";"+writeNumberToExcel(bNumOk/bNum)+";"+writeNumberToExcel(bNumBad/bNum)+";"+writeNumberToExcel(bNumNo/bNum)+"\n";
    }else{
        if(bNum === 0.0)
            resultString += "b;0;-1;-1;\n";
        else
            resultString += "$b$ & "+bNum + " & " + myRound(bNumGood/bNum)+" & "+myRound(bNumOk/bNum)+" & "+myRound(bNumBad/bNum)+" & "+myRound(bNumNo/bNum)+"\\\\ \n";
        }

    Strat_block_total[tPlayer-1]=bNum;
    return resultString;
}

function stringEvalFieldAttack(player, attack) {
    var numColsP = 1;
    for(var i=0; i<vLines.length; i++) {
        if(vLines[i]<0.49999999999999)
            numColsP++;
    }
    var numColsQ = 1;
    for(var i=0; i<vLines.length; i++) {
        if(vLines[i]>0.5000000000001)
            numColsQ++;
    }
    var numRows = hLines.length+1;

    // anlegen der Arrays für Anzahlen
    totalNum = new Array(numColsP);
    var successNum = new Array(numColsP);
    var devNum = new Array(numColsP);
    var faultNum = new Array(numColsP);
    for (var i = 0; i < numColsP; i++){
        totalNum[i] = new Array(numRows);
        successNum[i] = new Array(numRows);
        devNum[i] = new Array(numRows);
        faultNum[i] = new Array(numRows);
        for (var j = 0; j < numRows; j++){
            totalNum[i][j] = new Array(numColsQ);
            successNum[i][j] = new Array(numColsQ);
            devNum[i][j] = new Array(numColsQ);
            faultNum[i][j] = new Array(numColsQ);
            for (var k=0; k < numColsQ; k++){
                totalNum[i][j][k] = new Array(numRows);
                successNum[i][j][k] = new Array(numRows);
                devNum[i][j][k] = new Array(numRows);
                faultNum[i][j][k] = new Array(numRows);
                for (var l = 0; l < numRows; l++){
                    totalNum[i][j][k][l] = 0;
                    successNum[i][j][k][l] = 0;
                    devNum[i][j][k][l] = 0;
                    faultNum[i][j][k][l] = 0;
                }
            }
        }
    }

    // Auszählen der Datenpunkte
    for(var p1 = 0; p1 < numColsP; p1++) {
        var leftBorderFrom = p1 === 0? -2.0 : vLines[p1-1];
        var rightBorderFrom = p1 === (numColsP-1)?  0.5 : vLines[p1];
        for(var p2 = 0; p2 < numRows; p2++) {
            var upperBorderFrom = p2 === (numRows-1)?   -2.0 : hLines[numRows-p2-2];
            var lowerBorderFrom = p2 === 0? 2.0:  hLines[numRows-p2-1];
            for(var q1 = 0; q1 < numColsQ; q1++) {
                var rightBorderTo = q1 === 0?   2.0:    vLines[vLines.length-q1];
                var leftBorderTo = q1 === (numColsQ-1)? 0.5:   vLines[vLines.length-1-q1];
                for(var q2 = 0; q2<numRows; q2++) {
                    var upperBorderTo = q2 === 0? -2.0: hLines[q2-1];
                    var lowerBorderTo = q2 === (numRows-1)? 2.0:    hLines[q2];
                    for(var i = 0; i<gMDPDataJSON.length; i++) {
                        var tData = gMDPDataJSON[i];
                        if(tData.player === player && isInArray(tData.action,attack) &&
                                tData.from.x >= leftBorderFrom && tData.from.x < rightBorderFrom &&
                                tData.from.y >= upperBorderFrom && tData.from.y < lowerBorderFrom &&
                                tData.to.x >= leftBorderTo && tData.to.x < rightBorderTo &&
                                tData.to.y >= upperBorderTo && tData.to.y < lowerBorderTo) {
                            totalNum[p1][p2][q1][q2] = totalNum[p1][p2][q1][q2]+1.0;
                            if(tData.result == "success")
                                successNum[p1][p2][q1][q2] = successNum[p1][p2][q1][q2]+1.0;
                            if(tData.result == "deviation")
                                devNum[p1][p2][q1][q2] = devNum[p1][p2][q1][q2]+1.0;
                            if(tData.result == "fault")
                                faultNum[p1][p2][q1][q2] = faultNum[p1][p2][q1][q2]+1.0;
                        }else{
                        	// console.log("NO");
                        }
                    }
                }
            }
        }
    }

    if (estimateStrat){
    	for (var p1=0; p1<totalNum.length; p1++){
    			for (var p2=0; p2<totalNum[p1].length; p2++){
    				for (var q1=1; q1<=3; q1++){
    					for (var q2=1; q2<=4; q2++){
    						if (attack=="SJ"){
    							Strat_SJ_total[player-1]+=totalNum[p1][p2][q1][q2];
	    						// Randfeld
	    						if(q2==1 || q2 == 4){
	    							Strat_Serve_riskyField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// kein Randfeld
	    						if(q2==2 || q2 == 3){
	    							Strat_Serve_safeField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// on Player on right half = Q*1-Q*2
	    						if (q2==1 || q2 == 2){
	    							Strat_serve_on_Player[0]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// on Player on left half = Q*3-Q*4
	    						if (q2==3 || q2 == 4){
	    							Strat_serve_on_Player[1]+=totalNum[p1][p2][q1][q2];
	    						}
    						}
    						if (arraysEqual(attack,["ST","SF","SJF"])){
    							Strat_SF_total[player-1]+=totalNum[p1][p2][q1][q2];
	    						// Randfeld
	    						if(q2==1 || q2 == 4){
	    							Strat_Serve_riskyField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// kein Randfeld
	    						if(q2==2 || q2 == 3){
	    							Strat_Serve_safeField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// on Player on right half = Q*1-Q*2
	    						if (q2==1 || q2 == 2){
	    							Strat_serve_on_Player[0]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// on Player on left half = Q*3-Q*4
	    						if (q2==3 || q2 == 4){
	    							Strat_serve_on_Player[1]+=totalNum[p1][p2][q1][q2];
	    						}
    						}
    						if (attack=="FSM"){
    							Strat_FSM_total[player-1]+=totalNum[p1][p2][q1][q2];
	    						// Randfeld
	    						if(q2==1 || q2 == 4){
	    							Strat_field_riskyField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// kein Randfeld
	    						if(q2==2 || q2 == 3){
	    							Strat_field_safeField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
    						}
    						if (attack=="FP"){
    							Strat_FP_total[player-1]+=totalNum[p1][p2][q1][q2];
	    						// Randfeld
	    						if(q2==1 || q2 == 4){
	    							Strat_field_riskyField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
	    						// kein Randfeld
	    						if(q2==2 || q2 == 3){
	    							Strat_field_safeField[player-1]+=totalNum[p1][p2][q1][q2];
	    						}
    						}
    					}
    				}
    			}
    		}
    }

    var resultString = "";

    if (!renameFields){
	    if(!latex){
	        // keine Felder umbenennen und kein latex
	        for (var p1 = 0; p1 < numColsP; p1++){
	            for (var p2 = 0; p2 < numRows; p2++){
	                for (var q1=0; q1 < numColsQ; q1++){
	                    for (var q2=0; q2<numRows; q2++){
	                        if (totalNum[p1][p2][q1][q2]==0){
	                        // resultString += "P"+p1+","+p2+";Q"+q1+","+q2+";";
	                        // resultString+= "0;-1;-1;;"
	                        }else{
	                            resultString += "P"+p1+";"+p2+";Q"+q1+";"+q2+";";
	                            resultString+= totalNum[p1][p2][q1][q2] + ";"+(successNum[p1][p2][q1][q2]/totalNum[p1][p2][q1][q2])+";"+
	                            //(devNum[p1][p2][q1][q2]/totalNum[p1][p2][q1][q2])+";"+
	                            (faultNum[p1][p2][q1][q2]/totalNum[p1][p2][q1][q2])+"\n";
	                        }
	                    }
	                }
	            }
	        }
	    }else{
	    	console.log("WARNING: Latex ohne Feldzusammenfassungen (renameFields=true) macht keinen Sinn! nicht implementiert!");
	    	process.exit();
	    }
    }

    // den Abschnitt brauchte mal mal für den Input in die calibrated.xls Datei.
    // if (renameFields && !latex && estimateStrat){
    //     // Felder umbenennen und Latex
    //     // ab hier identisch zu Latex
    //     if (arraysEqual(attack,["ST","SF","SJF"]) || arraysEqual(attack,["SJ"])){
    //         totalNum_sum = new Array(3);
    //         succNum_sum = new Array(3);
    //         faultNum_sum = new Array(3);
    //         // Spaltenweise
    //         for (var q2 = 1; q2<=4; q2++){
    //             totalNum_sum[q2-1] = 0;
    //             succNum_sum[q2-1] = 0;
    //             faultNum_sum[q2-1] = 0;
    //             for (var p2 = 1; p2 <=4; p2++){
    //                 // Summe über die Zeilen
    //                 for(var q1 = 1; q1<=3; q1++){
    //                 	// machmal ist Ball schon im Feld aufgenommen worden, war aber ein gültiger Aufschlag deswegen mitzählen
    //                 	for (var p1=0;p1<=1; p1++){
	   //                  	totalNum_sum[q2-1] += totalNum[p1][p2][q1][q2];
	   //                      succNum_sum[q2-1] += successNum[p1][p2][q1][q2];
	   //                      faultNum_sum[q2-1] += faultNum[p1][p2][q1][q2];
    //                 	}
    //                 }
    //             }
    //         }
    //         // ab hier anders
    //         for (var i=0; i<totalNum_sum.length; i++){
    //             resultString+= "P01-P04;Q1"+(i+1)+"-Q3"+(i+1)+";"
    //             if (totalNum_sum[i] == 0){
    //                 resultString+= "0;-1;-1;\n";
    //             }else{
    //                 resultString+= ""
    //                 + totalNum_sum[i] + ";"
    //                 + myRound(succNum_sum[i]/totalNum_sum[i])
    //                 + ";" + myRound(faultNum_sum[i]/totalNum_sum[i])
    //                 + "\n";
    //             }
    //         }
    //     }
    //     // wie unten bei Latex
    //     if (isInArray(attack,["FSM", "FSH", "FP"])){
    //         // für jede Spalte ein Array
    //         totalNum_sum = new Array(4);
    //         succNum_sum = new Array(4);
    //         faultNum_sum = new Array(4);
    //         // for (var p1=0; p1<4; p1++){
    //         //     totalNum_sum[p1] = new Array(3);
    //         //     succNum_sum[p1] = new Array(3);
    //         //     faultNum_sum[p1] = new Array(3);
    //         // }


    //         // for (var p1=1; p1<4; p1++){
    //         // für jede Spalte im Zielfeld
    //         for (var q2 = 1; q2<=4; q2++){
    //             totalNum_sum[q2-1] = 0;
    //             succNum_sum[q2-1] = 0;
    //             faultNum_sum[q2-1] = 0;
    //             // Summe über alle von-Felder
    //             for (var p1=1; p1<4; p1++){
    //                 for (var p2 = 1; p2 <=4; p2++){
    //                     // Summe über alle Zeilen im Zielfeld
    //                     for(var q1 = 1; q1<=3; q1++){
    //                         totalNum_sum[q2-1] += totalNum[p1][p2][q1][q2];
    //                         succNum_sum[q2-1] += successNum[p1][p2][q1][q2];
    //                         faultNum_sum[q2-1] += faultNum[p1][p2][q1][q2];
    //                     }
    //                 }
    //             }
    //             // VonFeld P00-P05
    //             for (var p2 = 0; p2 <=5; p2++){
    //                 for(var q1 = 1; q1<=3; q1++){
    //                     totalNum_sum[q2-1] += totalNum[0][p2][q1][q2];
    //                     succNum_sum[q2-1] += successNum[0][p2][q1][q2];
    //                     faultNum_sum[q2-1] += faultNum[0][p2][q1][q2];
    //                 }
    //             }
    //             // VonFelder P10-P30 & P15-P35
    //             for (var p1 = 1; p1<=3; p1++){
    //                 for(var q1 = 1; q1<=3; q1++){
    //                     totalNum_sum[q2-1] += totalNum[p1][5][q1][q2] + totalNum[p1][0][q1][q2];
    //                     succNum_sum[q2-1] += successNum[p1][5][q1][q2] + successNum[p1][0][q1][q2];
    //                     faultNum_sum[q2-1] += faultNum[p1][5][q1][q2] + faultNum[p1][0][q1][q2];
    //                 }
    //             }
    //         }

    //         // ab hier anders
    //         // for (var p1=0; p1<4; p1++){
    //             for (var i=0; i<totalNum_sum.length; i++){
    //                 // if (p1==0)
    //                 //     resultString+= "out,Q"+i+"1-Q"+i+"4;"
    //                 // if (p1==1)
    //                 //     resultString+= "P11-P14,Q"+i+"1-Q"+i+"4;"
    //                 // if (p1==2)
    //                 //     resultString+= "P21-P24,Q"+i+"1-Q"+i+"4;"
    //                 // if (p1==3)
    //                     resultString+= "P**-P**;Q1"+(i+1)+"-Q3"+(i+1)+";"

    //                 if (totalNum_sum[i] == 0){
    //                     resultString+= "0;-1;-1;\n";
    //                 }else{
    //                     resultString+= "" + totalNum_sum[i] + ";"
    //                     + myRound(succNum_sum[i]/totalNum_sum[i])
    //                     + ";" + myRound(faultNum_sum[i]/totalNum_sum[i]) + "\n";
    //                 }
    //             }
    //         // }
    //     }
    // }


    if (renameFields && !latex){
        // Felder umbenennen und Latex
        // ab hier identisch zu Latex
        if (arraysEqual(attack,["ST","SF","SJF"]) || arraysEqual(attack,["SJ"])){
            totalNum_sum = new Array(3);
            succNum_sum = new Array(3);
            faultNum_sum = new Array(3);
            for (var q1 = 1; q1<=3; q1++){
                totalNum_sum[q1-1] = 0;
                succNum_sum[q1-1] = 0;
                faultNum_sum[q1-1] = 0;
                for (var p2 = 1; p2 <=4; p2++){
                    for(var q2 = 1; q2<=4; q2++){
                    	// MERKE: auch wenn Ball schon vor der Linie war mitzählen
                    	for (p1=0; p1<=1; p1++){
	                        totalNum_sum[q1-1] += totalNum[p1][p2][q1][q2];
	                        succNum_sum[q1-1] += successNum[p1][p2][q1][q2];
	                        faultNum_sum[q1-1] += faultNum[p1][p2][q1][q2];
	                	}
                    }
                }
            }
            // ab hier anders
            for (var i=0; i<totalNum_sum.length; i++){
                resultString+= "P01-P04;Q"+i+"1-Q"+(i+1)+"4;"
                if (totalNum_sum[i] == 0){
                    resultString+= "0;-1;-1;\n";
                }else{
                    resultString+= ""
                    + totalNum_sum[i] + ";"
                    + writeNumberToExcel(succNum_sum[i]/totalNum_sum[i])
                    + ";" + writeNumberToExcel(faultNum_sum[i]/totalNum_sum[i])
                    + "\n";
                }
            }
        }

        // wie unten bei Latex
        if (isInArray(attack,["FSM", "FSH", "FP"])){
            totalNum_sum = new Array(4);
            succNum_sum = new Array(4);
            faultNum_sum = new Array(4);
            for (var p1=0; p1<4; p1++){
                totalNum_sum[p1] = new Array(3);
                succNum_sum[p1] = new Array(3);
                faultNum_sum[p1] = new Array(3);
            }

            for (var p1=1; p1<4; p1++){
                for (var q1 = 1; q1<=3; q1++){
                    totalNum_sum[p1][q1-1] = 0;
                    succNum_sum[p1][q1-1] = 0;
                    faultNum_sum[p1][q1-1] = 0;
                    for (var p2 = 1; p2 <=4; p2++){
                        for(var q2 = 1; q2<=4; q2++){
                            totalNum_sum[p1][q1-1] += totalNum[p1][p2][q1][q2];
                            succNum_sum[p1][q1-1] += successNum[p1][p2][q1][q2];
                            faultNum_sum[p1][q1-1] += faultNum[p1][p2][q1][q2];
                        }
                    }
                }
            }
            // Spezialfall Von-Feld ist Ausfeld
            for (var q1 = 1; q1<=3; q1++){
                // Aufsummieren nach Q1*, Q2*, Q3*
                totalNum_sum[0][q1-1] = 0;
                succNum_sum[0][q1-1] = 0;
                faultNum_sum[0][q1-1] = 0;
                // VonFeld P00-P05
                for (var p2 = 0; p2 <=5; p2++){
                    for(var q2 = 1; q2<=4; q2++){
                        totalNum_sum[0][q1-1] += totalNum[0][p2][q1][q2];
                        succNum_sum[0][q1-1] += successNum[0][p2][q1][q2];
                        faultNum_sum[0][q1-1] += faultNum[0][p2][q1][q2];
                    }
                }
                // VonFelder P10-P30 & P15-P35
                for (var p1 = 1; p1<=3; p1++){
                    for(var q2 = 1; q2<=4; q2++){
                        totalNum_sum[0][q1-1] += totalNum[p1][5][q1][q2] + totalNum[p1][0][q1][q2];
                        succNum_sum[0][q1-1] += successNum[p1][5][q1][q2] + successNum[p1][0][q1][q2];
                        faultNum_sum[0][q1-1] += faultNum[p1][5][q1][q2] + faultNum[p1][0][q1][q2];
                    }
                }
            }

            // ab hier anders
            for (var p1=0; p1<4; p1++){
                for (var i=0; i<totalNum_sum[p1].length; i++){
                    if (p1==0)
                        resultString+= "out;Q"+i+"1-Q"+i+"4;"
                    if (p1==1)
                        resultString+= "P11-P14;Q"+i+"1-Q"+i+"4;"
                    if (p1==2)
                        resultString+= "P21-P24;Q"+i+"1-Q"+i+"4;"
                    if (p1==3)
                        resultString+= "P31-P34;Q"+i+"1-Q"+i+"4;"

                    if (totalNum_sum[p1][i] == 0){
                        resultString+= "0;-1;-1;\n";
                    }else{
                        resultString+= "" + totalNum_sum[p1][i] + ";"
                        + writeNumberToExcel(succNum_sum[p1][i]/totalNum_sum[p1][i])
                        + ";" + writeNumberToExcel(faultNum_sum[p1][i]/totalNum_sum[p1][i]) + "\n";
                    }
                }
            }
        }
    }

    if (renameFields && latex){
        // Felder umbenennen und Latex
        if (arraysEqual(attack,["ST","SF","SJF"]) || arraysEqual(attack,["SJ"])){
            totalNum_sum = new Array(3);
            succNum_sum = new Array(3);
            faultNum_sum = new Array(3);
            for (var q1 = 1; q1<=3; q1++){
                totalNum_sum[q1-1] = 0;
                succNum_sum[q1-1] = 0;
                faultNum_sum[q1-1] = 0;
                for (var p2 = 1; p2 <=4; p2++){
                    for(var q2 = 1; q2<=4; q2++){
                    	// MERKE: auch wenn Ball schon vor der Linie war mitzählen
                    	for (p1=0; p1<=1; p1++){
	                        totalNum_sum[q1-1] += totalNum[p1][p2][q1][q2];
	                        succNum_sum[q1-1] += successNum[p1][p2][q1][q2];
	                        faultNum_sum[q1-1] += faultNum[p1][p2][q1][q2];
	                    }
                    }
                }
            }

            for (var i=0; i<totalNum_sum.length; i++){
                if (totalNum_sum[i] == 0){
                    resultString+= " & - & - & -";
                }else{
                    resultString+= " & " + totalNum_sum[i] + " & " + myRound(succNum_sum[i]/totalNum_sum[i]) + " & " + myRound(faultNum_sum[i]/totalNum_sum[i])
                }
                if (i==totalNum_sum.length-1){
                    resultString+="\\\\\n"
                }
            }
        }
        if (isInArray(attack,["FSM", "FSH", "FP"])){
            totalNum_sum = new Array(4);
            succNum_sum = new Array(4);
            faultNum_sum = new Array(4);
            for (var p1=0; p1<4; p1++){
                totalNum_sum[p1] = new Array(3);
                succNum_sum[p1] = new Array(3);
                faultNum_sum[p1] = new Array(3);
            }

            for (var p1=1; p1<4; p1++){
                for (var q1 = 1; q1<=3; q1++){
                    totalNum_sum[p1][q1-1] = 0;
                    succNum_sum[p1][q1-1] = 0;
                    faultNum_sum[p1][q1-1] = 0;
                    for (var p2 = 1; p2 <=4; p2++){
                        for(var q2 = 1; q2<=4; q2++){
                            totalNum_sum[p1][q1-1] += totalNum[p1][p2][q1][q2];
                            succNum_sum[p1][q1-1] += successNum[p1][p2][q1][q2];
                            faultNum_sum[p1][q1-1] += faultNum[p1][p2][q1][q2];
                        }
                    }
                }
            }
            // Spezialfall Von-Feld ist Ausfeld
            for (var q1 = 1; q1<=3; q1++){
                // Aufsummieren nach Q1*, Q2*, Q3*
                totalNum_sum[0][q1-1] = 0;
                succNum_sum[0][q1-1] = 0;
                faultNum_sum[0][q1-1] = 0;
                // VonFeld P00-P05
                for (var p2 = 0; p2 <=5; p2++){
                    for(var q2 = 1; q2<=4; q2++){
                        totalNum_sum[0][q1-1] += totalNum[0][p2][q1][q2];
                        succNum_sum[0][q1-1] += successNum[0][p2][q1][q2];
                        faultNum_sum[0][q1-1] += faultNum[0][p2][q1][q2];
                    }
                }
                // VonFelder P10-P30 & P15-P35
                for (var p1 = 1; p1<=3; p1++){
                    for(var q2 = 1; q2<=4; q2++){
                        totalNum_sum[0][q1-1] += totalNum[p1][5][q1][q2] + totalNum[p1][0][q1][q2];
                        succNum_sum[0][q1-1] += successNum[p1][5][q1][q2] + successNum[p1][0][q1][q2];
                        faultNum_sum[0][q1-1] += faultNum[p1][5][q1][q2] + faultNum[p1][0][q1][q2];
                    }
                }
            }

            for (var p1=0; p1<4; p1++){
                for (var i=0; i<totalNum_sum[p1].length; i++){
                    if (p1==0 && i==0)
                        resultString+= "& out"
                    if (p1==1 && i==0)
                        resultString+= "& \\TeamA11-\\TeamA14"
                    if (p1==2 && i==0)
                        resultString+= "& \\TeamA21-\\TeamA24"
                    if (p1==3 && i==0)
                        resultString+= "& \\TeamA31-\\TeamA34"

                    if (totalNum_sum[p1][i] == 0){
                        resultString+= " & - & - & -";
                    }else{
                        resultString+= " & " + totalNum_sum[p1][i] + " & " + myRound(succNum_sum[p1][i]/totalNum_sum[p1][i]) + " & " + myRound(faultNum_sum[p1][i]/totalNum_sum[p1][i]);
                    }
                    if (i==totalNum_sum[p1].length-1){
                        resultString+="\\\\\n"
                    }
                }
            }
        }
    }
    return resultString;
}

function writeNumberToExcel(d){
	// return d.toString().replace(/[.]/, ",");
	return d;
}

function isInArray(string, array){
    if (!Array.isArray(array)){
        if(string == array){
            return true;
        }else{
            return false;
        }
    }

    for (var i = 0; i<array.length; i++){
        if (array[i]==string) return true;
    }
    return false;
}

function arraysEqual(arr1, arr2) {
    if(arr1.length !== arr2.length)
        return false;
    for(var i = arr1.length; i--;) {
        if(arr1[i] !== arr2[i])
            return false;
    }

    return true;
}

function myRoundN(zahl,n){
	var fact = 1;
	for (var i=0; i<n; i++){
		fact=fact*10;
	}
    return Math.round(zahl*fact)/fact;
}

function myRound(zahl){
    return Math.round(zahl*100)/100;
}